{DSLstats - a program for monitoring the state of a DSL internet connection
Copyright (C) 2012-2013 Eric Chapman

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License version 2
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

You can contact the author via the website http://dslstats.plainroad.me.uk
}

unit Bitchart1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dateutils, LCLType;

type
  TBitchartType = (btBitloading, btBitswap);
  TLegendPosition = (lpbLeft, lpbRight);
  TBitchart = class(TGraphicControl)
  private
    { Private declarations }
    ABitmap: TBitmap;
    FBitchartType: TBitchartType;
    FBitsArray: Array[0..4095] of Integer;
    FDownTones: Array[0..7] of Integer;
    FFirstTone, FLastTone: Integer;
    FLegendPosition: TLegendPosition;
    FMaxreported: Word;
    FMaxTone, FToneIncrement: Integer;
    FTonesperpixel: Shortint;
    FSNRArray: Array[0..4095] of Single;
    FShowLegend, FSNRpertone, FVDSL2, FScaleSnap: Boolean;
    FSnapHeight, FSnapWidth: Integer;
    FUpColour, FDownColour, FOtherColour, FBitchartColour, FGraphColour, FSNRColour: TColor;
    FUpTones: Array[0..7] of Integer;
    FVersionString: string;
    FXOffset, FYOffset: Integer;
    FYMin, FYMax, FYIncrement: Integer;
    Timestamp, FStartTime: TDateTime;
    XMultiplier: Single;
    function GetBitsArray(Index: Integer): Integer;
    function GetPenColour(Tone: Integer): TColor;
    function GetSNRArray(Index: Integer): Single;
    procedure SetBitchartColour(Value: TColor);
    procedure SetBitchartType(Value: TBitchartType);
    procedure SetBitsArray(Index: Integer; Value: Integer);
    procedure SetDownColour(Value: TColor);
    procedure SetDownTones(Index: Integer; Value: Integer);
    procedure SetFirstTone(Value: Integer);
    procedure SetGraphColour(Value: TColor);
    procedure SetLegendPosition(Value: TLegendPosition);
    procedure SetOtherColour(Value: TColor);
    procedure SetShowLegend(Value: Boolean);
    procedure SetSNRArray(Index: Integer; Value: Single);
    procedure SetSNRColour(Value: TColor);
    procedure SetSNRpertone(Value: Boolean);
    procedure SetStartTime(Value: TDateTime);
    procedure SetToneIncrement(Value: Integer);
    procedure SetTonesperpixel(Value: Shortint);
    procedure SetUpColour(Value: TColor);
    procedure SetUpTones(Index: Integer; Value: Integer);
    procedure SetVDSL2(Value: Boolean);
    procedure SetVersionString(Value: string);
    procedure SetXOffset(Value: Integer);
    procedure SetYIncrement(Value: Integer);
    procedure SetYMax(Value: Integer);
    procedure SetYMin(Value: Integer);
    procedure SetYOffset(Value: Integer);
  protected
    { Protected declarations }
    procedure Paint; override;
    procedure Resize; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetBits(Tone: Integer): Integer;
    function GetSNR(Tone: Integer): Single;
    function SaveToPNGFile(Filename: string): Boolean;
    procedure DrawBitmap(Bmp: TBitmap);
    procedure Drawbits(HighestTone: Integer);
    property BitsArray[Index: Integer]: Integer read GetBitsArray write SetBitsArray;
    property DownTones[Index: Integer]: Integer write SetDownTones;
    property SNRArray[Index: Integer]: Single read GetSNRArray write SetSNRArray;
    property StartTime: TDateTime read FStartTime write SetStartTime;
    property UpTones[Index: Integer]: Integer write SetUpTones;
  published
    { Published declarations }
    property Align default alNone;
    property Anchors;
    property BitchartColour: TColor read FBitchartColour write SetBitchartColour;
    property BitchartType: TBitchartType read FBitchartType write SetBitchartType;
    property Caption;
    property Color;
    property DoSNRpertone: Boolean read FSNRpertone write SetSNRpertone;
    property DownColour: TColor read FDownColour write SetDownColour;
    property DragCursor;
    property DragMode;
    property Enabled;
    property FirstTone: Integer read FFirstTone write SetFirstTone;
    property Font;
    property GraphColour: TColor read FGraphColour write SetGraphColour;
    property LegendPosition: TLegendPosition read FLegendPosition write SetLegendPosition;
    property Maxreported: Word read FMaxreported write FMaxreported;
    property MaxTone: Integer read FMaxTone;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OtherColour: TColor read FOtherColour write SetOtherColour;
    property ParentColor;
    property ParentFont;
    property PopupMenu;
    property ScaleSnap: Boolean read FScaleSnap write FScaleSnap;
    property ShowHint;
    property ShowLegend: Boolean read FShowLegend write SetShowLegend;
    property SnapHeight: Integer read FSnapHeight write FSnapHeight;
    property SnapWidth: Integer read FSnapWidth write FSnapWidth;
    property SNRColour: TColor read FSNRColour write SetSNRColour;
    property ToneIncrement: Integer read FToneIncrement write SetToneIncrement;
    property Tonesperpixel: Shortint read FTonesperpixel write SetTonesperpixel;
    property UpColour: TColor read FUpColour write SetUpColour;
    property VDSL2: Boolean read FVDSL2 write SetVDSL2;
    property VersionString: string read FVersionString write SetVersionString;
    property Visible;
    property XOffset: Integer read FXOffset write SetXOffset;
    property YIncrement: Integer read FYIncrement write SetYIncrement;
    property YMax: Integer read FYMax write SetYMax;
    property YMin: Integer read FYMin write SetYMin;
    property YOffset: Integer read FYOffset write SetYOffset;
  end;

procedure Register;

implementation

const
  {$IFDEF CpuArm}
  LastToneNumber = 4095;
  {$ELSE}
  LastToneNumber = 8191;
  {$ENDIF}

constructor TBitchart.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Self.Width := 200;
  Self.Height := 150;
  Self.Align := alNone;
  FXOffset := 50;
  FYOffset := 40;
  FBitchartType := btBitloading;
  FFirstTone := 0;
  FLastTone := 511;
  FLegendPosition := lpbRight;
  FShowLegend := True;
  FTonesperpixel := 1;
  XMultiplier := 1;
  FBitchartColour := clWindow;
  FGraphColour := $D8FFFF;
  Canvas.Brush.Color := FBitchartColour;
  FUpColour := clGreen;
  FDownColour := clRed;
  FYMin := 0;
  FYMax := 20;
  FYIncrement := 2;
  FMaxreported := LastToneNumber;
  Timestamp := Now;
  FSNRpertone := False;
  VDSL2 := False;
  FSnapHeight := 400;
  FSnapWidth := 600;
  FScaleSnap := False;
  ABitmap := TBitmap.Create;
end;

destructor TBitchart.Destroy;
begin
  ABitmap.Free;
  inherited Destroy;
end;

procedure TBitchart.Paint;
begin
  try
    DrawBitmap(ABitmap);  //Do all the drawing on an in-memory bitmap before copying it to the bitchart canvas
    Canvas.Draw(0, 0, ABitmap);
  except
    on E: Exception do;  //do nothing
  end;
end;

procedure TBitchart.DrawBitmap(Bmp: TBitmap);
var
  X1, Y1, X2, Y2, N, YInc, Tone, Len, Bitload, Tonesper50, Tick, SNRval,
    SNRIncrement, XPos, LastXPos, YPos, LastYPos: Integer;
  S: string;
  Dotsperdb, Dotsperbit: Single;

  {X1, X2, Y1, Y2 - define the actual graph area in pixels
  XInc, YInc - the number of pixels from one axis mark to the next
  Tone - the tone number
  Len - used to store the length of a string
  Bitload - used in a loop to hold the left Y axis value
  TonesDone - used to count the number of tones per plotted point in compressed view
  Tonesper50 - the number of tones per 50 horizontal pixels in compressed view
  Tick - used in a loop to position the intermediate axis ticks
  SNRVal - derived from Bitload to define the right Y axis values
  SNRIncrement - derived from FYIncrement to define the gap between values on the right Y axis
  XPos - value of X to plot one sample
  YPos - value of Y to plot one sample
  LastXPos - the value of X for the previous sample
  LastYPos - the value of Y for the previous sample
  S - general purpose string variable
  Dest, Source - rectangles used when copying between canvases
  Dotsperdb - the number of pixels per dB on the right Y axis
  Dotsperbit - the number of pixels per bit on the left Y axis
  }

begin
  if FBitchartType = btBitswap then FYOffset := 65 else FYOffset := 40;
  X1 := FXOffset;
  Y1 := 0;
  if FSNRpertone then X2 := Width - FXOffset + 1
  else X2 := Width;
  Y2 := Height - FYOffset;
  Bmp.Width := Width;
  Bmp.Height := Height;
  Tonesper50 := Round(50 * XMultiplier);
  FToneIncrement := Tonesper50;
  FMaxTone := Round((X2 - X1) * XMultiplier);
  YInc := ((Y2 - 8) * FYIncrement) div FYMax;
  Dotsperdb := YInc * FYIncrement / 12;
  with Bmp.Canvas do
  begin
    Font.Assign(Self.Font);
    Font.CharSet := ANSI_CHARSET;
    Brush.Color := FBitchartColour;
    Pen.Color := clBlack;
    Pen.Style := psSolid;
    Pen.Width := 1;
    {$IFDEF Windows}
    Rectangle(0, 0, Width, Height);
    FillRect(1, 1, Width - 1, Height - 1);
    {$ELSE}
    Rectangle(0, 0, Width, Height - 1);
    FillRect(1, 1, Width - 1, Height - 2);
    Pen.Color := clBackground;
    MoveTo(0, Height - 1);
    LineTo(Width, Height - 1);
    Pen.Color := clBlack;
    {$ENDIF}
    Brush.Color := FGraphColour;
    Rectangle(X1, Y1, X2, Y2);
    Brush.Color := FBitchartColour;

    //Draw grid

    //Vertical lines and X axis marks
    XPos := X1;
    Tone := FFirstTone;
    Font.Size := 8;
    Font.Orientation := 0;
    while XPos <= X2 do
    begin
      Tick := 0; //Ticks every 10 pixels
      while Tick < 50 do
      begin
        Pen.Style := psSolid;
        if Tick = 0 then MoveTo(XPos + Tick, Y2 + 4)
        else MoveTo(XPos + Tick, Y2 + 2);
        LineTo(XPos + Tick, Y2 - 2);
        if (Tick = 0) and (Tone > 0) then
        begin
          Pen.Style := psDot;
          LineTo(XPos, 0);
        end;
        Tick := Tick + 10;
        if XPos + Tick > X2 then Break;
      end;
      S := IntToStr(Tone);
      Len := TextWidth(S);
      TextOut(XPos  - (Len shr 1), Y2 + 4, S);
      FMaxTone := Tone;
      XPos := XPos + 50;
      Tone := Tone + Tonesper50;
    end;
    {$IFDEF Windows}
      Font.Size := 8;
    {$ELSE}
      Font.Size := 9;
    {$ENDIF}
    S := FormatDateTime('dd mmm yyyy hh:mm:ss', Timestamp);
    TextOut(X1, Y2 + 20, S);
    if (FBitchartType = btBitswap) and (FStartTime <> TDateTime(0)) then
    begin
      S := FormatDateTime('dd mmm yyyy hh:mm:ss', FStartTime);
      S := 'Graph reset at: ' + S;
      TextOut(X1, Y2 + 40, S);
    end;
    S := 'Tone number';
    Len := TextWidth(S);
    TextOut(X1 + ((X2 - X1) shr 1)- (Len shr 1), Y2 + 20, S);
    Len := TextWidth(FVersionString);
    TextOut(Width - Len - 10, Y2 + 20, FVersionString);

    //Horizontal lines and Y axis marks
    N := Y2 - ((YInc * FYMax) div FYIncrement) - 1;
    Bitload := FYMax;
    SNRval := Bitload * 3;
    SNRIncrement := FYIncrement * 3;
    Font.Size := 8;
    while N <= Y2 do
    begin
      Pen.Style := psSolid;
      Moveto(X1 - 4, N);   //tick marks LHS
      Lineto(X1 + 4, N);
      MoveTo(X1 - 2, N - (YInc shr 1)); //intermediate tick marks LHS
      LineTo(X1 + 2, N - (YInc shr 1));
      if FSNRpertone then
      begin
        Moveto(X2 - 4, N);   //tick marks RHS
        Lineto(X2 + 4, N);
        MoveTo(X2 - 2, N - (YInc shr 1)); //intermediate tick marks RHS
        LineTo(X2 + 2, N - (YInc shr 1));
      end;
      MoveTo(X1 + 4, N);  //grid
      if N < Y2 - 1 then
      begin
        Pen.Style := psDot;
        Lineto(X2, N);
      end;
      S := IntToStr(BitLoad);
      Len := TextWidth(S);
      TextOut(X1 - Len - 4, N - 6, S);
      if FSNRpertone then
      begin
        S := IntToStr(SNRval);
        Font.Color := FSNRColour;
        TextOut(X2 + 4, N - 6, S);
      end;
      Font.Color := clBlack;
      N := N + YInc;
      Bitload := Bitload - FYIncrement;
      SNRval := SNRval - SNRIncrement;
    end;

    //Y axis labels
    {$IFDEF Windows}
      Font.Size := 8;
    {$ELSE}
      Font.Size := 9;
    {$ENDIF}
    Font.Orientation := 900;
    if FBitchartType = btBitswap then S := 'Bits swapped' else S := 'Bits';
    Len := TextWidth(S);
    TextOut(3, (Y1 + Y2) shr 1 + (Len shr 1), S);
    if FSNRpertone then
    begin
      Font.Color := FSNRColour;
      Len := TextWidth('SNR (dB)');
      TextOut(X2 + 20, (Y1 + Y2) shr 1 + (Len shr 1), 'SNR (dB)');
    end;
    Font.Color := clBlack;

    //Draw graphs

    if FLastTone > 0 then //Tone data has been stored
    begin
      Dotsperbit := YInc / FYIncrement;
      XPos := X1;
      Tone := FFirstTone;
      while (XPos < X2) and (Tone <= FLastTone) do
      begin
        if Tone > FMaxreported then Pen.Style := psDot else Pen.Style := psSolid;
        //if FBitloading or (FBitchartType = btBitswap) then
        begin
          Pen.Color := GetPenColour(Tone);
          {$IFDEF Windows}
          Moveto(XPos, Y2 - 2);
          Lineto(XPos, Y2 - Round(FBitsArray[Tone] * Dotsperbit) - 2);
          {$ELSE}
          Moveto(XPos, Y2 - 1);
          Lineto(XPos, Y2 - Round(FBitsArray[Tone] * Dotsperbit) - 1);
          {$ENDIF}
        end;
        if FSNRpertone then
        begin
          Pen.Color := FSNRColour;
          Pen.Style := psSolid;
          YPos := Y2 - Round(FSNRArray[Tone] * Dotsperdb) - 1;
          if Tone > FFirstTone then
          begin
            if Tone > FFirstTone + 1 then MoveTo(LastXPos, LastYPos)
            else MoveTo(XPos - 1, LastYPos);
            LineTo(XPos, YPos);
          end;
          LastXPos := XPos;
          LastYPos := YPos;
        end;
        Inc(Tone);
        XPos := X1 + Round((Tone - FFirstTone) * 50 / Tonesper50);
      end;
    end;

    //Draw the legend in top right or top left corner

    if FShowLegend then
    begin
      Brush.Color := clWhite;
      Pen.Color := clBlack;
      Pen.Style := psSolid;
      Pen.Width := 1;
      if VDSL2 then
      begin
        if FLegendPosition = lpbRight then Rectangle(X2 - 110, 5, X2 - 5, 58)
        else Rectangle(FXOffset + 5, 5, FXOffset + 110, 58);
      end
      else begin
        if FLegendPosition = lpbRight then Rectangle(X2 - 110, 5, X2 - 5, 43)
        else Rectangle(FXOffset + 5, 5, FXOffset + 110, 43);
      end;
      Pen.Width := 2;
      Pen.Color := FUpColour;
      if FLegendPosition = lpbRight then
      begin
        MoveTo(X2 - 105, 15);
        LineTo(X2 - 85, 15);
      end
      else begin
        MoveTo(FXOffset + 10, 15);
        LineTo(FXOffset + 30, 15);
      end;
      Pen.Color := FDownColour;
      if FLegendPosition = lpbRight then
      begin
        MoveTo(X2 - 105, 30);
        LineTo(X2 - 85, 30);
      end
      else begin
        MoveTo(FXOffset + 10, 30);
        LineTo(FXOffset + 30, 30);
      end;
      if VDSL2 then
        begin
        Pen.Color := FOtherColour;
        if FLegendPosition = lpbRight then
        begin
          MoveTo(X2 - 105, 45);
          LineTo(X2 - 85, 45);
        end
        else begin
          MoveTo(FXOffset + 10, 45);
          LineTo(FXOffset + 30, 45);
        end;
      end;
      Font.Size := 8;
      Font.Orientation := 0;
      if FLegendPosition = lpbRight then
      begin
        TextOut(X2 - 83, 8, 'Upstream');
        TextOut(X2 - 83, 23, 'Downstream');
        if VDSL2 then TextOut(X2 - 83, 38, 'Other');
      end
      else begin
        TextOut(FXOffset + 32, 8, 'Upstream');
        TextOut(FXOffset + 32, 23, 'Downstream');
        if VDSL2 then TextOut(FXOffset + 32, 38, 'Other');
      end;
    end;
  end;
end;

//Get the display colour for a VDSL2 tone
function TBitchart.GetPenColour(Tone: Integer): TColor;
begin
  if ((Tone >= FUpTones[0]) and (Tone <= FUpTones[1]))
  or ((Tone >= FUpTones[2]) and (Tone <= FUpTones[3]))
  or ((Tone >= FUpTones[4]) and (Tone <= FUpTones[5]))
  or ((Tone >= FUpTones[6]) and (Tone <= FUpTones[7]))
  then Result := FUpColour
  else if ((Tone >= FDownTones[0]) and (Tone <= FDownTones[1]))
  or ((Tone >= FDownTones[2]) and (Tone <= FDownTones[3]))
  or ((Tone >= FDownTones[4]) and (Tone <= FDownTones[5]))
  or ((Tone >= FDownTones[6]) and (Tone <= FDownTones[7]))
  then Result := FDownColour
  else Result := FOtherColour;
end;

//Save current view of the bitloading chart
function TBitchart.SaveToPNGFile(Filename: string): Boolean;
var
  Png: TPortableNetworkGraphic;
  Bmp: TBitmap;
begin
  Result := True;
  Bmp := TBitmap.Create;
  try
    DrawBitmap(Bmp);
  except
    on E: Exception do
    begin
      Result := False;
      Bmp.Free;
      Exit;
    end;
  end;
  Png := TPortableNetworkGraphic.Create;
  try
    if FScaleSnap then
    begin
      Png.Width := FSnapWidth;
      Png.Height := FSnapHeight;
      Png.Canvas.StretchDraw(Rect(0, 0, FSnapWidth - 1, FSnapHeight - 1), Bmp);
    end
    else Png.Assign(Bmp);
    Png.SaveToFile(Filename);
  except
    on E: Exception do
       Result := False;
  end;
  Png.Free;
  Bmp.Free;
end;

//Respond to a resize of the main window
procedure TBitchart.Resize;
begin
  inherited Resize;
  FMaxTone := FFirstTone + Round((Self.Width - FXOffset) * XMultiplier);
  Invalidate;
end;

//The trigger to redraw the chart with a new sample of values
procedure TBitchart.Drawbits(HighestTone: Integer);
begin
  FLastTone := HighestTone;
  Timestamp := Now;
  Invalidate;
end;

// **** RESPOND TO CHANGES IN PROPERTY VALUES **** //

procedure TBitchart.SetXOffset(Value: Integer);
begin
  if Value < Self.Width shr 1 then
  begin
    FXOffset := Value;
    Invalidate;
  end;
end;

procedure TBitchart.SetYOffset(Value: Integer);
begin
    if Value < Self.Height shr 1 then
  begin
    FYOffset := Value;
    Invalidate;
  end;
end;

procedure TBitchart.SetToneIncrement(Value: Integer);
begin
  FToneIncrement := Value;
  Invalidate
end;

procedure TBitchart.SetYMin(Value: Integer);
begin
  if Value < FYMax then FYMin := Value;
  Invalidate;
end;

procedure TBitchart.SetYMax(Value: Integer);
begin
  if (Value > FYMin) and (Value > 1) then FYMax := Value;
  Invalidate;
end;

procedure TBitchart.SetYIncrement(Value: Integer);
begin
  FYIncrement := Value;
  Invalidate;
end;

procedure TBitchart.SetFirstTone(Value: Integer);
begin
  if Value <> FFirstTone then
  begin
    FFirstTone := Value;
    if FBitchartType = btBitswap then FMaxTone := FFirstTone + (Self.Width - FXOffset) * FTonesperpixel
    else FMaxTone := FFirstTone + (Self.Width - FXOffset) div 3;
    Invalidate;
  end;
end;

procedure TBitChart.SetGraphColour(Value: TColor);
begin
  if Value <> FGraphColour then
  begin
    FGraphColour := Value;
    Invalidate;
  end;
end;

procedure TBitchart.SetUpColour(Value: TColor);
begin
  if Value <> FUpColour then
  begin
    FUpColour := Value;
    Invalidate;
  end;
end;

procedure TBitchart.SetDownColour(Value: TColor);
begin
  if Value <> FDOwnColour then
  begin
    FDownColour := Value;
    Invalidate;
  end;
end;

procedure TBitchart.SetOtherColour(Value: TColor);
begin
  if Value <> FOtherColour then
  begin
    FOtherColour := Value;
    Invalidate;
  end;
end;

procedure TBitchart.SetBitchartColour(Value: TColor);
begin
  if Value <> FBitchartColour then
  begin
    FBitchartColour := Value;
    Invalidate;
  end;
end;

procedure TBitchart.SetLegendPosition(Value: TLegendPosition);
begin
  if Value <> FLegendPosition then
  begin
    FLegendPosition := Value;
    Invalidate;
  end;
end;

procedure TBitchart.SetShowLegend(Value: Boolean);
begin
  if Value <> FShowLegend then
  begin
    FShowLegend := Value;
    Invalidate;
  end;
end;

procedure TBitchart.SetSNRColour(Value: TColor);
begin
  if Value <> FSNRColour then
  begin
    FSNRColour := Value;
    Invalidate;
  end;
end;

procedure TBitchart.SetBitsArray(Index: Integer; Value: Integer);
begin
  FBitsArray[Index] := Value;
  if Value > FYMax then
  begin
    if Value <= 200 then FYMax := ((Value div 20) + 1) * 20
    else if Value <= 2000 then FYMax := ((Value div 100) + 1) * 100
    else if Value <= 20000 then FYMax := ((Value div 1000) + 1) * 1000
    else FYMax := ((Value div 10000) + 1) * 10000;
    FYIncrement := (FYMax) div 10;
  end;
end;

procedure TBitchart.SetVDSL2(Value: Boolean);
begin
  if Value <> FVDSL2 then
  begin
    FVDSL2 := Value;
    Invalidate;
  end;
end;

procedure TBitchart.SetSNRArray(Index: Integer; Value: Single);
begin
  try
    FSNRArray[Index] := Value;
  except
    on E: Exception do;  //Do nothing
  end;
end;

procedure TBitchart.SetUpTones(Index: Integer; Value: Integer);
begin
  try
    FUpTones[Index] := Value;
  except
    on E: Exception do;  //Do nothing
  end;
end;

procedure TBitchart.SetDownTones(Index: Integer; Value: Integer);
begin
  try
    FDownTones[Index] := Value;
  except
    on E: Exception do;  //Do nothing
  end;
end;

procedure TBitchart.SetTonesperpixel(Value: Shortint);
begin
  if Value <> FTonesperpixel then
  begin
    FTonesperpixel := Value;
    case Value of
      10: XMultiplier := 10;
      9:  XMultiplier := 9;
      8:  XMultiplier := 8;
      7:  XMultiplier := 7;
      6:  XMultiplier := 6;
      5:  XMultiplier := 5;
      4:  XMultiplier := 4;
      3:  XMultiplier := 3;
      2:  XMultiplier := 1.8;
      1:  XMultiplier := 0.85;
      0:  XMultiplier := 0.55;
      -1: XMultiplier := 0.35;
      -2: XMultiplier := 0.25;
      -3: XMultiplier := 0.2;
    else
      XMultiplier := 1;
    end;
    Invalidate;
  end;
end;

procedure TBitchart.SetSNRpertone(Value: Boolean);
begin
  if Value <> FSNRpertone then
  begin
    FSNRpertone := Value;
    Invalidate;
  end;
end;

procedure TBitchart.SetBitchartType(Value: TBitchartType);
begin
  if Value <> FBitchartType then
  begin
    FBitchartType := Value;
    Invalidate;
  end;
end;

procedure TBitchart.SetStartTime(Value: TDateTime);
begin
  if Value <> FStartTime then
  begin
    FStartTime := Value;
    Invalidate;
  end;
end;

procedure TBitchart.SetVersionString(Value: string);
begin
  if Value <> FVersionString then
  begin
    FVersionString := Value;
    Invalidate;
  end;
end;

//**** GET ARRAY VALUES ****

function TBitChart.GetBits(Tone: Integer): Integer;
begin
  try
    Result := FBitsArray[Tone];
  except
    on E: Exception do Result := 0;
  end;
end;

function TBitchart.GetBitsArray(Index: Integer): Integer;
begin
  try
    Result := FBitsArray[Index];
  except
    on E: Exception do Result := 0;
  end;
end;

function TBitchart.GetSNR(Tone: Integer): Single;
begin
  try
    Result := FSNRArray[Tone];
  except
    on E: Exception do Result := 0;
  end;
end;

function TBitchart.GetSNRArray(Index: Integer): Single;
begin
  try
    Result := FSNRArray[Index];
  except
    on E: Exception do Result := 0;
  end;
end;

procedure Register;
begin
  RegisterComponents('Eric',[TBitchart]);
end;

initialization
{$I dsgraphs.lrs}

end.
