{DSLstats - a program for monitoring the state of a DSL internet connection
Copyright (C) 2012-2013 Eric Chapman

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License version 2
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

You can contact the author via the website http://dslstats.plainroad.me.uk
}

unit linechart;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, dateutils, ExtCtrls, Math, LCLType;

type
  TChartPoint = record
    XValue: TDateTime;
    UpYValue, DownYValue: Single;
    DrawBlank: Boolean;
  end;

  TLinechartType = (ltMargin, ltSpeed, ltCRC, ltFEC, ltSwap, ltTraffic, ltPing, ltSES, ltGeneral);
  TGraphType = (gtLine, gtBar, gtDot);
  TLegendPosition = (lplLeft, lplRight);
  TLinechart = class(TGraphicControl)
  private
    { Private declarations }
    ABitmap: TBitmap;
    ArraySize, FNumSamples: Integer;
    FChartType: TLinechartType;
    FGraphType: TGraphType;
    FHoursperpage, FThickness: Word;
    FLegendPosition: TLegendPosition;
    FLogYScale, FShowMaxUp, FShowMaxDown, FPlotUpstream, FPlotDownstream, FAutoscale, FPerSample: Boolean;
    FMaxYValue, FMinYValue, FClipValue, HighestY: Single;
    FPageStart: TDateTime;
    FPerMinuteMultiplier: Single;
    FShowLegend, FUseClipping, FScaleSnap: Boolean;
    FSnapHeight, FSnapWidth: Integer;
    FUpColour, FDownColour, FLinechartColour, FGraphColour: TColor;
    FVersionString, FYCaption: string;
    FXOffset, FYOffset: Word;
    SampleRecord: TChartPoint;
    procedure SetAutoscale(Value: Boolean);
    procedure SetChartType(Value: TLinechartType);
    procedure SetClipValue(Value: Single);
    procedure SetDownColour(Value: TColor);
    procedure SetGraphColour(Value: TColor);
    procedure SetGraphType(Value: TGraphType);
    procedure SetHoursperpage(Value: Word);
    procedure SetLegendPosition(Value: TLegendPosition);
    procedure SetLinechartColour(Value: TColor);
    procedure SetLogYScale(Value: Boolean);
    procedure SetMaxYValue(Value: Single);
    procedure SetMinYValue(Value: Single);
    procedure SetPerSample(Value: Boolean);
    procedure SetPlotDownstream(Value: Boolean);
    procedure SetPlotUpstream(Value: Boolean);
    procedure SetShowLegend(Value: Boolean);
    procedure SetShowMaxDown(Value: Boolean);
    procedure SetShowMaxUp(Value: Boolean);
    procedure SetThickness(Value: Word);
    procedure SetUpColour(Value: TColor);
    procedure SetUseClipping(Value: Boolean);
    procedure SetVersionString(Value: string);
    procedure SetXOffset(Value: Word);
    procedure SetYCaption(Value: string);
    procedure SetYOffset(Value: Word);
  protected
    { Protected declarations }
    procedure Paint; override;
  public
    { Public declarations }
    SampleArray, MaxSampleArray: array of TChartPoint;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function AddXY(T: TDateTime; UpYValue: Single; DownYValue: Single; DrawBlank: Boolean): Boolean;
    function AddXYmax(T: TDateTime; UpYValue: Single; DownYValue: Single; DrawBlank: Boolean): Boolean;
    function DoAddXY(T: TDateTime; UpYValue: Single; DownYValue: Single; DrawBlank, PlotMax: Boolean): Boolean;
    function SaveToPNGFile(Filename: string): Boolean;
    procedure ClearArray;
    procedure DrawBitmap(Bmp: TBitmap);
    procedure SetPageStart(Value: TDateTime);
    property NumSamples: Integer read FNumSamples;
    property PerMinuteMultiplier: Single read FPerminuteMultiplier write FPerminuteMultiplier;
  published
    { Published declarations }
    property Align default alNone;
    property Anchors;
    property Autoscale: Boolean read FAutoscale write SetAutoscale;
    property Caption;
    property ChartType: TLinechartType read FChartType write SetChartType;
    property ClipValue: Single read FClipValue write SetClipValue default 20;
    property Color;
    property DownColour: TColor read FDownColour write SetDownColour;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property GraphColour: TColor read FGraphColour write SetGraphColour;
    property GraphThickness: Word read FThickness write SetThickness default 1;
    property GraphType: TGraphType read FGraphType write SetGraphType;
    property Hoursperpage: Word read FHoursperpage write SetHoursperpage default 2;
    property LegendPosition: TLegendPosition read FLegendPosition write SetLegendPosition;
    property LinechartColour: TColor read FLinechartColour write SetLinechartColour;
    property LogYScale: Boolean read FLogYScale write SetLogYScale;
    property MaxYValue: Single read FMaxYValue write SetMaxYValue default 10;
    property MinYValue: Single read FMinYValue write SetMinYValue default 0;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property PageStart: TDateTime read FPageStart;
    property ParentColor;
    property ParentFont;
    property PerSample: Boolean read FPerSample write SetPerSample;
    property PlotDownstream: Boolean read FPlotDownstream write SetPlotDownstream;
    property PlotUpstream: Boolean read FPlotUpstream write SetPlotUpstream;
    property PopupMenu;
    property ScaleSnap: Boolean read FScaleSnap write FScaleSnap;
    property ShowHint;
    property ShowLegend: Boolean read FShowLegend write SetShowLegend;
    property ShowMaxDown: Boolean read FShowMaxDown write SetShowMaxDown;
    property ShowMaxUp: Boolean read FShowMaxUp write SetShowMaxUp;
    property SnapHeight: Integer read FSnapHeight write FSnapHeight;
    property SnapWidth: Integer read FSnapWidth write FSnapWidth;
    property UpColour: TColor read FUpColour write SetUpColour;
    property UseClipping: Boolean read FUseClipping write SetUseClipping;
    property VersionString: string read FVersionString write SetVersionString;
    property Visible;
    property XOffset: Word read FXOffset write SetXOffset default 40;
    property YCaption: string read FYCaption write SetYCaption;
    property YOffset: Word read FYOffset write SetYOffset default 60;
  end;

procedure Register;

implementation

constructor TLinechart.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Self.Width := 200;
  Self.Height := 150;
  Self.Align := alNone;
  FXOffset := 40;
  FYOffset := 65;
  FShowLegend := True;
  FShowMaxUp := True;
  FShowMaxDown := True;
  FPlotUpstream := True;
  FPlotDownstream := True;
  FThickness := 1;
  FHoursperpage := 2;
  FPageStart := Now;
  FChartType := ltMargin;
  FGraphType := gtLine;
  FLegendPosition := lplRight;
  FLinechartColour := clWindow;
  FGraphColour := $D8FFFF;
  FMaxYValue := 10;
  FMinYValue := 0;
  FClipValue := 20;
  FUseClipping := False;
  FAutoscale := True;
  FPerMinuteMultiplier := 2;
  FPerSample := False;
  FYCaption := 'X';
  HighestY := 0;
  FSnapHeight := 400;
  FSnapWidth := 600;
  FScaleSnap := False;
  ArraySize := 1000;
  SetLength(SampleArray, ArraySize);
  SetLength(MaxSampleArray, ArraySize);
  FNumSamples := 0;
  ABitmap := TBitmap.Create;
end;

destructor TLinechart.Destroy;
begin
  if FNumSamples > 0 then
  begin
    SampleArray := nil;
    if FChartType = ltSpeed then MaxSampleArray := nil;
  end;
  ABitmap.Free;
  inherited Destroy;
end;

procedure TLinechart.Paint;
begin
  try
    DrawBitmap(ABitmap);  //Do all the drawing on an in-memory bitmap before copying it to the linechart canvas
    Canvas.Draw(0, 0, ABitmap);
  except
    on E: Exception do;  //Do nothing, just trap exception
  end;
end;

procedure TLinechart.DrawBitmap(Bmp: TBitmap);
var
  XPos, YPos, PrevYPos, X1, Y1, X2, Y2, XEnd, MinutesIncrement, XIncrement, YIncrement, Len, Y, Yy,
     Magnitude, Nn: Integer;
  YVal, YValInc, N, N1: Int64;
  AYear, AMonth, ADay, AHour, AMinute, ASecond, AMillisecond, Seconds: Word;
  S, Timestring: string;
  XFraction, Value, YMax, HY, MY: Single;
  XTime, PageEnd: TDateTime;

  {XPos, YPos - plotting points
  X1, X2, Y1, Y2 - define the actual graph area
  XEnd - the X value of the last visible time marker
  MinutesIncrement - the number of minutes between successive X axis markers
  XIncrement, YIncrement - the number of pixels between successive X and Y axis markers
  Len - teporary storage of the length of a string
  YVal - the value associated with the current Y axis mark
  YValInc - the difference between successive Y axis values
  Y, Yy - temporary Y values
  N - loop counter
  N1 - used to hold temporary value in a loop
  X - temporary X value
  Magnitude - the power of 10 for the max Y value in logarithmic mode
  }

  function GetPlottedValue(StoredValue: Single): Single;
  begin
    if N1 = 0 then Result := 0
    else if FPerSample then Result := StoredValue
    else begin
      Seconds := SecondsBetween(SampleArray[N1].XValue, SampleArray[N1 - 1].XValue);
      if Seconds > 0 then Result := StoredValue * 60 / Seconds else Result := 0;
    end;
  end;

begin
  HY := HighestY;
  MY := FMaxYValue;
  if (FChartType in [ltCRC, ltFEC, ltSES, ltGeneral]) and not FPerSample then
  begin
    HY := HY * FPerMinuteMultiplier;
    MY := MY * FPerMinuteMultiplier;
  end;
  if HY > 0 then S := IntToStr(Round(HY * 10))
  else S := IntToStr(Round(MY * 10));
  Len := Bmp.Canvas.TextWidth(S);
  FXOffset := Len + 25;
  X1 := FXOffset;
  Y1 := 0;
  X2 := Width;
  Y2 := Height - FYOffset;
  PageEnd := FPageStart + EncodeTimeInterval(FHoursperpage, 0, 0, 0);
  MinutesIncrement := FHoursperpage * 5;
  DecodeDateTime(FPageStart, AYear, AMonth, ADay, AHour, AMinute, ASecond, AMillisecond);
  if ASecond > 29 then    //round to the nearest minute
  begin
    Inc(AMinute);
    if AMinute = 60 then
    begin
      AMinute := 0;
      Inc(AHour);
      if AHour = 24 then AHour := 0;
    end;
  end;

  //Calculate X axis values
  XIncrement := (Width - FXOffset - 15) div 12;  //Distance between axis marks

  //Calculate Y axis values
  if FAutoscale then     //Calculate automatic scaling values
  begin
    if FUseClipping then
    begin
      if MY > FClipValue then MY := FClipValue;
      if (HY > 0) and (MY > HY) then MY := HY;
    end
    else if HY > MY then MY := HY;
    YMax := MY;
    if YMax < 10 then YMax := 10;
    Y := Round(YMax + 0.5);
    Yy := 0;
    if Y <= 10 then
    begin
      YIncrement := Round((Height - FYOffset - 10) / (YMax - FMinYValue) - 0.5);
      YValInc := 1;
    end
    else begin
      if Y <= 100 then   //make it a multiple of 10
      begin
        repeat
          Yy := Yy + 10;
        until Yy >= Y;
      end
      else if Y <= 1000 then  //make it a multiple of 100
      begin
        repeat
          Yy := Yy + 100;
        until Yy >= Y;
      end
      else if Y <= 10000 then   //make it a multiple of 1000
      begin
        repeat
          Yy := Yy + 1000;
        until Yy >= Y;
      end
      else if Y <= 100000 then   //make it a multiple of 10000
      begin
        repeat
          Yy := Yy + 10000;
        until Yy >= Y;
      end
      else if Y <= 1000000 then   //make it a multiple of 100000
      begin
        repeat
          Yy := Yy + 100000;
        until Yy >= Y;
      end
      else begin       //make it a multiple of 1000000
        repeat
          Yy := Yy + 1000000;
        until Yy >= Y;
      end;
      YValInc := Yy div 10;
      YIncrement := Round((Height - FYOffset - 10) / 10 - 0.5);
    end;
  end
  else begin     //Use max/min Y values specified by the user
    YMax := MY;
    if YMax - FMinYValue > 20 then YValInc := 2 else YValInc := 1;
    YIncrement := Round((Height - FYOffset - 10) * YValInc / (YMax - FMinYValue) - 0.5);
  end;

  Bmp.Width := Width;
  Bmp.Height := Height;
  with Bmp.Canvas do
  begin
    Font.Assign(Self.Font);
    Font.CharSet := ANSI_CHARSET;
    Brush.Color := FLinechartColour;
    Pen.Color := clBlack;
    Pen.Style := psSolid;
    Pen.Width := 1;
    {$IFDEF Windows}
    Rectangle(0, 0, Width, Height);
    FillRect(1, 1, Width - 1, Height - 1);
    {$ELSE}
    Rectangle(0, 0, Width, Height - 1);
    FillRect(1, 1, Width - 1, Height - 2);
    Pen.Color := clBackground;
    MoveTo(0, Height - 1);
    LineTo(Width, Height - 1);
    Pen.Color := clBlack;
    {$ENDIF}
    Brush.Color := FGraphColour;
    Rectangle(X1, Y1, X2, Y2);
    Brush.Color := FLinechartColour;

    //Draw grid

    //X axis marks and vertical lines
    XPos := X1 + 5;
    N := 0;
    Font.Orientation := 900;
    Font.Size := 8;
    repeat
      Pen.Style := psSolid;
      MoveTo(XPos + (XIncrement shr 1), Y2 + 2); //intermediate marks
      LineTo(XPos + (XIncrement shr 1), Y2 - 2);
      MoveTo(XPos, Y2 + 4); //main marks
      LineTo(XPos, Y2 - 4);
      Pen.Style := PsDot;
      LineTo(XPos, Y1);
      S := IntToStr(AHour);
      if AHour < 10 then S := '0' + S;
      Timestring := S + ':';
      S := IntToStr(AMinute);
      if AMinute < 10 then S := '0' + S;
      Timestring := Timestring + S;
      Len := TextWidth(Timestring);
      {$IFDEF Windows}
      TextOut(XPos - 7, Y2 + Len + 6, Timestring);
      {$ELSE}
      TextOut(XPos - 5, Y2 + Len + 6, Timestring);
      {$ENDIF}
      XEnd := XPos;
      XPos := XPos + XIncrement;
      Inc(N);
      AMinute := AMinute + MinutesIncrement;
      if AMinute > 59 then
      repeat
        AMinute := AMinute - 60;
        Inc(AHour);
        if AHour = 24 then AHour := 0;
      until AMinute < 60;
    until XPos > Width;
    S := 'Time';
    {$IFDEF Windows}
      Font.Size := 8;
    {$ELSE}
      Font.Size := 9;
    {$ENDIF}
    Len := TextWidth(S);
    Font.Orientation := 0;
    TextOut((Width shr 1) - (Len shr 1), Height - 20, S);
    S := '|<-- ' + FormatDateTime('dd mmm yyyy', FPageStart);
    TextOut(FXOffset, Height - 20, S);
    Len := TextWidth(FVersionString);
    TextOut(X2 - Len - 10, Height - 20, FVersionString);

    //Y axis marks and horizontal lines
    YPos := Y2 - 1;
    YVal := Round(FMinYValue);
    Font.Orientation := 0;
    Font.Size := 8;
    if FLogYScale then //Vertical scale is logarithmic
    begin
      Magnitude := 0;  //Calculate number of powers of 10
      if HY > 0 then Yy := Round(HY + 0.5)
      else Yy := Round(YMax + 0.5);
      repeat
        Inc(Magnitude);
        Yy := Yy div 10;
      until Yy = 0;
      if FChartType in [ltCRC, ltFEC, ltSES, ltGeneral] then Inc(Magnitude)
      else if FChartType = ltSpeed then Dec(Magnitude);
      if Magnitude = 0 then Magnitude := 1;
      YIncrement := (Y2 - 10) div Magnitude;  //Pixel spacing between powers of 10
      Yy := Round(log10(3) * YIncrement);    //Pixel spacing of 3s above 10s
      Y := Magnitude;
      if FChartType in [ltCRC, ltFEC, ltSES, ltGeneral] then N := 1
      else if FChartType = ltSpeed then N := 100
      else N := 10;
      repeat
        Pen.Style := psSolid;
        MoveTo(X1 - 4, YPos);
        LineTo(X1 + 4, YPos);
        if Y < Magnitude then
        begin
          Pen.Style := psDot;
          LineTo(X2, YPos);
          S := IntToStr(N);
          N := N * 10;
        end
        else begin
          if FChartType in [ltCRC, ltFEC, ltSES, ltGeneral] then S := '0.1'
          else if FChartType = ltSpeed then S := '10'
          else S := '1';
        end;
        Len := TextWidth(S);
        TextOut(FXOffset - Len - 5, YPos - 5, S);
        Pen.Style := psSolid;
        MoveTo(X1 - 4, YPos - Yy);
        LineTo(X1 + 4, YPos - Yy);
        Pen.Style := psDot;
        LineTo(X2, YPos - Yy);
        if (FChartType in [ltCRC, ltFEC, ltSES, ltGeneral]) and (N < 10) then S := FloatToStrF(N * 0.3, ffFixed, 0, 1)
        else S := IntToStr(N * 3 div 10);
        Len := TextWidth(S);
        TextOut(FXOffset - Len - 5, YPos - Yy - 5, S);
        Pen.Style := psSolid;
        for Nn := 1 to 9 do
        begin
          YVal := YPos - Round(Log10(Nn) * YIncrement);
          MoveTo(X1 - 2, YVal);
          LineTo(X1 + 2, YVal);
        end;
        Dec(Y);
        YPos := YPos - YIncrement;
      until Y < 0;
    end
    else repeat        //Vertical scale is linear
      Pen.Style := psSolid;
      MoveTo(X1 - 2, YPos - (YIncrement shr 1));  //intermediate marks
      LineTo(X1 + 2, YPos - (YIncrement shr 1));
      if (FChartType = ltMargin) and (YMax - FMinYValue < 7) then  //draw intermediate lines as well
      begin
        Pen.Style := psDot;
        LineTo(X2, YPos - (YIncrement shr 1));
        Pen.Style := psSolid;
      end;
      MoveTo(X1 - 4, YPos);  //main marks
      LineTo(X1 + 4, YPos);
      Pen.Style := psDot;
      LineTo(X2, YPos);
      S := IntToStr(YVal);
      Len := TextWidth(S);
      TextOut(FXOffset - Len - 5, YPos - 5, S);
      YVal := YVal + YValInc;
      YPos := YPos - YIncrement;
    until YPos < Y1;
    Font.Orientation := 900;
    {$IFDEF Windows}
      Font.Size := 8;
    {$ELSE}
      Font.Size := 9;
    {$ENDIF}
    case FChartType of
      ltMargin: S := 'SNR Margin (dB)';
      ltSpeed: S := 'Connection speed (kbps)';
      ltCRC: if FPerSample then S := 'CRC per sample' else S := 'CRC per min';
      ltFEC: if FPerSample then S := 'FEC per sample' else S := 'FEC per min';
      ltSwap: S := 'Bitswaps per min';
      ltTraffic: S := 'Traffic (kB per min)';
      ltPing: S := 'Latency (ms)';
      ltSES: if FPerSample then S := 'SES per sample' else S := 'SES per min';
      ltGeneral: if FPerSample then S := FYCaption + ' per sample' else S := FYCaption + ' per min';
    end;
    Len := TextWidth(S);
    TextOut(3, (Y1 + Y2) shr 1 + (Len shr 1), S);

    //Plot the graphs
    if FNumSamples > 1 then
    begin
      Pen.Style := psSolid;
      Pen.Width := FThickness;
      N := FNumSamples;
      repeat
        Dec(N);
        if N = -1 then Break;
      until SampleArray[N].XValue < FPageStart;
      Inc(N);   //N is now first sample to display

      if FPlotUpstream then
      begin
        //Plot upstream values
        Pen.Color := FUpColour;
        Brush.Color := FUpColour;

        //Plot regular values
        N1 := N;
        XTime := SampleArray[N].XValue;
        if XTime = FPagestart then XFraction := 0
        else XFraction := (XTime - FPageStart) / (PageEnd - FPagestart);
        XPos := X1 + 5 + Round(XFraction * (XEnd - X1 - 5));
        if (FChartType in [ltCRC, ltFEC, ltSES, ltGeneral]) then Value := GetPlottedValue(SampleArray[N1].UpYValue)
        else Value := SampleArray[N1].UpYValue;
        if Value < FMinYValue then Value := FMinYValue;
        if FLogYScale then
        begin
          if (FChartType in [ltCRC, ltFEC, ltSES, ltGeneral]) and (Value < 0.1) then Value := 0.1
          else if (FChartType = ltTraffic) and (Value < 1) then Value := 1
          else if (FChartType = ltSpeed) and (Value < 10) then Value := 10;
          if FChartType in [ltCRC, ltFEC, ltSES, ltGeneral] then YPos := Y2 - 1 - Round(Log10(Value * 10) * YIncrement)
          else if FChartType = ltSpeed then YPos := Y2 - 1 - Round(Log10(Value / 10) * YIncrement)
          else YPos := Y2 - 1 - Round(Log10(Value) * YIncrement);
        end
        else begin
          if FUseClipping and (Value > FClipValue) then Value := FClipValue;
          YPos := Y2 - 1 - Round((Value - FMinYValue) * YIncrement / YValInc);
        end;
        MoveTo(XPos, YPos);
        if FGraphType = gtBar then LineTo(XPos, Y2)
        else if FGraphType = gtDot then Ellipse(XPos - 2, YPos - 2, XPos + 2, YPos + 2);
        repeat
          PrevYPos := YPos;
          Inc(N1);
          XTime := SampleArray[N1].XValue;
          XFraction := (XTime - FPageStart) / (PageEnd - FPagestart);
          XPos := X1 + 5 + Round(XFraction * (XEnd - X1 - 5));
          if (FChartType in [ltCRC, ltFEC, ltSES, ltGeneral]) then Value := GetPlottedValue(SampleArray[N1].UpYValue)
          else Value := SampleArray[N1].UpYValue;
          if Value < FMinYValue then Value := FMinYValue;
          if FLogYScale then
          begin
            if (FChartType in [ltCRC, ltFEC, ltSES, ltGeneral]) and (Value < 0.1) then Value := 0.1
            else if (FChartType = ltTraffic) and (Value < 1) then Value := 1
            else if (FChartType = ltSpeed) and (Value < 10) then Value := 10;
            if FChartType in [ltCRC, ltFEC, ltSES, ltGeneral] then YPos := Y2 - 1 - Round(Log10(Value * 10) * YIncrement)
            else if FChartType = ltSpeed then YPos := Y2 - 1 - Round(Log10(Value / 10) * YIncrement)
            else YPos := Y2 - 1 - Round(Log10(Value) * YIncrement);
          end
          else begin
            if FUseClipping and (Value > FClipValue) then Value := FClipValue;
            YPos := Y2 - 1 - Round((Value - FMinYValue) * YIncrement / YValInc);
          end;
          if SampleArray[N1].DrawBlank then
          begin
            if FThickness > 1 then Pen.Width := 1 else Pen.Style := psDot;
          end;
          if FGraphType = gtBar then
          begin
            MoveTo(XPos, YPos);
            LineTo(XPos, Y2);
          end
          else if FGraphType = gtDot then Ellipse(XPos - 2, YPos - 2, XPos + 2, YPos + 2)
          else begin
            if FLogYScale and (PrevYPos = Y2 - 1) and (YPos = PrevYPos) then MoveTo(XPos, YPos)  //Don't plot two successive zeros in log mode
            else LineTo(XPos, YPos);
          end;
          Pen.Style := psSolid;
          Pen.Width := FThickness;
        until N1 = FNumSamples - 1;

        //Plot max attainable values
        if FShowMaxUp and (FChartType = ltSpeed) then
        begin
          if FThickness = 1 then Pen.Style := psDot else Pen.Width := 1;
          N1 := N;
          XTime := SampleArray[N].XValue;
          if XTime = FPagestart then XFraction := 0
          else XFraction := (XTime - FPageStart) / (PageEnd - FPagestart);
          XPos := X1 + 5 + Round(XFraction * (XEnd - X1 - 5));
          Value := MaxSampleArray[N1].UpYValue;
          if FLogYScale then
          begin
            if (FChartType = ltSpeed) and (Value < 10) then Value := 10;
            if FChartType = ltSpeed then YPos := Y2 - 1 - Round(Log10(Value / 10) * YIncrement)
            else YPos := Y2 - 1 - Round(Log10(Value) * YIncrement);
          end
          else YPos := Y2 - 1 - Round(Value * YIncrement / YValInc);
          MoveTo(XPos, YPos);
          repeat
            Inc(N1);
            XTime := MaxSampleArray[N1].XValue;
            XFraction := (XTime - FPageStart) / (PageEnd - FPagestart);
            XPos := X1 + 5 + Round(XFraction * (XEnd - X1 - 5));
            Value := MaxSampleArray[N1].UpYValue;
            if FLogYScale then
            begin
              if (FChartType = ltSpeed) and (Value < 10) then Value := 10;
              if FChartType = ltSpeed then YPos := Y2 - 1 - Round(Log10(Value / 10) * YIncrement)
              else YPos := Y2 - 1 - Round(Log10(Value) * YIncrement);
            end
            else YPos := Y2 - 1 - Round(Value * YIncrement / YValInc);
            if MaxSampleArray[N1].DrawBlank then
            begin
              if FThickness = 1 then Pen.Style := psClear else Pen.Style := psDot;
              LineTo(XPos, YPos);
              if FThickness = 1 then Pen.Style := psDot else Pen.Style := psSolid;
            end
            else LineTo(XPos, YPos);
          until N1 = FNumSamples - 1;
          Pen.Style := psSolid;
          Pen.Width := FThickness;
        end;
      end;

      if FPlotDownstream then
      begin
        //Plot downstream values
        Pen.Color := FDownColour;
        Brush.Color := FDownColour;

        //Plot regular values
        N1 := N;
        XTime := SampleArray[N].XValue;
        if XTime = FPagestart then XFraction := 0
        else XFraction := (XTime - FPageStart) / (PageEnd - FPagestart);
        XPos := X1 + 5 + Round(XFraction * (XEnd - X1 - 5));
        if (FChartType in [ltCRC, ltFEC, ltSES, ltGeneral]) then Value := GetPlottedValue(SampleArray[N1].DownYValue)
        else Value := SampleArray[N1].DownYValue;
        if Value < FMinYValue then Value := FMinYValue;
        if FLogYScale then
        begin
          if (FChartType in [ltCRC, ltFEC, ltSES, ltGeneral]) and (Value < 0.1) then Value := 0.1
          else if (FChartType = ltTraffic) and (Value < 1) then Value := 1
          else if (FChartType = ltSpeed) and (Value < 10) then Value := 10;
          if FChartType in [ltCRC, ltFEC, ltSES, ltGeneral] then YPos := Y2 - 1 - Round(Log10(Value * 10) * YIncrement)
          else if FChartType = ltSpeed then YPos := Y2 - 1 - Round(Log10(Value / 10) * YIncrement)
          else YPos := Y2 - 1 - Round(Log10(Value) * YIncrement);
        end
        else begin
          if FUseClipping and (Value > FClipValue) then Value := FClipValue;
          YPos := Y2 - 1 - Round((Value - FMinYValue) * YIncrement / YValInc);
        end;
        MoveTo(XPos, YPos);
        if FGraphType = gtBar then LineTo(XPos, Y2)
        else if FGraphType = gtDot then Ellipse(XPos - 2, YPos - 2, XPos + 2, YPos + 2);
        repeat
          PrevYPos := YPos;
          Inc(N1);
          XTime := SampleArray[N1].XValue;
          XFraction := (XTime - FPageStart) / (PageEnd - FPagestart);
          XPos := X1 + 5 + Round(XFraction * (XEnd - X1 - 5));
          if (FChartType in [ltCRC, ltFEC, ltSES, ltGeneral]) then Value := GetPlottedValue(SampleArray[N1].DownYValue)
          else Value := SampleArray[N1].DownYValue;
          if Value < FMinYValue then Value := FMinYValue;
          if FLogYScale then
          begin
            if (FChartType in [ltCRC, ltFEC, ltSES, ltGeneral]) and (Value < 0.1) then Value := 0.1
            else if (FChartType = ltTraffic) and (Value < 1) then Value := 1
            else if (FChartType = ltSpeed) and (Value < 10) then Value := 10;
            if FChartType in [ltCRC, ltFEC, ltSES, ltGeneral] then YPos := Y2 - 1 - Round(Log10(Value * 10) * YIncrement)
            else if FChartType = ltSpeed then YPos := Y2 - 1 - Round(Log10(Value / 10) * YIncrement)
            else YPos := Y2 - 1 - Round(Log10(Value) * YIncrement);
          end
          else begin
            if FUseClipping and (Value > FClipValue) then Value := FClipValue;
            YPos := Y2 - 1 - Round((Value - FMinYValue) * YIncrement / YValInc);
          end;
          if SampleArray[N1].DrawBlank then
          begin
            if FThickness > 1 then Pen.Width := 1 else Pen.Style := psDot;
          end;
          if FGraphType = gtBar then
          begin
            MoveTo(XPos, YPos);
            LineTo(XPos, Y2);
          end
          else if FGraphType = gtDot then Ellipse(XPos - 2, YPos - 2, XPos + 2, YPos + 2)
          else begin
            if FLogYScale and (PrevYPos = Y2 - 1) and (YPos = PrevYPos) then MoveTo(XPos, YPos)  //Don't plot two successive zeros in log mode
            else LineTo(XPos, YPos);
          end;
          Pen.Style := psSolid;
          Pen.Width := FThickness;
        until N1 = FNumSamples - 1;

        //Plot max attainable values
        if FShowMaxDown and (FChartType = ltSpeed) then
        begin
          if FThickness = 1 then Pen.Style := psDot else Pen.Width := 1;
          N1 := N;
          XTime := SampleArray[N].XValue;
          if XTime = FPagestart then XFraction := 0
          else XFraction := (XTime - FPageStart) / (PageEnd - FPagestart);
          XPos := X1 + 5 + Round(XFraction * (XEnd - X1 - 5));
          Value := MaxSampleArray[N1].DownYValue;
          if FLogYScale then
          begin
            if (FChartType = ltSpeed) and (Value < 10) then Value := 10;
            if FChartType = ltSpeed then YPos := Y2 - 1 - Round(Log10(Value / 10) * YIncrement)
            else YPos := Y2 - 1 - Round(Log10(Value) * YIncrement);
          end
          else YPos := Y2 - 1 - Round(Value * YIncrement / YValInc);
          MoveTo(XPos, YPos);
          repeat
            Inc(N1);
            XTime := MaxSampleArray[N1].XValue;
            XFraction := (XTime - FPageStart) / (PageEnd - FPagestart);
            XPos := X1 + 5 + Round(XFraction * (XEnd - X1 - 5));
            Value := MaxSampleArray[N1].DownYValue;
            if FLogYScale then
            begin
              if (FChartType = ltSpeed) and (Value < 10) then Value := 10;
              if FChartType = ltSpeed then YPos := Y2 - 1 - Round(Log10(Value / 10) * YIncrement)
              else YPos := Y2 - 1 - Round(Log10(Value) * YIncrement);
            end
            else YPos := Y2 - 1 - Round(Value * YIncrement / YValInc);
            if MaxSampleArray[N1].DrawBlank then
            begin
              if FThickness = 1 then Pen.Style := psClear else Pen.Style := psDot;
              LineTo(XPos, YPos);
              if FThickness = 1 then Pen.Style := psDot else Pen.Style := psSolid;
            end
            else LineTo(XPos, YPos);
          until N1 = FNumSamples - 1;
          Pen.Style := psSolid;
          Pen.Width := FThickness;
        end;
      end;
    end;

    //Draw the legend in top right or top left corner

    if FShowLegend then
    begin
      Brush.Color := clWhite;
      Pen.Color := clBlack;
      Pen.Style := psSolid;
      Pen.Width := 1;
      if FLegendPosition = lplRight then Rectangle(X2 - 110, 5, X2 - 5, 43)
      else Rectangle(FXOffset + 5, 5, FXOffset + 110, 43);
      Pen.Width := 2;
      Pen.Color := FUpColour;
      if FLegendPosition = lplRight then
      begin
        MoveTo(X2 - 105, 15);
        LineTo(X2 - 85, 15);
      end
      else begin
        MoveTo(FXOffset + 10, 15);
        LineTo(FXOffset + 30, 15);
      end;
      Pen.Color := FDownColour;
      if FLegendPosition = lplRight then
      begin
        MoveTo(X2 - 105, 30);
        LineTo(X2 - 85, 30);
      end
      else begin
        MoveTo(FXOffset + 10, 30);
        LineTo(FXOffset + 30, 30);
      end;
      Font.Size := 8;
      Font.Orientation := 0;
      if FLegendPosition = lplRight then
      begin
        TextOut(X2 - 83, 8, 'Upstream');
        TextOut(X2 - 83, 23, 'Downstream');
      end
      else begin
        TextOut(FXOffset + 32, 8, 'Upstream');
        TextOut(FXOffset + 32, 23, 'Downstream');
      end;
    end;
  end;
end;

//Process a new sample
//Return value is True if the plotted value will be clipped when redrawing after receiving this sample

//Normal sample
function TLinechart.AddXY(T: TDateTime; UpYValue: Single; DownYValue: Single; DrawBlank: Boolean): Boolean;
begin
  Result := DoAddXY(T, UpYValue, DownYValue, DrawBlank, False);
end;

//Max attainable speed sample
function TLinechart.AddXYmax(T: TDateTime; UpYValue: Single; DownYValue: Single; DrawBlank: Boolean): Boolean;
begin
  Result := DoAddXY(T, UpYValue, DownYValue, DrawBlank, True);
end;

function TLinechart.DoAddXY(T: TDateTime; UpYValue: Single; DownYValue: Single; DrawBlank, PlotMax: Boolean): Boolean;
var
  Larger, Smaller: Single;
  Seconds: Word;
begin
  Result := False;
  SampleRecord.XValue := T;
  SampleRecord.UpYValue := UpYValue;
  SampleRecord.DownYValue := DownYValue;
  SampleRecord.DrawBlank := DrawBlank;
  if PlotMax then MaxSampleArray[FNumSamples] := SampleRecord
  else begin
    SampleArray[FNumSamples] := SampleRecord;
    Inc(FNumSamples);
  end;
  if UpYValue > DownYValue then
  begin
    Larger := UpYValue;
    Smaller := DownYValue;
  end
  else begin
    Larger := DownYValue;
    Smaller := UpYValue;
  end;
  if Larger > FClipValue then Result := True;
  if Larger > HighestY then HighestY := Larger;
  if FAutoscale then
  begin
    if Larger > FMaxYValue then FMaxYValue := Larger * 1.05;
    if Smaller < FMinYValue then FMinYValue := Round(Smaller - 0.5);
  end;
  if PlotMax then Exit;
  if ArraySize - FNumSamples < 50 then
  begin
    ArraySize := ArraySize + 1000;
    SetLength(SampleArray, ArraySize);
    if FChartType = ltSpeed then SetLength(MaxSampleArray, ArraySize);
  end;
  FPageStart := T - EncodeTimeInterval(FHoursperpage, 0, 0, 0);
  if FPageStart < SampleArray[0].XValue then FPageStart := SampleArray[0].XValue;
  Invalidate;
end;

//Reset the sample array to initial state
procedure TLinechart.ClearArray;
begin
  FPageStart := Now;
  if FNumSamples = 0 then Exit;
  SampleArray := nil;
  MaxSampleArray := nil;
  ArraySize := 1000;
  SetLength(SampleArray, ArraySize);
  SetLength(MaxSampleArray, ArraySize);
  FNumSamples := 0;
  HighestY := 0;
end;

procedure TLinechart.SetAutoscale(Value: Boolean);
begin
  if Value <> FAutoscale then
  begin
    FAutoscale := Value;
    Invalidate;
  end;
end;

procedure TLinechart.SetChartType(Value: TLinechartType);
begin
  if Value <> FChartType then
  begin
    FChartType := Value;
    Invalidate;
  end;
end;

procedure TLinechart.SetGraphType(Value: TGraphType);
begin
  if Value <> FGraphType then
  begin
    FGraphType := Value;
    Invalidate;
  end;
end;

procedure TLinechart.SetLegendPosition(Value: TLegendPosition);
begin
  if Value <> FLegendPosition then
  begin
    FLegendPosition := Value;
    Invalidate;
  end;
end;

//Save a snapshot in .png format
function TLinechart.SaveToPNGFile(Filename: string): Boolean;
var
  Png: TPortableNetworkGraphic;
  Bmp: TBitmap;
begin
  Result := True;
  Bmp := TBitmap.Create;
  try
    DrawBitmap(Bmp);
  except
    on E: Exception do
    begin
      Result := False;
      Bmp.Free;
      Exit;
    end;
  end;
  Png := TPortableNetworkGraphic.Create;
  try
    if FScaleSnap then
    begin
      Png.Width := FSnapWidth;
      Png.Height := FSnapHeight;
      Png.Canvas.StretchDraw(Rect(0, 0, FSnapWidth - 1, FSnapHeight - 1), Bmp);
    end
    else Png.Assign(Bmp);
    Png.SaveToFile(Filename);
  except
    on E: Exception do
      Result := False;
  end;
  Png.Free;
  Bmp.Free;
end;

procedure TLinechart.SetThickness(Value: Word);
begin
  if Value <> FThickness then
  begin
    FThickness := Value;
    Invalidate;
  end;
end;

procedure TLinechart.SetXOffset(Value: Word);
begin
  if Value < Self.Width shr 1 then
  begin
    FXOffset := Value;
    Invalidate;
  end;
end;

procedure TLinechart.SetYOffset(Value: Word);
begin
    if Value < Self.Height shr 1 then
  begin
    FYOffset := Value;
    Invalidate;
  end;
end;

procedure TLineChart.SetGraphColour(Value: TColor);
begin
  if Value <> FGraphColour then
  begin
    FGraphColour := Value;
    Invalidate;
  end;
end;

procedure TLinechart.SetUpColour(Value: TColor);
begin
  if Value <> FUpColour then
  begin
    FUpColour := Value;
    Invalidate;
  end;
end;

procedure TLinechart.SetDownColour(Value: TColor);
begin
  if Value <> FDOwnColour then
  begin
    FDownColour := Value;
    Invalidate;
  end;
end;

procedure TLineChart.SetPageStart(Value: TDateTime);
begin
  if Value <> FPageStart then
  begin
    FPageStart := Value;
    Invalidate;
  end;
end;

procedure TLinechart.SetLinechartColour(Value: TColor);
begin
  if Value <> FLinechartColour then
  begin
    FLinechartColour := Value;
    Invalidate;
  end;
end;

procedure TLinechart.SetHoursperpage(Value: Word);
begin
  if Value <> FHoursperpage then
  begin
    FHoursperpage := Value;
    Invalidate;
  end;
end;

procedure TLinechart.SetMaxYValue(Value: Single);
begin
  if (Value <> FMaxYValue) and (Value > FMinYValue + 0.5) then
  begin
    FMaxYValue := Value;
    Invalidate;
  end;
end;

procedure TLinechart.SetMinYValue(Value: Single);
begin
  if (Value <> FMinYValue) and (Value < FMaxYValue - 0.5) then
  begin
    FMinYValue := Value;
    Invalidate;
  end;
end;

procedure TLinechart.SetClipValue(Value: Single);
begin
  if Value <> FClipValue then
  begin
    FClipValue := Value;
    if Value > FMaxYValue then FMaxYValue := Value;
    Invalidate;
  end;
end;

procedure TLinechart.SetUseClipping(Value: Boolean);
begin
  if Value <> FUseClipping then
  begin
    FUseClipping := Value;
    Invalidate;
  end;
end;

procedure TLineChart.SetPerSample(Value: Boolean);
begin
  if Value <> FPerSample then
  begin
    FPerSample := Value;
    Invalidate;
  end;
end;

procedure TLinechart.SetShowLegend(Value: Boolean);
begin
  if Value <> FShowLegend then
  begin
    FShowLegend := Value;
    Invalidate;
  end;
end;

procedure TLinechart.SetLogYScale(Value: Boolean);
begin
  if Value <> FLogYScale then
  begin
    FLogYScale := Value;
    if FLogYScale and (FChartType in [ltCRC, ltFEC, ltSES, ltGeneral]) then FMinYValue := 0.1;
    Invalidate;
  end;
end;

procedure TLinechart.SetShowMaxUp(Value: Boolean);
begin
  if Value <> FShowMaxUp then
  begin
    FShowMaxUp := Value;
    Invalidate;
  end;
end;

procedure TLinechart.SetShowMaxDown(Value: Boolean);
begin
  if Value <> FShowMaxDown then
  begin
    FShowMaxDown := Value;
    Invalidate;
  end;
end;

procedure TLinechart.SetVersionString(Value: string);
begin
  if Value <> FVersionString then
  begin
    FVersionString := Value;
    Invalidate;
  end;
end;

procedure TLinechart.SetPlotUpstream(Value: Boolean);
begin
  if Value <> FPlotUpstream then
  begin
    FPlotUpstream := Value;
    Invalidate;
  end;
end;

procedure TLinechart.SetPlotDownstream(Value: Boolean);
begin
  if Value <> FPlotDownstream then
  begin
    FPlotDownstream := Value;
    Invalidate;
  end;
end;

procedure TLinechart.SetYCaption(Value: string);
begin
  if Value <> FYCaption then
  begin
    FYCaption := Value;
    Invalidate;
  end;
end;

procedure Register;
begin
  RegisterComponents('Eric',[TLinechart]);
end;

initialization
{$I dsgraphs.lrs}

end.
