{DSLstats - a program for monitoring the state of a DSL internet connection
Copyright (C) 2012-2013 Eric Chapman

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License version 2
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

You can contact the author via the website http://dslstats.plainroad.me.uk
}

unit Linediagchart1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, dateutils, LCLType;

type
  TGraphType = (gtQLN, gtHLog, gtSNR, gtBitswap);
  TLegendPosition = (lpldLeft, lpldRight);
  TDSLAMType = (dslNone, dslHuawei, dslECI);
  TLinediagchart = class(TGraphicControl)
  private
    { Private declarations }
    ABitmap: TBitmap;
    Displaying, FExtendedToneRange, FPlotBaseline, FShowLegend, FVDSL2, FScaleSnap: Boolean;
    FDownTones: Array[0..7] of Integer;
    FDSLAMType: TDSLAMType;
    FFirstTone, FLastTone: Integer;
    {$IFDEF CpuArm}
    FGraphArray: Array[0..4095] of Single;
    {$ELSE}
    FGraphArray: Array[0..8191] of Single;
    {$ENDIF}
    FGraphType: TGraphType;
    FLegendPosition: TLegendPosition;
    FLinediagchartColour, FGraphColour, FUpColour, FDownColour, FOtherColour: TColor;
    FMaxTone, FToneIncrement: Integer;
    FTonesperpixel: Shortint;
    FSnapHeight, FSnapWidth: Integer;
    FUpTones: Array[0..7] of Integer;
    FVersionString: string;
    FXOffset, FYOffset, FThickness: Integer;
    FYMin, FYMax, FYIncrement: Integer;
    NormalHighestTone: Word;
    RefreshTime: TDateTime;
    XMultiplier: Single;
    function GetPenColour(Count: Integer): TColor;
    procedure SetDownColour(Value: TColor);
    procedure SetDownTones(Index: Integer; Value: Integer);
    procedure SetDSLAMType(Value: TDSLAMType);
    procedure SetExtendedToneRange(Value: Boolean);
    procedure SetFirstTone(Value: Integer);
    procedure SetGraphColour(Value: TColor);
    procedure SetGraphType(Value: TGraphType);
    procedure SetLastTone(Value: Integer);
    procedure SetLegendPosition(Value: TLegendPosition);
    procedure SetLinediagchartColour(Value: TColor);
    procedure SetPlotBaseline(Value: Boolean);
    procedure SetOtherColour(Value: TColor);
    procedure SetShowLegend(Value: Boolean);
    procedure SetThickness(Value: Integer);
    procedure SetToneIncrement(Value: Integer);
    procedure SetTonesperpixel(Value: Shortint);
    procedure SetUpColour(Value: TColor);
    procedure SetUpTones(Index: Integer; Value: Integer);
    procedure SetVDSL2(Value: Boolean);
    procedure SetVersionString(Value: string);
    procedure SetXOffset(Value: Integer);
    procedure SetYMax(Value: Integer);
    procedure SetYMin(Value: Integer);
    procedure SetYOffset(Value: Integer);
  protected
    { Protected declarations }
    procedure Paint; override;
    procedure DrawBitmap(Bmp: TBitmap);
    procedure SetGraphArray(Index: Word; Value: Single);
    function GetGraphArray(Index: Word): Single;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function SaveToPNGFile(Filename: string): Boolean;
    procedure DrawGraph(HighestTone: Integer);
    property DownTones[Index: Integer]: Integer write SetDownTones;
    property GraphArray[Index: Word]: Single read GetGraphArray write SetGraphArray;
    property UpTones[Index: Integer]: Integer write SetUpTones;
  published
    { Published declarations }
    property Align default alNone;
    property Anchors;
    property Caption;
    property Color;
    property DownColour: TColor read FDownColour write SetDownColour;
    property DragCursor;
    property DragMode;
    property DSLAMType: TDSLAMType read FDSLAMType write SetDSLAMType;
    property Enabled;
    property ExtendedToneRange: Boolean read FExtendedToneRange write SetExtendedToneRange;
    property FirstTone: Integer read FFirstTone write SetFirstTone;
    property Font;
    property GraphColour: TColor read FGraphColour write SetGraphColour;
    property GraphThickness: Integer read FThickness write SetThickness default 1;
    property GraphType: TGraphType read FGraphType write SetGraphType;
    property LastTone: Integer read FLastTone write SetLastTone;
    property LegendPosition: TLegendPosition read FLegendPosition write SetLegendPosition;
    property LinediagchartColour: TColor read FLinediagchartColour write SetLinediagchartColour;
    property MaxTone: Integer read FMaxTone;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OtherColour: TColor read FOtherColour write SetOtherColour;
    property ParentColor;
    property ParentFont;
    property PlotBaseline: Boolean read FPlotBaseline write SetPlotBaseline;
    property PopupMenu;
    property ScaleSnap: Boolean read FScaleSnap write FScaleSnap;
    property ShowHint;
    property ShowLegend: Boolean read FShowLegend write SetShowLegend;
    property SnapHeight: Integer read FSnapHeight write FSnapHeight;
    property SnapWidth: Integer read FSnapWidth write FSnapWidth;
    property ToneIncrement: Integer read FToneIncrement write SetToneIncrement;
    property Tonesperpixel: Shortint read FTonesperpixel write SetTonesperpixel;
    property UpColour: TColor read FUpColour write SetUpColour;
    property VDSL2: Boolean read FVDSL2 write SetVDSL2;
    property VersionString: string read FVersionString write SetVersionString;
    property Visible;
    property XOffset: Integer read FXOffset write SetXOffset;
    property YIncrement: Integer read FYIncrement;
    property YMax: Integer read FYMax write SetYMax;
    property YMin: Integer read FYMin write SetYMin;
    property YOffset: Integer read FYOffset write SetYOffset;
  end;

procedure Register;

implementation

const
  {$IFDEF CpuArm}
  LastToneNumber = 4095;
  {$ELSE}
  LastToneNumber = 8191;
  {$ENDIF}

constructor TLinediagchart.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  RefreshTime := Now;
  Self.Width := 200;
  Self.Height := 150;
  Self.Align := alNone;
  FXOffset := 50;
  FYOffset := 40;
  FThickness := 1;
  FFirstTone := 0;
  FLastTone := 0;
  FLegendPosition := lpldRight;
  FShowLegend := True;
  FTonesperpixel := 1;
  FExtendedToneRange := False;
  FPlotBaseline := False;
  XMultiplier := 1;
  NormalHighestTone := 511;
  Displaying := False;
  FLineDiagchartColour := clWindow;
  FGraphColour := $D8FFFF;
  FUpColour := clGreen;
  FDownColour := clRed;
  FOtherColour := clBlue;
  FYMin := -170;
  FYMax := -60;
  FYIncrement := 20;
  FGraphType := gtQLN;
  VDSL2 := False;
  FSnapHeight := 400;
  FSnapWidth := 600;
  FScaleSnap := False;
  ABitmap := TBitmap.Create;
end;

destructor TLinediagchart.Destroy;
begin
  ABitmap.Free;
  inherited Destroy;
end;

procedure TLinediagchart.SetGraphType(Value: TGraphType);
begin
  if Value <> FGraphType then
  begin
    FGraphType := Value;
    if FGraphType = gtSNR then
    begin
      FYMin := -20;
      FYMax := 70;
    end
    else if FGraphType = gtBitswap then
    begin
      FYMin := 0;
      FYMax := 10;
    end
    else if FGraphType = gtQLN then
    begin
      FYMin := -170;
      FYMax := -60;
    end
    else begin         //gtHlog
      FYMin := -120;
      FYMax := 0;
    end;
    Invalidate;
  end;
end;


procedure TLinediagchart.Paint;
begin
  try
    DrawBitmap(ABitmap);  //Do all the drawing on an in-memory bitmap before copying it to the linediagchart canvas
    Canvas.Draw(0, 0, ABitmap);
  except
    on E: Exception do;  //do nothing
  end;
end;

procedure TLinediagchart.DrawBitmap(Bmp: TBitmap);
var
  X1, X2, XPos, PrevX, ThisX, Y1, Y2, YUsed, YInc, Y, YVal, Tone, Tick, Len, Tonesper50, ThisY, PrevY: Integer;
  NumIncrements: Word;
  S: string;
  ToneValue: single;
  BaselineStyle: TPenStyle;
begin
  X1 := FXOffset;
  Y1 := 0;
  X2 := Width;
  Y2 := Height-FYOffset;
  Tonesper50 := Round(50 * XMultiplier);
  FToneIncrement := Tonesper50;
  if (FGraphType in [gtQLN, gtHLog, gtSNR]) then FYIncrement := 10
  else FYIncrement := (FYMax - FYMin) div 10;
  Bmp.Width := Width;
  Bmp.Height := Height;
  if FPlotBaseline then BaselineStyle := psDot else BaselineStyle := psClear;
  with Bmp.Canvas do
  begin
    Font.Assign(Self.Font);
    Font.CharSet := ANSI_CHARSET;
    Brush.Color := FLinediagchartColour;
    Pen.Color := clBlack;
    Pen.Style := psSolid;
    Pen.Width := 1;
    {$IFDEF Windows}
    Rectangle(0, 0, Width, Height);
    FillRect(1, 1, Width - 1, Height - 1);
    {$ELSE}
    Rectangle(0, 0, Width, Height - 1);
    FillRect(1, 1, Width - 1, Height - 2);
    Pen.Color := clBackground;
    MoveTo(0, Height - 1);
    LineTo(Width, Height - 1);
    Pen.Color := clBlack;
    {$ENDIF}
    Brush.Color := FGraphColour;
    Rectangle(X1, Y1, X2, Y2);
    Brush.Color := FLinediagchartColour;

    //Draw grid

    //Vertical lines and X axis marks
    XPos := X1;
    Tone := FFirstTone;
    Font.Size := 8;
    Font.Orientation := 0;
    while XPos <= X2 do
    begin
      Tick := 0; //Ticks every 10 pixels
      while Tick < 50 do
      begin
        Pen.Style := psSolid;
        if Tick = 0 then MoveTo(XPos + Tick, Y2 + 4)
        else MoveTo(XPos + Tick, Y2 + 2);
        LineTo(XPos + Tick, Y2 - 2);
        if (Tick = 0) and (Tone > 0) then
        begin
          Pen.Style := psDot;
          LineTo(XPos, 0);
        end;
        Tick := Tick + 10;
        if XPos + Tick > X2 then Break;
      end;
      S := IntToStr(Tone);
      Len := TextWidth(S);
      TextOut(XPos  - (Len shr 1), Y2 + 4, S);
      FMaxTone := Tone;
      XPos := XPos + 50;
      Tone := Tone + Tonesper50;
    end;
    {$IFDEF Windows}
      Font.Size := 8;
    {$ELSE}
      Font.Size := 9;
    {$ENDIF}
    S := FormatDateTime('dd mmm yyyy hh:mm:ss', RefreshTime);
    TextOut(X1, Y2 + 20, S);
    S := 'Tone number';
    Len := TextWidth(S);
    TextOut(X1 + ((X2 - X1) shr 1)- (Len shr 1), Y2 + 20, S);
    Len := TextWidth(FVersionString);
    TextOut(Width - Len - 10, Y2 + 20, FVersionString);

    //Horizontal lines and Y axis marks
    if (FGraphType in [gtQLN, gtHLog, gtSNR]) then NumIncrements := (FYMax - FYMin) div 10
    else NumIncrements := 10;
    YInc := (Y2 - 8) div NumIncrements;   //pixels per axis mark
    YUsed := YInc * NumIncrements;
    YVal := FYMin;
    Y := Y2 - 1;
    Font.Size := 8;
    repeat
      Pen.Style := psSolid;
      MoveTo(X1 - 2, Y - (YInc shr 1));  //intermediate tick marks
      LineTo(X1 + 2, Y - (YInc shr 1));
      Moveto(X1 - 4, Y);   //tick marks
      Lineto(X1 + 4, Y);
      Pen.Style := psDot;
      if Y < Y2 then LineTo(X2, Y);
      S := IntToStr(YVal);
      Len := TextWidth(S);
      TextOut(X1 - Len - 4, Y - 6, S);
      Y := Y - YInc;
      YVal := YVal + FYIncrement;
    until Y < 0;

    //Y axis labels
    {$IFDEF Windows}
      Font.Size := 8;
    {$ELSE}
      Font.Size := 9;
    {$ENDIF}
    if FGRaphType = gtQLN then S := 'QLN (dBm/Hz)'
    else if FGRaphType = gtHLog then S := 'HLog (dB)'
    else if FGraphType = gtBitswap then S := 'Bitswaps'
    else S := 'SNR per tone (dB)';
    Len := TextWidth(S);
    Font.Orientation := 900;
    TextOut(6, (Y1 + Y2) shr 1 + (Len shr 1), S);

    //Draw graphs

    if Displaying then  //Tone data has been stored
    begin
      Pen.Style := psSolid;
      Pen.Width := FThickness;
      XPos := X1;
      Tone := FFirstTone;
      repeat
        XPos := X1 + Round((Tone - FFirstTone) * 50 / Tonesper50);
        ToneValue := FGraphArray[Tone];
        if Tone = FFirstTone then
        begin
          PrevX := XPos;
          PrevY := Y2 - 1 - YUsed + Round((FYMax - ToneValue) / (FYMax - FYMin) * YUsed);
        end
        else begin
          ThisX := XPos;
          ThisY := Y2 - 1 - YUsed + Round((FYMax - ToneValue) / (FYMax - FYMin) * YUsed);
          if PrevY <> 0 then
          begin
            if ((FGraphType = gtQLN) and ((ToneValue < -159) or (FGraphArray[Tone - 1] < -159)))
            or ((FGraphType = gtHLog) and ((ToneValue < -95) or (FGraphArray[Tone - 1] < -95))) then Pen.Style := BaselineStyle
            else Pen.Style := psSolid;         //Don't plot baseline values or plot them dotted
            Pen.Color := GetPenColour(Tone);
            MoveTo(PrevX, PrevY);
            LineTo(ThisX, ThisY);
          end;
          PrevY := ThisY;
          PrevX := ThisX;
        end;
        Inc(Tone);
      until (XPos > X2) or (Tone = FLastTone);
    end;

    //Draw the legend in top right or top left corner

    if FShowLegend then
    begin
      Brush.Color := clWhite;
      Pen.Color := clBlack;
      Pen.Style := psSolid;
      Pen.Width := 1;
      if VDSL2 then
      begin
        if FLegendPosition = lpldRight then Rectangle(X2 - 110, 5, X2 - 5, 58)
        else Rectangle(FXOffset + 5, 5, FXOffset + 110, 58);
      end
      else begin
        if FLegendPosition = lpldRight then Rectangle(X2 - 110, 5, X2 - 5, 43)
        else Rectangle(FXOffset + 5, 5, FXOffset + 110, 43);
      end;
      Pen.Width := 2;
      Pen.Color := FUpColour;
      if FLegendPosition = lpldRight then
      begin
        MoveTo(X2 - 105, 15);
        LineTo(X2 - 85, 15);
      end
      else begin
        MoveTo(FXOffset + 10, 15);
        LineTo(FXOffset + 30, 15);
      end;
      Pen.Color := FDownColour;
      if FLegendPosition = lpldRight then
      begin
        MoveTo(X2 - 105, 30);
        LineTo(X2 - 85, 30);
      end
      else begin
        MoveTo(FXOffset + 10, 30);
        LineTo(FXOffset + 30, 30);
      end;
      if VDSL2 then
      begin
        Pen.Color := FOtherColour;
        if FLegendPosition = lpldRight then
        begin
          MoveTo(X2 - 105, 45);
          LineTo(X2 - 85, 45);
        end
        else begin
          MoveTo(FXOffset + 10, 45);
          LineTo(FXOffset + 30, 45);
        end;
      end;
      Font.Size := 8;
      Font.Orientation := 0;
      if FLegendPosition = lpldRight then
      begin
        TextOut(X2 - 83, 8, 'Upstream');
        TextOut(X2 - 83, 23, 'Downstream');
        if VDSL2 then TextOut(X2 - 83, 38, 'Other');
      end
      else begin
        TextOut(FXOffset + 32, 8, 'Upstream');
        TextOut(FXOffset + 32, 23, 'Downstream');
        if VDSL2 then TextOut(FXOffset + 32, 38, 'Other');
      end;
    end;
  end;
end;

//Get the display colour for a tone
function TLinediagchart.GetPenColour(Count: Integer): TColor;
begin
  if FLastTone < 512 then
  begin
    if Count < 32 then Result := FUpColour
    else Result := FDownColour;
  end
  else begin
    if ((Count >= FUpTones[0]) and (Count <= FUpTones[1]))
    or ((Count >= FUpTones[2]) and (Count <= FUpTones[3]))
    or ((Count >= FUpTones[4]) and (Count <= FUpTones[5]))
    or ((Count >= FUpTones[6]) and (Count <= FUpTones[7]))
    then Result := FUpColour
    else if ((Count >= FDownTones[0]) and (Count <= FDownTones[1]))
    or ((Count >= FDownTones[2]) and (Count <= FDownTones[3]))
    or ((Count >= FDownTones[4]) and (Count <= FDownTones[5]))
    or ((Count >= FDownTones[6]) and (Count <= FDownTones[7]))
    then Result := FDownColour
    else Result := FOtherColour;
  end;
end;

//Save snapshot
function TLinediagchart.SaveToPNGFile(Filename: string): Boolean;
var
  Png: TPortableNetworkGraphic;
  Bmp: TBitmap;
begin
  Result := True;
  Bmp := TBitmap.Create;
  try
    DrawBitmap(Bmp);
  except
    on E: Exception do
    begin
      Result := False;
      Bmp.Free;
      Exit;
    end;
  end;
  Png := TPortableNetworkGraphic.Create;
  try
    if FScaleSnap then
    begin
      Png.Width := FSnapWidth;
      Png.Height := FSnapHeight;
      Png.Canvas.StretchDraw(Rect(0, 0, FSnapWidth - 1, FSnapHeight - 1), Bmp);
    end
    else Png.Assign(Bmp);
    Png.SaveToFile(Filename);
  except
    on E: Exception do
       Result := False;
  end;
  Png.Free;
  Bmp.Free;
end;

function TLinediagChart.GetGraphArray(Index: Word): Single;
begin
  try
    Result := FGraphArray[Index];
  except
    on E: Exception do Result := 0;
  end;
end;

procedure TLinediagchart.SetGraphArray(Index: Word; Value: Single);
begin
  try
    FGraphArray[Index] := Value;
  except
    on E: Exception do;   //Do nothing
  end;
end;


procedure TLinediagchart.DrawGraph(HighestTone: Integer);
begin
  if FExtendedToneRange then FLastTone := LastToneNumber
  else FLastTone := HighestTone;
  NormalHighestTone := HighestTone;
  Displaying := True;
  RefreshTime := Now;
  Invalidate;
end;

procedure TLinediagchart.SetThickness(Value: Integer);
begin
  if Value <> FThickness then
  begin
    FThickness := Value;
    Invalidate;
  end;
end;

procedure TLinediagchart.SetLegendPosition(Value: TLegendPosition);
begin
  if Value <> FLegendPosition then
  begin
    FLegendPosition := Value;
    Invalidate;
  end;
end;

procedure TLinediagchart.SetShowLegend(Value: Boolean);
begin
  if Value <> FShowLegend then
  begin
    FShowLegend := Value;
    Invalidate;
  end;
end;

procedure TLinediagchart.SetLinediagchartColour(Value: TColor);
begin
  if Value <> FLinediagchartColour then
  begin
    FLinediagchartColour := Value;
    Invalidate;
  end;
end;

procedure TLinediagchart.SetVDSL2(Value: Boolean);
begin
  if Value <> FVDSL2 then
  begin
    FVDSL2 := Value;
    Invalidate;
  end;
end;

procedure TLinediagchart.SetXOffset(Value: Integer);
begin
  if Value < Self.Width shr 1 then
  begin
    FXOffset := Value;
    Invalidate;
  end;
end;

procedure TLinediagchart.SetYOffset(Value: Integer);
begin
    if Value < Self.Height shr 1 then
  begin
    FYOffset := Value;
    Invalidate;
  end;
end;

procedure TLinediagchart.SetToneIncrement(Value: Integer);
begin
  FToneIncrement := Value;
  Invalidate
end;

procedure TLinediagchart.SetYMin(Value: Integer);
begin
    if (Value <> FYMin) and (Value < FYMax) then FYMin := Value;
    Invalidate;
end;

procedure TLinediagchart.SetYMax(Value: Integer);
begin
    if (Value <> FYMax) and (Value > FYMin) then FYMax := Value;
    Invalidate;
end;

procedure TLinediagchart.SetFirstTone(Value: Integer);
begin
  if Value < 0 then Value := 0;
  if Value <> FFirstTone then
  begin
    FFirstTone := Value;
    Invalidate;
  end;
end;

procedure TLinediagchart.SetLastTone(Value: Integer);
begin
  if Value <> FLastTone then
  begin
    FLastTone := Value;
    Invalidate;
  end;
end;

procedure TLinediagchart.SetGraphColour(Value: TColor);
begin
  if Value <> FGraphColour then
  begin
    FGraphColour := Value;
    Invalidate;
  end;
end;

procedure TLinediagchart.SetUpColour(Value: TColor);
begin
  if Value <> FUpColour then
  begin
    FUpColour := Value;
    Invalidate;
  end;
end;

procedure TLinediagchart.SetDownColour(Value: TColor);
begin
  if Value <> FDOwnColour then
  begin
    FDownColour := Value;
    Invalidate;
  end;
end;

procedure TLinediagchart.SetOtherColour(Value: TColor);
begin
  if Value <> FOtherColour then
  begin
    FOtherColour := Value;
    Invalidate;
  end;
end;

procedure TLinediagchart.SetUpTones(Index: Integer; Value: Integer);
begin
  FUpTones[Index] := Value;
end;

procedure TLineDiagchart.SetTonesperpixel(Value: Shortint);
begin
  if Value <> FTonesperpixel then
  begin
    FTonesperpixel := Value;
    case Value of
      10: XMultiplier := 10;
      9:  XMultiplier := 9;
      8:  XMultiplier := 8;
      7:  XMultiplier := 7;
      6:  XMultiplier := 6;
      5:  XMultiplier := 5;
      4:  XMultiplier := 4;
      3:  XMultiplier := 3;
      2:  XMultiplier := 1.8;
      1:  XMultiplier := 0.85;
      0:  XMultiplier := 0.55;
      -1: XMultiplier := 0.35;
      -2: XMultiplier := 0.25;
      -3: XMultiplier := 0.2;
    else
      XMultiplier := 1;
    end;
    Invalidate;
  end;
end;

procedure TLinediagchart.SetDownTones(Index: Integer; Value: Integer);
begin
  FDownTones[Index] := Value;
end;

procedure TLinediagchart.SetExtendedToneRange(Value: Boolean);
begin
  if Value <> FExtendedToneRange then
  begin
    FExtendedToneRange := Value;
    if FExtendedToneRange then FLastTone := LastToneNumber
    else begin
      FLastTone := NormalHighestTone;
      FFirstTone := 0;
    end;
    Invalidate;
  end;
end;

procedure TLinediagchart.SetPlotBaseline(Value: Boolean);
begin
  if Value <> FPlotBaseline then
  begin
    FPlotBaseline := Value;
    if FPlotBaseline then FLastTone := LastToneNumber
    else begin
      FLastTone := NormalHighestTone;
      FFirstTone := 0;
    end;
    Invalidate;
  end;
end;

procedure TLinediagchart.SetDSLAMType(Value: TDSLAMType);
begin
  if Value <> FDSLAMType then
  begin
    FDSLAMType := Value;
    Invalidate;
  end;
end;

procedure TLinediagChart.SetVersionString(Value: string);
begin
  if Value <> FVersionString then
  begin
    FVersionString := Value;
    Invalidate;
  end;
end;

procedure Register;
begin
  RegisterComponents('Eric',[TLinediagchart]);
end;

initialization
{$I dsgraphs.lrs}

end.
