unit SNRMperband;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dateutils, Dialogs, LCLType;

type

  TBandType = (btUp, btDown);
  TLegendPosition = (lppLeft, lppRight);

  TBandPoint = record
    XValue: TDateTime;
    YValues: array[0..4] of Single;
    DrawBlank: Boolean;
  end;
  PBandPoint = ^TBandPoint;

  TGraphPoints = record
    XPos: Integer;
    YPos: array[0..4] of Integer;
  end;

  TSNRMperband = class(TGraphicControl)
  private
    { Private declarations }
    ABitmap: TBitmap;
    ArraySize: Integer;
    FBandchartColour, FGraphColour: TColor;
    FBandColours: array[0..4] of TColor;
    FBandsIncluded, BandsUsed: array[0..4] of Boolean;
    FBandType: TBandType;
    FBrush: TBrush;
    FLegendPosition: TLegendPosition;
    FMaxYValue, FMinYValue, FClipValue, HighestY: Single;
    FNumSamples: Integer;
    FPageStart: TDateTime;
    FPen: TPen;
    FShowLegend, FUseClipping, FScaleSnap: Boolean;
    FSnapHeight, FSnapWidth: Integer;
    FVersionString: string;
    FXOffset, FYOffset, FThickness, FHoursperpage: Word;
    NumBandsUsed: Byte;
    PreviousPoints, NewPoints: TGraphPoints;
    function GetBandColours(Index: Byte): TColor;
    procedure SetBandchartColour(Value: TColor);
    procedure SetBandColours(Index: Byte; Value: TColor);
    procedure SetBandType(Value: TBandType);
    procedure SetClipValue(Value: Single);
    procedure SetGraphColour(Value: TColor);
    procedure SetHoursperpage(Value: Word);
    procedure SetLegendPosition(Value: TLegendPosition);
    procedure SetMaxYValue(Value: Single);
    procedure SetMinYValue(Value: Single);
    procedure SetShowLegend(Value: Boolean);
    procedure SetThickness(Value: Word);
    procedure SetUseClipping(Value: Boolean);
    procedure SetVersionString(Value: string);
    procedure SetXOffset(Value: Word);
    procedure SetYOffset(Value: Word);
  protected
    { Protected declarations }
    procedure DrawBitmap(Bmp: TBitmap);
    procedure Paint; override;
  public
    { Public declarations }
    SampleArray: array of TBandPoint;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function SaveToPNGFile(Filename: string): Boolean;
    procedure AddRecord(Rec: PBandPoint);
    procedure ClearArray;
    procedure SetBandsIncluded(Index: Byte; Value: Boolean);
    procedure SetPageStart(Value: TDateTime);
    property BandColours[Index: Byte]: TColor read GetBandColours write SetBandColours;
    property NumSamples: Integer read FNumSamples;
  published
    { Published declarations }
    property Align default alNone;
    property Anchors;
    property BandchartColour: TColor read FBandchartColour write SetBandchartColour;
    property BandType: TBandType read FBandType write SetBandType;
    property Caption;
    property ClipValue: Single read FClipValue write SetClipValue default 20;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property GraphColour: TColor read FGraphColour write SetGraphColour;
    property GraphThickness: Word read FThickness write SetThickness default 1;
    property Hoursperpage: Word read FHoursperpage write SetHoursperpage default 2;
    property LegendPosition: TLegendPosition read FLegendPosition write SetLegendPosition;
    property MaxYValue: Single read FMaxYValue write SetMaxYValue default 10;
    property MinYValue: Single read FMinYValue write SetMinYValue default 0;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property PageStart: TDateTime read FPageStart;
    property ParentColor;
    property ParentFont;
    property ScaleSnap: Boolean read FScaleSnap write FScaleSnap;
    property ShowHint;
    property ShowLegend: Boolean read FShowLegend write SetShowLegend;
    property SnapHeight: Integer read FSnapHeight write FSnapHeight;
    property SnapWidth: Integer read FSnapWidth write FSnapWidth;
    property UseClipping: Boolean read FUseClipping write SetUseClipping;
    property VersionString: string read FVersionString write SetVersionString;
    property Visible;
    property XOffset: Word read FXOffset write SetXOffset default 40;
    property YOffset: Word read FYOffset write SetYOffset default 60;
  end;

procedure Register;

implementation

constructor TSNRMperband.Create(AOwner: TComponent);
var
  N: Byte;
const
  Colours: array[0..4] of TColor = (clPurple, clGreen, clRed, clBlue, $004080);
begin
  inherited Create(AOwner);
  Self.Width := 200;
  Self.Height := 150;
  Self.Align := alNone;
  FXOffset := 40;
  FYOffset := 65;
  FShowLegend := True;
  FBandType := btUp;
  FBrush := TBrush.Create;
  FPen := TPen.Create;
  FGraphColour := $d8ffff;
  FBandchartColour := clWindow;
  for N := 0 to 4 do
  begin
    FBandColours[N] := Colours[N];
    FBandsIncluded[N] := True;
    BandsUsed[N] := True;
  end;
  NumBandsUsed := 5;
  FThickness := 1;
  FHoursperpage := 2;
  FPageStart := Now;
  FMaxYValue := 10;
  FMinYValue := 0;
  FClipValue := 20;
  FUseClipping := False;
  FNumSamples := 0;
  FSnapHeight := 400;
  FSnapWidth := 600;
  FScaleSnap := False;
  ArraySize := 1000;
  SetLength(SampleArray, ArraySize);
  ABitmap := TBitmap.Create;
end;

destructor TSNRMperband.Destroy;
begin
  ABitmap.Free;
  FPen.Free;
  FBrush.Free;
  inherited Destroy;
end;

procedure TSNRMperband.Paint;
begin
  try
    DrawBitmap(ABitmap);  //Do all the drawing on an in-memory bitmap before copying it to the linechart canvas
    Canvas.Draw(0, 0, ABitmap);
  except
    on E: Exception do;  //Do nothing, just trap exception
  end;
end;

procedure TSNRMperband.DrawBitmap(Bmp: TBitmap);
var
  N, N1, XPos, XEnd, MinutesIncrement, XIncrement, Len, YPos, YIncrement,
    YVal, YValInc: Integer;
  Loop, X1, Y1, X2, Y2: Word;
  AYear, AMonth, ADay, AHour, AMinute, ASecond, AMillisecond: Word;
  Source, Dest: TRect;
  PageEnd, XTime: TDateTime;
  XFraction, Value: Single;
  S, TimeString: string;
begin
  for N := 0 to 4 do BandsUsed[N] := True;
  NumBandsUsed := 5;
  if HighestY > 0 then S := IntToStr(Round(HighestY * 10))
  else S := IntToStr(Round(FMaxYValue * 10));
  Len := Bmp.Canvas.TextWidth(S);
  FXOffset := Len + 25;
  X1 := FXOffset;
  Y1 := 0;
  X2 := Width;
  Y2 := Height - FYOffset;
  PageEnd := FPageStart + EncodeTimeInterval(FHoursperpage, 0, 0, 0);
  MinutesIncrement := FHoursperpage * 5;
  DecodeDateTime(FPageStart, AYear, AMonth, ADay, AHour, AMinute, ASecond, AMillisecond);
  if ASecond > 29 then    //round to the nearest minute
  begin
    Inc(AMinute);
    if AMinute = 60 then
    begin
      AMinute := 0;
      Inc(AHour);
      if AHour = 24 then AHour := 0;
    end;
  end;

  //Calculate X axis values

  XIncrement := (Width - FXOffset - 15) div 12;  //Distance between axis marks

  //Calculate Y axis values

  if FUseClipping and (FMaxYValue > FClipValue) then FMaxYValue := FClipValue;
  if FMaxYValue < 10 then FMaxYValue := 10;
  YIncrement := Round((Height - FYOffset - 10) / 10 - 0.5);
  YValInc := Round(FMaxYValue / 10);

  Bmp.Width := Width;
  Bmp.Height := Height;
  with Bmp.Canvas do
  begin
    Font.Assign(Self.Font);
    Font.CharSet := ANSI_CHARSET;
    Brush.Color := FBandchartColour;
    Pen.Color := clBlack;
    Pen.Style := psSolid;
    Pen.Width := 1;
    {$IFDEF Windows}
    Rectangle(0, 0, Width, Height);
    FillRect(1, 1, Width - 1, Height - 1);
    {$ELSE}
    Rectangle(0, 0, Width, Height - 1);
    FillRect(1, 1, Width - 1, Height - 2);
    Pen.Color := clBackground;
    MoveTo(0, Height - 1);
    LineTo(Width, Height - 1);
    Pen.Color := clBlack;
    {$ENDIF}
    Brush.Color := FGraphColour;
    Rectangle(X1, Y1, X2, Y2);
    Brush.Color := FBandchartColour;

    //Draw grid

    //X axis marks and vertical lines

    XPos := X1 + 5;
    N := 0;
    Font.Orientation := 900;
    repeat
      Pen.Style := psSolid;
      MoveTo(XPos + (XIncrement shr 1), Y2 + 2); //intermediate marks
      LineTo(XPos + (XIncrement shr 1), Y2 - 2);
      MoveTo(XPos, Y2 + 4); //main marks
      LineTo(XPos, Y2 - 4);
      Pen.Style := PsDot;
      LineTo(XPos, Y1);
      S := IntToStr(AHour);
      if AHour < 10 then S := '0' + S;
      Timestring := S + ':';
      S := IntToStr(AMinute);
      if AMinute < 10 then S := '0' + S;
      Timestring := Timestring + S;
      Len := TextWidth(Timestring);
      {$IFDEF Windows}
      TextOut(XPos - 7, Y2 + Len + 6, Timestring);
      {$ELSE}
      TextOut(XPos - 5, Y2 + Len + 6, Timestring);
      {$ENDIF}
      XEnd := XPos;
      XPos := XPos + XIncrement;
      Inc(N);
      AMinute := AMinute + MinutesIncrement;
      if AMinute > 59 then
      repeat
        AMinute := AMinute - 60;
        Inc(AHour);
        if AHour = 24 then AHour := 0;
      until AMinute < 60;
    until XPos > Width;
    S := 'Time';
    {$IFDEF Windows}
      Font.Size := 8;
    {$ELSE}
      Font.Size := 9;
    {$ENDIF}
    Len := TextWidth(S);
    Font.Orientation := 0;
    TextOut((Width shr 1) - (Len shr 1), Height - 20, S);
    S := '|<-- ' + FormatDateTime('dd mmm yyyy', FPageStart);
    TextOut(FXOffset, Height - 20, S);
    Len := TextWidth(FVersionString);
    TextOut(X2 - Len - 10, Height - 20, FVersionString);

    //Y axis marks and horizontal lines

    YPos := Y2 - 1;
    YVal := Round(FMinYValue);
    Font.Orientation := 0;
    Font.Size := 8;
    repeat
      Pen.Style := psSolid;
      MoveTo(X1 - 2, YPos - (YIncrement shr 1));  //intermediate marks
      LineTo(X1 + 2, YPos - (YIncrement shr 1));
      MoveTo(X1 - 4, YPos);  //main marks
      LineTo(X1 + 4, YPos);
      Pen.Style := psDot;
      LineTo(X2, YPos);
      S := IntToStr(YVal);
      Len := TextWidth(S);
      TextOut(FXOffset - Len - 5, YPos - 5, S);
      YVal := YVal + YValInc;
      YPos := YPos - YIncrement;
    until YVal > FMaxYValue;
    Font.Orientation := 900;
    {$IFDEF Windows}
      Font.Size := 8;
    {$ELSE}
      Font.Size := 9;
    {$ENDIF}
    S := 'SNRM per band (dB)';
    Len := TextWidth(S);
    TextOut(3, (Y1 + Y2) shr 1 + (Len shr 1), S);

    //Plot the graphs

    if FNumSamples > 1 then
    begin
      Pen.Style := psSolid;
      Pen.Width := FThickness;
      N := FNumSamples;
      repeat
        Dec(N);
        if N = -1 then Break;
      until SampleArray[N].XValue < FPageStart;
      Inc(N);   //N is now first sample to display
      XTime := SampleArray[N].XValue;
      if XTime = FPagestart then XFraction := 0
      else XFraction := (XTime - FPageStart) / (PageEnd - FPagestart);
      XPos := X1 + 5 + Round(XFraction * (XEnd - X1 - 5));
      PreviousPoints.XPos := XPos;
      for Loop := 0 to 4 do
      begin
        Value := SampleArray[N].YValues[Loop];
        if Value = -255 then   //Don't plot and don't include in legend
        begin
          YPos := -1;
          BandsUsed[Loop] := False;
          Dec(NumBandsUsed);
        end
        else begin
          if Value < FMinYValue then Value := FMinYValue;
          if FUseClipping and (Value > FClipValue) then Value := FClipValue;
          if not FBandsIncluded[Loop] then YPos := -1     //Don't plot
          else YPos := Y2 - 1 - Round((Value - FMinYValue) * YIncrement / YValInc);
        end;
        PreviousPoints.YPos[Loop] := YPos;
      end;
      N1 := N;
      repeat
        Inc(N1);
        XTime := SampleArray[N1].XValue;
        XFraction := (XTime - FPageStart) / (PageEnd - FPagestart);
        XPos := X1 + 5 + Round(XFraction * (XEnd - X1 - 5));
        NewPoints.XPos := XPos;
        for Loop := 0 to 4 do
        begin
          Value := SampleArray[N1].YValues[Loop];
          if (Value = -255) or not FBandsIncluded[Loop] then YPos := -1     //Don't plot
          else begin
            if Value < FMinYValue then Value := FMinYValue;
            if FUseClipping and (Value > FClipValue) then Value := FClipValue;
            YPos := Y2 - 1 - Round((Value - FMinYValue) * YIncrement / YValInc);
          end;
          NewPoints.YPos[Loop] := YPos;
        end;
        for Loop := 0 to 4 do
        begin
          Pen.Color := FBandColours[Loop];
          MoveTo(PreviousPoints.XPos, PreviousPoints.YPos[Loop]);
          LineTo(NewPoints.XPos, NewPoints.YPos[Loop]);
        end;
        PreviousPoints := NewPoints;
      until N1 = FNumSamples - 1;
    end;

    //Draw the legend in top right corner

    if FShowLegend then
    begin
      Font.Orientation := 0;
      Brush.Color := clWhite;
      Pen.Color := clBlack;
      Pen.Style := psSolid;
      Pen.Width := 1;
      if FLegendPosition = lppRight then Rectangle(X2 - 55, 5, X2 - 5, 12 + (NumBandsUsed * 14))
      else Rectangle(FXOffset + 5, 5, FXOffset + 55, 12 + (NumBandsUsed * 14));
      Pen.Width := 2;
      N1 := 0;
      for N := 0 to 4 do
      if BandsUsed[N] then
      begin
        Pen.Color := FBandColours[N];
        if FBandType = btUp then S := 'U' + IntToStr(N) else S := 'D' + IntToStr(N);
        if FLegendPosition = lppRight then
        begin
          MoveTo(X2 - 50, 14 * (N1 + 1));
          Lineto(X2 - 30, 14 * (N1 + 1));
          Font.Size := 8;
          TextOut(X2 - 25, 9 + (N1 * 14), S);
        end
        else begin
          MoveTo(FXOffset + 10, 14 * (N1 + 1));
          Lineto(FXOffset + 30, 14 * (N1 + 1));
          Font.Size := 8;
          TextOut(FXOffset + 35, 9 + (N1 * 14), S);
        end;
        Inc(N1);
      end;
    end;
  end;
end;

//Add a new sample record
procedure TSNRMperband.AddRecord(Rec: PBandPoint);
var
  N: Byte;
begin
  SampleArray[FNumSamples] := Rec^;
  Inc(FNumSamples);
  for N := 0 to 4 do
  begin
    if Rec^.YValues[N] > FMaxYValue then FMaxYValue := Rec^.YValues[N];
  end;
  if FMaxYValue > 10 then
  begin
    if FMaxYValue > 20 then FMaxYValue := 40 else FMaxYValue := 20;
  end;
  if FNumSamples > ArraySize - 50 then
  begin
    ArraySize := ArraySize + 1000;
    SetLength(SampleArray, ArraySize);
  end;
  FPageStart := Rec^.XValue - EncodeTimeInterval(FHoursperpage, 0, 0, 0);
  if FPageStart < SampleArray[0].XValue then FPageStart := SampleArray[0].XValue;
  Invalidate;
end;

//Reset the array
procedure TSNRMperband.ClearArray;
begin
  FPageStart := Now;
  if FNumSamples = 0 then Exit;
  FNumSamples := 0;
  SampleArray := nil;
  SetLength(SampleArray, 1000);
  FMaxYValue := 10;
  FMinYValue := 0;
end;

//Save a snapshot in .png format
function TSNRMperband.SaveToPNGFile(Filename: string): Boolean;
var
  Png: TPortableNetworkGraphic;
  Bmp: TBitmap;
begin
  Result := True;
  Bmp := TBitmap.Create;
  try
    DrawBitmap(Bmp);
  except
    on E: Exception do
    begin
      Result := False;
      Bmp.Free;
      Exit;
    end;
  end;
  Png := TPortableNetworkGraphic.Create;
  try
    if FScaleSnap then
    begin
      Png.Width := FSnapWidth;
      Png.Height := FSnapHeight;
      Png.Canvas.StretchDraw(Rect(0, 0, FSnapWidth - 1, FSnapHeight - 1), Bmp);
    end
    else Png.Assign(Bmp);
    Png.SaveToFile(Filename);
  except
    on E: Exception do
      Result := False;
  end;
  Png.Free;
  Bmp.Free;
end;

procedure TSNRMperband.SetBandType(Value: TBandType);
begin
  if Value <> FBandType then
  begin
    FBandType := Value;
    Invalidate;
  end;
end;

procedure TSNRMperband.SetLegendPosition(Value: TLegendPosition);
begin
  if Value <> FLegendPosition then
  begin
    FLegendPosition := Value;
    Invalidate;
  end;
end;

procedure TSNRMperband.SetBandsIncluded(Index: Byte; Value: Boolean);
begin
  FBandsIncluded[Index] := Value;
  Invalidate;
end;

procedure TSNRMperband.SetXOffset(Value: Word);
begin
  if Value < Self.Width shr 1 then
  begin
    FXOffset := Value;
    Invalidate;
  end;
end;

procedure TSNRMperband.SetYOffset(Value: Word);
begin
    if Value < Self.Height shr 1 then
  begin
    FYOffset := Value;
    Invalidate;
  end;
end;

procedure TSNRMperband.SetThickness(Value: Word);
begin
  if Value <> FThickness then
  begin
    FThickness := Value;
    Invalidate;
  end;
end;

procedure TSNRMperband.SetBandchartColour(Value: TColor);
begin
  if Value <> FBandchartColour then
  begin
    FBandchartColour := Value;
    Invalidate;
  end;
end;

procedure TSNRMperband.SetGraphColour(Value: TColor);
begin
  if Value <> FBrush.Color then
  begin
    FBrush.Color := Value;
    FGraphColour := Value;
    Invalidate;
  end;
end;

procedure TSNRMperband.SetPageStart(Value: TDateTime);
begin
  if Value <> FPageStart then
  begin
    FPageStart := Value;
    Invalidate;
  end;
end;

procedure TSNRMperband.SetHoursperpage(Value: Word);
begin
  if Value <> FHoursperpage then
  begin
    FHoursperpage := Value;
    Invalidate;
  end;
end;

function TSNRMperband.GetBandColours(Index: Byte): TColor;
begin
  try
    Result := FBandColours[Index];
  except
    on E: Exception do Result := clBlack;
  end;
end;

procedure TSNRMperband.SetBandColours(Index: Byte; Value: TColor);
begin
  if Value <> FBandColours[Index] then
  begin
    FBandColours[Index] := Value;
    Invalidate;
  end;
end;

procedure TSNRMperband.SetMaxYValue(Value: Single);
begin
  if (Value <> FMaxYValue) and (Value > FMinYValue) then
  begin
    FMaxYValue := Value;
    Invalidate;
  end;
end;

procedure TSNRMperband.SetMinYValue(Value: Single);
begin
  if (Value <> FMinYValue) and (Value < FMaxYValue) then
  begin
    FMinYValue := Value;
    Invalidate;
  end;
end;

procedure TSNRMperband.SetClipValue(Value: Single);
begin
  if Value <> FClipValue then
  begin
    FClipValue := Value;
    if Value > FMaxYValue then FMaxYValue := Value;
    Invalidate;
  end;
end;

procedure TSNRMperband.SetUseClipping(Value: Boolean);
begin
  if Value <> FUseClipping then
  begin
    FUseClipping := Value;
    Invalidate;
  end;
end;

procedure TSNRMperband.SetShowLegend(Value: Boolean);
begin
  if Value <> FShowLegend then
  begin
    FShowLegend := Value;
    Invalidate;
  end;
end;

procedure TSNRMperband.SetVersionString(Value: string);
begin
  if Value <> FVersionString then
  begin
    FVersionString := Value;
    Invalidate;
  end;
end;

procedure Register;
begin
  RegisterComponents('Eric',[TSNRMperband]);
end;

initialization
{$I dsgraphs.lrs}

end.
