{DSLstats - a program for monitoring the state of a DSL internet connection
Copyright (C) 2012-2013 Eric Chapman

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License version 2
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

You can contact the author via the website http://dslstats.plainroad.me.uk
}

unit traffic;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, Dateutils, LCLType;

type
  TTrafficType = (utHourly, utDaily, utMonthly);
  TLegendPosition = (lptLeft, lptRight);

  TTrafficValues = record
    UpTraffic, DownTraffic: Int64;
  end;

  TESperhour = record
    UpES, DownES: Integer;
  end;

  TMonths = (fmJan, fmFeb, fmMar, fmApr, fmMay, fmJun, fmJul, fmAug, fmSep, fmOct, fmNov, fmDec);
  TDaysAgo = 0..100;

  TTraffic = class(TGraphicControl)
  private
    { Private declarations }
    ABitmap: TBitmap;
    DailyTraffic: array[0..1, 1..31] of TTrafficValues;
    DataList: TStringList;
    FAutoscale: Boolean;
    FBillingDay: Byte;
    FDaysAgo: TDaysAgo;
    FESDate: TDateTime;
    FVersionString: string;
    FFirstMonth: TMonths;
    FFirstYear, FMonthlyLimit, FHourlyMaxY: Word;
    FGraphType: TTrafficType;
    FLegendPosition: TLegendPosition;
    FShowLegend, FMonthlyLimitDownOnly, FShowBilling, FScaleSnap: Boolean;
    FSnapHeight, FSnapWidth: Integer;
    FUpColour, FDownColour, FBothColour, FTrafficColour, FGraphColour, FLimitColour: TColor;
    FUpString, FDownString, FTotalString: string;
    FXOffset, FYOffset: Integer;
    HourlyES: array[0..101, 1..24] of TESperhour;     //One extra day for overflow during display
    HourlyMax, DailyMax, MonthlyMax, MaxYValue: Int64;
    MonthlyTraffic: array[1..12] of TTrafficValues;
    XPositions: array[1..49] of Word;
    function GetXPosition(Index: Byte): Word;
    function ScaleValue(Value, MaxValue: Cardinal): Cardinal;
    procedure SetAutoscale(Value: Boolean);
    procedure SetBillingDay(Value: Byte);
    procedure SetBothColour(Value: TColor);
    procedure SetDaysAgo(Value: TDaysAgo);
    procedure SetDownColour(Value: TColor);
    procedure SetDownString(Value: string);
    procedure SetFirstMonth(Value: TMonths);
    procedure SetFirstYear(Value: Word);
    procedure SetGraphColour(Value: TColor);
    procedure SetGraphType(Value: TTrafficType);
    procedure SetHourlyMaxY(Value: Word);
    procedure SetLegendPosition(Value: TLegendPosition);
    procedure SetLimitColour(Value: TColor);
    procedure SetMonthlyLimit(Value: Word);
    procedure SetMonthlyLimitDownOnly(Value: Boolean);
    procedure SetShowBilling(Value: Boolean);
    procedure SetShowLegend(Value: Boolean);
    procedure SetTrafficColour(Value: TColor);
    procedure SetTotalString(Value: string);
    procedure SetUpColour(Value: TColor);
    procedure SetUpString(Value: string);
    procedure SetVersionString(Value: string);
    procedure SetXOffset(Value: Integer);
    procedure SetYOffset(Value: Integer);
  protected
    { Protected declarations }
    procedure Paint; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function LoadHourlyES(Filename: string): Boolean;
    function SaveHourlyES(Filename: string): Boolean;
    function SaveToPNGFile(Filename: string): Boolean;
    procedure ClearDaily;
    procedure ClearESArray;
    procedure DrawBitmap(Bmp: TBitmap);
    procedure GetDailyTraffic(AMonth, Index: Byte; var UpTraffic, DownTraffic: Int64);
    procedure GetHourlyES(DaysAgo: TDaysAgo; Hour: Byte; var UpES, DownES: Word);
    procedure GetMonthlyTraffic(Index: Byte; var UpTraffic, DownTraffic: Int64);
    procedure IncrementMonth;
    procedure LoadTrafficData(HomePath: string);
    procedure SaveTrafficData(HomePath: string);
    procedure Scrolldays(Days: Word);
    procedure SetDailyTraffic(AMonth, Index: Byte; UPTraffic, DownTraffic: Int64);
    procedure SetESDate(Value: TDateTime);
    procedure SetHourlyES(DaysAgo: TDaysAgo; Hour: Byte; UpES, DownES : Word);
    procedure SetHourlyMax(Higher: Word);
    procedure SetMonthlyTraffic(Index: Byte; UpTraffic, DownTraffic: Int64);
    procedure ZeroData;
    property XPosition[Index: Byte]: Word read GetXPosition;
  published
    { Published declarations }
    property Align default alNone;
    property Anchors;
    property Autoscale: Boolean read FAutoscale write SetAutoscale;
    property BillingDay: Byte read FBillingDay write SetBillingDay;
    property BothColour: TColor read FBothColour write SetBothColour;
    property Caption;
    property Color;
    property DaysAgo: TDaysAgo read FDaysAgo write SetDaysAgo;
    property DownColour: TColor read FDownColour write SetDownColour;
    property DownString: string read FDownString write SetDownString;
    property Enabled;
    property ESDate: TDateTime read FESDate write SetESDate;
    property FirstMonth: TMonths read FFirstMonth write SetFirstMonth;
    property FirstYear: Word read FFirstYear write SetFirstYear;
    property Font;
    property GraphColour: TColor read FGraphColour write SetGraphColour;
    property GraphType: TTrafficType read FGraphType write SetGraphType;
    property Hint;
    property HourlyMaxY: Word read FHourlyMaxY write SetHourlyMaxY;
    property LegendPosition: TLegendPosition read FLegendPosition write SetLegendPosition;
    property LimitColour: TColor read FLimitColour write SetLimitColour;
    property MonthlyLimit: Word read FMonthlyLimit write SetMonthlyLimit;
    property MonthlyLimitDownOnly: Boolean read FMonthlyLimitDownOnly write SetMonthlyLimitDownOnly;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property ParentColor;
    property ParentFont;
    property PopupMenu;
    property ScaleSnap: Boolean read FScaleSnap write FScaleSnap;
    property ShowHint;
    property ShowBilling: Boolean read FShowBilling write SetShowBilling;
    property ShowLegend: Boolean read FShowLegend write SetShowLegend;
    property SnapHeight: Integer read FSnapHeight write FSnapHeight;
    property SnapWidth: Integer read FSnapWidth write FSnapWidth;
    property TotalString: string read FTotalString write SetTotalString;
    property TrafficColour: TColor read FTrafficColour write SetTrafficColour;
    property UpColour: TColor read FUpColour write SetUpColour;
    property UpString: string read FUpString write SetUpString;
    property VersionString: string read FVersionString write SetVersionString;
    property Visible;
    property XOffset: Integer read FXOffset write SetXOffset default 40;
    property YOffset: Integer read FYOffset write SetYOffset default 60;
  end;

procedure Register;

implementation

constructor TTraffic.Create(AOwner: TComponent);
var
  Hour, Day: Word;
  N: Byte;
begin
  inherited Create(AOwner);
  for Day := 0 to 101 do
    for Hour := 1 to 24 do
    begin
      HourlyES[Day, Hour].UpES := 0;
      HourlyES[Day, Hour].DownES := 0;
    end;
  for N := 1 to 49 do XPositions[N] := 0;
  Self.Width := 200;
  Self.Height := 150;
  Self.Align := alNone;
  FGraphType := utDaily;
  FUpColour := clGreen;
  FDownColour := clRed;
  FBothColour := clAqua;
  FGraphColour := $D8FFFF;
  FTrafficColour := clWindow;
  FLimitColour := clAqua;
  FUpString := '';
  FDownString := '';
  FTotalString := '';
  FAutoscale := True;
  FHourlyMaxY := 10;
  FXOffset := 40;
  FYOffset := 50;
  MaxYValue := 10;
  HourlyMax := 10;
  DailyMax := 10;
  MonthlyMax := 10;
  FMonthlyLimit := 0;
  FMonthlyLimitDownOnly := False;
  FDaysAgo := 0;
  FESDate := 0;
  FFirstMonth := fmJan;
  FFirstYear := 2012;
  FShowBilling := False;
  FShowLegend := True;
  FSnapHeight := 400;
  FSnapWidth := 600;
  FScaleSnap := False;
  ABitmap := TBitmap.Create;
  DataList := TStringList.Create;
end;

destructor TTraffic.Destroy;
begin
  ABitmap.Free;
  DataList.Free;
  inherited Destroy;
end;

procedure TTraffic.ClearESArray;
var
  Day, Hour: Byte;
begin
  for Day := 0 to 101 do
    for Hour := 1 to 24 do
    begin
      HourlyES[Day, Hour].UpES := 0;
      HourlyES[Day, Hour].DownES := 0;
    end;
  HourlyMax := 10;
  Invalidate;
end;

procedure TTraffic.Paint;
begin
  try
    DrawBitmap(ABitmap);  //Do all the drawing on an in-memory bitmap before copying it to the component canvas
    Canvas.Draw(0, 0, ABitmap);
  except
    on E: Exception do;  //Do nothing, just trap exception
  end;
end;

procedure TTraffic.DrawBitmap(Bmp: TBitmap);
var
  S, YLabel: string;
  N, N1, X, Y, Len, X1, X2, Y1, Y2, XIncrement, YIncrement, XNum, Digits,
    TopY, UpY, DownY, BothY: Integer;
  YDivider, Higher: Word;
  UpValue, DownValue, BothValue, YMultiplier: Single;

begin
  Bmp.Width := Width;
  Bmp.Height := Height;
  if FGraphType = utHourly then
  begin
    if FAutoscale then
    begin
      Higher := 10;
      for N := 1 to 24 do
      begin
        if HourlyES[FDaysAgo, N].DownES > Higher then Higher := HourlyES[FDaysAgo, N].DownES;
        if HourlyES[FDaysAgo + 1, N].DownES > Higher then Higher := HourlyES[FDaysAgo + 1, N].DownES;
        if HourlyES[FDaysAgo, N].UpES > Higher then Higher := HourlyES[FDaysAgo, N].UpES;
        if HourlyES[FDaysAgo + 1, N].UpES > Higher then Higher := HourlyES[FDaysAgo + 1, N].UpES;
      end;
      HourlyMax := 10;
      SetHourlyMax(Higher);
    end;
    MaxYValue := HourlyMax;
    YLabel := 'ES per hour';
  end
  else begin
    if FGraphType = utDaily then MaxYValue := DailyMax
    else MaxYValue := MonthlyMax;
    YDivider := 1;
    if MaxYValue > 1000 then
    begin
      YDivider := 1000;
      MaxYValue := MaxYValue div 1000;
      YLabel := 'Traffic (GB)';
    end
    else YLabel := 'Traffic (MB)';
  end;
  if MaxYValue div 10 * 10 = MaxYValue then Digits := 0 else Digits := 1;
  S := FloatToStrF(MaxYValue, ffFixed, 0, Digits);
  Len := Bmp.Canvas.TextWidth(S);
  FXOffset := Len + 30;
  X1 := FXOffset;
  Y1 := 0;
  X2 := Width;
  Y2 := Height - FYOffset;
  YIncrement := (Y2 - Y1 - 8) div 10;
  if FGraphType = utHourly then XNum := 48
  else if FGraphType = utDaily then XNum := 31
  else XNum := 12;
  XIncrement := (X2 - X1 - 10) div XNum;
  YMultiplier := MaxYValue / 10;
  with Bmp.Canvas do
  begin
    Font.Assign(Self.Font);
    Font.CharSet := ANSI_CHARSET;
    Brush.Color := FTrafficColour;
    Pen.Color := clBlack;
    Pen.Style := psSolid;
    Pen.Width := 1;
    {$IFDEF Windows}
    Rectangle(0, 0, Width, Height);
    FillRect(1, 1, Width - 1, Height - 1);
    {$ELSE}
    Rectangle(0, 0, Width, Height - 1);
    FillRect(1, 1, Width - 1, Height - 2);
    Pen.Color := clBackground;
    MoveTo(0, Height - 1);
    LineTo(Width, Height - 1);
    Pen.Color := clBlack;
    {$ENDIF}
    Brush.Color := FGraphColour;
    Rectangle(X1, Y1, X2, Y2);
    Brush.Color := FTrafficColour;

    //Draw horizontal lines and Y axis marks
    Y := Y2;
    Font.Size := 8;
    for N := 0 to 10 do
    begin
      Pen.Style := psSolid;
      if N < 10 then  //Intermediate marks
      begin
        MoveTo(X1 - 2, Y - (YIncrement shr 1));
        LineTo(X1 + 2, Y - (YIncrement shr 1));
      end;
      MoveTo(X1 - 4, Y);
      LineTo(X1 + 4, Y);
      Pen.Style := psDot;
      LineTo(X2, Y);
      if N = 0 then S := '0'
      else S := FloatToStrF(N * YMultiplier, ffFixed, 0, Digits);
      Len := TextWidth(S);
      TextOut(X1 - Len - 5, Y - 7, S);
      Y := Y - YIncrement;
    end;
    TopY := Y + YIncrement;
    {$IFDEF Windows}
      Font.Size := 8;
    {$ELSE}
      Font.Size := 9;
    {$ENDIF}
    Font.Orientation := 900;
    Len := TextWidth(YLabel);
    TextOut(5, (Y1 + Y2) shr 1 + (Len shr 1), YLabel);
    Font.Orientation := 0;
    if (FMonthlyLimit > 0) and (FGraphType = utMonthly) then  //Draw max limit line if in range
    begin
      if (YDivider = 1000) and (FMonthlyLimit <= MaxYValue) then
      begin
        try
          Y := Y2 - Round(FMonthlyLimit / MaxYValue * (Y2 - Y1 - 5));
        except
          on E: Exception do Y := Y2;
        end;
        Pen.Color := FLimitColour;
        Pen.Style := psSolid;
        Pen.Width := 3;
        MoveTo(X1 + 2, Y);
        LineTo(X2 - 3, Y);
        Pen.Color := clBlack;
        Pen.Width := 1;
      end;
    end;

    //Draw vertical lines and X axis marks
    X := X1 + 5;
    Pen.Style := psSolid;
    {$IFDEF Windows}
      Font.Size := 8;
    {$ELSE}
      Font.Size := 9;
    {$ENDIF}
    N1 := Ord(FFirstMonth) + 1;
    for N := 1 to XNum + 1 do
    begin
      XPositions[N] := X;
      MoveTo(X, Y2);
      LineTo(X, Y2 + 4);
      if (FGraphType = utHourly) and ((N = 1) or (N = 25)) then
      begin
        MoveTo(X, Y2 + 25);
        LineTo(X, Y2 + 45);
      end;
      if ((FGraphType = utHourly) and (N <= XNum + 1)) or ((FGraphType = utMonthly) or(N shr 1 shl 1 < N) and (N <= XNum)) then
      begin
        if FGraphType = utHourly then
        begin
          if N > 24 then S := IntToStr(N - 25)
          else S := IntToStr(N - 1);
        end
        else if FgraphType = utDaily then S := IntToStr(N)
        else S := FormatSettings.ShortMonthNames[N1];
        Len := TextWidth(S);
        if FGraphType = utHourly then
        begin
          if (N - 1) shr 1 shl 1 = N - 1 then     //Even number
            TextOut(X - (Len shr 1), Y2 + 6, S);
        end
        else TextOut(X + (XIncrement - Len) shr 1, Y2 + 6, S);
        Inc(N1);
        if N1 = 13 then N1 := 1;
      end;
      X := X + XIncrement;
    end;
    if XNum < 31 then for N := XNum + 1 to 31 do XPositions[N] := 0;
    if FGraphType = utHourly then S := '<-- ' + FormatDateTime('dd mmm yyyy', FESDate)
    else if FGraphType = utMonthly then S := 'Month'
    else S := FormatDateTime('mmm yyyy', Now);
    Len := TextWidth(S);
    if FGraphType = utHourly then TextOut(((X - XIncrement - X1) shr 1) + X1 + 10, Y2 + 30, S)
    else TextOut((X2 - Len) shr 1, Y2 + 30, S);
    if FGraphType = utHourly then TextOut(X1 + 10, Y2 + 30, '<-- ' + FormatDateTime('dd mmm yyyy', FESDate - 1))
    else if FGraphType = utMonthly then
    begin
      S := IntToStr(FFirstYear);
      Len := TextWidth(S);
      TextOut(X1 + 5 + (XIncrement - Len) shr 1, Y2 + 30, S);
      if FFirstMonth <> fmJan then S := IntToStr(FFirstYear + 1);
      Len := TextWidth(S);
      TextOut(X1 + 5 + (11 * XIncrement) + (XIncrement - Len) shr 1, Y2 + 30, S);
    end;
    Len := TextWidth(FVersionString);
    TextOut(Width - Len - 10, Y2 + 30, FVersionString);

    //Plot values
    X := X1 + 5;
    for N := 1 to XNum do
    begin
      if FGraphType = utHourly then
      begin
        if N < 25 then  //Previous day
        begin
          UpValue := HourlyES[FDaysAgo + 1, N].UpES;
          DownValue := HourlyES[FDaysAgo + 1, N].DownES;
        end
        else begin      //Current day
          UpValue := HourlyES[FDaysAgo, N - 24].UpES;
          DownValue := HourlyES[FDaysAgo, N - 24].DownES;
        end;
      end
      else if FGraphType = utDaily then
      begin
        UpValue := DailyTraffic[0, N].UpTraffic / YDivider;
        DownValue := DailyTraffic[0, N].DownTraffic / YDivider;
      end
      else begin
        UpValue := MonthlyTraffic[N].UpTraffic / YDivider;
        DownValue := MonthlyTraffic[N].DownTraffic / YDivider;
        BothValue := UpValue + DownValue;
        BothY := Round(BothValue / MaxYValue * (Y2 - TopY));
      end;
      UpY := Round(UpValue / MaxYValue * (Y2 - TopY));
      DownY := Round(DownValue / MaxYValue * (Y2 - TopY));
      if (FGraphType = utHourly) or (FGraphType = utDaily) or (FMonthlyLimit = 0) or FMonthlyLimitDownOnly then
      begin
        Brush.Color := FUpColour;
        FillRect(X, Y2 - UpY, X + (Xincrement shr 1) + 1, Y2);
        Brush.Color := FDownColour;
        FillRect(X + (XIncrement shr 1) + 1, Y2 - DownY, X + XIncrement, Y2);
      end
      else begin
        Brush.Color := FUpColour;
        FillRect(X, Y2 - UpY, X + (XIncrement div 3) + 1, Y2);
        Brush.Color := FDownColour;
        FillRect(X + (XIncrement div 3) + 1, Y2 - DownY, X + (XIncrement shl 1 div 3) + 1, Y2);
        Brush.Color := FBothColour;
        FillRect(X + (XIncrement shl 1 div 3) + 1, Y2 - BothY, X + XIncrement, Y2);
      end;
      X := X + XIncrement;
    end;

    //Draw the legend

    if FShowLegend then
    begin
      Brush.Color := clWhite;
      Pen.Color := clBlack;
      Pen.Style := psSolid;
      if (FGraphType = utDaily) or (FMonthlyLimit = 0) or FMonthlyLimitDownOnly then
      begin
        if FLegendPosition = lptRight then Rectangle(X2 - 110, 5, X2 - 5, 43)
        else Rectangle(FXOffset + 5, 5, FXOffset + 110, 43);
      end
      else begin
        if FLegendPosition = lptRight then Rectangle(X2 - 110, 5, X2 - 5, 58)
        else Rectangle(FXOffset + 5, 5, FXOffset + 110, 58);
      end;
      Pen.Width := 3;
      Pen.Color := FUpColour;
      if FLegendPosition = lptRight then
      begin
        MoveTo(X2 - 105, 15);
        LineTo(X2 - 85, 15);
      end
      else begin
        MoveTo(FXOffset + 10, 15);
        LineTo(FXOffset + 30, 15);
      end;
      Pen.Color := FDownColour;
      if FLegendPosition = lptRight then
      begin
        MoveTo(X2 - 105, 30);
        LineTo(X2 - 85, 30);
      end
      else begin
        MoveTo(FXOffset + 10, 30);
        LineTo(FXOffset + 30, 30);
      end;
      if (FGraphType = utMonthly) and (FMonthlyLimit > 0) and not FMonthlyLimitDownOnly then
      begin
        Pen.Color := FBothColour;
        if FLegendPosition = lptRight then
        begin
          MoveTo (X2 - 105, 45);
          LineTo(X2 - 86, 45);
        end
        else begin
          MoveTo(FXOffset + 10, 45);
          LineTo(FXOffset + 30, 45);
        end;
      end;
      Font.Size := 8;
      Font.Orientation := 0;
      if FLegendPosition = lptRight then
      begin
        TextOut(X2 - 83, 8, 'Upstream');
        TextOut(X2 - 83, 23, 'Downstream');
        if (FGraphType = utMonthly) and (FMonthlyLimit > 0) and not FMonthlyLimitDownOnly then TextOut(X2 - 83, 38, 'Up + Down');
      end
      else begin
        TextOut(FXOffset + 32, 8, 'Upstream');
        TextOut(FXOffset + 32, 23, 'Downstream');
        if (FGraphType = utMonthly) and (FMonthlyLimit > 0) and not FMonthlyLimitDownOnly then TextOut(FXOffset + 32, 38, 'Up + Down');
      end;
    end;
    if FShowBilling and (FGraphType <> utHourly) then
    begin
      Brush.Color := clWhite;
      Pen.Color := clBlack;
      Pen.Width := 1;
      Pen.Style := psSolid;
      Rectangle(FXOffset + 5, 5, FXOffset + 150, 100);
      Font.Size := 8;
      Font.Orientation := 0;
      Font.Style := [fsBold];
      TextOut(FXOffset + 10, 8, 'Billing month data');
      Font.Style := [];
      TextOut(FXOffset + 10, 23, 'First day: ' + IntToStr(FBillingDay));
      TextOut(FXOffset + 10, 38, 'Traffic up: ' + FUpString);
      TextOut(FXOffset + 10, 53, 'Traffic down: ' + FDownString);
      TextOut(FXOffset + 10, 68, 'Traffic total: ' + FTotalString);
      TextOut(FXOffset + 10, 83, 'Monthly limit: ' + IntToStr(FMonthlyLimit) + ' GB');
    end;
  end;
end;

procedure TTraffic.Scrolldays(Days: Word);
var
  N, N1: Byte;
begin
  for N := 100 - Days downto 0 do
    HourlyES[N + Days] := HourlyES[N];
  for N := 0 to Days - 1 do
    for N1 := 1 to 24 do
    begin
      HourlyES[N, N1].UpES := 0;
      HourlyES[N, N1].DownES := 0;
    end;
  if FAutoscale then HourlyMax := 10;
  Invalidate;
end;

procedure TTraffic.GetHourlyES(DaysAgo: TDaysAgo; Hour: Byte; var UpES, DownES: Word);
begin
  UpES := HourlyES[DaysAgo, Hour].UpES;
  DownES := HourlyES[DaysAgo, Hour].DownES;
end;

function TTraffic.GetXPosition(Index: Byte): Word;
begin
  try
    Result := XPositions[Index];
  except
    on E: Exception do Result := 0;
  end;
end;

procedure TTraffic.SetAutoscale(Value: Boolean);
var
  N: Byte;
  Higher: Word;
begin
  if Value <> FAutoscale then
  begin
    FAutoscale := Value;
    if FAutoscale then
    begin
      Higher := 10;
      for N := 1 to 24 do
      begin
        if HourlyES[FDaysAgo, N].DownES > Higher then Higher := HourlyES[FDaysAgo, N].DownES;
        if HourlyES[FDaysAgo + 1, N].DownES > Higher then Higher := HourlyES[FDaysAgo + 1, N].DownES;
        if HourlyES[FDaysAgo, N].UpES > Higher then Higher := HourlyES[FDaysAgo, N].UpES;
        if HourlyES[FDaysAgo + 1, N].UpES > Higher then Higher := HourlyES[FDaysAgo + 1, N].UpES;
      end;
      HourlyMax := 10;
      SetHourlyMax(Higher);
    end
    else HourlyMax := FHourlyMaxY;
    Invalidate;
  end;
end;

//Set max Y value manually
procedure TTraffic.SetHourlyMaxY(Value: Word);
begin
  if Value <> FHourlyMaxY then
  begin
    FHourlyMaxY := Value;
    if not FAutoscale then
    begin
      HourlyMax := Value;
      Invalidate;
    end;
  end;
end;

//Set max Y value automatically
procedure TTraffic.SetHourlyMax(Higher: Word);
begin
  if FAutoscale and (Higher > HourlyMax) then
  begin
    case Higher of
      11..20: HourlyMax := 20;
      21..40: HourlyMax := 40;
      41..100: HourlyMax := 100;
      101..200: HourlyMax := 200;
      201..400: HourlyMax := 400;
      401..1000: HourlyMax := 1000;
      1001..2000: HourlyMax := 2000;
      2001..4000: HourlyMax := 4000;
      4001..10000: if FGraphType = utHourly then HourlyMax := 4000 else HourlyMax := 10000;
      10001..65535: if FGraphType = utHourly then HourlyMax := 4000 else HourlyMax := 40000;
    end;
  end;
end;

procedure TTraffic.SetHourlyES(DaysAgo: TDaysAgo; Hour: Byte; UpES, DownES : Word);
var
  Higher: Word;
begin
  if (UpES < 0) or (UpES > 3600) then UpES := 0;    //ES per hour can't be negative or greater than 3600
  if (DownES < 0) or (DownES > 3600) then DownES := 0;
  HourlyES[DaysAgo, Hour].UpES := UpES;
  HourlyES[DaysAgo, Hour].DownES := DownES;
  if UpES > DownES then Higher := UpES else Higher := DownES;
  if DaysAgo = 0 then SetHourlyMax(Higher);
  Invalidate;
end;

procedure TTraffic.SetDailyTraffic(AMonth, Index: Byte; UpTraffic, DownTraffic: Int64);
var
  Higher: Int64;
begin
  DailyTraffic[AMonth, Index].UpTraffic := UpTraffic;
  DailyTraffic[AMonth, Index].DownTraffic := DownTraffic;
  if UpTraffic > DownTraffic then Higher := UpTraffic else Higher := DownTraffic;
  DailyMax := ScaleValue(Higher, DailyMax);
  Invalidate;
end;

procedure TTraffic.SetMonthlyTraffic(Index: Byte; UpTraffic, DownTraffic: Int64);
var
  Higher: Int64;
  N: Byte;
begin
  MonthlyTraffic[Index].UpTraffic := UpTraffic;
  MonthlyTraffic[Index].DownTraffic := DownTraffic;
  if (UpTraffic = 0) and (DownTraffic = 0) then   //Zero all the days in the current month
  begin
    for N := 1 to 31 do
    begin
      DailyTraffic[0, N].UpTraffic := 0;
      DailyTraffic[0, N].DownTraffic := 0;
    end;
    DailyMax := 10;
  end
  else begin
    if UpTraffic > DownTraffic then Higher := UpTraffic else Higher := DownTraffic;
    MonthlyMax := ScaleValue(Higher, MonthlyMax);
  end;
  Invalidate;
end;

procedure TTraffic.GetDailyTraffic(AMonth, Index: Byte; var UpTraffic, DownTraffic: Int64);
begin
  UpTraffic := DailyTraffic[AMonth, Index].UpTraffic;
  DownTraffic := DailyTraffic[AMonth, Index].DownTraffic;
end;

procedure TTraffic.GetMonthlyTraffic(Index: Byte; var UpTraffic, DownTraffic: Int64);
begin
  UpTraffic := MonthlyTraffic[Index].UpTraffic;
  DownTraffic := MonthlyTraffic[Index].DownTraffic;
end;

//Set all all daily values to zero
procedure TTraffic.ClearDaily;
var
  N: Byte;
begin
  for N := 1 to 31 do
  begin
    DailyTraffic[0, N].UpTraffic := 0;
    DailyTraffic[0, N].DownTraffic := 0;
  end;
  Invalidate;
end;

//Increment the first month and shift the array contents
procedure TTraffic.IncrementMonth;
var
  N: Byte;
begin
  for N := 1 to 31 do
  begin
    DailyTraffic[1, N] := DailyTraffic[0, N];        //This month's daily traffic becomes last month
    DailyTraffic[0, N].UpTraffic := 0;
    DailyTraffic[0, N].DownTraffic := 0;
  end;
  for N := 1 to 11 do
    MonthlyTraffic[N] := MonthlyTraffic[N + 1];
  MonthlyTraffic[12].UpTraffic := 0;
  MonthlyTraffic[12].DownTraffic := 0;
  if Ord(FFirstMonth) < 11 then FFirstMonth := TMonths(Succ(Ord(FFirstMonth)))
  else begin
    FFirstMonth := fmJan;
    Inc(FFirstYear);
  end;
  Invalidate;
end;

//Save a snapshot in .png format
function TTraffic.SaveToPNGFile(Filename: string): Boolean;
var
  Png: TPortableNetworkGraphic;
  Bmp: TBitmap;
begin
  Result := True;
  Bmp := TBitmap.Create;
  try
    DrawBitmap(Bmp);
  except
    on E: Exception do
    begin
      Result := False;
      Bmp.Free;
      Exit;
    end;
  end;
  Png := TPortableNetworkGraphic.Create;
  try
    if FScaleSnap then
    begin
      Png.Width := FSnapWidth;
      Png.Height := FSnapHeight;
      Png.Canvas.StretchDraw(Rect(0, 0, FSnapWidth - 1, FSnapHeight - 1), Bmp);
    end
    else Png.Assign(Bmp);
    Png.SaveToFile(Filename);
  except
    on E: Exception do
      Result := False;
  end;
  Png.Free;
  Bmp.Free;
end;

//Save hourly ES data to file
function TTraffic.SaveHourlyES(Filename: string): Boolean;
var
  ESList: TStringList;
  Day, Hour: Byte;
begin
  Result := True;
  ESList := TStringList.Create;
  try
    ESList.Add(FloatToStr(Now));
    for Day := 0 to 100 do
    begin
      ESList.Add('Days ago = ' + IntToStr(Day));
      for Hour := 1 to 24 do
        ESList.Add(IntToStr(HourlyES[Day, Hour].UpES) + ',' + IntToStr(HourlyES[Day, Hour].DownES));
    end;
    ESList.SaveToFile(Filename);
  except
    on E: Exception do
      Result := False;
  end;
  ESList.Free;
end;

//Load hourly ES from file
function TTraffic.LoadHourlyES(Filename: string): Boolean;
var
  ESList: TStringList;
  Day, Hour, N: Byte;
  LineNumber, Highest: Word;
  Days: Integer;
  S, Up, Down: string;
  TD: TDateTime;
begin
  Result := True;
  Highest := 0;
  ESList := TStringList.Create;
  try
    ESList.LoadFromFile(Filename);
    TD := StrToFloat(ESList[0]);
    Days := DaysBetween(DateOf(Now), DateOf(TD));

    // New section to be tested

    // End of new section

    LineNumber := 1;
    for Day := 0 to 100 do
    begin
      Inc(LineNumber);
      for Hour := 1 to 24 do
      begin
        S := ESList[LineNumber];
        N := Pos(',', S);
        try
          Up := S[1..N - 1];
          Down := S[N + 1..Length(S)];
        except
          on E: Exception do
          begin
            Up := '0';
            Down := '0';
          end;
        end;
        HourlyES[Day, Hour].UpES := StrToInt(Up);
        HourlyES[Day, Hour].DownES := StrToInt(Down);
        if Day = 0 then
        begin
          if HourlyES[Day, Hour].UpES > Highest then Highest := HourlyES[Day, Hour].UpES;
          if HourlyES[Day, Hour].DownES > Highest then Highest := HourlyES[Day, Hour].DownES;
          SetHourlyMax(Highest);
        end;
        Inc(LineNumber);
      end;
    end;
    Invalidate;
  except
    on E: Exception do
      Result := False;
  end;
  ESList.Free;
end;

procedure TTraffic.SetGraphColour(Value: TColor);
begin
  if Value <> FGraphColour then
  begin
    FGraphColour := Value;
    Invalidate;
  end;
end;

procedure TTraffic.SetTrafficColour(Value: TColor);
begin
  if Value <> FTrafficColour then
  begin
    FTrafficColour := Value;
    Invalidate;
  end;
end;

procedure TTraffic.SetGraphType(Value: TTrafficType);
begin
  if Value <> FGraphType then
  begin
    FGraphType := Value;
    Invalidate;
  end;
end;

procedure TTraffic.SetLegendPosition(Value: TLegendPosition);
begin
  if Value <> FLegendPosition then
  begin
    FLegendPosition := Value;
    Invalidate;
  end;
end;

procedure TTraffic.SetUpColour(Value: TColor);
begin
  if Value <> FUpColour then
  begin
    FUpColour := Value;
    Invalidate;
  end;
end;

procedure TTraffic.SetDownColour(Value: TColor);
begin
  if Value <> FDownColour then
  begin
    FDownColour := Value;
    Invalidate;
  end;
end;

procedure TTraffic.SetBothColour(Value: TColor);
begin
  if Value <> FBothColour then
  begin
    FBothColour := Value;
    Invalidate;
  end;
end;

procedure TTraffic.SetLimitColour(Value: TColor);
begin
  if Value <> FLimitColour then
  begin
    FLimitColour := Value;
    Invalidate;
  end;
end;

procedure TTraffic.SetXOffset(Value: Integer);
begin
    if Value < Self.Width shr 1 then
  begin
    FXOffset := Value;
    Invalidate;
  end;
end;

procedure TTraffic.SetYOffset(Value: Integer);
begin
    if Value < Self.Height shr 1 then
  begin
    FYOffset := Value;
    Invalidate;
  end;
end;

procedure TTraffic.SetESDate(Value: TDateTime);
begin
  if Value <> FESDate then
  begin
    FESDate := Value;
    Invalidate;
  end;
end;

procedure TTraffic.SetFirstMonth(Value: TMonths);
begin
  if Value <> FFirstMonth then
  begin
    FFirstMonth := Value;
    Invalidate;
  end;
end;

procedure TTraffic.SetFirstYear(Value: Word);
begin
  if Value <> FFirstYear then
  begin
    FFirstYear := Value;
    Invalidate;
  end;
end;

procedure TTraffic.SetDaysAgo(Value: TDaysAgo);
var
  N: Byte;
  Higher: Word;
begin
  if Value <> FDaysAgo then
  begin
    FDaysAgo := Value;
    if FAutoscale then
      begin
      HourlyMax := 10;
      Higher := 10;
      for N := 1 to 24 do
      begin
        if HourlyES[FDaysAgo, N].UpES > Higher then Higher := HourlyES[FDaysAgo, N].UpES;
        if HourlyES[FDaysAgo, N].DownES > Higher then Higher := HourlyES[FDaysAgo, N].DownES;
      end;
    SetHourlyMax(Higher);
    end;
    Invalidate;
  end;
end;

procedure TTraffic.SetBillingDay(Value: Byte);
begin
  if Value <> FBillingDay then
  begin
    FBillingDay := Value;
    Invalidate;
  end;
end;
procedure TTraffic.SetShowBilling(Value: Boolean);
begin
  if Value <> FShowBilling then
  begin
    FShowBilling := Value;
    Invalidate;
  end;
end;

procedure TTraffic.SetShowLegend(Value: Boolean);
begin
  if Value <> FShowLegend then
  begin
    FShowLegend := Value;
    Invalidate;
  end;
end;

procedure TTraffic.SetMonthlyLimit(Value: Word);
begin
  if Value <> FMonthlyLimit then
  begin
    FMonthlyLimit := Value;
    Invalidate;
  end;
end;

procedure TTraffic.SetMonthlyLimitDownOnly(Value: Boolean);
begin
  if Value <> FMonthlyLimitDownOnly then
  begin
    FMonthlyLimitDownOnly := Value;
    Invalidate;
  end;
end;

procedure TTraffic.SetDownString(Value: string);
begin
  if Value <> FDownString then
  begin
    FDownString := Value;
    Invalidate;
  end;
end;

procedure TTraffic.SetUpString(Value: string);
begin
  if Value <> FUpString then
  begin
    FUpString := Value;
    Invalidate;
  end;
end;

procedure TTraffic.SetTotalString(Value: string);
begin
  if Value <> FTotalString then
  begin
    FTotalString := Value;
    Invalidate;
  end;
end;

procedure TTraffic.SetVersionString(Value: string);
begin
  if Value <> FVersionString then
  begin
    FVersionString := Value;
    Invalidate;
  end;
end;

function TTraffic.ScaleValue(Value, MaxValue: Cardinal): Cardinal;
begin
  if Value > MaxValue then
  begin
    case Value of
      11..40: Result := 40;
      41..100: Result := 100;
      101..400: Result := 400;
      401..1000: Result := 1000;
      1001..4000: Result := 4000;
      4001..10000: Result := 10000;
      10001..40000: Result := 40000;
      40001..100000: Result := 100000;
      100001..400000: Result := 400000;
      400001..1000000: Result := 1000000;
    end;
  end
  else Result := MaxValue;
end;

procedure TTraffic.SaveTrafficData(HomePath: string);
var
  N: Byte;
  S: string;
begin
  try
    DataList.Clear;
    for N := 1 to 31 do
    begin
      S := IntToStr(DailyTraffic[0, N].UpTraffic) + ',' + IntToStr(DailyTraffic[0, N].DownTraffic) + ','
        + IntToStr(DailyTraffic[1, N].UpTraffic) + ',' + IntToStr(DailyTraffic[1, N].DownTraffic) + ',';
      if N < 13 then S := S + IntToStr(MonthlyTraffic[N].UpTraffic) + ',' + IntToStr(MonthlyTraffic[N].DownTraffic)
      else S := S + '0,0';
      DataList.Add(S);
    end;
    DataList.SaveToFile(HomePath + 'traffic.txt');
  except
    on E: Exception do ShowMessage('Unable to save traffic monitoring data');
  end;
  DataList.SaveToFile(HomePath + 'traffic.txt');
end;

procedure TTraffic.LoadTrafficData(HomePath: string);
var
  DayList: TStringList;
  N: Byte;
  Up0, Up1, Down0, Down1, MonthUp, MonthDown, Higher, MonthHigher: Int64;
begin
  if FileExists(HomePath + 'traffic.txt') then
  try
    DayList := TStringList.Create;
    try
      DataList.LoadFromFile(HomePath + 'traffic.txt');
      for N := 1 to 31 do
      begin
        DayList.CommaText := DataList[N - 1];
        Higher := 10;
        MonthHigher := 10;
        Up0 := StrToInt(DayList[0]);
        if Up0 > Higher then Higher := Up0;
        Up1 := StrToInt(DayList[1]);
        if Up1 > Higher then Higher := Up1;
        Down0 := StrToInt(DayList[2]);
        if Down0 > Higher then Higher := Down0;
        Down1 := StrToInt(DayList[3]);
        if Down1 > Higher then Higher := Down1;
        DailyMax := ScaleValue(Higher, DailyMax);
        DailyTraffic[0, N].UpTraffic := Up0;
        DailyTraffic[0, N].DownTraffic := Up1;
        DailyTraffic[1, N].UpTraffic := Down0;
        DailyTraffic[1, N].DownTraffic := Down1;
        if N < 13 then
        begin
          MonthUp := StrToInt(DayList[4]);
          if MonthUp > MonthHigher then MonthHigher := MonthUp;
          MonthDown := StrToInt(DayList[5]);
          if MonthDown > MonthHigher then MonthHigher := MonthDown;
          MonthlyMax := ScaleValue(MonthHigher, MonthlyMax);
          MonthlyTraffic[N].UpTraffic := MonthUp;
          MonthlyTraffic[N].DownTraffic := MonthDown;
        end;
      end;
    finally
      DayList.Free;
    end;
  except
    on E: Exception do
    begin
      ShowMessage('Unable to restore saved traffic monitoring data');
      ZeroData;
    end;
  end
  else ZeroData;
end;

procedure TTraffic.ZeroData;
var
  N: Byte;
begin
  for N := 1 to 31 do
  begin
    DailyTraffic[0, N].UpTraffic := 0;
    DailyTraffic[0, N].DownTraffic := 0;
    DailyTraffic[1, N].UpTraffic := 0;
    DailyTraffic[1, N].DownTraffic := 0;
    if N < 13 then
    begin
      MonthlyTraffic[N].UpTraffic := 0;
      MonthlyTraffic[N].DownTraffic := 0;
    end;
  end;
end;

procedure Register;
begin
  RegisterComponents('Eric',[TTraffic]);
end;

initialization
{$I dsgraphs.lrs}

end.
