unit telnetunit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  lNetComponents, lNet, dateutils;

type

  TLogEvent = (leSuccess, leBusy, leNoshowtime, leNoresponse, leNologin, leNosecondstage, leNoCLI, leNodata,
    leNostats, leNobits, leNosnr, leNoqln, leNohlog, leNopbparams, leNovendor, leNouptime,
    leReadonlydir, leStart, leResume, lePause, leStop, leBadpassword,
    leCLIfail, leNorouterresponse, leResync, leAutosnapshot, leLoginException,{ leAttenSaved,}
    leDownloadFailed, leDownloadSucceeded, leSnapshotFailed, leNospeciallogin, leErrorsreset, leErrorsnotsaved,
    leAdvancedTweaking, leNoadvancedtweak, leIPAddress, leNoIPaddress, leGraphrestore, leReturntonormal,
    leNographrestore, leNoreturntonormal, leXMLsavefailed, leXMLoadfailed, leDeletesucceeded, leDeletefailed,
    leNoTraffic, leImplausibleup, leImplausibledown, leDayreset, leMonthreset, leSamplemissed,
    leSynchronise, leUnsync, leIsrunningfailed, leMisc, leUnknown, leTimeout, leHG622Error, leNoVectoring);

  TTelnetOperation = (toCollectstats, toAdvancedTweak);

  { TTelnetForm }

  TTelnetForm = class(TForm)
    LTelnetClient: TLTelnetClientComponent;
    procedure FormCreate(Sender: TObject);
    procedure LTelnetClientError(const Msg: string; aSocket: TLSocket);
    procedure LTelnetClientReceive(aSocket: TLSocket);
  private
    { private declarations }
    procedure Disconnect;
  public
    { public declarations }
    Mode, DelayTime: Integer;
    Statscommand, Bitscommand, Paramscommand, SNRcommand, QLNcommand, Hlogcommand, VendorCommand, Uptimecommand,
       AdvancedTweakCommand, VersionCommand, VersionText, Shelltext, Connectresult, Commandresult,
       LogText, DiagsText, TrafficCommand, LineDiagCommand, GInpCommand, VectoringCommand: string;
    Stats, Bits, Params, SNR, QLN, HLog, Vendor, Uptime, Traffic, Linediags, GInpStats, Vectoring: string;
    Username, Password, HostName: string;
    LE: TLogEvent;
    TelnetOperation: TTelnetOperation;
    UseSecondStage, DoBitloading, DoSNRpertone, DoQLN, QLNDone, DoHLog, HLogDone,
       BandsDone, DoTweak, Special, DoInsertDelay, IsThomson, CustomDone, FinalCustom,
       DSLdown, IsHG622, DoSignalAttn, DoFirmware, DoVendor, DoSysuptime, DoVectoring: Boolean;
    CustomCommands: array[1..4] of string;
    CustomIncludes: array[1..4, 1..6] of Boolean;  //Indexes are the same as CommandNumbers on the main form
    DoCustom: array[1..6] of Boolean;   //Indexes correspond with the six "When to send" events on the main form
    CustomOutputs: array[1..4] of string;
    procedure DoTelnet;
  end;

var
  TelnetForm: TTelnetForm;
  DoTraffic: Boolean;  //Must be in unit, not form, because they are addressed
  PortNumber: Word;    //before the form is created

implementation

{$R *.lfm}

uses main;

{ TTelnetForm }

procedure TTelnetForm.FormCreate(Sender: TObject);
var
  N: Byte;
begin
  IsHG622 := False;
  for N := 1 to 6 do DoCustom[N] := False;
  FinalCustom := False;
  CustomDone := False;
  DSLdown := False;
  QLNDone := False;
  HLogDone := False;
  Mode := -1;
end;

{Log in by telnet, process custom commands, and collect all the required data
or process the advanced tweaks

Every message sent to the modem is accompanied by a timeout, so the program shouldn't
hang if the modem doesn't respond for any reason.}

procedure TTelnetForm.DoTelnet;
var
  S, Prompt: string;
  N, N1: Word;
  Len: Integer;
  TimeoutTime: TDateTime;
  TimedOut: Boolean;
begin
  CommandResult := '';
  try
    LTelnetClient.Connect(HostName, PortNumber);
  except
    on E: Exception do
    begin
      LE := leLoginexception;
      Exit;
    end;
  end;
  TimeoutTime := Now + EncodeTimeInterval(0, 0, 3, 0);
  //TimeoutTime := Now + EncodeTimeInterval(0, 0, 10, 0);
  repeat
    Application.ProcessMessages;
    Sleep(10);
  until LTelnetClient.Connected or (Now > TimeoutTime);
  if LTelnetClient.Connected then
  begin
    TimedOut := False;
    TimeoutTime := Now + EncodeTimeInterval(0, 0, 3, 0);     //Wait up to 3 sec for prompt
    repeat
      Application.ProcessMessages;
      Sleep(10);
      if Now > TimeoutTime then
      begin
        TimedOut := True;
        Break;
      end;
    until (Pos('#', CommandResult) > 0)
    or (Pos('$', CommandResult) > 0)
    or (Pos('>', CommandResult) > 0)
    or (Pos(':', CommandResult) > 0);
    if TimedOut and not IsHG622 then
    begin
      Disconnect;
      LE := leNologin;
      Exit;
    end;

    // ** This is a fudge for some modems such as HG622 **

    if IsHG622 then    //Try fudge if IsHG622 override is set
    try
      Sleep(50);
      TimedOut := False;
      TimeoutTime := Now + EncodeTimeInterval(0, 0, 3, 0);
      CommandResult := '';
      LTelnetClient.SendMessage('xxxxxxxx' + #13#10);
      repeat
        Application.ProcessMessages;
        Sleep(50);
        if Now > TimeoutTime then
        begin
          TimedOut := True;
          Break;
        end;
      until (Pos('login', LowerCase(CommandResult)) > 0)
      or (Pos('username', LowerCase(CommandResult)) > 0);
      if TimedOut then
      begin
        Disconnect;
        LE := leHG622Error;
        Exit;
      end;
    except
      on E: Exception do
      begin
        Disconnect;
        Mainform.MiscString := 'Error running the HG622 workaround';
        Mainform.LogEvent(leMisc);
        LE := leHG622Error;
        Exit;
      end;
    end;

    // ** End of fudge **

    Sleep(50);
    if UserName > '' then   //Send username
    try
      CommandResult := '';
      LTelnetClient.SendMessage(UserName + #13#10);
      TimeoutTime := Now + EncodeTimeInterval(0, 0, 3, 0);
      repeat
        Application.ProcessMessages;
        Sleep(10);
        if Now > TimeoutTime then
        begin
          Disconnect;
          Mainform.MiscString := 'No response to login user name.';
          MainForm.LogEvent(leMisc);
          LE := leNologin;
          Exit;
        end;
      until Pos('assw', CommandResult) > 0;
    except
      on E: Exception do
      begin
        Mainform.MiscString := 'Error sending the login username';
        Mainform.LogEvent(leMisc);
        Exit;
      end;
    end;

    if Password > '' then   //Send password
    try
      CommandResult := '';
      LTelnetClient.SendMessage(Password + #13#10);
      TimeoutTime := Now + EncodeTimeInterval(0, 0, 2, 0);   //Wait 2 sec for a response
      repeat
        Application.ProcessMessages;
        Sleep(10);
        if Now > TimeoutTime then  //Didn't respond within 2 seconds, so give it another prod
        begin
          LTelnetClient.SendMessage(#13#10);
          //TimeoutTime := Now + EncodeTimeInterval(0, 0, 0, 500);   //Wait another 500 ms
          TimeoutTime := Now + EncodeTimeInterval(0, 0, 1, 0);   //Wait another second
          repeat
            Application.ProcessMessages;
            Sleep(10);
            if Now > TimeoutTime then
            begin
              Disconnect;
              LE := leBadpassword;
              Exit;
            end;
          until (Pos('#', CommandResult) > 0) or (Pos('$', CommandResult) > 0) or (Pos('>', CommandResult) > 0);
        end;
      until (Pos('#', CommandResult) > 0) or (Pos('$', CommandResult) > 0) or (Pos('>', CommandResult) > 0);
    except
      on E: Exception do
      begin
        Disconnect;
        Mainform.MiscString := 'Error sending the login password';
        Mainform.LogEvent(leMisc);
        Exit;
      end;
    end;

    // Get prompt character

    if not UseSecondStage then
    begin
      if Pos('#', CommandResult) > 0 then Prompt := '#'
      else if Pos('$', CommandResult) > 0 then Prompt := '$'
      else Prompt := '>';
    end;
    CommandResult := '';

    if not IsThomson then // Thomson login is complete at this stage
    begin

      // Special login

      if Special then
      begin
        if DoInsertDelay then Sleep(DelayTime);
        N := 3;
        try
          repeat
            S := Mainform.Responses[N].Text;
            if S > '' then
            begin
              CommandResult := '';
              LTelnetClient.SendMessage(S + #13#10);
              //TimeoutTime := Now + EncodeTimeInterval(0, 0, 2, 0);
              TimeoutTime := Now + EncodeTimeInterval(0, 0, 3, 0);
              repeat
                Application.ProcessMessages;
                Sleep(10);
                if Now > TimeoutTime then
                begin
                  Disconnect;
                  LE := leNologin;
                  Exit;
                end;
              until (N = 10) or (Pos(Mainform.Prompts[N + 1].Text, CommandResult) > 0) or (Pos('#', CommandResult) > 0);
              DiagsText := CommandResult;
            end
            else Break;
            Inc(N);
          until N = 11;
        except
          on E: Exception do
          begin
            Disconnect;
            LE := leNospeciallogin;
            LogText := 'Error during response number ' + IntToStr(N);
            Exit;
          end;
        end;
        Prompt := '#';
        if Pos(Prompt, CommandResult) = 0 then    //No BusyBox prompt
        begin
          Disconnect;
          LE := leNologin;
          Exit;
        end;
      end

      //Second stage login

      else if UseSecondStage then
      begin
        Sleep(100);   //Just a precaution
        CommandResult := '';
        try
          LTelnetClient.SendMessage(Shelltext + #13#10);
          //TimeoutTime := Now + EncodeTimeInterval(0, 0, 2, 0);
          TimeoutTime := Now + EncodeTimeInterval(0, 0, 3, 0);
          repeat
            Application.ProcessMessages;
            Sleep(10);
            if Now > TimeoutTime then
            begin
              Disconnect;
              LE := leNosecondstage;
              Exit;
            end;
          until (Pos('#', CommandResult) > 0) or (Pos('$', CommandResult) > 0) or (Pos('>', CommandResult) > 0);
          if Pos('#', CommandResult) > 0 then Prompt := '#'
          else if Pos('$', CommandResult) > 0 then Prompt := '$'
          else Prompt := '>';
        except
          on E: Exception do
          begin
            Disconnect;
            LE := leNosecondstage;
            Exit;
          end;
        end;
        if Pos(Prompt, CommandResult) = 0 then    //No CLI prompt
        begin
          Disconnect;
          LE := leNosecondstage;
          Exit;
        end;
      end;
    end;

    // ** End of login **

    case TelnetOperation of

      toCollectStats: begin    //Collect data from modem/router

                        Sleep(50);
                        TimeoutTime := Now + EncodeTimeInterval(0, 0, 15, 0);    //15 seconds, just so it doesn't hang
                        CommandResult := '';
                        for N := 1 to 4 do   //Process custom commands
                        begin
                          for N1 := 1 to 6 do
                          begin
                            if (N1 <> 3) and DoCustom[N1] then   //Don't do DSL-down here
                            begin
                              if CustomIncludes[N, N1] then
                              begin
                                try
                                  CommandResult := '';
                                  LTelnetClient.SendMessage(CustomCommands[N] + #13#10);
                                  repeat
                                    Application.ProcessMessages;
                                    Sleep(10);
                                    if Now > TimeoutTime then
                                    begin
                                      Disconnect;
                                      Mainform.MiscString := 'Timeout while processing custom commands';
                                      MainForm.LogEvent(leMisc);
                                      LE := leTimeout;
                                      Exit;
                                    end;
                                  until Pos(Prompt, CommandResult) > 0;
                                  CustomOutputs[N] := CommandResult;
                                  CustomDone := True;
                                except
                                  on E: Exception do
                                    CustomOutputs[N] := 'Command number ' + IntToStr(N) + ' failed';
                                end;
                                CustomDone := True;
                                Break;
                              end;
                            end;
                          end;
                        end;

                        //Get modem/router stats
                        try
                          Sleep(100);
                          CommandResult := '';
                          LTelnetClient.SendMessage(StatsCommand + #13#10);
                          repeat
                            Application.ProcessMessages;
                            Sleep(10);
                            Len := Length(CommandResult);
                            if Now > TimeoutTime then
                            begin
                              Disconnect;
                              Mainform.MiscString := 'Timeout while retrieving stats';
                              MainForm.LogEvent(leMisc);
                              LE := leNostats;
                              Exit;
                            end;
                          until (Len > 500)
                          and (Pos(Prompt, CommandResult) > 400);
                          Stats := CommandResult;
                          Sleep(100);
                          if IsThomson then
                          begin
                            if (Length(Stats) < 200) and (Pos('down', Stats) > 0) then  //DSL connection down
                            begin
                              for N := 1 to 4 do
                                if CustomIncludes[N, 3] then  //Process custom command
                                begin
                                  try
                                    CommandResult := '';
                                    LTelnetClient.SendMessage(CustomCommands[N] + #13#10);
                                    repeat
                                      Application.ProcessMessages;
                                      Sleep(10);
                                      if Now > TimeoutTime then
                                      begin
                                        Disconnect;
                                        Mainform.MiscString := 'Timeout while processing custom commands';
                                        MainForm.LogEvent(leMisc);
                                        LE := leTimeout;
                                        Exit;
                                      end;
                                    until Pos(Prompt, CommandResult) > 0;
                                    CustomOutputs[N] := CommandResult;
                                    CustomDone := True;
                                  except
                                    on E: Exception do
                                      CustomOutputs[N] := 'Command number ' + IntToStr(N) + ' failed';
                                  end;
                                end;
                              DSLdown := True;
                              Disconnect;
                              LE := leNoshowtime;
                              Exit;
                            end;
                          end;
                          if Length(Stats) < 100 then
                          begin
                            Disconnect;
                            LE := leNostats;
                            Exit;
                          end;

                          if not IsThomson and (Pos('Showtime', Stats) = 0) then    //DSL down
                          begin
                            for N := 1 to 4 do
                              if CustomIncludes[N, 3] then  //Process custom command
                              begin
                                try
                                  CommandResult := '';
                                  LTelnetClient.SendMessage(CustomCommands[N] + #13#10);
                                  repeat
                                    Application.ProcessMessages;
                                    Sleep(10);
                                    if Now > TimeoutTime then
                                    begin
                                      Disconnect;
                                      Mainform.MiscString := 'Timeout while processing custom commands';
                                      MainForm.LogEvent(leMisc);
                                      LE := leTimeout;
                                      Exit;
                                    end;
                                  until Pos(Prompt, CommandResult) > 0;
                                  CustomOutputs[N] := CommandResult;
                                  CustomDone := True;
                                except
                                  on E: Exception do
                                    CustomOutputs[N] := 'Command number ' + IntToStr(N) + ' failed';
                                end;
                              end;
                            DSLdown := True;
                            Disconnect;
                            LE := leNoshowtime;
                            Exit;
                          end
                          else begin
                            if DSLdown then  //Was down, now back up, i.e. a resync
                            begin
                              DSLdown := False;
                              for N := 1 to 4 do
                                if CustomIncludes[N, 2] then  //Process custom command
                                begin
                                  try
                                    CommandResult := '';
                                    LTelnetClient.SendMessage(CustomCommands[N] + #13#10);
                                    repeat
                                      Application.ProcessMessages;
                                      Sleep(10);
                                      if Now > TimeoutTime then
                                      begin
                                        Disconnect;
                                        Mainform.MiscString := 'Timeout while processing custom commands';
                                        MainForm.LogEvent(leMisc);
                                        LE := leTimeout;
                                        Exit;
                                      end;
                                    until Pos(Prompt, CommandResult) > 0;
                                    CustomOutputs[N] := CommandResult;
                                    CustomDone := True;
                                  except
                                    on E: Exception do
                                      CustomOutputs[N] := 'Command number ' + IntToStr(N) + ' failed';
                                  end;
                                end;
                            end;
                          end;

                          if IsThomson then     //Collect G.INP data if available
                          begin
                            CommandResult := '';
                            LTelnetClient.SendMessage(GInpCommand + #13#10);
                            repeat
                              Application.ProcessMessages;
                              Sleep(10);
                              Len := Length(CommandResult);
                              if Now > TimeoutTime then
                              begin
                                Disconnect;
                                Mainform.MiscString := 'Timeout while retrieving stats';
                                MainForm.LogEvent(leMisc);
                                LE := leNostats;
                                Exit;
                              end;
                            until Pos(Prompt, CommandResult) > 10;
                            GInpStats := CommandResult;
                          end;

                        except
                          on E: Exception do
                          begin
                            LE := leNostats;
                            Disconnect;
                            Exit;
                          end;
                        end;

                        //Get bitloading data
                        if DoBitloading then
                        try
                          CommandResult := '';
                          LTelnetClient.SendMessage(#13#10);
                          repeat
                            Application.ProcessMessages;
                            Sleep(10);
                            if Now > TimeoutTime then
                            begin
                              Disconnect;
                              Mainform.MiscString := 'Timeout while collecting bitloading data';
                              MainForm.LogEvent(leMisc);
                              LE := leTimeout;
                              Exit;
                            end;
                          until Pos(Prompt, CommandResult) > 0;
                          CommandResult := '';
                          LTelnetClient.SendMessage(BitsCommand + #13#10);
                          repeat
                            Application.ProcessMessages;
                            Sleep(10);
                            Len := Length(CommandResult);
                            if Now > TimeoutTime then
                            begin
                              Disconnect;
                              Mainform.MiscString := 'Timeout while collecting bitloading data';
                              MainForm.LogEvent(leMisc);
                              LE := leTimeout;
                              Exit;
                            end;
                          until (Len > 500)
                          and (Pos(Prompt, CommandResult) > 400);
                          Bits := CommandResult;
                        except
                          on E: Exception do
                          begin
                            LE := leNobits;
                            Disconnect;
                           Exit;
                          end;
                        end;

                        //Get Traffic data
                        if DoTraffic then
                        try
                          CommandResult := '';
                          LTelnetClient.SendMessage(TrafficCommand + #13#10);
                          repeat
                            Application.ProcessMessages;
                            Sleep(10);
                            Len := Length(CommandResult);
                            if Now > TimeoutTime then
                            begin
                              Disconnect;
                              Mainform.MiscString := 'Timeout while collecting traffic data';
                              MainForm.LogEvent(leMisc);
                              LE := leTimeout;
                              Exit;
                            end;
                          until (Len > 100)
                          and (Pos(Prompt, CommandResult) > 100);
                          Traffic := CommandResult;
                        except
                          on E: Exception do
                          begin
                            LE := leNoTraffic;
                            Disconnect;
                            Traffic := 'blank';
                            Exit;
                          end;
                        end;

                        //End of Thomson/Technicolor options
                        if IsThomson then
                        begin
                          LE := leSuccess;
                          Disconnect;
                          VersionText := 'Not reported';
                          Vendor := 'Not reported';
                          Exit;
                        end;

                        //Get SNR per tone with --SNR command
                        if DoSNRpertone then
                        try
                          CommandResult := '';
                          LTelnetClient.SendMessage(SNRCommand + #13#10);
                          repeat
                            Application.ProcessMessages;
                            Sleep(10);
                            Len := Length(CommandResult);
                            if Now > TimeoutTime then
                            begin
                              Disconnect;
                              Mainform.MiscString := 'Timeout while collecting SNR per tone';
                              MainForm.LogEvent(leMisc);
                              LE := leTimeout;
                              Exit;
                            end;
                          until (Len > 500)
                          and (Pos(Prompt, CommandResult) > 400);   //((CommandResult[Len] = Prompt) or (CommandResult[Len - 1] = Prompt));  //Pos(Prompt, CommandResult) > 0;
                          SNR := CommandResult;
                        except
                          on E: Exception do
                          begin
                            LE := leNosnr;
                            Disconnect;
                            Exit;
                          end;
                        end;

                        //Get QLN data with --QLN command
                        if DoQLN and (not QLNDone) then
                        try
                          CommandResult := '';
                          LTelnetClient.SendMessage(QLNCommand + #13#10);
                          repeat
                            Application.ProcessMessages;
                            Sleep(10);
                            Len := Length(CommandResult);
                            if Now > TimeoutTime then
                            begin
                              Disconnect;
                              Mainform.MiscString := 'Timeout while collecting QLN data';
                              MainForm.LogEvent(leMisc);
                              LE := leTimeout;
                              Exit;
                            end;
                            until (Len > 500)
                            and (Pos(Prompt, CommandResult) > 400);   //((CommandResult[Len] = Prompt) or (CommandResult[Len - 1] = Prompt));  //Pos(Prompt, CommandResult) > 0;
                          QLN := CommandResult;
                          QLNDone := True;
                        except
                          on E: Exception do
                          begin
                            LE := leNoqln;
                            Disconnect;
                            Exit;
                          end;
                        end;

                        //Get HLog data with --HLog command
                        if DoHlog and (not HLogDone) then
                        try
                          CommandResult := '';
                          LTelnetClient.SendMessage(HLogCommand + #13#10);
                          repeat
                            Application.ProcessMessages;
                            Sleep(10);
                            Len := Length(CommandResult);
                            if Now > TimeoutTime then
                            begin
                              Disconnect;
                              Mainform.MiscString := 'Timeout while collecting HLog data';
                              MainForm.LogEvent(leMisc);
                              LE := leTimeout;
                              Exit;
                            end;
                          until (Len > 500)
                          and (Pos(Prompt, CommandResult) > 400);   //((CommandResult[Len] = Prompt) or (CommandResult[Len - 1] = Prompt));  //Pos(Prompt, CommandResult) > 0;
                          HLog := CommandResult;
                          HLogDone := True;
                        except
                          on E: Exception do
                          begin
                            LE := leNohlog;
                            Disconnect;
                            Exit;
                          end;
                        end;

                        //Establish connection mode with --pbParams command
                        if (Mode = -1) or (Mode = 2) then
                        try
                          CommandResult := '';
                          LTelnetClient.SendMessage(ParamsCommand + #13#10);
                          repeat
                            Application.ProcessMessages;
                            Sleep(10);
                            if Now > TimeoutTime then
                            begin
                              Disconnect;
                              Mainform.MiscString := 'Timeout while collecting pbParams data';
                              MainForm.LogEvent(leMisc);
                              LE := leTimeout;
                              Exit;
                            end;
                          until Pos(Prompt, CommandResult) > 0;
                          Params := CommandResult;
                        except
                          on E: Exception do
                          begin
                            LE := leNopbparams;
                            Disconnect;
                            Params := 'blank';
                            Exit;
                          end;
                        end;

                        //Get signal attenuation (if available) - not for VDSL2
                        if DoSignalAttn and ((Mode in [0, 1]) or ((Mode = -1) and (Pos('Discovery', Params) = 0))) then
                        try
                          CommandResult := '';
                          LTelnetClient.SendMessage(LinediagCommand + #13#10);
                          repeat
                            Application.ProcessMessages;
                            Sleep(10);
                            Len := Length(CommandResult);
                            if Now > TimeoutTime then
                            begin
                              Disconnect;
                              Mainform.MiscString := 'Timeout while collecting signal attn data';
                              MainForm.LogEvent(leMisc);
                              LE := leTimeout;
                              Exit;
                            end;
                          until (Len > 500)
                          and (Pos(Prompt, CommandResult) > 400);
                          Linediags := CommandResult;

                        except
                          on E: Exception do
                          begin
                            Disconnect;
                            MainForm.MiscString := 'Unknown error while collecting signal attenuation';
                            MainForm.LogEvent(leMisc);
                            Exit;
                          end;
                        end;

                        //Get make of DSLAM/MSAN with --vendor command
                        if DoVendor then
                        try
                          CommandResult := '';
                          LTelnetClient.SendMessage(VendorCommand + #13#10);
                          repeat
                            Application.ProcessMessages;
                            Sleep(10);
                            if Now > TimeoutTime then
                            begin
                              Disconnect;
                              Mainform.MiscString := 'Timeout while collecting vendor data';
                              MainForm.LogEvent(leMisc);
                              LE := leTimeout;
                              Exit;
                            end;
                          until Pos(Prompt, CommandResult) > 0;
                          Vendor := CommandResult;
                        except
                          on E: Exception do
                          begin
                            LE := leNovendor;
                            Disconnect;
                            Vendor := 'blank';
                            Exit;
                          end;
                        end
                        else Vendor := 'Not monitored';

                        //Get status of vectoring with --vectoring command
                        if DoVectoring then
                        try
                          CommandResult := '';
                          LTelnetClient.SendMessage(VectoringCommand + #13#10);
                          repeat
                            Application.ProcessMessages;
                            Sleep(10);
                            if Now > TimeoutTime then
                            begin
                              Disconnect;
                              Mainform.MiscString := 'Timeout while collecting vectoring data';
                              MainForm.LogEvent(leMisc);
                              LE := leTimeout;
                              Exit;
                            end;
                          until Pos(Prompt, CommandResult) > 0;
                          Vectoring := CommandResult;
                        except
                          on E: Exception do
                          begin
                            LE := leNovectoring;
                            Disconnect;
                            Vectoring := 'blank';
                            Exit;
                          end;
                        end
                        else Vectoring := 'Not monitored';

                        //Get system uptime with command 'cat /proc/uptime'
                        if DoSysuptime then
                        try
                          CommandResult := '';
                          LTelnetClient.SendMessage(UptimeCommand + #13#10);
                          repeat
                            Application.ProcessMessages;
                            Sleep(10);
                            if Now > TimeoutTime then
                            begin
                              Disconnect;
                              Mainform.MiscString := 'Timeout while collecting system uptime';
                              MainForm.LogEvent(leMisc);
                              LE := leTimeout;
                              Exit;
                            end;
                          until Pos(Prompt, CommandResult) > 0;
                          Uptime := CommandResult;
                        except
                          on E: Exception do
                          begin
                            LE := leNouptime;
                            Disconnect;
                            Uptime := 'Unknown';
                            Exit;
                          end;
                        end
                        else Uptime := 'Not monitored';

                        //Get firmware version
                        if DoFirmware then
                        try
                          CommandResult := '';
                          LTelnetClient.SendMessage(VersionCommand + #13#10);
                          repeat
                            Application.ProcessMessages;
                            Sleep(10);
                            if Now > TimeoutTime then
                            begin
                              Disconnect;
                              Mainform.MiscString := 'Timeout while collecting firmware version';
                              MainForm.LogEvent(leMisc);
                              LE := leTimeout;
                              Exit;
                            end;
                          until Pos(Prompt, CommandResult) > 0;
                          VersionText := CommandResult;
                        except
                          on E: Exception do VersionText := 'Unknown';
                        end
                        else VersionText := 'Not monitored';

                        Disconnect;
                        LE := leSuccess;
                      end;

      toAdvancedTweak:begin    //Apply advanced tweaks
                        try
                          Sleep(50);
                          CommandResult := '';
                          LTelnetClient.SendMessage(AdvancedTweakCommand + #13#10);
                          repeat
                            Application.ProcessMessages;
                            Sleep(10);
                            if Now > TimeoutTime then
                            begin
                              Disconnect;
                              Mainform.MiscString := 'Timeout while processing advanced tweaks';
                              MainForm.LogEvent(leMisc);
                              LE := leTimeout;
                              Exit;
                            end;
                          until Pos(Prompt, CommandResult) > 0;
                          LE := leSuccess;
                        except
                          on E: Exception do LE := leNoadvancedtweak;
                        end;
                        Disconnect;
                      end;
     end;
  end
  else begin
    LE := leNologin;
  end;
end;

//Close the telnet session
procedure TTelnetForm.Disconnect;
var
  N: Byte;
begin
  try
    N := 1;
    repeat   //Try up to 5 times to disconnect telnet session
      LTelnetClient.Disconnect;
      Sleep(50);
      if not LTelnetClient.Connected then Break;
      Inc(N);
    until N = 6;
    if N > 1 then
    begin
      if N = 6 then Mainform.MiscString := 'Unable to disconnect telnet session after 5 attempts'
      else Mainform.MiscString := IntToStr(N) + ' attempts needed to disconnect telnet session';
      Mainform.LogEvent(leMisc);
    end;
  except
    on E: Exception do; //Do nothing, it was already disconnected
  end;
end;

procedure TTelnetForm.LTelnetClientError(const Msg: string; aSocket: TLSocket);
begin
  //Not used at present
end;

procedure TTelnetForm.LTelnetClientReceive(aSocket: TLSocket);
var
  S: string;
begin
  if LTelnetClient.GetMessage(S) > 0 then CommandResult := CommandResult + S;
end;

end.

