{DSLstats - a program for monitoring the state of a DSL internet connection

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License version 2
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

You can contact the author via the website http://dslstats.plainroad.me.uk
}

unit main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ComCtrls,
  LCLType, StdCtrls, Buttons, Spin, ExtCtrls, ColorBox, EditBtn, inifiles,
  dateutils, Bitchart1, linechart, Linediagchart1, traffic, SNRMperband,
  Clipbrd, maxmin, telnetunit1, LCLIntf, Menus, httpsend,
  strutils, vinfo, Blowfish, Base64, pbmaxmin, synautil, ftpsend,
  fphttpclient, smtpsend, ssl_openssl, blcksock, process, zipper,
  fpwebfile, fphttpserver, types, popup{$IFDEF Windows}, jwatlhelp32,
  windows, ShellAPI{$ELSE}, Unix, BaseUnix{$ENDIF}, tonerange;

type
  TDataPoint = record
    MaxUpValue, MinUpValue, MaxDownValue, MinDownValue: Single;
    MaxUpTime, MinUpTime, MaxDownTime, MinDownTime: TDateTime;
  end;

  TpbDataPoint = record
    MaxUValues, MinUValues, MaxDValues, MinDValues: array[0..4] of Single;
    MaxUTimes, MinUTimes, MaxDTimes, MinDTimes: array[0..4] of TDateTime;
  end;

  TErrors1 = record
    CRCup, CRCdown, FECup, FECdown, HECup, HECdown, ESup, ESdown, SESup, SESdown: Int64;
    TD: TDateTime;
    Seconds: Cardinal;
  end;

  TUtilityOp = (Tweaking, AdvancedTweaking);

  TVersionResult = (vrUpdateYes, vrUpdateNo, vrNoResult);

  TAlertType = (atSNRM, atSNRMreturn, atCRC, atFEC, atES, atResync, atTest, atDS, atFTPFail);

  TUploadSource = (usWSTest, usDSTest, usSSTest, usDataStore, usWebserver, usSnapshots);

  TRouter = packed record
    Manufacturer: string[16];
    Model: string[16];
    IPAddress: string[16];
    LoginName: string[16];
    Password: string[16];
    SecondStage: Boolean;
    ShellCommand: string[8];
    CommandPrefix: string[10];
  end;

  TSpecialRouter = packed record
    Manufacturer: string[24];
    Model: string[24];
    IPAddress: string[16];
    Prompts, Responses: array[1..10] of string[24];
    CommandPrefix: string[16];
    DelayTime: Word;
  end;

  TTrafficConfig = record
    Name: string[20];
    Command: string[20];
    IFace: string[12];
    Other: string[12];
    Reversed: string[1];
  end;

  TXMLSave = record
    SNRMFile, ConSpeedFile, CRCFile, FECFile, BitswapsFile: string;
    SNRM, ConSpeed, CRC, FEC, Bitswaps: Boolean;
    SNRMStart, ConSpeedStart, CRCStart, FECStart, BitswapsStart, SNRMEnd, ConSpeedEnd, CRCEnd, FECEnd, BitswapsEnd: TDateTime;
  end;

  {THTTPServerThread}

  THTTPServerThread = class(TThread)
  private
    FServer: TFPHTTPServer;
  public
    constructor Create(APort: Word; const OnRequest: THTTPServerRequestHandler);
    procedure Execute; override;
    procedure DoTerminate; override;
    property Server: TFPHTTPServer read FServer;
  end;

  { Tmainform }

  Tmainform = class(TForm)
    Activitylight: TLabel;
    AdvancedTab: TTabSheet;
    AdvancedTweaksTab: TTabSheet;
    AlertsTab: TTabSheet;
    AllActiveWeb: TCheckBox;
    AllConfiguredWeb: TCheckBox;
    AllTextWeb: TCheckBox;
    Annexm: TCheckBox;
    ApplicationProperties1: TApplicationProperties;
    ApplyButton: TBitBtn;
    AttenuationLogText: TCheckBox;
    AttenuationTab: TTabSheet;
    AutoDetectBox: TCheckBox;
    AutoscaleBox: TCheckBox;
    AutostartBox: TCheckBox;
    AverageDateBox: TComboBox;
    AverageErrorsText: TCheckBox;
    AveragesTab1: TTabSheet;
    AverageTitle1: TLabel;
    BaselineBox: TCheckBox;
    BaselineBox1: TCheckBox;
    BHideLegend: TCheckBox;
    FTPemail: TCheckBox;
    FTPFailBox: TCheckBox;
    DoUploadData: TCheckBox;
    DoUploadWS: TCheckBox;
    DoUploadSS: TCheckBox;
    DSCurlLabel: TLabel;
    DSCurlLocation: TFileNameEdit;
    DSFTPButton: TRadioButton;
    DSFTPPort: TEdit;
    WSFTPSButton: TRadioButton;
    SSFTPSButton: TRadioButton;
    DSRevealFTPpassword: TCheckBox;
    DSSFTPButton: TRadioButton;
    DSTLSButton: TRadioButton;
    DSUploadDir: TEdit;
    DSUploadPassword: TEdit;
    DSUploadURL: TEdit;
    DSUploadUsername: TEdit;
    FTPresult: TLabel;
    Label147: TLabel;
    Label148: TLabel;
    Label149: TLabel;
    Label150: TLabel;
    Label162: TLabel;
    Label175: TLabel;
    Label176: TLabel;
    Label177: TLabel;
    Label178: TLabel;
    Label179: TLabel;
    Label180: TLabel;
    Label181: TLabel;
    Label182: TLabel;
    Label183: TLabel;
    Label184: TLabel;
    Label185: TLabel;
    FTPUploadTab: TTabSheet;
    Label186: TLabel;
    Label187: TLabel;
    Label188: TLabel;
    Label189: TLabel;
    Label190: TLabel;
    Label191: TLabel;
    Label192: TLabel;
    Label193: TLabel;
    Label194: TLabel;
    Label197: TLabel;
    PageControl11: TPageControl;
    DSFTPSbutton: TRadioButton;
    TestUploadButton: TBitBtn;
    WSCurlLabel: TLabel;
    SSCurlLabel: TLabel;
    WSCurlLocation: TFileNameEdit;
    SSCurlLocation: TFileNameEdit;
    WSFTPButton: TRadioButton;
    SSFTPButton: TRadioButton;
    WSFTPPort: TEdit;
    SSFTPPort: TEdit;
    WSFTPresult: TLabel;
    SSFTPResult: TLabel;
    WSRevealFTPpassword: TCheckBox;
    SSRevealFTPpassword: TCheckBox;
    WSSFTPButton: TRadioButton;
    SSSFTPButton: TRadioButton;
    WSTestUploadButton: TBitBtn;
    SSTestUploadButton: TBitBtn;
    WSTLSButton: TRadioButton;
    SSTLSButton: TRadioButton;
    WSUploadDir: TEdit;
    SSUploadDir: TEdit;
    WSUploadPassword: TEdit;
    SSUploadPassword: TEdit;
    WSUploadRate: TSpinEdit;
    WSUploadTab: TTabSheet;
    SSUploadTab: TTabSheet;
    DSUploadTab: TTabSheet;
    MultipleButton : TRadioButton ;
    Panel18 : TPanel ;
    SingleButton : TRadioButton ;
    Label17: TLabel;
    WebserverSaveBox: TCheckBox;
    WebserverDirEdit: TDirectoryEdit;
    DataFolderEdit: TDirectoryEdit;
    Label143: TLabel;
    Label144: TLabel;
    Label145: TLabel;
    ESTotalDown: TLabel;
    ESTotalUp: TLabel;
    Label146: TLabel;
    Label15: TLabel;
    Resyncemail: TCheckBox;
    SNRMemail: TCheckBox;
    CRCemail: TCheckBox;
    FECemail: TCheckBox;
    ESemail: TCheckBox;
    DSemail: TCheckBox;
    Label142: TLabel;
    SMTPPort: TEdit;
    PlainButton: TRadioButton;
    SSLButton: TRadioButton;
    StartTLSButton: TRadioButton;
    SaveClearButton: TBitBtn;
    EmailFrom: TEdit;
    ESWeb: TCheckBox;
    Label105: TLabel;
    Label33: TLabel;
    PlannedEventsBox: TCheckBox;
    FontSelectButton: TBitBtn;
    FontDialog2: TFontDialog;
    FontSelectButton1: TBitBtn;
    FontSelectButton10: TBitBtn;
    FontSelectButton11: TBitBtn;
    FontSelectButton12: TBitBtn;
    FontSelectButton2: TBitBtn;
    FontSelectButton3: TBitBtn;
    FontSelectButton4: TBitBtn;
    FontSelectButton5: TBitBtn;
    FontSelectButton6: TBitBtn;
    FontSelectButton7: TBitBtn;
    FontSelectButton8: TBitBtn;
    FontSelectButton9: TBitBtn;
    DaysPerPageEdit: TSpinEdit;
    ToneRangeS: TBitBtn;
    ToneRangeQ: TBitBtn;
    CustomPrefixEdit: TEdit;
    CustomPrefixLabel: TLabel;
    DownstreamBoxM: TCheckBox;
    Frequency2: TLabel;
    Label155: TLabel;
    Label159: TLabel;
    Label160: TLabel;
    Label153: TLabel;
    Label79: TLabel;
    Label80: TLabel;
    Marginchart: TLinechart;
    MarginClip: TSpinEdit;
    MaxYEdit: TSpinEdit;
    MClipLabel: TLabel;
    MdbLabel: TLabel;
    MHideLegend: TCheckBox;
    minLabel: TLabel;
    hrLabel: TLabel;
    MinYEdit: TSpinEdit;
    MSnapButton: TSpeedButton;
    PageControl10: TPageControl;
    Panel4: TPanel;
    Panel9: TPanel;
    RightClickLabel1: TLabel;
    SNR2: TLabel;
    SNRchart: TLinediagchart;
    SNRHideLegend: TCheckBox;
    SNRSnapButton: TSpeedButton;
    StartofRecording: TRadioButton;
    FirstAtTime: TRadioButton;
    SCustomPrefixEdit: TEdit;
    SCustomPrefixLabel: TLabel;
    ScaleBox: TCheckBox;
    HeightLabel: TLabel;
    SnapshotHours: TSpinEdit;
    SnapshotMinutes: TSpinEdit;
    SNRTabSheet: TTabSheet;
    SNRTab: TTabSheet;
    SNRMTab: TTabSheet;
    ToneNumber2: TLabel;
    ToneRangeH: TBitBtn;
    ToneRangeBS: TBitBtn;
    ToneRangeBL: TBitBtn;
    UpstreamBoxM: TCheckBox;
    WidthLabel: TLabel;
    PixelLabel2: TLabel;
    PixelLabel1: TLabel;
    ShowBillingBox: TCheckBox;
    ClipButton16: TSpeedButton;
    Label173: TLabel;
    MaxUpLabel: TLabel;
    MaxDownLabel: TLabel;
    Panel38: TPanel;
    ResetTodayButton: TBitBtn;
    Bitchart1: TBitchart;
    BitloadingTab: TTabSheet;
    BitloadingText: TCheckBox;
    Bitloadingweb: TCheckBox;
    BitsList: TMemo;
    Bitsnap: TCheckBox;
    Bitswapchart: TBitchart;
    BitswapSaveBox: TCheckBox;
    BitswapSaveButton: TBitBtn;
    BitswapSaveEBox: TCheckBox;
    BitswapsperminLabel: TLabel;
    BitswapTab: TTabSheet;
    BitswapToggle: TRadioGroup;
    BsHideLegend: TCheckBox;
    BSnapButton: TSpeedButton;
    CategoryBox: TCheckBox;
    CClipLabel: TLabel;
    CEnableClippingBox: TCheckBox;
    Changelog: TMemo;
    CheckUpdateBox: TCheckBox;
    CheckUpdateButton: TBitBtn;
    CHideLegend: TCheckBox;
    ClearLogButton: TBitBtn;
    ClearTextButton: TBitBtn;
    ClipButton10: TSpeedButton;
    ClipButton11: TSpeedButton;
    ClipButton12: TSpeedButton;
    ClipButton13: TSpeedButton;
    ClipButton14 : TSpeedButton ;
    ClipButton15: TSpeedButton;
    ClipButton2: TSpeedButton;
    ClipButton3: TSpeedButton;
    ClipButton4: TSpeedButton;
    ClipButton5: TSpeedButton;
    ClipButton6: TSpeedButton;
    ClipButton7: TSpeedButton;
    ClipButton8: TSpeedButton;
    ClipButton9: TSpeedButton;
    Colour0: TColorBox;
    Colour1: TColorBox;
    Colour2: TColorBox;
    Colour3: TColorBox;
    Colour4: TColorBox;
    CombineWeb: TCheckBox;
    CominmgnToggle: TRadioGroup;
    Commandbox: TComboBox;
    CommandGroup: TRadioGroup;
    CommandHours: TSpinEdit;
    CommandMinutes: TSpinEdit;
    CommandOutput: TMemo;
    ConfigurationTab: TTabSheet;
    ConfirmCloseBox: TCheckBox;
    ConfirmRefreshButton: TBitBtn;
    ConnectionStatsText: TCheckBox;
    ConnspeedLogY: TCheckBox;
    Connspeedweb: TCheckBox;
    CperminLabel: TLabel;
    CRCAverageDown: TLabel;
    CRCAverageUp: TLabel;
    CRCchart: TLinechart;
    CRCClip: TSpinEdit;
    CRCLogY: TCheckBox;
    CRCpm: TRadioButton;
    CRCps: TRadioButton;
    CRCriseBox: TCheckBox;
    CRCSaveBox: TCheckBox;
    CRCSaveButton: TBitBtn;
    CRCSaveEBox: TCheckBox;
    CRCsnap: TCheckBox;
    CRCTab: TTabSheet;
    CRCTrigger: TEdit;
    CRCweb: TCheckBox;
    CSnapButton: TSpeedButton;
    CustomBox: TCheckBox;
    CustomButton: TRadioButton;
    CustomCommandsText: TCheckBox;
    CustomEdit: TEdit;
    CustomTab: TTabSheet;
    D0: TCheckBox;
    D1: TCheckBox;
    D2: TCheckBox;
    D3: TCheckBox;
    D4: TCheckBox;
    DailyBox: TCheckBox;
    DailyButton: TRadioButton;
    Dateorder: TRadioButton;
    DaysAgoEdit: TSpinEdit;
    DelayTimeEdit: TSpinEdit;
    DeleteOlderBox: TCheckBox;
    DeleteWebBox: TCheckBox;
    DiagnosticsBox: TCheckBox;
    DisabledLabel: TLabel;
    DisabledLabel1: TLabel;
    DisableSpecial: TCheckBox;
    DisplayHereButton: TRadioButton;
    DoAverages: TCheckBox;
    DoBitloading: TCheckBox;
    Dobitswap: TCheckBox;
    DoBitswaps: TCheckBox;
    DoCoMinMgn: TCheckBox;
    DoCRC: TCheckBox;
    DoES: TCheckBox;
    DoFEC: TCheckBox;
    DoFirmware: TCheckBox;
    DoHlog: TCheckBox;
    Doi24k: TCheckBox;
    DoMargin: TCheckBox;
    DonothingButton: TBitBtn;
    DontAskButton: TCheckBox;
    DoQLN: TCheckBox;
    DoSES: TCheckBox;
    Dosesdrop: TCheckBox;
    DoSignalAttn: TCheckBox;
    DoSNRM: TCheckBox;
    DoSNRMperband: TCheckBox;
    DoSNRpertone: TCheckBox;
    DoSpeed: TCheckBox;
    Dosra: TCheckBox;
    DoSwapspermin: TCheckBox;
    DoSysuptime: TCheckBox;
    DoTrellis: TCheckBox;
    DoVectoring: TCheckBox;
    DoVendor: TCheckBox;
    DoSaveData: TCheckBox;
    Downcolour: TColorBox;
    DownDay: TLabel;
    DownloadButton: TBitBtn;
    DownloadDirEdit: TDirectoryEdit;
    DownloadingLabel: TLabel;
    DownMonth: TLabel;
    DownonlyButton: TRadioButton;
    DownSNRM: TRadioButton;
    DownSNRMargin: TLabel;
    Downspeed: TLabel;
    DownstreamBoxC: TCheckBox;
    DownstreamBoxF: TCheckBox;
    DownstreamBoxLEFTRS: TCheckBox;
    DownstreamBoxO: TCheckBox;
    DownstreamBoxR: TCheckBox;
    DownstreamBoxrtxc: TCheckBox;
    DownstreamBoxrtxtx: TCheckBox;
    DownstreamBoxrtxuc: TCheckBox;
    DownstreamBoxSES: TCheckBox;
    DownstreamBoxT: TCheckBox;
    EHideLegend: TCheckBox;
    EitherSNRM: TRadioButton;
    EmailAddress: TEdit;
    EmailPassword: TEdit;
    EmailSubject: TEdit;
    EmailUsername: TEdit;
    ErrorResetButton1: TBitBtn;
    ErrorTab: TTabSheet;
    ES: TLabel;
    ESAutoscale: TRadioButton;
    ESAverageDown: TLabel;
    ESAverageUp: TLabel;
    ESChart: TTraffic;
    ESMax: TComboBox;
    ESnapButton: TSpeedButton;
    ESRiseBox: TCheckBox;
    ESSetMax: TRadioButton;
    ESSnap: TCheckBox;
    ESTab: TTabSheet;
    ESTrigger: TEdit;
    Eventlist: TMemo;
    EventlogBox: TCheckBox;
    EventLogButton: TRadioButton;
    EventlogTab: TTabSheet;
    EventLogText: TCheckBox;
    EveryHoursLabel: TLabel;
    ExitButton: TSpeedButton;
    ExtendedBox: TCheckBox;
    ExtendedBox1: TCheckBox;
    FailedLabel: TLabel;
    FailLabel: TLabel;
    FClipLabel: TLabel;
    FECAverageDown: TLabel;
    FECAverageUp: TLabel;
    FECchart: TLinechart;
    FECClip: TSpinEdit;
    FECLogY: TCheckBox;
    FECpm: TRadioButton;
    FECps: TRadioButton;
    FECriseBox: TCheckBox;
    FECSaveBox: TCheckBox;
    FECSaveButton: TBitBtn;
    FECSaveEBox: TCheckBox;
    FECsnap: TCheckBox;
    FECTab: TTabSheet;
    FECTrigger: TEdit;
    FECweb: TCheckBox;
    FEnableClippingBox: TCheckBox;
    FHideLegend: TCheckBox;
    FontButton: TSpeedButton;
    FontDialog1: TFontDialog;
    FperminLabel: TLabel;
    Frequency: TLabel;
    FrequencyBox: TSpinEdit;
    FSnapButton: TSpeedButton;
    Fullstatsweb: TCheckBox;
    GBLabel: TLabel;
    GDotInpList: TMemo;
    GDotINPTab: TTabSheet;
    GetIPButton: TBitBtn;
    GInpList : TMemo ;
    Graphcolour: TColorBox;
    GraphsTab: TTabSheet;
    GroupBox2: TGroupBox;
    GroupBox3: TGroupBox;
    GroupBox4: TGroupBox;
    HECAverageDown: TLabel;
    HECAverageUp: TLabel;
    Helpbutton: TSpeedButton;
    HLogchart: TLinediagchart;
    HLogHideLegend: TCheckBox;
    HLogList: TMemo;
    HLogsnap: TCheckBox;
    HLogTab: TTabSheet;
    HLogText: TCheckBox;
    HSnapButton: TSpeedButton;
    i24kToggle: TRadioGroup;
    InsertDelayBox: TCheckBox;
    InterfaceGroup: TRadioGroup;
    Interleave: TLabel;
    IPedit: TEdit;
    IsHG622Box: TCheckBox;
    KeepLevelsBox: TCheckBox;
    Label1: TLabel;
    Label10: TLabel;
    Label100: TLabel;
    Label101: TLabel;
    Label102: TLabel;
    Label103: TLabel;
    Label104: TLabel;
    Label106: TLabel;
    Label107: TLabel;
    Label108: TLabel;
    Label109: TLabel;
    Label11: TLabel;
    Label110: TLabel;
    Label111: TLabel;
    Label112: TLabel;
    Label113: TLabel;
    Label114: TLabel;
    Label115: TLabel;
    Label116: TLabel;
    Label117: TLabel;
    Label118: TLabel;
    Label119: TLabel;
    Label12: TLabel;
    Label120: TLabel;
    Label121: TLabel;
    Label122: TLabel;
    Label123: TLabel;
    Label124: TLabel;
    Label125: TLabel;
    Label126: TLabel;
    Label127: TLabel;
    Label128: TLabel;
    Label129: TLabel;
    Label13: TLabel;
    Label130: TLabel;
    Label131: TLabel;
    Label132: TLabel;
    Label133: TLabel;
    Label134: TLabel;
    Label135: TLabel;
    Label136: TLabel;
    Label137: TLabel;
    Label138: TLabel;
    Label139: TLabel;
    Label14: TLabel;
    Label140: TLabel;
    Label141: TLabel;
    Label151: TLabel;
    Label152: TLabel;
    Label154: TLabel;
    Label156: TLabel;
    Label157: TLabel;
    Label158: TLabel;
    Label16: TLabel;
    Label161: TLabel;
    Label163: TLabel;
    Label164: TLabel;
    Label165: TLabel;
    Label166: TLabel;
    Label167: TLabel;
    Label168: TLabel;
    Label169: TLabel;
    Label170: TLabel;
    Label171: TLabel;
    Label172: TLabel;
    Label174: TLabel;
    Label18: TLabel;
    Label19: TLabel;
    Label2: TLabel;
    Label20: TLabel;
    Label21: TLabel;
    Label22: TLabel;
    Label23: TLabel;
    Label24: TLabel;
    Label25: TLabel;
    Label26: TLabel;
    Label27: TLabel;
    Label28: TLabel;
    Label29: TLabel;
    Label3: TLabel;
    Label30: TLabel;
    Label31: TLabel;
    Label32: TLabel;
    Label34: TLabel;
    Label35: TLabel;
    Label36: TLabel;
    Label37: TLabel;
    Label38: TLabel;
    Label39: TLabel;
    Label4: TLabel;
    Label40: TLabel;
    Label41: TLabel;
    Label42: TLabel;
    Label43: TLabel;
    Label44: TLabel;
    Label45: TLabel;
    Label46: TLabel;
    Label47: TLabel;
    Label48: TLabel;
    Label49: TLabel;
    Label5: TLabel;
    Label50: TLabel;
    Label51: TLabel;
    Label52: TLabel;
    Label53: TLabel;
    Label54: TLabel;
    Label55: TLabel;
    Label56: TLabel;
    Label57: TLabel;
    Label58: TLabel;
    Label59: TLabel;
    Label6: TLabel;
    Label60: TLabel;
    Label61: TLabel;
    Label62: TLabel;
    Label63: TLabel;
    Label64: TLabel;
    Label65: TLabel;
    Label66: TLabel;
    Label67: TLabel;
    Label68: TLabel;
    Label69: TLabel;
    Label7: TLabel;
    Label70: TLabel;
    Label71: TLabel;
    Label72: TLabel;
    Label73: TLabel;
    Label74: TLabel;
    Label75: TLabel;
    Label76: TLabel;
    Label77: TLabel;
    Label78: TLabel;
    Label8: TLabel;
    Label81: TLabel;
    Label82: TLabel;
    Label83: TLabel;
    Label84: TLabel;
    Label85: TLabel;
    Label86: TLabel;
    Label87: TLabel;
    Label88: TLabel;
    Label89: TLabel;
    Label9: TLabel;
    Label90: TLabel;
    Label91: TLabel;
    Label92: TLabel;
    Label93: TLabel;
    Label94: TLabel;
    Label95: TLabel;
    Label96: TLabel;
    Label97: TLabel;
    Label98: TLabel;
    Label99: TLabel;
    LAttenuationLog: TMemo;
    LEFTRSChart: TLinechart;
    LEFTRSClip: TSpinEdit;
    LEFTRSClipLabel: TLabel;
    LEFTRSEnableClippingBox: TCheckBox;
    LEFTRSHideLegend: TCheckBox;
    LEFTRSLogY: TCheckBox;
    LEFTRSperminLabel: TLabel;
    LEFTRSSnapButton: TSpeedButton;
    LegendLeft: TRadioButton;
    LegendRight: TRadioButton;
    LimitColour: TColorBox;
    LimitEdit: TEdit;
    LimitLabel: TLabel;
    LimitWarningBox: TCheckBox;
    LocationLabel: TLabel;
    LoggingButton: TBitBtn;
    LoginTab: TTabSheet;
    DSalertBox: TCheckBox;
    DSbits: TCheckBox;
    DShlog: TCheckBox;
    DSqln: TCheckBox;
    DSsnr: TCheckBox;
    MenuItem1: TMenuItem;
    MinimiseItem: TMenuItem;
    MinimiseoncloseBox: TCheckBox;
    MinimisetotrayBox: TCheckBox;
    MiscTab: TTabSheet;
    MonitorTrafficBox: TCheckBox;
    MonthlyButton: TRadioButton;
    Nameedit: TEdit;
    Nameorder: TRadioButton;
    NoResult: TLabel;
    NormalButton: TBitBtn;
    NormalLabel: TLabel;
    Numberofbits: TLabel;
    OClipLabel: TLabel;
    OEnableClippingBox: TCheckBox;
    OffsetLabel: TLabel;
    OHFErrChart: TLinechart;
    OHFErrClip: TSpinEdit;
    OHFErrLogY: TCheckBox;
    OHideLegend: TCheckBox;
    OlderDays: TSpinEdit;
    OpenDialog1: TOpenDialog;
    OperminLabel: TLabel;
    OptionsTab: TTabSheet;
    OSAdvised: TRadioButton;
    OSnapButton: TSpeedButton;
    Othercolour: TColorBox;
    OtherInterfaceEdit: TEdit;
    OtherInterfaceLabel: TLabel;
    PageControl1: TPageControl;
    PageControl2: TPageControl;
    PageControl3: TPageControl;
    PageControl4: TPageControl;
    PageControl5: TPageControl;
    PageControl6: TPageControl;
    PageControl7: TPageControl;
    PageControl8: TPageControl;
    PageControl9: TPageControl;
    Panel1: TPanel;
    Panel10: TPanel;
    Panel11: TPanel;
    Panel12: TPanel;
    Panel13: TPanel;
    Panel14: TPanel;
    Panel15: TPanel;
    Panel16: TPanel;
    Panel17: TPanel;
    Panel19: TPanel;
    Panel2: TPanel;
    Panel20: TPanel;
    Panel21: TPanel;
    Panel22: TPanel;
    Panel23: TPanel;
    Panel24: TPanel;
    Panel25: TPanel;
    Panel26: TPanel;
    Panel27: TPanel;
    Panel28: TPanel;
    Panel29: TPanel;
    Panel3: TPanel;
    Panel30: TPanel;
    Panel31: TPanel;
    Panel32: TPanel;
    Panel33: TPanel;
    Panel34: TPanel;
    Panel35: TPanel;
    Panel36: TPanel;
    Panel37 : TPanel ;
    Panel5: TPanel;
    Panel6: TPanel;
    Panel7: TPanel;
    Panel8: TPanel;
    ParamsList: TMemo;
    Passwordedit: TEdit;
    PauseBox: TCheckBox;
    Pausebutton: TSpeedButton;
    pbParamsText: TCheckBox;
    PerDay: TRadioButton;
    PerMinute: TRadioButton;
    PerMinuteBoxC: TRadioButton;
    PerMinuteBoxF: TRadioButton;
    PerMinuteBoxLEFTRS: TRadioButton;
    PerMinuteBoxO: TRadioButton;
    PerMinuteBoxR: TRadioButton;
    PerMinuteBoxrtxc: TRadioButton;
    PerMinuteBoxrtxtx: TRadioButton;
    PerMinuteBoxrtxuc: TRadioButton;
    PerMinuteBoxSES: TRadioButton;
    PerMonth: TRadioButton;
    PerSampleBoxC: TRadioButton;
    PerSampleBoxF: TRadioButton;
    PerSampleBoxLEFTRS: TRadioButton;
    PerSampleBoxO: TRadioButton;
    PerSampleBoxR: TRadioButton;
    PerSampleBoxrtxc: TRadioButton;
    PerSampleBoxrtxtx: TRadioButton;
    PerSampleBoxrtxuc: TRadioButton;
    PerSampleBoxSES: TRadioButton;
    PMClip: TSpinEdit;
    PMClipLabel: TLabel;
    PMHideLegend: TCheckBox;
    PMSnapButton: TSpeedButton;
    PopupTimer: TTimer;
    PortEdit: TEdit;
    ProgressBar2: TProgressBar;
    QLNchart: TLinediagchart;
    QLNHideLegend: TCheckBox;
    QLNList: TMemo;
    QLNsnap: TCheckBox;
    QLNTab: TTabSheet;
    QLNText: TCheckBox;
    QSnapButton: TSpeedButton;
    QuitItem: TMenuItem;
    RClipLabel: TLabel;
    RefreshEdit: TSpinEdit;
    REnableClippingBox: TCheckBox;
    ResetButton: TBitBtn;
    ResetDayButton: TBitBtn;
    ResetLoginButton: TBitBtn;
    ResetMonthButton: TBitBtn;
    ResetResyncButton: TBitBtn;
    RestartBox: TCheckBox;
    RestartEdit: TSpinEdit;
    RestoreButton: TSpeedButton;
    RestoredLabel: TLabel;
    RestoreGraphButton: TBitBtn;
    RestoreItem: TMenuItem;
    ResyncBox: TCheckBox;
    RevealBox: TCheckBox;
    RevealBoxS: TCheckBox;
    RevealEPBox: TCheckBox;
    ReverseBox: TCheckBox;
    RHideLegend: TCheckBox;
    RightClickLabel: TLabel;
    RouterBox: TComboBox;
    RouterTrafficBox: TComboBox;
    RperminLabel: TLabel;
    RSCorrchart: TLinechart;
    RSCorrClip: TSpinEdit;
    RSCorrLogY: TCheckBox;
    RSnapButton: TSpeedButton;
    rtxcChart: TLinechart;
    rtxcClip: TSpinEdit;
    rtxcClipLabel: TLabel;
    rtxcEnableClippingBox: TCheckBox;
    rtxcHideLegend: TCheckBox;
    rtxcLogY: TCheckBox;
    rtxcperminLabel: TLabel;
    rtxcSnapButton: TSpeedButton;
    rtxtxChart: TLinechart;
    rtxtxClip: TSpinEdit;
    rtxtxClipLabel: TLabel;
    rtxtxEnableClippingBox: TCheckBox;
    rtxtxHideLegend: TCheckBox;
    rtxtxLogY: TCheckBox;
    rtxtxperminLabel: TLabel;
    rtxtxSnapButton: TSpeedButton;
    rtxucChart: TLinechart;
    rtxucClip: TSpinEdit;
    rtxucClipLabel: TLabel;
    rtxucEnableClippingBox: TCheckBox;
    rtxucHideLegend: TCheckBox;
    rtxucLogY: TCheckBox;
    rtxucperminLabel: TLabel;
    rtxucSnapButton: TSpeedButton;
    SameAsBox: TCheckBox;
    SameButton: TRadioButton;
    SampleRateEdit: TSpinEdit;
    SAnnexM: TCheckBox;
    SAttenuationLog: TMemo;
    Save1: TLabel;
    Save2: TLabel;
    Save3: TLabel;
    Save4: TLabel;
    Save5: TLabel;
    SaveAfterBox: TCheckBox;
    SaveAveragesButton: TSpeedButton;
    SaveGdotInpData: TSpeedButton;
    SaveGInpButton : TSpeedButton ;
    SaveLogButton: TSpeedButton;
    SaveLogButton1: TSpeedButton;
    SaveonexitBox: TCheckBox;
    SaveStatsButton: TSpeedButton;
    SaveTDBits: TSpeedButton;
    SaveTDHLog: TSpeedButton;
    SaveTDParams: TSpeedButton;
    SaveTDQLN: TSpeedButton;
    SaveTDSNR: TSpeedButton;
    SaveTDStats: TSpeedButton;
    SaveTDTraffic: TSpeedButton;
    SaveTDVectoring: TSpeedButton;
    SaveTextButton: TBitBtn;
    SaveXMLTab: TTabSheet;
    SCommandBox: TComboBox;
    Secondstage: TCheckBox;
    SelectXMLButton: TBitBtn;
    SendEventlogBox: TCheckBox;
    SendNowButton: TBitBtn;
    SESAverageDown: TLabel;
    SESAverageUp: TLabel;
    SESchart: TLinechart;
    SESClip: TSpinEdit;
    SESClipLabel: TLabel;
    SesdropToggle: TRadioGroup;
    SESEnableClippingBox: TCheckBox;
    SESHideLegend: TCheckBox;
    SESLogY: TCheckBox;
    SESperminLabel: TLabel;
    SESsnap: TCheckBox;
    SESSnapButton: TSpeedButton;
    SEStab: TTabSheet;
    SESWeb: TCheckBox;
    Shellcommand: TEdit;
    SHideLegend: TCheckBox;
    ShowHintsBox: TCheckBox;
    ShowMaxDownBox: TCheckBox;
    ShowMaxUpBox: TCheckBox;
    SIPEdit: TEdit;
    SMTPserver: TEdit;
    SnapallBox: TCheckBox;
    SnapallButton: TSpeedButton;
    SnapAllConfiguredBox: TCheckBox;
    SnapDirEdit: TDirectoryEdit;
    SnapshotsTab: TTabSheet;
    SNR: TLabel;
    SNRColour: TColorBox;
    SNRList: TMemo;
    SNRMbar: TTrackBar;
    SNRMbox: TGroupBox;
    SNRMdownPanel: TPanel;
    SNRMdownTab: TTabSheet;
    SNRMfallBox: TCheckBox;
    SNRMpbChartdown: TSNRMperband;
    SNRMpbChartup: TSNRMperband;
    SNRMpbSnap: TCheckBox;
    SNRMpbweb: TCheckBox;
    SNRMperbandTab: TTabSheet;
    SNRMreturnBox: TCheckBox;
    SNRMreturnTrigger: TEdit;
    SNRMTrigger: TEdit;
    SNRMupPanel: TPanel;
    SNRMupTab: TTabSheet;
    SNRMweb: TCheckBox;
    SNRptsnap: TCheckBox;
    SNRSaveBox: TCheckBox;
    SNRSaveButton: TBitBtn;
    SNRSaveEBox: TCheckBox;
    SNRseparate: TCheckBox;
    SNRsnap: TCheckBox;
    SNRText: TCheckBox;
    SNRwithbitloading: TCheckBox;
    SpBHideLegendDown: TCheckBox;
    SpBHideLegendUp: TCheckBox;
    SpBSnapButtonDown: TSpeedButton;
    SpBSnapButtonUp: TSpeedButton;
    SpecialLabel: TLabel;
    SpecialLogin: TCheckBox;
    SpecialTab: TTabSheet;
    Speedchart1: TLinechart;
    SpeedSaveBox: TCheckBox;
    SpeedSaveButton: TBitBtn;
    SpeedSaveEBox: TCheckBox;
    Speedsnap: TCheckBox;
    SpeedTab: TTabSheet;
    DayEdit: TSpinEdit;
    HeightEdit: TSpinEdit;
    WidthEdit: TSpinEdit;
    SPortEdit: TEdit;
    SraToggle: TRadioGroup;
    SRouterBox: TComboBox;
    SSnapButton: TSpeedButton;
    StartButton: TSpeedButton;
    StartDateTime: TLabel;
    StartingLabel1: TLabel;
    StartMaximisedBox: TCheckBox;
    StartminimisedBox: TCheckBox;
    StatsList: TMemo;
    StatsTab: TTabSheet;
    StatsText: TMemo;
    StatsText1: TCheckBox;
    Statsweb: TCheckBox;
    Stopbutton: TSpeedButton;
    SuccessLabel: TLabel;
    Swappmsnap: TCheckBox;
    Swapptsnap: TCheckBox;
    Swapsperminchart: TLinechart;
    Swapspermintab: TTabSheet;
    SWSnapButton: TSpeedButton;
    Synchronise: TCheckBox;
    SysUptime: TLabel;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    TabSheet5: TTabSheet;
    TabSheet6: TTabSheet;
    TabSheet7: TTabSheet;
    TabSheet8: TTabSheet;
    TelnetVectoringTab: TTabSheet;
    TakeEveryLabel: TLabel;
    TDownBox: TCheckBox;
    TelnetBitTab: TTabSheet;
    TelnetDataTab: TTabSheet;
    TelnetGInpTab : TTabSheet ;
    TelnetHlogTab: TTabSheet;
    TelnetParamsTab: TTabSheet;
    TelnetQLNTab: TTabSheet;
    TelnetSNRTab: TTabSheet;
    TelnetTab: TTabSheet;
    TelnetTrafficTab: TTabSheet;
    TestButton: TButton;
    ThicknessEdit: TSpinEdit;
    ThomBox: TCheckBox;
    TimeperpageEdit: TSpinEdit;
    TimepersnapshotEdit: TSpinEdit;
    Timer1: TTimer;
    Timer2: TTimer;
    Timer3: TTimer;
    Timer4: TTimer;
    Timer5: TTimer;
    Timer6: TTimer;
    Timer7: TTimer;
    Timer8: TTimer;
    Timer9: TTimer;
    Tonenumber1: TLabel;
    TonesTab: TTabSheet;
    Toolbar1: TPanel;
    TrafficChart: TTraffic;
    TrafficConfigTab: TTabSheet;
    TrafficList: TMemo;
    VectoringList: TMemo;
    Trafficpermin: TRadioButton;
    TrafficperminChart: TLinechart;
    TrafficSnap: TCheckBox;
    TrafficTab: TTabSheet;
    TrafficText: TCheckBox;
    TrayIcon1: TTrayIcon;
    TrayMenu: TPopupMenu;
    TrellisToggle: TRadioGroup;
    TUpBox: TCheckBox;
    TweakingCommand: TLabel;
    U0: TCheckBox;
    U1: TCheckBox;
    U2: TCheckBox;
    U3: TCheckBox;
    U4: TCheckBox;
    UHideLegend: TCheckBox;
    Upcolour: TColorBox;
    UpDay: TLabel;
    UpdownPanel: TPanel;
    UpgradeTab: TTabSheet;
    UpMonth: TLabel;
    UpPlusDownButton: TRadioButton;
    UpPlusDownColour: TColorBox;
    UpSNRM: TRadioButton;
    UpSNRMargin: TLabel;
    Upspeed: TLabel;
    UpstreamBoxC: TCheckBox;
    UpstreamBoxF: TCheckBox;
    UpstreamBoxLEFTRS: TCheckBox;
    UpstreamBoxO: TCheckBox;
    UpstreamBoxR: TCheckBox;
    UpstreamBoxrtxc: TCheckBox;
    UpstreamBoxrtxtx: TCheckBox;
    UpstreamBoxrtxuc: TCheckBox;
    UpstreamBoxSES: TCheckBox;
    UpstreamBoxT: TCheckBox;
    Uptime: TLabel;
    USnapButton: TSpeedButton;
    VersionResult: TLabel;
    WaitLabel1: TLabel;
    WebPortEdit: TEdit;
    WebserverBox: TCheckBox;
    WebServerTab: TTabSheet;
    WebsiteButton: TBitBtn;
    DataStoreTab: TTabSheet;
    WithOther: TRadioButton;
    WSUploadURL: TEdit;
    SSUploadURL: TEdit;
    WSUploadUsername: TEdit;
    SSUploadUsername: TEdit;
    XMLFileEdit: TEdit;
    procedure AllActiveWebChange(Sender: TObject);
    procedure AnnexmChange(Sender: TObject);
    procedure ApplicationProperties1Minimize(Sender: TObject);
    procedure ApplyButtonClick(Sender: TObject);
    procedure AutoscaleBoxChange(Sender: TObject);
    procedure AverageDateBoxChange(Sender: TObject);
    procedure BaselineBoxChange(Sender: TObject);
    procedure DoUploadSSChange(Sender: TObject);
    procedure DoUploadWSChange(Sender: TObject);
    //procedure DSbitsChange(Sender: TObject);
    //procedure DSbitsUploadChange(Sender: TObject);
    procedure MultipleButtonChange(Sender: TObject);
    procedure DSRevealFTPpasswordChange(Sender: TObject);
    procedure DataFolderEditChange(Sender: TObject);
    procedure DoUploadDataChange(Sender: TObject);
    procedure TestUploadButtonClick(Sender: TObject);
    procedure EmailFromExit(Sender: TObject);
    procedure FontSelectButtonClick(Sender: TObject);
    procedure SaveClearButtonClick(Sender: TObject);
    procedure SMTPPortChange(Sender: TObject);
    procedure SNRMemailChange(Sender: TObject);
    procedure SNRTabExit(Sender: TObject);
    procedure ToneRangeClick(Sender: TObject);
    procedure Bitchart1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Bitchart1MouseLeave(Sender: TObject);
    procedure Bitchart1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure Bitchart1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Bitchart1MouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure Bitchart1MouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure BitloadingTabMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure CEnableClippingBoxChange(Sender: TObject);
    procedure CheckAlerts(AlertType: TAlertType; Number1, Number2: Extended);
    procedure HeightEditChange(Sender: TObject);
    procedure ScaleBoxChange(Sender: TObject);
    procedure DayEditKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState
      );
    procedure CheckUpdateButtonClick(Sender: TObject);
    procedure ClearLogButtonClick(Sender: TObject);
    procedure ClearTextButtonClick(Sender: TObject);
    procedure ClipButton10Click(Sender: TObject);
    procedure ClipButton14Click(Sender: TObject);
    procedure ClipButton11Click(Sender: TObject);
    procedure ClipButton12Click(Sender: TObject);
    procedure ClipButton15Click(Sender: TObject);
    procedure ClipButton16Click(Sender: TObject);
    procedure ClipButton2Click(Sender: TObject);
    procedure ClipButton3Click(Sender: TObject);
    procedure ClipButton4Click(Sender: TObject);
    procedure ClipButton5Click(Sender: TObject);
    procedure ClipButton6Click(Sender: TObject);
    procedure ClipButton7Click(Sender: TObject);
    procedure ClipButton8Click(Sender: TObject);
    procedure ClipButton9Click(Sender: TObject);
    procedure ClipButton13Click(Sender: TObject);
    procedure CommandboxChange(Sender: TObject);
    procedure CommandGroupSelectionChanged(Sender: TObject);
    procedure ConfirmRefreshButtonClick(Sender: TObject);
    procedure ConnspeedLogYChange(Sender: TObject);
    procedure CRCClipChange(Sender: TObject);
    procedure CRCLogYChange(Sender: TObject);
    procedure CustomBoxChange(Sender: TObject);
    procedure DailyBoxChange(Sender: TObject);
    procedure DailyButtonChange(Sender: TObject);
    procedure DayEditChange(Sender: TObject);
    procedure DaysAgoEditChange(Sender: TObject);
    procedure DisableSpecialChange(Sender: TObject);
    procedure DisplayHereButtonChange(Sender: TObject);
    procedure DoBitloadingChange(Sender: TObject);
    procedure DoBitScroll(Sender: TObject; Down: Boolean);
    procedure DobitswapChange(Sender: TObject);
    procedure DoLinechartScroll(Sender: TObject; Down: Boolean);
    procedure DoMarginChange(Sender: TObject);
    procedure DonothingButtonClick(Sender: TObject);
    procedure DontAskButtonChange(Sender: TObject);
    procedure DoSnapshots(Sender: TObject; IsWebserver: Boolean);
    procedure DoSNRMpbchartScroll(Sender: TObject; Down: Boolean);
    procedure DoSNRpertoneChange(Sender: TObject);
    procedure DoToneScroll(Sender: TObject; Down: Boolean);
    procedure DownloadButtonClick(Sender: TObject);
    procedure DownonlyButtonChange(Sender: TObject);
    procedure DownstreamBoxCChange(Sender: TObject);
    procedure DownstreamBoxFChange(Sender: TObject);
    procedure DownstreamBoxLEFTRSChange(Sender: TObject);
    procedure DownstreamBoxMChange(Sender: TObject);
    procedure DownstreamBoxOChange(Sender: TObject);
    procedure DownstreamBoxRChange(Sender: TObject);
    procedure DownstreamBoxrtxtxChange(Sender: TObject);
    procedure DownstreamBoxrtxcChange(Sender: TObject);
    procedure DownstreamBoxrtxucChange(Sender: TObject);
    procedure DownstreamBoxSESChange(Sender: TObject);
    procedure DownstreamBoxTChange(Sender: TObject);
    procedure ErrorResetButton1Click(Sender: TObject);
    procedure ESAutoscaleChange(Sender: TObject);
    procedure ESMaxChange(Sender: TObject);
    procedure EveryChange(Sender: TObject);
    procedure ExitbuttonClick(Sender: TObject);
    procedure ExtendedBoxChange(Sender: TObject);
    procedure FECClipChange(Sender: TObject);
    procedure FECLogYChange(Sender: TObject);
    procedure FEnableClippingBoxChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
    procedure FrequencyBoxChange(Sender: TObject);
    procedure GetIPButtonClick(Sender: TObject);
    procedure GraphcolourChange(Sender: TObject);
    procedure HelpbuttonClick(Sender: TObject);
    procedure InterfaceGroupSelectionChanged(Sender: TObject);
    procedure IPeditKeyPress(Sender: TObject; var Key: char);
    procedure IsHG622BoxChange(Sender: TObject);
    procedure LEFTRSClipChange(Sender: TObject);
    procedure LEFTRSEnableClippingBoxChange(Sender: TObject);
    procedure LEFTRSLogYChange(Sender: TObject);
    procedure LegendLeftChange(Sender: TObject);
    procedure LimitEditChange(Sender: TObject);
    procedure LimitEditKeyPress(Sender: TObject; var Key: char);
    procedure LimitWarningBoxChange(Sender: TObject);
    procedure LoggingButtonClick(Sender: TObject);
    procedure MakeWebserverDir;
    procedure ManualSaveTextPages(Sender: TObject);
    procedure MarginchartMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure MarginchartMouseLeave(Sender: TObject);
    procedure MarginchartMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure MarginchartMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure MarginchartMouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure MarginchartMouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure MarginClipChange(Sender: TObject);
    procedure MaxYEditChange(Sender: TObject);
    procedure MaxYEditKeyPress(Sender: TObject; var Key: char);
    procedure DSstatsChange(Sender: TObject);
    procedure MHideLegendChange(Sender: TObject);
    procedure MinimiseItemClick(Sender: TObject);
    procedure MinimiseoncloseBoxChange(Sender: TObject);
    procedure MonitorTrafficBoxChange(Sender: TObject);
    procedure MsnapbuttonClick(Sender: TObject);
    procedure NormalButtonClick(Sender: TObject);
    procedure OEnableClippingBoxChange(Sender: TObject);
    procedure OHFErrClipChange(Sender: TObject);
    procedure OHFErrLogYChange(Sender: TObject);
    procedure OSAdvisedChange(Sender: TObject);
    procedure PerSampleBoxCChange(Sender: TObject);
    procedure PerSampleBoxFChange(Sender: TObject);
    procedure PerSampleBoxLEFTRSChange(Sender: TObject);
    procedure PerSampleBoxOChange(Sender: TObject);
    procedure PerSampleBoxRChange(Sender: TObject);
    procedure PerSampleBoxrtxtxChange(Sender: TObject);
    procedure PerSampleBoxrtxcChange(Sender: TObject);
    procedure PerSampleBoxrtxucChange(Sender: TObject);
    procedure PerSampleBoxSESChange(Sender: TObject);
    procedure PMClipChange(Sender: TObject);
    procedure PopupTimerTimer(Sender: TObject);
    procedure PorteditChange(Sender: TObject);
    procedure PorteditKeyPress(Sender: TObject; var Key: char);
    procedure QLNchartMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure QLNchartMouseLeave(Sender: TObject);
    procedure QLNchartMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure QLNchartMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure REnableClippingBoxChange(Sender: TObject);
    procedure ResetButtonClick(Sender: TObject);
    procedure ResetDayButtonClick(Sender: TObject);
    procedure ResetLoginButtonClick(Sender: TObject);
    procedure ResetMonthButtonClick(Sender: TObject);
    procedure ResetResyncButtonClick(Sender: TObject);
    procedure ResetTodayButtonClick(Sender: TObject);
    procedure RestoreButtonClick(Sender: TObject);
    procedure RestoreGraphButtonClick(Sender: TObject);
    procedure RevealBoxChange(Sender: TObject);
    procedure RevealBoxSChange(Sender: TObject);
    procedure RevealEPBoxChange(Sender: TObject);
    procedure RouterBoxChange(Sender: TObject);
    procedure RouterTrafficBoxChange(Sender: TObject);
    procedure RSCorrClipChange(Sender: TObject);
    procedure RSCorrLogYChange(Sender: TObject);
    procedure rtxtxClipChange(Sender: TObject);
    procedure rtxtxEnableClippingBoxChange(Sender: TObject);
    procedure rtxtxLogYChange(Sender: TObject);
    procedure rtxcClipChange(Sender: TObject);
    procedure rtxcEnableClippingBoxChange(Sender: TObject);
    procedure rtxcLogYChange(Sender: TObject);
    procedure rtxucClipChange(Sender: TObject);
    procedure rtxucEnableClippingBoxChange(Sender: TObject);
    procedure rtxucLogYChange(Sender: TObject);
    procedure SameAsBoxChange(Sender: TObject);
    procedure SampleRateEditChange(Sender: TObject);
    procedure SaveTextButtonClick(Sender: TObject);
    procedure SCommandBoxChange(Sender: TObject);
    procedure SecondstageChange(Sender: TObject);
    procedure SelectXMLButtonClick(Sender: TObject);
    procedure SendNowButtonClick(Sender: TObject);
    procedure SESClipChange(Sender: TObject);
    procedure SESEnableClippingBoxChange(Sender: TObject);
    procedure SESLogYChange(Sender: TObject);
    procedure ShowBillingBoxChange(Sender: TObject);
    procedure ShowHintsBoxChange(Sender: TObject);
    procedure ShowMaxDownBoxChange(Sender: TObject);
    procedure ShowMaxUpBoxChange(Sender: TObject);
    procedure SMTPPortKeyPress(Sender: TObject; var Key: char);
    procedure SnapallBoxChange(Sender: TObject);
    procedure SnapallButtonClick(Sender: TObject);
    procedure SnapDirEditChange(Sender: TObject);
    procedure SnapshotHoursChange(Sender: TObject);
    procedure SNRchartMouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure SNRchartMouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure SNRMbarChange(Sender: TObject);
    procedure SNRMpbChartupMouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure SNRMpbChartupMouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure SNRMTriggerKeyPress(Sender: TObject; var Key: char);
    procedure SNRseparateChange(Sender: TObject);
    procedure SNRsnapChange(Sender: TObject);
    procedure SNRTabMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SNRwithbitloadingChange(Sender: TObject);
    procedure SpecialLoginChange(Sender: TObject);
    procedure FontButtonClick(Sender: TObject);
    procedure SPortEditChange(Sender: TObject);
    procedure SRouterBoxChange(Sender: TObject);
    procedure StartButtonClick(Sender: TObject);
    procedure StartMaximisedBoxChange(Sender: TObject);
    procedure StartminimisedBoxChange(Sender: TObject);
    procedure StartofRecordingChange(Sender: TObject);
    procedure StopbuttonClick(Sender: TObject);
    procedure SynchroniseChange(Sender: TObject);
    procedure TestButtonClick(Sender: TObject);
    procedure ThicknessEditChange(Sender: TObject);
    procedure ThomBoxChange(Sender: TObject);
    procedure TimeperpageEditChange(Sender: TObject);
    procedure TimepersnapshotEditChange(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure Timer2Timer(Sender: TObject);
    procedure Timer3Timer(Sender: TObject);
    procedure Timer4Timer(Sender: TObject);
    procedure Timer5Timer(Sender: TObject);
    procedure Timer6Timer(Sender: TObject);
    procedure Timer7Timer(Sender: TObject);
    procedure Timer8Timer(Sender: TObject);
    procedure Timer9Timer(Sender: TObject);
    procedure Tonesperpx(Value: Smallint);
    procedure TrafficChartMouseLeave(Sender: TObject);
    procedure TrafficChartMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure TrayIcon1Click(Sender: TObject);
    procedure TUpBoxChange(Sender: TObject);
    procedure U0Change(Sender: TObject);
    procedure DSUploadURLChange(Sender: TObject);
    procedure UpstreamBoxCChange(Sender: TObject);
    procedure UpstreamBoxFChange(Sender: TObject);
    procedure UpstreamBoxLEFTRSChange(Sender: TObject);
    procedure UpstreamBoxMChange(Sender: TObject);
    procedure UpstreamBoxOChange(Sender: TObject);
    procedure UpstreamBoxRChange(Sender: TObject);
    procedure UpstreamBoxrtxtxChange(Sender: TObject);
    procedure UpstreamBoxrtxcChange(Sender: TObject);
    procedure UpstreamBoxrtxucChange(Sender: TObject);
    procedure UpstreamBoxSESChange(Sender: TObject);
    procedure UpstreamBoxTChange(Sender: TObject);
    procedure WebserverDirEditChange(Sender: TObject);
    procedure WebPortEditKeyPress(Sender: TObject; var Key: char);
    procedure WebserverBoxChange(Sender: TObject);
    procedure WebserverSaveBoxChange(Sender: TObject);
    procedure DSCurlLocationChange(Sender: TObject);
    procedure WSFTPPortKeyPress(Sender: TObject; var Key: char);
    procedure SSRevealFTPpasswordChange(Sender: TObject);
    procedure WSRevealFTPpasswordChange(Sender: TObject);
    procedure SSTestUploadButtonClick(Sender: TObject);
    procedure WSTestUploadButtonClick(Sender: TObject);
    procedure WSUploadDirKeyPress(Sender: TObject; var Key: char);
  private
    { private declarations }
    {$IFDEF Windows}
    function CheckModemStats: Boolean;
    {$ENDIF}
    function CheckVersion(DoNow: Boolean): TVersionResult;
    function CurlUploadFiles(Source: TUploadSource; LocalDir, RemoteURL, RemoteDir, Username, Password, Port: string; FTPFiles: TStringList): Boolean;
    function ConvertToLabel(Value: Int64): string;
    function Decrypt(Value: string): string;
    function DeleteFiles(Dir: string): Boolean;
    function Encrypt(Value: string): string;
    function GetTriggerFP(AlertType: TAlertType; Value: string): Extended;
    function LoadXML(Graph: TLinechart; Filename: string): Boolean;
    {$IFDEF Windows}
    function MoveConfigFiles(FromDir, ToDir: string): Boolean;
    function ProcessExists(Filename: string; var Num: Word): Boolean;
    {$ENDIF}
    function ProcessOperation(Sender: Tobject; TelnetOp: TTelnetOperation): TLogEvent;
    function RunProcess(Proc: TProcess; out OutputString, StderrString: string; out ExitStatus: Integer): Integer;
    function Testwrite(Dir: string): Boolean;
    function Timestring(Secs: Longword): string;
    procedure AdjustErrorArray1;
    procedure CalculateUsage;
    procedure CheckConfig;
    procedure DeleteOlder;
    procedure DeleteWebserverDir(Path: string);
    procedure DisplayErrors1;
    procedure DoAdvancedTweak(TweakOK: Boolean);
    procedure Dobandline(Dataline: string; Isdownstream: Boolean);
    procedure DoGraphFonts;
    procedure DoTextFonts;
    procedure DoHandleRequest(Sender: TObject; var ARequest: TFPHTTPConnectionRequest; var AResponse: TFPHTTPConnectionResponse);
    procedure DoSnapTimes;
    procedure DoSynchronise;
    procedure DoThickness;
    procedure DownloadYes;
    procedure ExtractBands;
    procedure Extractnumbers(Dataline: string; Pathline: Boolean);
    procedure FillGap;
    procedure FillInStats;
    procedure GetIPAddress(Manual: Boolean);
    procedure LogLAttenuation(Up, Down: string);
    procedure LogSAttenuation(Up, Down: string);
    procedure ModeChange;
    procedure PopulateWebserver;
    procedure PrefixSuffix(var Prefix, Suffix, Snapdir: string; Extension, Category: string);
    procedure ProcessTrafficValues;
    procedure ReadErrorFile1;
    procedure ReadInifile(Pathname: string);
    procedure ReadTrafficData;
    procedure ResetErrorElement(Element: Byte);
    procedure ResetErrors1;
    procedure ResetTriggers;
    procedure ResetValues;
    //procedure SaveTextPages;
    procedure SaveXML(Chart: TLinechart; XMLName, Filename: string);
    procedure SendAlert(AlertType: TAlertType; Up: Boolean; Value: Extended);
    procedure Setcolours;
    procedure SetOptions;
    procedure SetSnapSize;
    procedure SetTabs;
    procedure SetTrafficLimits;
    procedure SetupWebserver;
    procedure StartPulsing;
    procedure StopPulsing;
    procedure TidypbParams;
    procedure TransferCommands;
    procedure TransferLoginValues;
    procedure WriteErrorFile1;
    procedure WriteInifile;
    procedure WriteTrafficData;
  public
    { public declarations }
    procedure Gototime(ATime: TDateTime);
    procedure GototimePB(ATime: TDateTime);
    procedure LogEvent(LE: TLogEvent);
  public
    HomePath, Inipath, LC, WebserverPath, DataStorePath, SnapshotsPath, Modestring, MiscString,
       Vendor, Version, GraphType, IPAddress, DeletingDir, AttenFile, IsRunning,
       CommandPrefix, SCommandPrefix, Number1String, Number2String, Mp, Profile, PreviousLAttLine,
       PreviousSAttLine, FontName, TextFontName, IconColour, IconColour1, IconColour2: string;
    Samplerate, LastIndex, LogNum, StartX, BitstartX, ToneRange,
       TweakValue, SecondsPassed, T3seconds, ErrorNumber, EveryMinutes, StartupLeft, StartupTop,
       StartupHeight, StartupWidth, RouterUptime, Timeouts: Integer;
    PreviousAS: Longword;
    Hoursperpage, Minutesperpage, Hourspersnapshot, Minutespersnapshot, CurrentDay, CurrentMonth,
       CurrentYear, TrafficConfigCount, CheckCount, WebPort, RefreshPeriod, ResyncCount,
       NoLoginCount, Tries, DSUploadFailed, WSUploadFailed, SSUploadFailed: Word;
    Unconnected, Bitscrolling, Swapscrolling, QScrolling, HScrolling, SNRscrolling,
       Marginscrolling, Auto, Modefound, Stopped, VendorWritten,
       Collecting, DelayedClose, DelayedStop, DelayedPause, Configured, BitsSaved, DoQuit,
       TestOK, Encrypted, WasStopped, DoCustomNow, WasRecording, VDSL2, DoLogging,
       TriggerSNRMDone, TriggerSNRMreturnDone,  TriggerCRCDone, TriggerFECDone, TriggerESDone, QGraphDone,
       HGraphDone, U0notused, WebserverRunning, Starting, FirstSetup, SNRMAlertDone, SNRMReturnDone,
       UpstreamIsLower, IsRalink, HG635messagedone, SyncOptionChanged, ErrorOccurred,
       WSBitsOK, WSSNROK, WSQLNOK, WSHLogOK, FirstSample, JustStarted, GdotInp,
       SendEmailAlerts, Terminated, EmailResync, DoZips: Boolean;
    PreviousSpeed, Oldupswaps, Olddownswaps, RSup, RSdown: Single;
    DailyCRCUp, DailyCRCDown, DailyFECUp, DailyFECDown, DailyHECUp, DailyHECDown, DailyESUp, DailyESDown,
       DailySESUp, DailySESDown: Double;
    Number1, Number2, TempNum, TriggerSNRM, TriggerSNRMreturn, TriggerCRC, TriggerFEC, TriggerES: Extended;
    UpTraffic, DownTraffic, DailyUpValue, DailyDownValue, MonthlyUpValue, MonthlyDownValue,
       ReportedUpTraffic, ReportedDownTraffic, FECup, FECdown, CRCup, CRCdown, HECup, HECdown,
       OHFErr0up, OHFErr0down, OHFErr1up, OHFErr1down, OldESUp, OldESDown, NewESUp, NewESDown, ESup, ESdown, SESup, SESdown,
       RSCorr0up, RSCorr0down, RSCorr1up, RSCorr1down, RSUnCorr0Up, RSUnCorr0Down, RSUnCorr1Up, RSUnCorr1Down,
       RxQ0Up, RxQ0Down, TxQ0Up, TxQ0Down, Framing0Up, Framing0Down, Lookback0Up, Lookback0Down, RRC0Up, RRC0Down,
       rtxtxUp, rtxtxDown, rtxcUp, rtxcDown, rtxucUp, rtxucDown, LEFTRSUp, LEFTRSDown, minEFTRUp, minEFTRDown,
       OHFErrUp, OHFErrDown, RSCorrUp, RSCorrDown, errFreeBitsUp, errFreeBitsDown, LOSup, LOSdown, LOFup, LOFdown,
       LOMup, LOMdown: Int64;
    LastYear, LastMonth, LastDay, LastHour: Word;
    StartTime, LastSnapshot, OldT, PauseTime, LastSample, LastUpdateCheck, LastCustom, LastDelete, DeleteAge,
       LastTrafficSave, LastESSave, SampleTime, ResyncBaseTime, CurrentErrorDay, SnapshotDate,
       LastTimeCheck, LastZipTime, TNow, LastUploadWS: TDateTime;
    TonesperPixel: SmallInt;
    VendorList, LinediagsList, UpdateList, VersionList, UploadList: TStringList;
    StatsArray: array[0..2, 0..19] of string;
    ToneBands: array[0..1, 0..7] of Integer;
    BandsReported: array[0..1, 0..4] of Boolean;
    VDSLAtt, SNRMpb: array[1..8] of string;
    DSstats: array[1..150] of string;
    SNRMrecord: TDataPoint;
    SNRMpbrecord: TpbDataPoint;
    UtilityOp: TUtilityOp;
    DSeparator: Char;
    {$IFDEF CpuArm}
    OldBits: array[0..4095] of Byte;
    {$ELSE}
    OldBits: array[0..8191] of Byte;
    {$ENDIF}
    DataArray: array[0..100] of TRouter;
    TrafficConfigArray: array[0..50] of TTrafficConfig;
    Datafile: file of TRouter;
    SpecialDataArray: array[0..20] of TSpecialRouter;
    SpecialDatafile: file of TSpecialRouter;
    DataDir: string;
    NumRecords, NumSpecials, CurrentRecord: Integer;
    Prompts, Responses: array[1..10] of TEdit;
    GridStyle: TTextStyle;
    ErrorArray1: array[0..10] of TErrors1;
    Commands: array[1..4] of TEdit;
    CommandNumbers: array[1..4, 1..6] of TCheckBox;
    XMLSave: TXMLSave;
    RouterTrafficList: TStringList;
    UpRecord, DownRecord: TBandPoint;
    FHandler: TFPCustomFileModule;
    FServerThread: THTTPServerThread;
    AProcess, FTPProcess: TProcess;
    GDotInpStrings: array[1..22] of string;
    SnapshotTimes: array[1..24] of TDateTime;
    SnapshotsDone: array[1..24] of Boolean;
  end;

var
  mainform: Tmainform;

implementation

{$R *.lfm}
{$R dslstatsicons.res}

{ Tmainform }
const
  FilesURL = 'http://dslstats.me.uk/files/';
  {$IFDEF CpuArm}
  LastToneNumber = 4095;
  {$ELSE}
  LastToneNumber = 8191;
  {$ENDIF}
  CommandList: array[0..12] of string = ('adsl', 'adslcfg','adslcmd', 'adslctl', 'adslphxcmd', 'xdslcfg',
      'xdslcmd', 'xdslctl', 'xdsl', 'xdslctl0', 'xdslctl1', 'Custom', 'Not configured');
  DefaultIndex = 12;  //Corresponds with last item of CommandList array
  ESMaxList: array[0..8] of string = ('10', '20', '40', '100', '200', '400', '1000', '2000', '4000');
  NumDSstats = 141;  //Number of used items in DSstats
  GDotInpTitles: array[1..22] of string = ('General', 'rtx_tx          ', 'rtx_c           ',
       'rtx_uc          ', 'LEFTRS          ', 'minEFTR         ', 'errFreeBits     ', 'Bearer 0',
       'RxQueue         ', 'TxQueue         ', 'G.INP Framing   ', 'G.INP Lookback  ', 'RRC Bits        ',
       'Interleave depth', 'INP             ', 'INPRein         ', 'Delay           ',
       'Bearer 1', 'Interleave depth', 'INP             ', 'INPRein         ', 'Delay           ');

  // **** INITIALISATION ****

procedure Tmainform.FormCreate(Sender: TObject);
var
  N, N1: Integer;
  S, RouterLine, TempPath: string;
  Info: TVersionInfo;
  TrafficConfig: TTrafficConfig;
  AYear, AMonth, ADay, AHour, AMinute, ASecond, AMillisecond: Word;
begin
  DecodeDateTime(Now, AYear, AMonth, ADay, AHour, AMinute, ASecond, AMillisecond);
  DSeparator := DefaultFormatSettings.DecimalSeparator;
  DoQuit := False;
  Collecting := False;
  Unconnected := True;
  Timer1.Enabled := False;
  BitsSaved := False;
  TestOK := False;
  Stopped := True;
  WasStopped := True;
  WasRecording := False;
  DoCustomNow := False;
  WebserverPath := '';
  WebserverRunning := False;
  DataStorePath := '';
  SnapshotsPath := '';
  DoZips := True;
  LastZipTime := 0;
  LastUploadWS := 0;
  IsRalink := False;
  HG635messagedone := False;
  SyncOptionChanged := False;
  PreviousSpeed := 0;
  TNow := Now;
  OldT := Now;
  LastCustom := Now;
  LastDelete := 0;
  LastTrafficSave := Now;
  LastESSave := Now;
  ToneRange := 255;
  CRCup := 0;
  CRCdown := 0;
  ESup := 0;
  ESdown := 0;
  SESup := 0;
  SESdown := 0;
  FECup := 0;
  FECdown := 0;
  HECup := 0;
  HECdown := 0;
  OHFErr0Up := 0;
  OHFErr0Down := 0;
  OHFErr1Up := 0;
  OHFErr1Down := 0;
  RSCorr0Up := 0;
  RSCorr0Down := 0;
  RSUnCorr0Up := 0;
  RSUnCorr0Down := 0;
  RSCorr1Up := 0;
  RSCorr1Down := 0;
  RSUnCorr1Up := 0;
  RSUnCorr1Down := 0;
  Oldupswaps := 0;
  Olddownswaps := 0;
  LOSup := 0;
  LOSdown := 0;
  LOFup := 0;
  LOFdown := 0;
  LOMup := 0;
  LOMdown := 0;
  CurrentErrorDay := StartOfTheDay(Now);
  ResyncCount := 0;
  ResyncBaseTime := 0;
  PreviousAS := 0;
  Mp := Label153.Caption;
  StatsArray[1, 5] := '0 (since ' + FormatDateTime('dd mmm yyyy hh:nn:ss', ResyncBaseTime) + ')';
  Marginscrolling := False;
  Bitscrolling := False;
  Swapscrolling := False;
  PauseTime := 0;
  LastSample := 0;
  TweakValue := 100;
  CheckCount := 0;
  TriggerSNRMDone := False;
  TriggerSNRMreturnDone := False;
  TriggerCRCDone := False;
  TriggerFECDone := False;
  TriggerESDone := False;
  SNRMAlertDone := False;
  SNRMReturnDone := False;
  QGraphDone := False;
  HGraphDone := False;
  EmailResync := False;
  Tries := 0;
  RouterUptime := -1;
  IconColour1 := 'RED';
  IconColour2 := 'GREEN';
  NoLoginCount := 0;
  Terminated := False;
  DSUploadFailed := 0;
  WSUploadFailed := 0;
  SSUploadFailed := 0;
  Timeouts := 0;
  LastTimeCheck := Now;
  for N := 0 to 8 do ESMax.Items.Add(ESMaxList[N]);    //Populate ESMax combobox
  ESMax.ItemIndex := 0;
  with SNRMrecord do
  begin
    MaxUpTime := 0;
    MinUpTime := 0;
    MaxDownTime := 0;
    MinDownTime := 0;
  end;
  with SNRMpbrecord do
    for N := 0 to 4 do
    begin
      MaxUTimes[N] := 0;
      MinUTimes[N] := 0;
      MaxDTimes[N] := 0;
      MinDTimes[N] := 0;
    end;
  with XMLSave do
  begin
    SNRM := False;
    ConSpeed := False;
    CRC := False;
    FEC := False;
    Bitswaps := False;
  end;
  for N := 0 to DefaultIndex do
  begin
    CommandBox.Items.Add(CommandList[N]);
    SCommandBox.Items.Add(CommandList[N]);
  end;
  for N := 1 to 10 do  //Create edit boxes for special login page
  begin
    Prompts[N] := TEdit.Create(Self);
    Prompts[N].Parent := SpecialTab;
    Prompts[N].Left := 44;
    Prompts[N].Top := 72 + (26 * N);
    Prompts[N].Width := 100;
    Responses[N] := TEdit.Create(Self);
    Responses[N].Parent := SpecialTab;
    Responses[N].Left := 160;
    Responses[N].Top := 72 + (26 * N);
    Responses[N].Width := 180;
  end;
  Responses[2].EchoMode := emPassword;
  for N := 1 to 4 do  //Create edit boxes and checkboxes for custom commands
  begin
    Commands[N] := TEdit.Create(Self);
    Commands[N].Parent := CustomTab;
    Commands[N].Left := 40;
    Commands[N].Top := 32 + (30 * N);
    Commands[N].Width := 256;
    for N1 := 1 to 6 do
    begin
      CommandNumbers[N, N1] := TCheckBox.Create(Self);
      CommandNumbers[N, N1].Parent := CustomTab;
      CommandNumbers[N, N1].Left := 20 + 44 * (N - 1);
      if N1 = 6 then CommandNumbers[N, N1].Top := 375
      else CommandNumbers[N, N1].Top := 214 + 26 * (N1 - 1);
      CommandNumbers[N, N1].Width := 40;
      CommandNumbers[N, N1].Caption := IntToStr(N);
    end;
  end;

  for N := 0 to 10 do          //Make sure that error array is cleared initially
  begin
    ErrorArray1[N].CRCdown := 0;
    ErrorArray1[N].CRCup := 0;
    ErrorArray1[N].ESdown := 0;
    ErrorArray1[N].ESup := 0;
    ErrorArray1[N].FECdown := 0;
    ErrorArray1[N].FECup := 0;
    ErrorArray1[N].HECdown := 0;
    ErrorArray1[N].HECup := 0;
    ErrorArray1[N].SESdown := 0;
    ErrorArray1[N].SESup := 0;
    ErrorArray1[N].TD := 0;
  end;

  for N := 0 to 4 do           //Initialise UpRecord and DownRecord
  begin
    UpRecord.YValues[N] := -255;
    DownRecord.YValues[N] := -255;
  end;
  VendorList := TStringList.Create;
  LinediagsList := TStringList.Create;
  for N := 0 to LastToneNumber do OldBits[N] := 0;
  for N := 1 to 150 do DSstats[N] := '0';
  UpdateList := TStringList.Create;
  VersionList := TStringList.Create;
  UploadList := TStringList.Create;

  //Load configuration data for login and special login
  DataDir := IncludeTrailingPathDelimiter(Application.Location);
  if FileExists(DataDir + 'specialrouters.dat') then
  begin
    AssignFile(SpecialDatafile, DataDir + 'specialrouters.dat');
    try
      Reset(SpecialDatafile);
      SRouterBox.Items.Add('Select modem/router model');
      while not EOF(SpecialDatafile) do
      begin
        Inc(NumSpecials);
        Read(SpecialDatafile, SpecialDataArray[NumSpecials]);
        SRouterBox.Items.Add(SpecialDataArray[NumSpecials].Manufacturer + ' ' + SpecialDataArray[NumSpecials].Model);
      end;
      SRouterBox.Items.Add('-----------------');
      SRouterBox.ItemIndex := 0;
      CloseFile(SpecialDatafile);
    except
      on E: Exception do
      begin
        NumSpecials := 0;
        ShowMessage('Unable to read data from file "specialrouters.dat"');
      end;
    end;
  end;
  if FileExists(DataDir + 'routers.dat') then
  begin
    AssignFile(DataFile, DataDir + 'routers.dat');
    try
      Reset(DataFile);
      RouterBox.Items.Add('Select modem/router model');
      while not EoF(DataFile) do
      begin
        Inc(NumRecords);
        Read(DataFile, DataArray[NumRecords]);
        RouterBox.Items.Add(DataArray[NumRecords].Manufacturer + ' ' + DataArray[NumRecords].Model);
        SRouterBox.Items.Add(DataArray[NumRecords].Manufacturer + ' ' + DataArray[NumRecords].Model);
      end;
      RouterBox.ItemIndex := 0;
      SRouterBox.ItemIndex := 0;
      CloseFile(DataFile);
    except
      on E: Exception do
      begin
        NumRecords := 0;
        ShowMessage('Unable to read data from file "routers.dat"');
      end;
    end;
  end;

  //Load configuration data for modem/router traffic monitoring
  RouterTrafficList := TStringList.Create;
  TrafficConfigCount := 0;
  if FileExists(DataDir + 'routertraffic.csv') then
  try
    RouterTrafficList.LoadFromFile(DataDir + 'routertraffic.csv');
    TrafficConfigCount := RouterTrafficList.Count;
    if TrafficConfigCount > 1 then
      for N := 1 to TrafficConfigCount - 1 do     //Ignore first line, which has column headings
      begin
        RouterLine := RouterTrafficList[N];
        S := '';
        N1 := 1;
        while not (RouterLine[N1] = ',') do    //Get name
        begin
          S := S + RouterLine[N1];
          Inc(N1);
        end;
        TrafficConfig.Name := S;
        Inc(N1);
        S := '';
        while not (RouterLine[N1] = ',') do      //Get command
        begin
          S := S + RouterLine[N1];
          Inc(N1);
        end;
        TrafficConfig.Command := S;
        Inc(N1);
        S := '';
        while not (RouterLine[N1] = ',') do       //Get interface
        begin
          S := S + RouterLine[N1];
          Inc(N1);
        end;
        TrafficConfig.IFace := S;
        Inc(N1);
        S := '';
        while not (RouterLine[N1] = ',') do       //Get 'other' interface
        begin
          S := S + RouterLine[N1];
          Inc(N1);
        end;
        TrafficConfig.Other := S;
        Inc(N1);
        S := '';
        while N1 <= Length(RouterLine) do      //Get reverse flag
        begin
          S := S + RouterLine[N1];
          Inc(N1);
        end;
        TrafficConfig.Reversed := S;
        TrafficConfigArray[N - 1] := TrafficConfig;
      end;
    TrafficConfigCount := N - 1;
  except
    on E: Exception do
    begin
      TrafficConfigCount := 0;
      MiscString := 'File "routertraffic.csv" may be corrupted';
      LogEvent(leMisc);
    end;
  end
  else begin
    TrafficConfig.Name := 'Custom';
    TrafficConfig.Command := '';
    TrafficConfig.IFace := '';
    TrafficConfig.Other := '';
    TrafficConfig.Reversed := '';
    TrafficConfigArray[0] := TrafficConfig;
  end;
  try
    for N := 0 to TrafficConfigCount do
      RouterTrafficBox.Items.Add(TrafficConfigArray[N].Name);
    RouterTrafficBox.ItemIndex := 0;
  except
    on E: Exception do;  //Do nothing
  end;

  VDSL2 := False;
  for N := 1 to 8 do
  begin
    VDSLAtt[N] := '0';
    SNRMpb[N] := '';
  end;

  //Copy two files for use in FTP test upload
  Fileutil.CopyFile(DataDir + 'crc.html', DataDir + 'FTPtest1');
  Fileutil.CopyFile(DataDir + 'fec.html', DataDir + 'FTPtest2');

  //Set up basic program configuration
  {$IFDEF Windows}
  HomePath := GetAppConfigDir(False);            //Always includes trailing path delimiter
  if FileExists(HomePath + 'dslstats.ini') then  //Config files are in OS advised location
  begin
    if FileExists(DataDir + 'dslstats.ini') then  //Check for duplicate in DataDir, and delete if found
    try
      SysUtils.DeleteFile(DataDir + 'dslstats.ini');
    except
      on E: Exception do
      begin
        MiscString := 'Two copies of dslstats.ini exist. Using the one in the OS advised location';
        LogEvent(leMisc);
      end;
    end;
  end
  else begin
    if FileExists(DataDir + 'dslstats.ini') then HomePath := DataDir
    else begin
      if not ForceDirectories(HomePath) then
      begin
        HomePath := DataDir;
        MiscString := 'Could not create directory "' + HomePath + '"';
        LogEvent(leMisc);
      end;
    end;
  end;
  if HomePath = DataDir then WithOther.Checked := True
  else OSAdvised.Checked := True;
  HomePath := IncludeTrailingPathDelimiter(HomePath);
  DSCurlLabel.Enabled := False;
  DSCurlLocation.Enabled := False;
  WSCurlLabel.Enabled := False;
  WSCurlLocation.Enabled := False;
  SSCurlLabel.Enabled := False;
  SSCurlLocation.Enabled := False;
  {$ELSE}
  HomePath := GetEnvironmentVariable('HOME');
  if HomePath = '' then
  begin
    HomePath := GetAppConfigDir(False);
    if not ForceDirectories(HomePath) then
    begin
      TempPath := GetTempDir(True);
      ShowMessage('Unable to create ' + HomePath + '. Using ' + TempPath + ' instead');
      HomePath := TempPath;
    end
    else FpChmod (HomePath, &777);
  end;
  HomePath := IncludeTrailingPathDelimiter(HomePath) + '.dslstats/';
  if not DirectoryExists(HomePath) then
  try
    if not ForceDirectories(HomePath) then
    begin
      ShowMessage('Unable to create directory ' + Homepath);
    end;
    fpChmod(Homepath, &775);
  except
    on E: Exception do ShowMessage('Unable to create writeable directory ' + Homepath);
  end;
  LocationLabel.Enabled := False;
  OSAdvised.Enabled := False;
  WithOther.Enabled := False;
  {$ENDIF}

  Synchronise.Enabled := True;

  DataStorePath := IncludeTrailingPathDelimiter(HomePath + 'DataStore');

  if FileExists(HomePath + 'traffic.ini') then ReadTrafficData
  else begin
    ResetMonthButtonClick(Self);
    if AMonth = 12 then
    begin
      N := 1;
      TrafficChart.FirstYear := AYear;
    end
    else begin
      N := AMonth + 1;
      TrafficChart.FirstYear := AYear - 1;
    end;
    TrafficChart.FirstMonth := TMonths(N - 1);
  end;
  MiscString := 'Configuration files stored in ' + HomePath;
  LogEvent(leMisc);

  Inipath := HomePath + 'dslstats.ini';
  Starting := True;     //Don't process WebserverBoxChange procedure before the webserver is set up
  LC := Label70.Caption;
  ReadInifile(IniPath);
  Starting := False;
  FirstSetup := True;
  if WebserverBox.Checked then WebserverBoxChange(Self);
  if WebserverSaveBox.Checked and not WebserverBox.Checked then WebserverSaveBoxChange(Self);

  if FileExists(Homepath + 'es_data.txt') then
  try
    ESChart.LoadHourlyES(Homepath + 'es_data.txt');
  except
    on E: Exception do
    begin
      MiscString := 'File "' + HomePath + 'es_data.txt" may be corrupted. Unable to read.';
      LogEvent(leMisc);
    end;
  end
  else if FileExists(Homepath + 'es.data') then
  try
    ESChart.LoadHourlyES(Homepath + 'es.data');
  except
    on E: Exception do
    begin
      MiscString := 'File "' + HomePath + 'es.data" may be corrupted. Unable to read.';
      LogEvent(leMisc);
    end;
  end;
  RSup := 0;
  RSdown := 0;
  SetTabs;
  SetOptions;
  StartTime := Now;
  LastSnapshot := StartTime;
  BitswapChart.StartTime := 0;
  Stopped := True;
  StatsArray[0, 0] := 'DSLAM type / SW version:';
  StatsArray[0, 1] := 'Modem/router firmware:  ';
  StatsArray[0, 2] := 'DSL mode:               ';
  StatsArray[0, 3] := 'Status:                 ';
  StatsArray[0, 4] := 'Uptime:                 ';
  StatsArray[0, 5] := 'Resyncs:                ';
  StatsArray[1, 7] := 'Downstream';
  StatsArray[2, 7] := 'Upstream';
  StatsArray[0, 8] := 'Line attenuation (dB):  ';
  StatsArray[0, 9] := 'Signal attenuation (dB):';
  StatsArray[0, 10] := 'Connection speed (kbps):';
  StatsArray[0, 11] := 'SNR margin (dB):        ';
  StatsArray[0, 12] := 'Power (dBm):            ';
  StatsArray[0, 13] := 'Interleave depth:       ';
  StatsArray[0, 14] := 'INP:                    ';
  StatsArray[0, 15] := 'G.INP:                  ';
  StatsArray[1, 15] := 'Not enabled';
  StatsArray[2, 15] := '';
  StatsArray[0, 16] := 'Vectoring status:       ';
  StatsArray[0, 17] := 'RSCorr/RS (%):          ';
  StatsArray[0, 18] := 'RSUnCorr/RS (%):        ';
  StatsArray[0, 19] := 'ES/hour:                ';
  Info := TVersionInfo.Create;
  Info.Load(HINSTANCE);
  Version := IntToStr(Info.FixedInfo.FileVersion[0]) + '.' + IntToStr(Info.FixedInfo.FileVersion[1]);
  if Info.FixedInfo.FileVersion[2] > 0 then Version := Version + '.' + IntToStr(Info.FixedInfo.FileVersion[2]);
  Info.Free;
  S := 'DSLstats v' + Version;
  Self.Caption := S;
  MarginChart.VersionString := S;
  Speedchart1.VersionString := S;
  SwapsperminChart.VersionString := S;
  CRCChart.VersionString := S;
  FECChart.VersionString := S;
  SESChart.VersionString := S;
  BitswapChart.VersionString := S;
  Bitchart1.VersionString := S;
  SNRChart.VersionString := S;
  QLNChart.VersionString := S;
  HLogChart.VersionString := S;
  TrafficChart.VersionString := S;
  TrafficperminChart.VersionString := S;
  ESChart.VersionString := S;
  SNRMpbChartup.VersionString := S;
  SNRMpbChartdown.VersionString := S;
  CheckConfig;
  if DaysBetween(Now, LastUpdateCheck) >= FrequencyBox.Value then CheckVersion(False);
  if ParamCount > 0 then
  begin
    for N := 1 to ParamCount do
    begin
      S := LowerCase(ParamStr(N));
      if (S = 'minimised') or (S = 'minimized') then
      begin
        if PageControl1.ActivePage = UpgradeTab then Break;  //Don't minimise if an upgrade has been detected
        WindowState := wsMinimized;
        Break;
      end
      else if (S = 'maximised') or (S = 'maximized') then
      begin
        WindowState := wsMaximized;
        Break;
      end;
    end;
  end
  else begin
    if StartMinimisedBox.Checked then
    begin
      if PageControl1.ActivePage = UpgradeTab then Exit;  //Don't minimise if an upgrade has been detected
      WindowState := wsMinimized;
    end
    else if StartMaximisedBox.Checked then WindowState := wsMaximized;
  end;
  {$IFDEF Windows}
  GroupBox2.Height := 150;    //On Windows, group boxes need to be higher
  GroupBox3.Height := 246;
  GroupBox4.Height := 226;
  {$ENDIF}
  LAttenuationLog.Lines.Add(Chr(9) + Chr(9) + Chr(9) + 'Down' + Chr(9) + 'Up');
  LAttenuationLog.Lines.Add('');
  SAttenuationLog.Lines.Add(Chr(9) + Chr(9) + Chr(9) + 'Down' + Chr(9) + 'Up');
  SAttenuationLog.Lines.Add('');
  ESChart.SetESDate(Today);
  ResetErrors1;
  ReadErrorFile1;
  DisplayErrors1;
  SetSnapSize;
end;

//Process change of location of config files (Windows only)
procedure Tmainform.OSAdvisedChange(Sender: TObject);
begin
  {$IFDEF Windows}
  if (HomePath = DataDir) and (OSAdvised.Checked) then
  begin
    HomePath := GetAppConfigDir(False);
    if not MoveConfigFiles(DataDir, HomePath) then
    begin
      HomePath := DataDir;
      WithOther.Checked := True;
    end;
  end
  else if (HomePath <> DataDir) and (WithOther.Checked) then
  begin
    MoveConfigFiles(HomePath, DataDir);
    HomePath := DataDir;
  end;
  Inipath := HomePath + 'dslstats.ini';
  {$ENDIF}
end;

//Move config files from one location to another (Windows only)
{$IFDEF Windows}
function TMainform.MoveConfigFiles(FromDir, ToDir: string): Boolean;
begin
  Result := True;
  if not DirectoryExists(ToDir) then ForceDirectories(ToDir);
  if DirectoryExists(ToDir) then
  begin
    try
      if FileExists(FromDir + 'dslstats.ini') then
      begin
        if Fileutil.CopyFile(FromDir + 'dslstats.ini', ToDir + 'dslstats.ini', True)
        then Sysutils.DeleteFile(FromDir + 'dslstats.ini');
      end;
    except
      on E: Exception do
      begin
        MiscString := 'Unable to move dslstats.ini. Config directory unchanged.';
        LogEvent(leMisc);
        Result := False;
        Exit;
      end;
    end;
    try
      if FileExists(FromDir + 'traffic.ini') then
      begin
        if Fileutil.CopyFile(FromDir + 'traffic.ini', ToDir + 'traffic.ini', True)
        then Sysutils.DeleteFile(FromDir + 'traffic.ini');
      end;
      if FileExists(FromDir + 'traffic.txt') then
      begin
        if Fileutil.CopyFile(FromDir + 'traffic.txt', ToDir + 'traffic.txt', True)
        then Sysutils.DeleteFile(FromDir + 'traffic.txt');
      end;
      if FileExists(FromDir + 'errors1.dat') then
      begin
        if Fileutil.CopyFile(FromDir + 'errors1.dat', ToDir + 'errors1.dat', True) then
        Sysutils.DeleteFile(FromDir + 'errors1.dat');
      end;
      if FileExists(FromDir + 'es_data.txt') then
      begin
        if Fileutil.CopyFile(FromDir + 'es_data.txt', ToDir + 'es_data.txt', True) then
        Sysutils.DeleteFile(FromDir + 'es_data.txt');
      end;
      if FileExists(FromDir + 'es.data') then
      begin
        if Fileutil.CopyFile(FromDir + 'es.data', ToDir + 'es.data', True) then
        Sysutils.DeleteFile(FromDir + 'es.data');
      end;
      MiscString := 'Config files moved from ' + FromDir + ' to ' + ToDir;
      LogEvent(leMisc);
    except
      on E: Exception do
      begin
        MiscString := 'Unable to move some of the config files to new directory.';
        LogEvent(leMisc);
      end;
    end;
  end
  else begin       //Failed to create the new config directory
    MiscString := 'Could not create directory "' + ToDir + '"';
    LogEvent(leMisc);
    Result := False;
  end;
end;
{$ENDIF}

//Check if this is clean start, if so, go to appropriate configuration page
procedure Tmainform.CheckConfig;
begin
  Configured := True;
  if SpecialLogin.Checked then
  begin
    if SIPEdit.Text = '' then
    begin
      Configured := False;
      PageControl1.ActivePage := ConfigurationTab;
      PageControl2.ActivePage := TabSheet1;
      PageControl6.ActivePage := SpecialTab;
    end
    else begin
      PageControl1.ActivePage := SNRTabSheet;
      PageControl10.ActivePage := SNRMTab;
    end;
  end
  else begin
    if IPEdit.Text = '' then
    begin
      Configured := False;
      PageControl1.ActivePage := ConfigurationTab;
      PageControl2.ActivePage := TabSheet1;
      PageControl6.ActivePage := LoginTab;
    end
    else begin
      PageControl1.ActivePage := SNRTabSheet;
      PageControl10.ActivePage := SNRMTab;
    end;
  end;
end;

//Process command line parameter 'startrecording'
procedure Tmainform.Timer5Timer(Sender: TObject);
var
  N: Integer;
  S: string;
begin
  Timer5.Enabled := False;
  if ParamCount > 0 then
  begin
    for N := 1 to ParamCount do
    begin
      S := LowerCase(ParamStr(N));
      if S = 'startrecording' then
      begin
        StartButtonClick(Self);
        Break;
      end;
    end;
  end
  else begin
    if AutostartBox.Checked then StartButtonClick(Self);
  end;
end;

//Disallow simultaneous enable of StartMaximised and StartMinimised
procedure Tmainform.StartMaximisedBoxChange(Sender: TObject);
begin
  if StartMaximisedBox.Checked then StartMinimisedBox.Checked := False;
end;

procedure Tmainform.StartminimisedBoxChange(Sender: TObject);
begin
  if StartMinimisedBox.Checked then StartMaximisedBox.Checked := False;
end;

//Restore size and position to where it was at startup
procedure Tmainform.RestoreButtonClick(Sender: TObject);
begin
  WindowState := wsNormal;
  SetBounds(StartupLeft, StartupTop, StartupWidth, StartupHeight);
end;

//Read saved traffic data
procedure Tmainform.ReadTrafficData;
var
  S: string;
  N: Byte;
  Up, Down: Int64;
  AYear, AMonth, ADay, AHour, AMinute, ASecond, AMillisecond, SavedDay, SavedMonth,
     NewFirstMonth: Word;
  NewDataFile: Boolean;
begin
  DecodeDateTime(Now, AYear, AMonth, ADay, AHour, AMinute, ASecond, AMillisecond);
  with TMemInifile.Create(HomePath + 'traffic.ini') do   //Read saved traffic data
  begin
    S := ReadString('traffic', 'Traffic command', 'ifconfig');
    with CommandGroup do
    begin
      ItemIndex := 0;
      for N := 0 to Items.Count - 1 do
      begin
        if S = Items[N] then
        begin
          ItemIndex := N;
          Break;
        end;
      end;
    end;
    S := ReadString('traffic', 'Traffic interface', 'ppp***');
    with InterfaceGroup do
    begin
      ItemIndex := 0;
      for N := 0 to Items.Count - 1 do
      begin
        if S = Items[N] then
        begin
          ItemIndex := N;
          if S = 'Other...' then
          begin
            OtherInterfaceLabel.Visible := True;
            OtherInterfaceEdit.Visible := True;
          end;
          Break;
        end;
      end;
    end;
    MonitorTrafficBox.Checked := ReadBool('traffic', 'Monitor traffic', False);
    N := ReadInteger('traffic', 'GraphType', 1);
    Trafficpermin.Checked := (N = 0);
    DailyButton.Checked := (N = 1);
    MonthlyButton.Checked := (N = 2);
    OtherInterfaceEdit.Text := ReadString('traffic', 'Other interface', '');
    LimitWarningBox.Checked := ReadBool('traffic', 'Limit warning', False);
    UpPlusDownButton.Checked := not ReadBool('traffic', 'Download only', False);
    LimitEdit.Text := ReadString('traffic', 'Limit', '0');
    DayEdit.Value := ReadInteger('traffic', 'Billing day', 1);
    ShowBillingBox.Checked := ReadBool('traffic', 'Billing data', False);

    UpTraffic := StrToInt64(ReadString('traffic', 'Up', '0'));        //Last values reported by modem/router
    DownTraffic := StrToInt64(ReadString('traffic', 'Down', '0'));

    DailyUpValue := StrToInt64(ReadString('traffic', 'DailyUpValue', '0'));       //Last plotted values
    DailyDownValue := StrToInt64(ReadString('traffic', 'DailyDownValue', '0'));
    MonthlyUpValue := StrToInt64(ReadString('traffic', 'MonthlyUpValue', '0'));
    MonthlyDownValue := StrToInt64(ReadString('traffic', 'MonthlyDownValue', '0'));
    SavedMonth := ReadInteger('traffic', 'First month', AMonth);
    TrafficChart.FirstMonth := TMonths(SavedMonth - 1);
    SavedDay := ReadInteger('traffic', 'Current day', 1);
    if SavedMonth = 1 then CurrentMonth := 12 else CurrentMonth := SavedMonth - 1;
    NewDataFile := ReadBool('traffic', 'NewDataFile', False);

    if NewDataFile then            //No longer using inifile for data
      TrafficChart.LoadTrafficData(HomePath)
    else begin                     //Using inifile for data
      for N := 1 to 12 do          //Restore monthly totals
      begin
        Up := StrToInt64(ReadString('traffic', 'Month' + IntToStr(N) + 'up', '0'));
        Down := StrToInt64(ReadString('traffic', 'Month' + IntToStr(N) + 'down', '0'));
        TrafficChart.SetMonthlyTraffic(N, Up, Down);
      end;

      for N := 1 to 31 do          //Restore daily totals
      begin
        Up := StrToInt64(ReadString('traffic', 'Day' + IntToStr(N) + 'up', '0'));
        Down := StrToInt64(ReadString('traffic', 'Day' + IntToStr(N) + 'down', '0'));
        TrafficChart.SetDailyTraffic(0, N, Up, Down);
        TrafficChart.SetDailyTraffic(1, N, 0, 0);
      end;
    end;
    Free;
  end;

  if CurrentMonth <> AMonth then   //Month has changed since program was closed down
  begin
    if AMonth = 12 then NewFirstMonth := 1
    else NewFirstMonth := AMonth + 1;
    repeat
      TrafficChart.IncrementMonth;
    until TrafficChart.FirstMonth = TMonths(NewFirstMonth - 1);
    TrafficChart.ClearDaily;
    DailyUpValue := 0;
    DailyDownValue := 0;
    MonthlyUpValue := 0;
    MonthlyDownValue := 0
  end
  else if SavedDay <> ADay then     //Same month, new day
  begin
    DailyUpValue := 0;
    DailyDownValue := 0;
  end;
  CurrentDay := ADay;
  if CurrentMonth = 12 then TrafficChart.FirstYear := AYear else TrafficChart.FirstYear := AYear - 1;
  CalculateUsage;
end;

//Read the saved configuration data
procedure Tmainform.ReadInifile(Pathname: string);
var
  SavedWidth, SavedHeight: Integer;
  N, N1, CommandIndex, SCommandIndex, EncryptionType: Word;
  S: string;
  AYear, AMonth, ADay, AHour, AMinute, ASecond, AMillisecond: Word;
begin
  //Get default month and year for traffic chart
  DecodeDateTime(Now, AYear, AMonth, ADay, AHour, AMinute, ASecond, AMillisecond);
  if AMonth < 12 then
  begin
    Dec(AYear);
    AMonth := AMonth + 1;
  end
  else AMonth := 1;

  with TMemIniFile.Create(Pathname) do //Collect saved configuration
  begin
    try
      Encrypted := ReadBool('login', 'Encrypted', False);
      HG635messagedone := ReadBool('login', 'HG635 message done', False);
      S := Readstring('login', 'Router', '');
      if (S > '') and (RouterBox.Items.Count > 0) then
      begin
        for N := 0 to RouterBox.Items.Count - 1 do
        begin
          if S = RouterBox.Items[N] then
          begin
            RouterBox.ItemIndex := N;
            Break;
          end;
        end;
        MiscString := 'The configured modem/router is ' + S;
        LogEvent(leMisc);
      end;
      IPEdit.Text := Readstring('login', 'IPaddress', '');
      NameEdit.Text := Readstring('login', 'Telnetname', '');
      S := Readstring('login', 'Telnetpassword', '');
      if Encrypted then S := Decrypt(S);
      PasswordEdit.Text := S;
      Secondstage.Checked := Readbool('login', 'Usesecondstage', False);
      Shellcommand.Text := Readstring('login', 'Shelltext', 'sh');
      PortEdit.Text := ReadString('login', 'TelnetPort', '23');
      Annexm.Checked := ReadBool('login', 'Annex M', False);
      ThomBox.Checked := ReadBool('login', 'Thomson', False);
      IsHG622Box.Checked := ReadBool('login', 'Is HG622', False);

      //Needs to be done before reading Synchronise value
      Samplerate := Readinteger('graphs', 'Samplerate', 30000);
      if (SampleRate < 1000) or (SampleRate > 900000) then SampleRate := 30000;  //Revert to default if value is inappropriate
      SampleRateEdit.Value := Samplerate div 1000;

      Synchronise.Checked := ReadBool('login', 'Synchronise', False);
    except
      on E: Exception do
      begin
        MiscString := 'Error restoring login configuration';
        LogEvent(leMisc);
      end;
    end;

    try
      S := Readstring('special login', 'Router', '');
      if (S > '') and (SRouterBox.Items.Count > 0) then
      begin
        for N := 0 to SRouterBox.Items.Count - 1 do
        begin
          if S = SRouterBox.Items[N] then
          begin
            SRouterBox.ItemIndex := N;
            Break;
          end;
        end;
      end;
      for N := 1 to 10 do
      begin
        S := 'Prompt' + IntToStr(N);
        Prompts[N].Text := ReadString('special login', S, '');
        S := 'Response' + IntToStr(N);
        Responses[N].Text := ReadString('special login', S, '');
        if Encrypted and (N = 2) then Responses[N].Text := Decrypt(Responses[N].Text);
      end;
      Encrypted := True;
      SIPEdit.Text := ReadString('special login', 'IPAddress', '');
      SPortEdit.Text := ReadString('special login', 'TelnetPort', '23');
      SAnnexM.Checked := ReadBool('special login', 'AnnexM', False);
      SpecialLogin.Checked := ReadBool('special login', 'Use special', False);
      InsertDelayBox.Checked := ReadBool('special login', 'Insert delay', False);
      DelayTimeEdit.Value := ReadInteger('special login', 'Delay time', 100);
    except
      on E: Exception do
      begin
        MiscString := 'Error restoring special login configuration';
        LogEvent(leMisc);
      end;
    end;

    try
      CommandPrefix := ReadString('stats', 'CommandPrefix', '');
      if CommandPrefix = '' then CommandIndex := DefaultIndex
      else if CommandPrefix = 'Custom' then
      begin
        CustomPrefixLabel.Enabled := True;
        CustomPrefixEdit.Enabled := True;
        CommandIndex := DefaultIndex - 1;
      end
      else begin
        for N := 0 to 12 do
          if LowerCase(CommandList[N]) = LowerCase(CommandPrefix) then
          begin
            CommandIndex := N;
            Break;
          end;
      end;
      CustomPrefixEdit.Text := ReadString('stats', 'CustomPrefix', '');
      CommandBox.ItemIndex := CommandIndex;
      CommandBoxChange(Self);
      SCommandPrefix := ReadString('stats', 'SCommandPrefix', '');
      if SCommandPrefix = '' then SCommandIndex := DefaultIndex
      else if SCommandPrefix = 'Custom' then
      begin
        SCustomPrefixLabel.Enabled := True;
        SCustomPrefixEdit.Enabled := True;
        SCommandIndex := DefaultIndex - 1;
      end
      else begin
        for N := 0 to 12 do
          if LowerCase(CommandList[N]) = LowerCase(SCommandPrefix) then
          begin
            SCommandIndex := N;
            Break;
          end;
      end;
      SCommandBox.ItemIndex := SCommandIndex;
    except
      on E: Exception do
      begin
        MiscString := 'Error restoring configured command prefix';
        LogEvent(leMisc);
      end;
    end;

    try
      TimeperpageEdit.Value := ReadInteger('graphs', 'Hoursperpage', 2);
      DaysperpageEdit.Value := ReadInteger('graphs', 'Daysperpage', 0);
      Hoursperpage := TimeperpageEdit.Value + (DaysperpageEdit.Value * 24);
      TimeperpageEditChange(Self);
      ThicknessEdit.Value := ReadInteger('graphs', 'Thickness', 1);
      DoThickness;
      FontName := ReadString('graphs', 'Fontname', 'default');
      DoGraphFonts;
      TextFontName := ReadString('graphs', 'TextFontname', 'Monospace');
      {$IFDEF Windows}
      if TextFontName = 'Monospace' then TextFontName := 'Courier New';
      {$ENDIF}
      DoTextFonts;
      DoMargin.Checked := Readbool('graphs', 'ShowSNRmargin', True);
      DoBitloading.Checked := Readbool('graphs', 'Showbitloading', True);
      DoSNRpertone.Checked := ReadBool('graphs', 'ShowSNRpertone', False);
      SNRwithbitloading.Checked := ReadBool('graphs', 'SNRwithbitloading', False);
      SNRwithbitloading.Enabled := DoSNRpertone.Checked and not ThomBox.Checked;
      SNRseparate.Checked := ReadBool('graphs', 'SNRseparate', False);
      SNRseparate.Enabled := DoSNRpertone.Checked and not ThomBox.Checked;
      Bitchart1.DoSNRpertone := DoSNRpertone.Checked;
      DoBitswaps.Checked := ReadBool('graphs', 'Bitswaps', False);
      DoSwapspermin.Checked := ReadBool('graphs', 'Swaps per min', False);
      TonesTab.TabVisible := (DoSNRpertone.Checked or DoBitloading.Checked or DoBitswaps.Checked or DoSwapspermin.Checked);
      BitloadingTab.TabVisible := DoBitloading.Checked;
      BitswapTab.TabVisible := DoBitswaps.Checked;
      Swapspermintab.TabVisible := DoSwapspermin.Checked;
      SNRTab.TabVisible := (DoSNRpertone.Checked and SNRseparate.Checked);
      DoSpeed.Checked := ReadBool('graphs', 'ShowSpeed', True);
      DoCRC.Checked := ReadBool('graphs', 'ShowCRC', False);
      DoSES.Checked := ReadBool('graphs', 'ShowSES', False);
      DoFEC.Checked := ReadBool('graphs', 'ShowFEC', False);
      DoES.Checked := ReadBool('graphs', 'ShowES', False);
      DoAverages.Checked := ReadBool('graphs', 'ShowAverages', False);
      DoQLN.Checked := ReadBool('graphs', 'ShowQLN', False);
      DoHlog.Checked := ReadBool('graphs', 'ShowHlog', False);
      DoSignalAttn.Checked := ReadBool('graphs', 'Signal atten', False);
      DoFirmware.Checked := ReadBool('graphs', 'Firmware version', True);
      DoVendor.Checked := ReadBool('graphs', 'DSLAM Vendor', True);
      DoVectoring.Checked := ReadBool('graphs', 'Vectoring', True);
      DoSysuptime.Checked := ReadBool('graphs', 'System uptime', True);
      DoSNRMperband.Checked := ReadBool('graphs', 'ShowSNRMperband', True);
      MarginClip.Value := ReadInteger('graphs', 'MarginClipValue', 20);
      Marginchart.ClipValue := MarginClip.Value;
      AutoscaleBox.Checked := ReadBool('graphs', 'Autoscale Y', False);
      MaxYEdit.Value := ReadInteger('graphs', 'MaxYValue', 10);
      MinYEdit.Value := ReadInteger('graphs', 'MinYValue', 0);
      CRCClip.Value := ReadInteger('graphs', 'CRCClipValue', 20);
      CRCchart.ClipValue := CRCClip.Value;
      FECClip.Value := ReadInteger('graphs', 'FECClipValue', 100);
      FECchart.ClipValue := FECClip.Value;
      LEFTRSClip.Value := ReadInteger('graphs', 'LEFTRSClipValue', 100);
      LEFTRSchart.ClipValue := LEFTRSClip.Value;
      LEFTRSEnableClippingBox.Checked := ReadBool('graphs', 'UsertxcClipping', True);
      LEFTRSLogY.Checked := ReadBool('graphs', 'rtxc Log Y', False);
      LEFTRSLogYChange(Self);
      SESClip.Value := ReadInteger('graphs', 'SESClipValue', 100);
      SESchart.ClipValue := SESClip.Value;
      PMClip.Value := ReadInteger('graphs', 'PMClipValue', 100);
      SwapsperminChart.ClipValue := PMClip.Value;
      rtxtxClip.Value := ReadInteger('graphs', 'rtxtxClipValue', 20);
      rtxtxchart.ClipValue := rtxtxClip.Value;
      rtxcClip.Value := ReadInteger('graphs', 'rtxcClipValue', 20);
      rtxcchart.ClipValue := rtxcClip.Value;
      rtxucClip.Value := ReadInteger('graphs', 'rtxucClipValue', 20);
      rtxucchart.ClipValue := rtxucClip.Value;
      TonesperPixel := ReadInteger('graphs', 'Tones per pixel', -1);
      Tonesperpx(TonesperPixel);
      CRCLogY.Checked := ReadBool('graphs', 'CRC Log Y', False);
      CRCLogYChange(Self);
      FECLogY.Checked := ReadBool('graphs', 'FEC Log Y', False);
      FECLogYChange(Self);
      PerSampleBoxC.Checked := ReadBool('graphs', 'CRC per sample', False);
      PerSampleBoxF.Checked := ReadBool('graphs', 'FEC per sample', False);
      PerSampleBoxLEFTRS.Checked := ReadBool('graphs', 'LEFTRS per sample', False);
      PerSampleBoxrtxtx.Checked := ReadBool('graphs', 'rtxtx per sample', False);
      PerSampleBoxrtxc.Checked := ReadBool('graphs', 'rtxc per sample', False);
      PerSampleBoxrtxuc.Checked := ReadBool('graphs', 'rtxuc per sample', False);
      rtxtxLogY.Checked := ReadBool('graphs', 'rtxtx Log Y', False);
      rtxtxLogYChange(Self);
      rtxcLogY.Checked := ReadBool('graphs', 'rtxc Log Y', False);
      rtxcLogYChange(Self);
      rtxucLogY.Checked := ReadBool('graphs', 'rtxuc Log Y', False);
      rtxucLogYChange(Self);
      ConnspeedLogY.Checked := ReadBool('graphs', 'Connspeed Log Y', False);
      ConnspeedLogYChange(Self);
      CEnableClippingBox.Checked := ReadBool('graphs', 'UseCRCClipping', True);
      CRCchart.UseClipping := CEnableClippingBox.Checked;
      FEnableClippingBox.Checked := ReadBool('graphs', 'UseFECClipping', True);
      FECchart.UseClipping := FEnableClippingBox.Checked;
      LEFTRSEnableClippingBox.Checked := ReadBool('graphs', 'UseLEFTRSClipping', True);
      LEFTRSchart.UseClipping := LEFTRSEnableClippingBox.Checked;
      rtxtxEnableClippingBox.Checked := ReadBool('graphs', 'UsertxtxClipping', True);
      rtxtxchart.UseClipping := rtxtxEnableClippingBox.Checked;
      rtxcEnableClippingBox.Checked := ReadBool('graphs', 'UsertxcClipping', True);
      rtxcchart.UseClipping := rtxcEnableClippingBox.Checked;
      rtxucEnableClippingBox.Checked := ReadBool('graphs', 'UsertxucClipping', True);
      rtxucchart.UseClipping := rtxucEnableClippingBox.Checked;
      LegendLeft.Checked := ReadBool('graphs', 'Legend on left', False);
      LegendLeftChange(Self);
      MHideLegend.Checked := ReadBool('graphs', 'MHideLegend', False);
      SHideLegend.Checked := ReadBool('graphs', 'SHideLegend', False);
      PMHideLegend.Checked := ReadBool('graphs', 'PMHideLegend', False);
      CHideLegend.Checked := ReadBool('graphs', 'CHideLegend', False);
      FHideLegend.Checked := ReadBool('graphs', 'FHideLegend', False);
      UHideLegend.Checked := ReadBool('graphs', 'UHideLegend', False);
      EHideLegend.Checked := ReadBool('graphs', 'EHideLegend', False);
      SESHideLegend.Checked := ReadBool('graphs', 'SESHideLegend', False);
      SpBHideLegendUp.Checked := ReadBool('graphs', 'SpBHideLegendUp', False);
      SpBHideLegendDown.Checked := ReadBool('graphs', 'SpBHideLegendDown', False);
      BHideLegend.Checked := ReadBool('graphs', 'BHideLegend', False);
      BsHideLegend.Checked := ReadBool('graphs', 'BsHideLegend', False);
      SNRHideLegend.Checked := ReadBool('graphs', 'SNRHideLegend', False);
      QLNHideLegend.Checked := ReadBool('graphs', 'QLNHideLegend', False);
      HLogHideLegend.Checked := ReadBool('graphs', 'HLogHideLegend', False);
      rtxtxHideLegend.Checked := ReadBool('graphs', 'rtxtxHideLegend', False);
      rtxcHideLegend.Checked := ReadBool('graphs', 'rtxcHideLegend', False);
      rtxucHideLegend.Checked := ReadBool('graphs', 'rtxucHideLegend', False);
      LEFTRSHideLegend.Checked := ReadBool('graphs', 'LEFTRSHideLegend', False);
      MHideLegendChange(Self);
      ShowMaxUpBox.Checked := ReadBool('graphs', 'Max attainable up', False);
      Speedchart1.ShowMaxUp := ShowMaxUpBox.Checked;
      ShowMaxDownBox.Checked := ReadBool('graphs', 'Max attainable down', False);
      Speedchart1.ShowMaxDown := ShowMaxDownBox.Checked;
      ESSetMax.Checked := ReadBool('graphs', 'Set max ES value', False);
      ESMax.ItemIndex := ReadInteger('graphs', 'Max ES index', 0);
      ESAutoscaleChange(Self);
      ScaleBox.Checked := ReadBool('graphs', 'Scale snapshots', False);
      HeightEdit.Value := ReadInteger('graphs', 'Snapshot height', 400);
      WidthEdit.Value := ReadInteger('graphs', 'Snapshot width', 600);
    except
      on E: Exception do
      begin
        MiscString := 'Error restoring graph configuration';
        LogEvent(leMisc);
      end;
    end;

    try
      Graphcolour.Selected := ReadInteger('colours', 'Graphcolour', $D8FFFF);
      Upcolour.Selected := ReadInteger('colours', 'Upcolour', clGreen);
      Downcolour.Selected := ReadInteger('colours', 'Downcolour', clRed);
      UpPlusDowncolour.Selected := ReadInteger('colours', 'UpPlusDowncolour', clAqua);
      Othercolour.Selected := ReadInteger('colours', 'Othercolour', clBlue);
      SNRColour.Selected := ReadInteger('colours', 'SNRcolour', clPurple);
      LimitColour.Selected := ReadInteger('colours', 'Limitcolour', clAqua);
      Colour0.Selected := ReadInteger('colours', 'U0D0Colour', clPurple);
      Colour1.Selected := ReadInteger('colours', 'U1D1Colour', clGreen);
      Colour2.Selected := ReadInteger('colours', 'U2D2Colour', clRed);
      Colour3.Selected := ReadInteger('colours', 'U3D3Colour', clBlue);
      Colour4.Selected := ReadInteger('colours', 'U4D4Colour', $004080);
      SetColours;
    except
      on E: Exception do
      begin
        MiscString := 'Error restoring graph colours configuration';
        LogEvent(leMisc);
      end;
    end;

    try
      SnapDirEdit.Directory := ReadString('snapshots', 'Snapshotdir', HomePath + 'Snapshots');
      SnapshotsPath := IncludeTrailingPathDelimiter(SnapDirEdit.Directory);
      SameAsBox.Checked := ReadBool('snapshots', 'UseHoursperpage', True);
      if SameAsBox.Checked then Hourspersnapshot := Hoursperpage
      else Hourspersnapshot := ReadInteger('snapshots', 'Hourspersnapshot', 2);
      TimepersnapshotEdit.Value := Hourspersnapshot;
      Minutespersnapshot := Hourspersnapshot * 60;
      StartofRecording.Checked := ReadBool('snapshots', 'Start of recording', True);
      FirstAtTime.Checked := ReadBool('snapshots', 'First at', False);
      SnapshotHours.Value := ReadInteger('snapshots', 'Hours', 0);
      SnapshotMinutes.Value := ReadInteger('snapshots', 'Minutes', 0);
      SNRsnap.Checked := ReadBool('snapshots', 'SNRsnapshot', False);
      SNRMpbsnap.Checked := ReadBool('snapshots', 'SNRpbsnapshot', False);
      Speedsnap.Checked := ReadBool('snapshots', 'Speedsnapshot', False);
      Bitsnap.Checked := ReadBool('snapshots', 'Bitsnapshot', False);
      CRCsnap.Checked := ReadBool('snapshots', 'CRCsnapshot', False);
      SESsnap.Checked := ReadBool('snapshots', 'SESsnapshot', False);
      FECsnap.Checked := ReadBool('snapshots', 'FECsnapshot', False);
      ESsnap.Checked := ReadBool('snapshots', 'ESsnapshot', False);
      SNRptsnap.Checked := ReadBool('snapshots', 'SNRptsnapshot', False);
      Swapptsnap.Checked := ReadBool('snapshots', 'Swapsptsnapshot', False);
      Swappmsnap.Checked := ReadBool('snapshots', 'Swapspmsnapshot', False);
      QLNsnap.Checked := ReadBool('snapshots', 'QLNsnapshot', False);
      HLogsnap.Checked := ReadBool('snapshots', 'HLogsnapshot', False);
      TrafficSnap.Checked := ReadBool('snapshots', 'TrafficSnapshot', False);
      PerMinute.Checked := ReadBool('snapshots', 'PerMinute', True);
      PerDay.Checked := ReadBool('snapshots', 'PerDay', False);
      PerMonth.Checked := ReadBool('snapshots', 'PerMonth', False);
      Dateorder.Checked := ReadBool('snapshots', 'Dateorder', False);
      ConnectionStatsText.Checked := ReadBool('snapshots', 'ConnectionStatsText', False);
      SNRText.Checked := ReadBool('snapshots', 'SNRText', False);
      BitloadingText.Checked := ReadBool('snapshots', 'BitloadingText', False);
      pbParamsText.Checked := ReadBool('snapshots', 'pbParamsText', False);
      QLNText.Checked := ReadBool('snapshots', 'QLNText', False);
      HLogText.Checked := ReadBool('snapshots', 'HLogText', False);
      AttenuationLogText.Checked := ReadBool('snapshots', 'AttenuationLogText', False);
      AverageErrorsText.Checked := ReadBool('snapshots', 'AverageErrorsText', False);
      TrafficText.Checked := ReadBool('snapshots', 'TrafficText', False);
      EventlogText.Checked := ReadBool('snapshots', 'EventlogSnapshot', False);
      CustomCommandsText.Checked := ReadBool('snapshots', 'CustomCommandsSnapshot', False);
      StatsText1.Checked := ReadBool('snapshots', 'StatsSnapshot', False);
      SnapallBox.Checked := ReadBool('snapshots', 'SnapOnExit', False);
      SnapallConfiguredBox.Checked := ReadBool('snapshots', 'SnapConfiguredOnExit', False);
      DailyBox.Checked := ReadBool('snapshots', 'Daily folders', False);
      CategoryBox.Checked := ReadBool('snapshots', 'Category folders', False);
      LogNum := ReadInteger('snapshots', 'LogNumber', 0);
      DeleteOlderBox.Checked := ReadBool('snapshots', 'DeleteOlder', False);
      OlderDays.Value := ReadInteger('snapshots', 'Olderthan', 7);
      DoUploadSS.Checked := ReadBool('snapshots', 'Upload snapshots', False);
      SSUploadURL.Text := ReadString('snapshots', 'UploadURL', '');
      SSUploadDir.Text := ReadString('snapshots', 'UploadDir', '');
      SSUploadUsername.Text := ReadString('snapshots', 'Username', '');
      S := ReadString('snapshots', 'Password', '');
      SSUploadPassword.Text := Decrypt(S);
      SSFTPPort.Text := ReadString('snapshots', 'FTP port', '');
      SSTLSButton.Checked := ReadBool('snapshots', 'FTP-TLS', False);
      SSFTPSButton.Checked := ReadBool('snapshots', 'FTPS', False);
      SSSFTPButton.Checked := ReadBool('snapshots', 'SFTP', False);
    except
      on E: Exception do
      begin
        MiscString := 'Error restoring snapshots configuration';
        LogEvent(leMisc);
      end;
    end;

    try
      Eventlogbox.Checked := ReadBool('misc', 'Eventlog', True);
      SaveonexitBox.Checked := ReadBool('misc', 'SaveEventLog', False);
      SaveAfterBox.Checked := ReadBool('misc', 'SaveEventLogAfter', False);
      ConfirmCloseBox.Checked := ReadBool('misc', 'Confirm close', True);
      MinimiseoncloseBox.Checked := ReadBool('misc', 'Minimise on close', False);
      MinimiseoncloseBoxChange(Self);
      MinimisetotrayBox.Checked := ReadBool('misc', 'Minimise to tray', False);
      DontAskButton.Checked := not CheckUpdateBox.Checked;
      DownloadDirEdit.Directory := ReadString('misc', 'DownloadDir', HomePath);
      DiagnosticsBox.Checked := ReadBool('misc', 'Diagnostics', True);
      PlannedEventsBox.Checked := ReadBool('misc', 'Planned events', True);
      PauseBox.Checked := ReadBool('misc', 'Pause on error', False);
      ShowHintsBox.Checked := ReadBool('misc', 'ShowHints', True);
      ShowHintsBoxChange(Self);
      AutostartBox.Checked := ReadBool('misc', 'Autostart', False);
      StartMinimisedBox.Checked := ReadBool('misc', 'Start minimised', False);
      StartMaximisedBox.Checked := ReadBool('misc', 'Start maximised', False);
      ErrorNumber := ReadInteger('misc', 'ErrorNumber', 1);
      RestartBox.Checked := ReadBool('misc', 'AutoRestart', False);
      RestartEdit.Value := ReadInteger('misc', 'RestartTime', 10);
      DoLogging := ReadBool('misc', 'Attenuation logging', True);
      if DoLogging then LoggingButton.Caption := 'Stop logging' else LoggingButton.Caption := 'Start logging';
    except
      on E: Exception do
      begin
        MiscString := 'Error restoring misc configuration';
        LogEvent(leMisc);
      end;
    end;

    try
      NewESUp := StrToInt64(ReadString('errors', 'NewESUp', '0'));
      NewESDown := StrToInt64(ReadString('errors', 'NewESDown', '0'));
      OldESUp := StrToInt64(ReadString('errors', 'OldESUp', '0'));
      OldESDown := StrToInt64(ReadString('errors', 'OldESDown', '0'));
      LastYear := ReadInteger('errors', 'Last year', 0);
      LastMonth := ReadInteger('errors', 'Last month', 0);
      LastDay := ReadInteger('errors', 'Last day', 0);
      LastHour := ReadInteger('errors', 'Last hour', 0);
    except
      on E: Exception do
      begin
        MiscString := 'Error restoring ES data';
        LogEvent(leMisc);
      end;
    end;

    try
      CheckUpdateBox.Checked := ReadBool('updates', 'Check updates', True);
      S := ReadString('updates', 'LastCheck', '');
      try
        if S = '' then LastUpdateCheck := 0 else LastUpdateCheck := StrToDateTime(S);
      except
        on E: Exception do LastUpdateCheck := 0;
      end;
      FrequencyBox.Value := ReadInteger('updates', 'Frequency', 1);
    except
      on E: Exception do
      begin
        MiscString := 'Error restoring updates data';
        LogEvent(leMisc);
      end;
    end;

    try
    for N := 1 to 4 do
    begin
      Commands[N].Text := ReadString('custom', 'Command' + IntToStr(N), '');
      S := ReadString('custom', 'Include' + IntToStr(N), '000000');
      for N1 := 1 to 6 do
        CommandNumbers[N, N1].Checked := (S[N1] = '1');
    end;
      EventLogButton.Checked := ReadBool('custom', 'InEventLog', False);
      CommandHours.Value := ReadInteger('custom', 'Hours', 0);
      CommandMinutes.Value := ReadInteger('custom', 'Minutes', 0);
      EveryChange(Self);
    except
      on E: Exception do
      begin
        MiscString := 'Error restoring custom commands configuration';
        LogEvent(leMisc);
      end;
    end;

    try
      if ReadBool('alerts', 'Send email', False) then
      begin
        SNRMemail.Checked := True;
        CRCemail.Checked := True;
        FECemail.Checked := True;
        ESemail.Checked := True;
        FTPemail.Checked := True;
        Resyncemail.Checked := True;
      end
      else begin
        SNRMemail.Checked := ReadBool('alerts', 'SNRM email', False);
        CRCemail.Checked := ReadBool('alerts', 'CRC email', False);
        FECemail.Checked := ReadBool('alerts', 'FEC email', False);
        ESemail.Checked := ReadBool('alerts', 'ES email', False);
        FTPemail.Checked := ReadBool('alerts', 'FTP email', False);
        Resyncemail.Checked := ReadBool('alerts', 'Resync email', False);
      end;
      SendEventlogBox.Checked := ReadBool('alerts', 'Send event log', False);
      SNRMfallBox.Checked := ReadBool('alerts', 'SNRM', False);
      SNRMTrigger.Text := ReadString('alerts', 'SNRM trigger', '0');
      SNRMreturnBox.Checked := ReadBool('alerts', 'SNRM return', False);
      SNRMreturnTrigger.Text := ReadString('alerts', 'SNRM return trigger', '0');
      CRCriseBox.Checked := ReadBool('alerts', 'CRC rise', False);
      CRCTrigger.Text := ReadString('alerts', 'CRC trigger', '0');
      CRCpm.Checked := ReadBool('alerts', 'CRC per minute', True);
      FECriseBox.Checked := ReadBool('alerts', 'FEC rise', False);
      FECTrigger.Text := ReadString('alerts', 'FEC trigger', '0');
      FECpm.Checked := ReadBool('alerts', 'FEC per minute', True);
      ESriseBox.Checked := ReadBool('alerts', 'ES rise', False);
      ESTrigger.Text := ReadString('alerts', 'ES trigger', '0');
      KeepLevelsBox.Checked := ReadBool('alerts', 'Keep trigger levels', False);
      N := ReadInteger('alerts', 'SNRM option', 1);
      if N = 1 then UpSNRM.Checked := True
      else if N = 2 then DownSNRM.Checked := True
      else EitherSNRM.Checked := True;
      FTPFailBox.Checked := ReadBool('alerts', 'FTP fail', False);
      ResyncBox.Checked := ReadBool('alerts', 'Resync', False);
      EmailAddress.Text := ReadString('alerts', 'Email address', '');
      EmailSubject.Text := ReadString('alerts', 'Email subject', 'DSLstats alert');
      SMTPserver.Text := ReadString('alerts', 'SMTP server', '');
      EmailUsername.Text := ReadString('alerts', 'Username', '');
      EmailFrom.Text := ReadString('alerts', 'Email from', EmailUsername.Text);      //This line must follow EmailUsername
      SMTPPort.Text := ReadString('alerts', 'SMTP port', '25');
      if SMTPPort.Text = '465' then EncryptionType := 1
      else EncryptionType := ReadInteger('alerts', 'Encryption type', 0);
      case EncryptionType of
        0: PlainButton.Checked := True;
        1: SSLButton.Checked := True;
        2: StartTLSButton.Checked := True;
      end;
      EMailPassword.Text := Decrypt(ReadString('alerts', 'Password', ''));
    except
      on E: Exception do
      begin
        MiscString := 'Error restoring alerts configuration';
        LogEvent(leMisc);
      end;
    end;

    try                                                          //Transitional code
      S := ReadString('webserver', 'WSDir', '');
      if S = '' then                                             //First run of this version
      begin
        S := ReadString('webserver', 'Webserverdir', HomePath);  //Check for setting from earlier version
        S := IncludeTrailingPathDelimiter(S);
        S := IncludeTrailingPathDelimiter(S + 'Webserver');
        if not ForceDirectories(S) then
        begin
          WebserverDirEdit.Directory := HomePath;
          MiscString := 'Unable to create the specified webserver folder. Reverting to ' + HomePath;
          LogEvent(leMisc);
        end
        else WebserverDirEdit.Directory := S;                    //At this point the directory exists
      end
      else begin                                                 //Check if the saved setting still exists
        if not ForceDirectories(S) then
        begin
          WebserverDirEdit.Directory := HomePath;
          MiscString := 'The saved webserver location does not exist. Reverting to ' + HomePath;
          LogEvent(leMisc);
        end
        else WebserverDirEdit.Directory := S;                    //The directory exists
      end;
      MiscString := 'Webserver folder is ' + WebserverDirEdit.Directory;
      LogEvent(leMisc);
      WebPortEdit.Text := ReadString('webserver', 'Port', '55555');
      RefreshPeriod := ReadInteger('webserver', 'Refresh period', 30);
      RefreshEdit.Value := RefreshPeriod;
      CustomButton.Checked := ReadBool('webserver', 'Custom refresh', False);
      WebserverBox.Checked := ReadBool('webserver', 'Enable webserver', False);
      WebserverSaveBox.Checked := ReadBool('webserver', 'Save webserver files', False);
      SNRMweb.Checked := ReadBool('webserver', 'SNRM', False);
      SNRMpbweb.Checked := ReadBool('webserver', 'SNRMpb', False);
      Connspeedweb.Checked := ReadBool('webserver', 'Connspeed', False);
      Bitloadingweb.Checked := ReadBool('webserver', 'Bitloading', False);
      CRCweb.Checked := ReadBool('webserver', 'CRC', False);
      ESweb.Checked := ReadBool('webserver', 'ES', False);
      SESweb.Checked := ReadBool('webserver', 'SES', False);
      FECweb.Checked := ReadBool('webserver', 'FEC', False);
      Statsweb.Checked := ReadBool('webserver', 'Stats', False);
      Fullstatsweb.Checked := ReadBool('webserver', 'Connection stats', False);
      Combineweb.Checked := ReadBool('webserver', 'Combined page', False);
      DeleteWebBox.Checked := ReadBool('webserver', 'Delete on disable', False);
      AllActiveWeb.Checked := ReadBool('webserver', 'All active', False);
      AllConfiguredWeb.Checked := ReadBool('webserver', 'All configured', False);
      AllTextWeb.Checked := ReadBool('webserver', 'All text', False);
      DoUploadWS.Checked := ReadBool('webserver', 'Do upload', False);
      WSUploadRate.Value := ReadInteger('webserver', 'Upload rate', 5);
      WSUploadURL.Text := ReadString('webserver', 'UploadURL', '');
      WSUploadDir.Text := ReadString('webserver', 'UploadDir', '');
      WSUploadUsername.Text := ReadString('webserver', 'Username', '');
      S := ReadString('webserver', 'Password', '');
      WSUploadPassword.Text := Decrypt(S);
      WSFTPPort.Text := ReadString('webserver', 'FTP port', '');
      WSTLSButton.Checked := ReadBool('webserver', 'FTP-TLS', False);
      WSFTPSButton.Checked := ReadBool('webserver', 'FTPS', False);
      WSSFTPButton.Checked := ReadBool('webserver', 'SFTP', False);
    except
      on E: Exception do
      begin
        MiscString := 'Error restoring webserver configuration';
        LogEvent(leMisc);
      end;
    end;

    DoSaveData.Checked := ReadBool('DataStore', 'Do data', False);
    DoUploadData.Checked := ReadBool('DataStore', 'Do upload', False);
    SingleButton.Checked := ReadBool('DataStore', 'Single file', True);
    S := ReadString('DataStore', 'DataStore path', '');                                   //Transitional code
    if S = '' then                                                                        //First run of this version
    begin
      S := IncludeTrailingPathDelimiter(ReadString('DataStore', 'Data path', HomePath));  //Check for setting from earlier version
      DataFolderEdit.Directory := S + 'DataStore';
    end
    else begin                                                                            //Just read the saved setting
      DataFolderEdit.Directory := S;
    end;
    DataStorePath := IncludeTrailingPathDelimiter(DataFolderEdit.Directory);
    DSbits.Checked := ReadBool('DataStore', 'Bitloading', True);
    DSsnr.Checked := ReadBool('DataStore', 'SNR per tone', True);
    DShlog.Checked := ReadBool('DataStore', 'HLog', True);
    DSqln.Checked := ReadBool('DataStore', 'QLN', True);
    //DSbitsUpload.Checked := ReadBool('DataStore', 'BitsUpload', False);
    //DSsnrUpload.Checked := ReadBool('DataStore', 'SNRUpload', False);
    //DShlogUpload.Checked := ReadBool('DataStore', 'HLogUpload', False);
    //DSqlnUpload.Checked := ReadBool('DataStore', 'QLNUpload', False);
    DoUploadData.Checked := ReadBool('DataStore', 'Upload data', False);
    DSUploadURL.Text := ReadString('DataStore', 'UploadURL', '');
    DSUploadDir.Text := ReadString('DataStore', 'UploadDir', '');
    DSUploadUsername.Text := ReadString('DataStore', 'Username', '');
    S := ReadString('DataStore', 'Password', '');
    DSUploadPassword.Text := Decrypt(S);
    DSFTPPort.Text := ReadString('DataStore', 'FTP port', '');
    DSTLSButton.Checked := ReadBool('DataStore', 'FTP-TLS', False);
    DSFTPSButton.Checked := ReadBool('DataStore', 'FTPS', False);
    DSSFTPButton.Checked := ReadBool('DataStore', 'SFTP', False);

    try
      SNRSaveBox.Checked := ReadBool('XML', 'SNRM', False);
      SpeedSaveBox.Checked := ReadBool('XML', 'ConSpeed', False);
      CRCSaveBox.Checked := ReadBool('XML', 'CRC', False);
      FECSaveBox.Checked := ReadBool('XML', 'FEC', False);
      BitswapSaveBox.Checked := ReadBool('XML', 'Bitswap', False);
      SNRSaveEBox.Checked := ReadBool('XML', 'SNRMx', False);
      SpeedSaveEBox.Checked := ReadBool('XML', 'ConSpeedx', False);
      CRCSaveEBox.Checked := ReadBool('XML', 'CRCx', False);
      FECSaveEBox.Checked := ReadBool('XML', 'FECx', False);
      BitswapSaveEBox.Checked := ReadBool('XML', 'Bitswapx', False);
    except
      on E: Exception do
      begin
        MiscString := 'Error restoring XML configuration';
        LogEvent(leMisc);
      end;
    end;

    try
      Self.Left := ReadInteger('positions', 'Left', 200);
      Self.Top := ReadInteger('positions', 'Top', 200);
      SavedWidth := ReadInteger('positions', 'Width', 100);
      if SavedWidth > 100 then Self.Width := SavedWidth;
      SavedHeight := ReadInteger('positions', 'Height', 100);
      if SavedHeight > 100 then Self.Height := SavedHeight;
      StartupLeft := Self.Left;
      StartupTop := Self.Top;
      StartupHeight := Self.Height;
      StartupWidth := Self.Width;
    except
      on E: Exception do
      begin
        MiscString := 'Error restoring window positions';
        LogEvent(leMisc);
      end;
    end;

    try
      Free;
    except
      on E: Exception do
      begin
        MiscString := 'Unable to release memory after reading dslstats.ini';
        LogEvent(leMisc);
      end;
    end;
  end;
  if not AutoscaleBox.Checked then //Restore the custom limits for the SNRM graph
  begin
    MarginChart.MaxYValue := MaxYEdit.Value;
    MarginChart.MinYValue := MinYEdit.Value;
  end;
  SetTabs;
end;

// **** CLOSING DOWN ****

//Save traffic data
procedure Tmainform.WriteTrafficData;
var
  N: Byte;
begin
  with TMemInifile.Create(HomePath + 'traffic.ini') do
  begin
    EraseSection('traffic');
    WriteString('traffic', 'Traffic command', CommandGroup.Items[CommandGroup.ItemIndex]);
    WriteString('traffic', 'Traffic interface', InterfaceGroup.Items[InterfaceGroup.ItemIndex]);
    WriteBool('traffic', 'Monitor traffic', MonitorTrafficBox.Checked);
    if Trafficpermin.Checked then N := 0
    else if DailyButton.Checked then N := 1
    else N := 2;
    WriteInteger('traffic', 'GraphType', N);
    WriteString('traffic', 'Other interface', OtherInterfaceEdit.Text);
    WriteString('traffic', 'Up', IntToStr(UpTraffic));
    WriteString('traffic', 'Down', IntToStr(DownTraffic));
    WriteInteger('traffic', 'First month', Ord(TrafficChart.FirstMonth) + 1);
    WriteInteger('traffic', 'Current day', CurrentDay);
    WriteString('traffic', 'DailyUpValue', IntToStr(DailyUpValue));
    WriteString('traffic', 'DailyDownValue', IntToStr(DailyDownValue));
    WriteString('traffic', 'MonthlyUpValue', IntToStr(MonthlyUpValue));
    WriteString('traffic', 'MonthlyDownValue', IntToStr(MonthlyDownValue));
    WriteBool('traffic', 'Limit warning', LimitWarningBox.Checked);
    WriteBool('traffic', 'Download only', DownOnlyButton.Checked);
    WriteString('traffic', 'Limit', LimitEdit.Text);
    WriteInteger('traffic', 'Billing day', DayEdit.Value);
    WriteBool('traffic', 'Billing data', ShowBillingBox.Checked);
    WriteBool('traffic', 'NewDataFile', True);
    Free;
  end;
  TrafficChart.SaveTrafficData(HomePath);
end;

//Save configuration
procedure Tmainform.WriteInifile;
var
  N, N1, EncryptionType: Word;
  S: string;
begin
  ErrorOccurred := False;
  with TMemIniFile.Create(Inipath) do
  begin
    try
      EraseSection('login');
      WriteBool('login', 'HG635 message done', HG635messagedone);
      WriteBool('login', 'Encrypted', Encrypted);
      Writestring('login', 'Router', RouterBox.Text);
      Writestring('login', 'IPaddress', IPEdit.Text);
      Writestring('login', 'Telnetname', NameEdit.Text);
      Writestring('login', 'Telnetpassword', Encrypt(PasswordEdit.Text));
      Writebool('login', 'Usesecondstage', SecondStage.Checked);
      Writestring('login', 'Shelltext', ShellCommand.Text);
      WriteString('login', 'TelnetPort', PortEdit.Text);
      WriteBool('login', 'Annex M', Annexm.Checked);
      WriteBool('login', 'Thomson', ThomBox.Checked);
      WriteBool('login', 'Is HG622', IsHG622Box.Checked);
      WriteBool('login', 'Synchronise', Synchronise.Checked);
    except
      MiscString := 'Error saving login configuration';
      LogEvent(leMisc);
      ErrorOccurred := True;
    end;
    try
      EraseSection('special login');
      Writestring('special login', 'Router', SRouterBox.Text);
      try
        for N := 1 to 10 do
        begin
          S := 'Prompt' + IntToStr(N);
          WriteString('special login', S, Prompts[N].Text);
          S := 'Response' + IntToStr(N);
          if N = 2 then WriteString('special login', S, Encrypt(Responses[N].Text))
          else WriteString('special login', S, Responses[N].Text);
        end;
      except
        on E: Exception do
        begin
          MiscString := 'Unable to save special login configuration';
          LogEvent(leMisc);
        end;
      end;
      WriteString('special login', 'IPAddress', SIPEdit.Text);
      WriteString('special login', 'TelnetPort', SPortEdit.Text);
      WriteBool('special login', 'AnnexM', SAnnexM.Checked);
      WriteBool('special login', 'Use special', SpecialLogin.Checked);
      WriteBool('special login', 'Insert delay', InsertDelayBox.Checked);
      WriteInteger('special login', 'Delay time', DelayTimeEdit.Value);
    except
      MiscString := 'Error saving special login configuration';
      LogEvent(leMisc);
      ErrorOccurred := True;
    end;
    try
      Erasesection('stats');
      WriteString('stats', 'CommandPrefix', CommandBox.Items[CommandBox.ItemIndex]);
      WriteString('stats', 'SCommandPrefix', SCommandBox.Items[SCommandBox.ItemIndex]);
      WriteString('stats', 'CustomPrefix', CustomPrefixEdit.Text);
    except
      MiscString := 'Error saving command prefix settings';
      LogEvent(leMisc);
      ErrorOccurred := True;
    end;
    try
      Erasesection('graphs');
      Writeinteger('graphs', 'Samplerate', Samplerate);
      Writeinteger('graphs', 'Hoursperpage', TimeperpageEdit.Value);
      Writeinteger('graphs', 'Daysperpage', DaysperpageEdit.Value);
      WriteInteger('graphs', 'Thickness', ThicknessEdit.Value);
      WriteString('graphs', 'Fontname', FontName);
      WriteString('graphs', 'TextFontname', TextFontName);
      Writebool('graphs', 'ShowSNRmargin', DoMargin.Checked);
      Writebool('graphs', 'Showbitloading', DoBitloading.Checked);
      Writebool('graphs', 'ShowSNRpertone', DoSNRpertone.Checked);
      Writebool('graphs', 'SNRwithbitloading', SNRwithbitloading.Checked);
      Writebool('graphs', 'SNRseparate', SNRseparate.Checked);
      WriteBool('graphs', 'Bitswaps', DoBitswaps.Checked);
      WriteBool('graphs', 'Swaps per min', DoSwapspermin.Checked);
      WriteBool('graphs', 'ShowSpeed', DoSpeed.Checked);
      WriteBool('graphs', 'ShowCRC', DoCRC.Checked);
      WriteBool('graphs', 'ShowSES', DoSES.Checked);
      WriteBool('graphs', 'ShowFEC', DoFEC.Checked);
      WriteBool('graphs', 'ShowES', DoES.Checked);
      WriteBool('graphs', 'ShowAverages', DoAverages.Checked);
      WriteBool('graphs', 'ShowQLN', DoQLN.Checked);
      WriteBool('graphs', 'ShowHlog', DoHlog.Checked);
      WriteBool('graphs', 'Signal atten', DoSignalAttn.Checked);
      WriteBool('graphs', 'Firmware version', DoFirmware.Checked);
      WriteBool('graphs', 'DSLAM vendor', DoVendor.Checked);
      WriteBool('graphs', 'Vectoring', DoVectoring.Checked);
      WriteBool('graphs', 'System uptime', DoSysuptime.Checked);
      WriteBool('graphs', 'ShowSNRMperband', DoSNRMperband.Checked);
      WriteInteger('graphs', 'MarginClipValue', MarginClip.Value);
      WriteBool('graphs', 'Autoscale Y', AutoscaleBox.Checked);
      WriteInteger('graphs', 'MaxYValue', MaxYEdit.Value);
      WriteInteger('graphs', 'MinYValue', MinYEdit.Value);
      WriteInteger('graphs', 'CRCClipValue', CRCClip.Value);
      WriteInteger('graphs', 'FECClipValue', FECClip.Value);
      WriteInteger('graphs', 'LEFTRSClipValue', LEFTRSClip.Value);
      WriteInteger('graphs', 'PMClipValue', PMClip.Value);
      WriteInteger('graphs', 'rtxtxClipValue', rtxtxClip.Value);
      WriteInteger('graphs', 'rtxcClipValue', rtxcClip.Value);
      WriteInteger('graphs', 'rtxucClipValue', rtxucClip.Value);
      WriteInteger('graphs', 'Tones per pixel', TonesperPixel);
      {WriteInteger('graphs', 'Tones per pixel Bitswaps', TonesperpxEdit.Position);
      WriteInteger('graphs', 'Tones per pixel Bitloading', TonesperpxEditB.Position);
      WriteInteger('graphs', 'Tones per pixel HLog', TonesperpxEditH.Position);
      WriteInteger('graphs', 'Tones per pixel QLN', TonesperpxEditQ.Position);
      WriteInteger('graphs', 'Tones per pixel SNR', TonesperpxEditS.Position);}
      WriteBool('graphs', 'CRC Log Y', CRCLogY.Checked);
      WriteBool('graphs', 'FEC Log Y', FECLogY.Checked);
      WriteBool('graphs', 'LEFTRS Log Y', LEFTRSLogY.Checked);
      WriteBool('graphs', 'rtxtx Log Y', rtxtxLogY.Checked);
      WriteBool('graphs', 'rtxc Log Y', rtxcLogY.Checked);
      WriteBool('graphs', 'rtxuc Log Y', rtxucLogY.Checked);
      WriteBool('graphs', 'CRC per sample', PerSampleBoxC.Checked);
      WriteBool('graphs', 'FEC per sample', PerSampleBoxF.Checked);
      WriteBool('graphs', 'LEFTRS per sample', PerSampleBoxLEFTRS.Checked);
      WriteBool('graphs', 'rtxtx per sample', PerSampleBoxrtxtx.Checked);
      WriteBool('graphs', 'rtxc per sample', PerSampleBoxrtxc.Checked);
      WriteBool('graphs', 'rtxuc per sample', PerSampleBoxrtxuc.Checked);
      WriteBool('graphs', 'Connspeed Log Y', ConnspeedLogY.Checked);
      WriteBool('graphs', 'UseCRCClipping', CEnableClippingBox.Checked);
      WriteBool('graphs', 'UseFECClipping', FEnableClippingBox.Checked);
      WriteBool('graphs', 'UseLEFTRSClipping', LEFTRSEnableClippingBox.Checked);
      WriteBool('graphs', 'UsertxtxClipping', rtxtxEnableClippingBox.Checked);
      WriteBool('graphs', 'UsertxcClipping', rtxcEnableClippingBox.Checked);
      WriteBool('graphs', 'UsertxucClipping', rtxucEnableClippingBox.Checked);
      WriteBool('graphs', 'Legend on left', LegendLeft.Checked);
      WriteBool('graphs', 'MHideLegend', MHideLegend.Checked);
      WriteBool('graphs', 'SHideLegend', SHideLegend.Checked);
      WriteBool('graphs', 'PMHideLegend', PMHideLegend.Checked);
      WriteBool('graphs', 'CHideLegend', CHideLegend.Checked);
      WriteBool('graphs', 'FHideLegend', FHideLegend.Checked);
      WriteBool('graphs', 'LEFTRSHideLegend', LEFTRSHideLegend.Checked);
      WriteBool('graphs', 'rtxtxHideLegend', rtxtxHideLegend.Checked);
      WriteBool('graphs', 'rtxcHideLegend', rtxcHideLegend.Checked);
      WriteBool('graphs', 'rtxucHideLegend', rtxucHideLegend.Checked);
      WriteBool('graphs', 'UHideLegend', UHideLegend.Checked);
      WriteBool('graphs', 'EHideLegend', EHideLegend.Checked);
      WriteBool('graphs', 'SESHideLegend', SESHideLegend.Checked);
      WriteBool('graphs', 'SpBHideLegendUp', SpBHideLegendUp.Checked);
      WriteBool('graphs', 'SpBHideLegendDown', SpBHideLegendDown.Checked);
      WriteBool('graphs', 'BHideLegend', BHideLegend.Checked);
      WriteBool('graphs', 'BsHideLegend', BsHideLegend.Checked);
      WriteBool('graphs', 'SNRHideLegend', SNRHideLegend.Checked);
      WriteBool('graphs', 'QLNHideLegend', QLNHideLegend.Checked);
      WriteBool('graphs', 'HLogHideLegend', HLogHideLegend.Checked);
      WriteBool('graphs', 'Max attainable up', ShowMaxUpBox.Checked);
      WriteBool('graphs', 'Max attainable down', ShowMaxDownBox.Checked);
      WriteBool('graphs', 'Set max ES value', ESSetMax.Checked);
      WriteInteger('graphs', 'Max ES index', ESMax.ItemIndex);
      WriteBool('graphs', 'Scale snapshots', ScaleBox.Checked);
      WriteInteger('graphs', 'Snapshot height', HeightEdit.Value);
      WriteInteger('graphs', 'Snapshot width', WidthEdit.Value);
    except
      MiscString := 'Error saving graph settings';
      LogEvent(leMisc);
      ErrorOccurred := True;
    end;
    try
      EraseSection('colours');
      WriteInteger('colours', 'Graphcolour', Graphcolour.Selected);
      WriteInteger('colours', 'Upcolour', Upcolour.Selected);
      WriteInteger('colours', 'Downcolour', Downcolour.Selected);
      WriteInteger('colours', 'UpPlusDowncolour', UpPlusDowncolour.Selected);
      WriteInteger('colours', 'Othercolour', Othercolour.Selected);
      WriteInteger('colours', 'SNRColour', SNRColour.Selected);
      WriteInteger('colours', 'Limitcolour', LimitColour.Selected);
      WriteInteger('colours', 'U0D0Colour', Colour0.Selected);
      WriteInteger('colours', 'U1D1Colour', Colour1.Selected);
      WriteInteger('colours', 'U2D2Colour', Colour2.Selected);
      WriteInteger('colours', 'U3D3Colour', Colour3.Selected);
      WriteInteger('colours', 'U4D4Colour', Colour4.Selected);
    except
      MiscString := 'Error saving colour settings';
      LogEvent(leMisc);
      ErrorOccurred := True;
    end;
    try
      EraseSection('snapshots');
      WriteString('snapshots', 'Snapshotdir', SnapshotsPath);
      WriteBool('snapshots', 'Dateorder', Dateorder.Checked);
      WriteBool('snapshots', 'UseHoursperpage', SameAsBox.Checked);
      WriteInteger('snapshots', 'Hourspersnapshot', Hourspersnapshot);
      WriteBool('snapshots', 'Start of recording', StartofRecording.Checked);
      WriteBool('snapshots', 'First at', FirstAtTime.Checked);
      WriteInteger('snapshots', 'Hours', SnapshotHours.Value);
      WriteInteger('snapshots', 'Minutes', SnapshotMinutes.Value);
      WriteBool('snapshots', 'SNRsnapshot', SNRsnap.Checked);
      WriteBool('snapshots', 'SNRpbsnapshot', SNRMpbsnap.Checked);
      WriteBool('snapshots', 'Speedsnapshot', Speedsnap.Checked);
      WriteBool('snapshots', 'Bitsnapshot', Bitsnap.Checked);
      WriteBool('snapshots', 'CRCsnapshot', CRCsnap.Checked);
      WriteBool('snapshots', 'SESsnapshot', SESsnap.Checked);
      WriteBool('snapshots', 'FECsnapshot', FECsnap.Checked);
      WriteBool('snapshots', 'ESsnapshot', ESsnap.Checked);
      WriteBool('snapshots', 'SNRptsnapshot', SNRptsnap.Checked);
      WriteBool('snapshots', 'Swapsptsnapshot', Swapptsnap.Checked);
      WriteBool('snapshots', 'Swapspmsnapshot', Swappmsnap.Checked);
      WriteBool('snapshots', 'QLNsnapshot', QLNsnap.Checked);
      WriteBool('snapshots', 'HLogsnapshot', HLogsnap.Checked);
      WriteBool('snapshots', 'TrafficSnapshot', TrafficSnap.Checked);
      WriteBool('snapshots', 'PerMinute', PerMinute.Checked);
      WriteBool('snapshots', 'PerDay', PerDay.Checked);
      WriteBool('snapshots', 'PerMonth', PerMonth.Checked);
      WriteBool('snapshots', 'ConnectionStatsText', ConnectionStatsText.Checked);
      WriteBool('snapshots', 'SNRText', SNRText.Checked);
      WriteBool('snapshots', 'BitloadingText', BitloadingText.Checked);
      WriteBool('snapshots', 'pbParamsText', pbParamsText.Checked);
      WriteBool('snapshots', 'QLNText', QLNText.Checked);
      WriteBool('snapshots', 'HLogText', HLogText.Checked);
      WriteBool('snapshots', 'AttenuationLogText', AttenuationLogText.Checked);
      WriteBool('snapshots', 'AverageErrorsText', AverageErrorsText.Checked);
      WriteBool('snapshots', 'TrafficText', TrafficText.Checked);
      WriteBool('snapshots', 'EventlogSnapshot', EventlogText.Checked);
      WriteBool('snapshots', 'CustomCommandsSnapshot', CustomCommandsText.Checked);
      WriteBool('snapshots', 'StatsSnapshot', StatsText1.Checked);
      WriteBool('snapshots', 'SnapOnExit', SnapallBox.Checked);
      WriteBool('snapshots', 'SnapConfiguredOnExit', SnapallConfiguredBox.Checked);
      WriteBool('snapshots', 'Daily folders', DailyBox.Checked);
      WriteBool('snapshots', 'Category folders', CategoryBox.Checked);
      WriteInteger('snapshots', 'LogNumber', LogNum);
      WriteBool('snapshots', 'DeleteOlder', DeleteOlderBox.Checked);
      WriteInteger('snapshots', 'Olderthan', OlderDays.Value);
      WriteBool('snapshots', 'Upload snapshots', DoUploadSS.Checked);
      WriteString('snapshots', 'UploadURL', SSUploadURL.Text);
      WriteString('snapshots', 'UploadDir', SSUploadDir.Text);
      WriteString('snapshots', 'Username', SSUploadUsername.Text);
      S := Encrypt(SSUploadPassword.Text);
      WriteString('snapshots', 'Password', S);
      WriteString('snapshots', 'FTP port', SSFTPPort.Text);
      WriteBool('snapshots', 'FTP-TLS', SSTLSButton.Checked);
      WriteBool('snapshots', 'FTPS', SSFTPSButton.Checked);
      WriteBool('snapshots', 'SFTP', SSSFTPButton.Checked);
    except
      MiscString := 'Error saving snapshot settings';
      LogEvent(leMisc);
      ErrorOccurred := True;
    end;
    try
      EraseSection('misc');
      WriteBool('misc', 'Eventlog', Eventlogbox.Checked);
      WriteBool('misc', 'SaveEventLog', SaveonexitBox.Checked);
      WriteBool('misc', 'SaveEventLogAfter', SaveAfterBox.Checked);
      WriteBool('misc', 'Confirm close', ConfirmCloseBox.Checked);
      WriteBool('misc', 'Minimise to tray', MinimisetotrayBox.Checked);
      WriteBool('misc', 'Minimise on close', MinimiseoncloseBox.Checked);
      WriteString('misc', 'DownloadDir', DownloadDirEdit.Directory);
      WriteBool('misc', 'Diagnostics', DiagnosticsBox.Checked);
      WriteBool('misc', 'Planned events', PlannedEventsBox.Checked);
      WriteBool('misc', 'Pause on error', PauseBox.Checked);
      WriteBool('misc', 'ShowHints', ShowHintsBox.Checked);
      WriteBool('misc', 'Autostart', AutostartBox.Checked);
      WriteBool('misc', 'Start minimised', StartMinimisedBox.Checked);
      WriteBool('misc', 'Start maximised', StartMaximisedBox.Checked);
      WriteInteger('misc', 'ErrorNumber', ErrorNumber);
      WriteBool('misc', 'AutoRestart', RestartBox.Checked);
      WriteInteger('misc', 'RestartTime', RestartEdit.Value);
      WriteBool('misc', 'Attenuation logging', DoLogging);
    except
      MiscString := 'Error saving misc settings';
      LogEvent(leMisc);
      ErrorOccurred := True;
    end;
    try
      EraseSection('errors');
      WriteString('errors', 'NewESUp', IntToStr(NewESUp));
      WriteString('errors', 'NewESDown', IntToStr(NewESDown));
      WriteString('errors', 'OldESUp', IntToStr(OldESUp));
      WriteString('errors', 'OldESDown', IntToStr(OldESDown));
      WriteInteger('errors', 'Last year', LastYear);
      WriteInteger('errors', 'Last month', LastMonth);
      WriteInteger('errors', 'Last day', LastDay);
      WriteInteger('errors', 'Last hour', LastHour);
    except
      MiscString := 'Error saving ES data';
      LogEvent(leMisc);
      ErrorOccurred := True;
    end;
    try
      EraseSection('updates');
      WriteBool('updates', 'Check updates', CheckUpdateBox.Checked);
      WriteString('updates', 'LastCheck', DateTimeToStr(LastUpdateCheck));
      WriteInteger('updates', 'Frequency', FrequencyBox.Value);
    except
      MiscString := 'Error saving update information';
      LogEvent(leMisc);
      ErrorOccurred := True;
    end;
    try
      EraseSection('custom');
      for N := 1 to 4 do
      begin
        WriteString('custom', 'Command' + IntToStr(N), Commands[N].Text);
        S := '';
        for N1 := 1 to 6 do
          if CommandNumbers[N, N1].Checked then S := S + '1' else S := S + '0';
        WriteString('custom', 'Include' + IntToStr(N), S);
      end;
      WriteBool('custom', 'InEventLog', EventLogButton.Checked);
      WriteInteger('custom', 'Hours', CommandHours.Value);
      WriteInteger('custom', 'Minutes', CommandMinutes.Value);
    except
      MiscString := 'Error saving custom command settings';
      LogEvent(leMisc);
      ErrorOccurred := True;
    end;
    try
      EraseSection('alerts');
      WriteBool('alerts', 'Send event log', SendEventlogBox.Checked);
      WriteBool('alerts', 'SNRM email', SNRMemail.Checked);
      WriteBool('alerts', 'CRC email', CRCemail.Checked);
      WriteBool('alerts', 'FEC email', FECemail.Checked);
      WriteBool('alerts', 'ES email', ESemail.Checked);
      WriteBool('alerts', 'FTP email', ESemail.Checked);
      WriteBool('alerts', 'Resync email', FTPemail.Checked);
      WriteBool('alerts', 'SNRM', SNRMfallBox.Checked);
      WriteString('alerts', 'SNRM trigger', SNRMTrigger.Text);
      WriteBool('alerts', 'SNRM return', SNRMreturnBox.Checked);
      WriteString('alerts', 'SNRM return trigger', SNRMreturnTrigger.Text);
      WriteBool('alerts', 'CRC rise', CRCriseBox.Checked);
      WriteString('alerts', 'CRC trigger', CRCTrigger.Text);
      WriteBool('alerts', 'CRC per minute', CRCpm.Checked);
      WriteBool('alerts', 'FEC rise', FECriseBox.Checked);
      WriteString('alerts', 'FEC trigger', FECTrigger.Text);
      WriteBool('alerts', 'FEC per minute', FECpm.Checked);
      WriteBool('alerts', 'ES rise', ESriseBox.Checked);
      WriteString('alerts', 'ES trigger', ESTrigger.Text);
      WriteBool('alerts', 'Keep trigger levels', KeepLevelsBox.Checked);
      if UpSNRM.Checked then N := 1
      else if DownSNRM.Checked then N := 2
      else N := 3;
      WriteInteger('alerts', 'SNRM option', N);
      WriteBool('alerts', 'FTP fail', FTPFailBox.Checked);
      WriteBool('alerts', 'Resync', ResyncBox.Checked);
      WriteString('alerts', 'Email address', EmailAddress.Text);
      WriteString('alerts', 'Email subject', EmailSubject.Text);
      WriteString('alerts', 'SMTP server', SMTPserver.Text);
      WriteString('alerts', 'Username', EmailUsername.Text);
      WriteString('alerts', 'Email from', EmailFrom.Text);
      WriteString('alerts', 'Password', Encrypt(EMailPassword.Text));
      WriteString('alerts', 'SMTP port', SMTPPort.Text);
      if PlainButton.Checked then EncryptionType := 0
      else if SSLButton.Checked then EncryptionType := 1
      else EncryptionType := 2;
      WriteInteger('alerts', 'Encryption type', EncryptionType);
    except
      MiscString := 'Error saving alerts settings';
      LogEvent(leMisc);
      ErrorOccurred := True;
    end;

    try
      EraseSection('webserver');
      WriteString('webserver', 'WSdir', WebserverDirEdit.Directory);
      WriteBool('webserver', 'Enable webserver', WebserverBox.Checked);
      WriteBool('webserver', 'Save webserver files', WebserverSaveBox.Checked);
      WriteInteger('webserver', 'Refresh period', RefreshPeriod);
      WriteBool('webserver', 'Custom refresh', CustomButton.Checked);
      WriteString('webserver', 'Port', WebPortEdit.Text);
      WriteBool('webserver', 'SNRM', SNRMweb.Checked);
      WriteBool('webserver', 'SNRMpb', SNRMpbweb.Checked);
      WriteBool('webserver', 'Connspeed', Connspeedweb.Checked);
      WriteBool('webserver', 'Bitloading', Bitloadingweb.Checked);
      WriteBool('webserver', 'CRC', CRCweb.Checked);
      WriteBool('webserver', 'ES', ESweb.Checked);
      WriteBool('webserver', 'SES', SESweb.Checked);
      WriteBool('webserver', 'FEC', FECweb.Checked);
      WriteBool('webserver', 'Stats', Statsweb.Checked);
      WriteBool('webserver', 'Connection stats', Fullstatsweb.Checked);
      WriteBool('webserver', 'Combined page', Combineweb.Checked);
      WriteBool('webserver', 'Delete on disable', DeleteWebBox.Checked);
      WriteBool('webserver', 'All active', AllActiveWeb.Checked);
      WriteBool('webserver', 'All configured', AllConfiguredWeb.Checked);
      WriteBool('webserver', 'All text', AllTextWeb.Checked);
      WriteBool('webserver', 'Do upload', DoUploadWS.Checked);
      WriteInteger('webserver', 'Upload rate', WSUploadRate.Value);
      WriteString('webserver', 'UploadURL', WSUploadURL.Text);
      WriteString('webserver', 'UploadDir', WSUploadDir.Text);
      WriteString('webserver', 'Username', WSUploadUsername.Text);
      WriteString('webserver', 'Password', Encrypt(WSUploadPassword.Text));
      WriteString('webserver', 'FTP port', WSFTPPort.Text);
      WriteBool('webserver', 'FTP-TLS', WSTLSButton.Checked);
      WriteBool('webserver', 'FTPS', WSFTPSButton.Checked);
      WriteBool('webserver', 'SFTP', WSSFTPButton.Checked);
    except
      MiscString := 'Error saving webserver configuration';
      LogEvent(leMisc);
      ErrorOccurred := True;
    end;

    EraseSection('mydslwebstats');       //Transitional
    EraseSection('DataStore');
    WriteBool('DataStore', 'Do data', DoSaveData.Checked);
    WriteBool('DataStore', 'Do upload', DoUploadData.Checked);
    WriteBool('DataStore', 'Single file', SingleButton.Checked);
    WriteString('DataStore', 'DataStore path', DataFolderEdit.Directory);
    WriteBool('DataStore', 'Bitloading', DSbits.Checked);
    WriteBool('DataStore', 'SNR per tone', DSsnr.Checked);
    WriteBool('DataStore', 'HLog', DShlog.Checked);
    WriteBool('DataStore', 'QLN', DSqln.Checked);
    //WriteBool('DataStore', 'BitsUpload', DSbitsUpload.Checked);
    //WriteBool('DataStore', 'SNRUpload', DSsnrUpload.Checked);
    //WriteBool('DataStore', 'HLogUpload', DShlogUpload.Checked);
    //WriteBool('DataStore', 'QLNUpload', DSqlnUpload.Checked);
    WriteBool('DataStore', 'Upload data', DoUploadData.Checked);
    WriteString('DataStore', 'UploadURL', DSUploadURL.Text);
    WriteString('DataStore', 'UploadDir', DSUploadDir.Text);
    WriteString('DataStore', 'Username', DSUploadUsername.Text);
    WriteString('DataStore', 'Password', Encrypt(DSUploadPassword.Text));
    WriteString('DataStore', 'FTP port', DSFTPPort.Text);
    WriteBool('DataStore', 'FTP-TLS', DSTLSButton.Checked);
    WriteBool('DataStore', 'FTPS', DSFTPSButton.Checked);
    WriteBool('DataStore', 'SFTP', DSSFTPButton.Checked);

    try
      EraseSection('XML');
      WriteBool('XML', 'SNRM', SNRSaveBox.Checked);
      WriteBool('XML', 'ConSpeed', SpeedSaveBox.Checked);
      WriteBool('XML', 'CRC', CRCSaveBox.Checked);
      WriteBool('XML', 'FEC', FECSaveBox.Checked);
      WriteBool('XML', 'Bitswap', BitswapSaveBox.Checked);
      WriteBool('XML', 'SNRMx', SNRSaveEBox.Checked);
      WriteBool('XML', 'ConSpeedx', SpeedSaveEBox.Checked);
      WriteBool('XML', 'CRCx', CRCSaveEBox.Checked);
      WriteBool('XML', 'FECx', FECSaveEBox.Checked);
      WriteBool('XML', 'Bitswapx', BitswapSaveEBox.Checked);
    except
      MiscString := 'Error saving XML settings';
      LogEvent(leMisc);
      ErrorOccurred := True;
    end;
    try
      EraseSection('positions');
      WriteInteger('positions', 'Left', Self.Left);
      WriteInteger('positions', 'Top', Self.Top);
      WriteInteger('positions', 'Width', Self.Width);
      WriteInteger('positions', 'Height', Self.Height);
    except
      MiscString := 'Error saving window positions';
      LogEvent(leMisc);
      ErrorOccurred := True;
    end;
    try
      Free;
    except
      on E: Exception do
      begin
        MiscString := 'Unknown error when saving configuration to ' + IniPath;
        LogEvent(leMisc);
      end;
    end;
  end;
  if ErrorOccurred then
  begin
    ManualSaveTextPages(SaveLogButton);
    MiscString := 'Error while saving configuration. Event log saved as ' + MiscString; //MiscString was set to SnapDir by ManualSaveTextPages
    if DiagnosticsBox.Checked then ShowMessage(MiscString);
  end;
end;

//Get confirmation of program close
procedure Tmainform.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  if DoQuit or not ConfirmCloseBox.Checked then
  begin
    CanClose := True;
    Exit;
  end;
  if MessageDlg('Close program', 'Do you want to close the program?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
    DoQuit := True
  else begin
    CanClose := False;
    DoQuit := False;
  end;
end;

//Tidy up when closing the program
procedure Tmainform.FormClose(Sender: TObject; var CloseAction: TCloseAction);
var
  N, N1: Word;
begin
  if MinimiseoncloseBox.Checked and not DoQuit then
  begin
    CloseAction := caMinimize;
    Exit;
  end;
  if ActivityLight.Visible then
  begin
    CloseAction := caNone;
    DelayedClose := True;
    ActivityLight.Caption := 'Closing...';
    Exit;
  end;
  Timer1.Enabled := False;
  for N := 1 to 4 do
    if CommandNumbers[N, 4].Checked then
    begin
      ActivityLight.Caption := 'Custom command...';
      ActivityLight.Show;
      TransferCommands;
      TelnetForm.DoCustom[4] := True;
      TelnetForm.FinalCustom := True;
      TransferLoginValues;
      ProcessOperation(Self, toCollectstats);
      ManualSaveTextPages(SaveTextButton);
      Break;
    end;
  if SaveonexitBox.Checked or ErrorOccurred then
    ManualSaveTextPages(SaveLogButton);   //Event log
  if SnapallBox.Checked then MsnapButtonClick(SnapallBox);  //Snap all active graphs
  if SnapallConfiguredBox.Checked then MsnapButtonClick(SnapallConfiguredBox);  //Snap all snapshot-configured graphs
  if SNRSaveEBox.Checked then MsnapButtonClick(SNRSaveButton);   //Save selected graphs as XML files
  if SpeedSaveEBox.Checked then MsnapButtonClick(SpeedSaveButton);
  if CRCSaveEBox.Checked then MsnapButtonClick(CRCSaveButton);
  if FECSaveEBox.Checked then MsnapButtonClick(FECSaveButton);
  if BitswapSaveEBox.Checked then MsnapButtonClick(BitswapSaveButton);
  WriteTrafficData;
  WriteInifile;
  if ESChart.SaveHourlyES(Homepath + 'es_data.txt') then
  begin
    {$IFDEF Windows}
    if FileExists(Homepath + 'es.data') then Deletefile(PChar(Homepath + 'es.data'));
    {$ELSE}
    if FileExists(Homepath + 'es.data') then Deletefile(Homepath + 'es.data');
    {$ENDIF}
  end;
  WriteErrorfile1;
  if FServerThread <> nil then FServerThread.Terminate;
  FHandler.Free;
  for N := 1 to 10 do
  begin
    Prompts[N].Free;
    Responses[N].Free;
    if N < 5 then
    begin
      Commands[N].Free;
    end;
  end;
  for N := 1 to 4 do
    for N1 := 1 to 6 do
      CommandNumbers[N, N1].Free;
  UpdateList.Free;
  VendorList.Free;
  LinediagsList.Free;
  RouterTrafficList.Free;
  VersionList.Free;
  UploadList.Free;
end;

//Process Exit or Quit button press
procedure Tmainform.ExitbuttonClick(Sender: TObject);
begin
  if Sender = QuitItem then
  begin
    DoQuit := True;
    Close;
    Exit;
  end;
  if MinimiseoncloseBox.Checked then
  begin
    Application.Minimize;
    Hide;
    Exit;
  end;
  Close;
end;

//Process click of Minimise in systray menu
procedure Tmainform.MinimiseItemClick(Sender: TObject);
begin
  Application.Minimize;
  Hide;
end;

//Restore minimised program
procedure Tmainform.TrayIcon1Click(Sender: TObject);
begin
  WindowState := wsNormal;
  Show;
end;

//Review current window state
procedure Tmainform.Timer4Timer(Sender: TObject);
begin
  if WindowState = wsMinimized then
  begin
    if MinimisetotrayBox.Checked then
    begin
      Hide;
    end;
  end;
  Timer4.Enabled := False;
end;

//Hide the taskbar entry on minimise (only works with Windows)
procedure Tmainform.ApplicationProperties1Minimize(Sender: TObject);
begin
  {$IFDEF Windows}
  if MinimiseToTrayBox.Checked then Hide;
  {$ENDIF}
end;

// **** LOGIN CONFIGURATION ****

//Process change of telnet port
procedure Tmainform.PorteditChange(Sender: TObject);
begin
  if PortEdit.Text > '' then TelnetUnit1.PortNumber := StrToInt(PortEdit.Text)
  else TelnetUnit1.PortNumber := 23;
end;

//Process change of special login telnet port
procedure Tmainform.SPortEditChange(Sender: TObject);
begin
  if SPortEdit.Text > '' then TelnetUnit1.PortNumber := StrToInt(SPortEdit.Text)
  else TelnetUnit1.PortNumber := 23;
end;

//Process change of CLI command prefix (basic config)
procedure Tmainform.CommandboxChange(Sender: TObject);
begin
  IsRalink := (CommandBox.Items[CommandBox.ItemIndex] = 'adslphxcmd');
  DoSaveData.Enabled := not IsRalink;
  DoUploadData.Enabled := not IsRalink;
  CustomPrefixLabel.Enabled := (CommandBox.ItemIndex = DefaultIndex - 1);
  CustomPrefixEdit.Enabled := (CommandBox.ItemIndex = DefaultIndex - 1);
  if not DoSaveData.Enabled then
  begin
    DoSaveData.Checked := False;
  end;
end;

//Process change of CLI command prefix (special login)
procedure Tmainform.SCommandBoxChange(Sender: TObject);
begin
  SCustomPrefixLabel.Enabled := (SCommandBox.ItemIndex = DefaultIndex - 1);
  SCustomPrefixEdit.Enabled := (SCommandBox.ItemIndex = DefaultIndex - 1);
end;

//Process change of second stage option
procedure Tmainform.SecondstageChange(Sender: TObject);
begin
  Shellcommand.Enabled := Secondstage.Checked;
end;

//Process change of modem/router model
procedure Tmainform.RouterBoxChange(Sender: TObject);
var
  S: string;
  N: Integer;
begin
  with RouterBox do
  if ItemIndex > 0 then
  begin
    S := DataArray[ItemIndex].Model;
    if (not HG635messagedone) and (S = 'HG635') then
    begin
      HG635messagedone := True;
      S := 'If you want to use custom commands or tweaks you may need'
        + sLineBreak + 'to use Special Login and select "Huawei HG635 (root)".'
        + sLineBreak + sLineBreak + '(This message will not be shown again)';
      MessageDlg('Please note', S, mtInformation, [mbOK], '');
    end;
    IPEdit.Text := DataArray[ItemIndex].IPAddress;
    NameEdit.Text := DataArray[ItemIndex].LoginName;
    PasswordEdit.Text := DataArray[ItemIndex].Password;
    PortEdit.Text := '23';
    SecondStage.Checked := DataArray[ItemIndex].SecondStage;
    ShellCommand.Text := DataArray[ItemIndex].ShellCommand;
    S := DataArray[ItemIndex].CommandPrefix;
    N := 0;
    CommandBox.ItemIndex := 0;
    if S > '' then
    repeat
      if S = CommandBox.Items[N] then Break;
      Inc(N);
    until N = CommandBox.Items.Count;
    if N < CommandBox.Items.Count then CommandBox.ItemIndex := N;
  end;
  ThomBox.Checked := (RouterBox.Items[RouterBox.ItemIndex][1..11] = 'Technicolor');
  IsHG622Box.Checked := (RouterBox.Items[RouterBox.ItemIndex] = 'Huawei HG622')
                     or (RouterBox.Items[RouterBox.ItemIndex] = 'Huawei HG635')
                     or (RouterBox.Items[RouterBox.ItemIndex] = 'Huawei HG658c');
  TelnetForm.IsHG622 := IsHG622Box.Checked;
  MiscString := 'The configured modem/router is ' + RouterBox.Items[RouterBox.ItemIndex];
  LogEvent(leMisc);
end;

//Process change of modem/router model for traffic monitoring
procedure Tmainform.RouterTrafficBoxChange(Sender: TObject);
var
  Index, N: Word;
begin
  Index := RouterTrafficBox.ItemIndex;
  with CommandGroup do
  begin
    for N := 0 to Items.Count - 1 do
    if Items[N] = TrafficConfigArray[Index].Command then
    begin
      ItemIndex := N;
      Break;
    end;
  end;
  with InterfaceGroup do
  begin
    for N := 0 to Items.Count - 1 do
    if Items[N] = TrafficConfigArray[Index].IFace then
    begin
      ItemIndex := N;
      if Items[N] = 'Other...' then OtherInterfaceEdit.Text := TrafficConfigArray[Index].Other;
      Break;
    end;
  end;
  ReverseBox.Checked := (TrafficConfigArray[Index].Reversed = 'Y');
end;

//Process change of special modem/router model
procedure Tmainform.SRouterBoxChange(Sender: TObject);
var
  Index, N, Offset: Word;
  S: string;
begin
  Offset := NumSpecials + 1;
  Index := SRouterBox.ItemIndex;
  if Index = Offset then Exit;
  if Index > 0 then
  begin
    if Index < Offset then  //Special modem/router not in main list
    begin
      for N := 1 to 10 do
      begin
        Prompts[N].Text := SpecialDataArray[Index].Prompts[N];
        Responses[N].Text := SpecialDataArray[Index].Responses[N];
      end;
      SIPEdit.Text := SpecialDataArray[Index].IPAddress;
      SPortEdit.Text := '23';
      S := SpecialDataArray[Index].CommandPrefix;
    end
    else begin   //Regular modem/router in main list
      N := 1;
      Index := Index - Offset;
      Prompts[N].Text := ':';
      Responses[N].Text := DataArray[Index].LoginName;
      Inc(N);
      Prompts[N].Text := ':';
      Responses[N].Text := DataArray[Index].Password;
      if DataArray[Index].SecondStage then
      begin
        Inc(N);
        Prompts[N].Text := '>';
        Responses[N].Text := DataArray[Index].ShellCommand;
      end;
      repeat
        Inc(N);
        Prompts[N].Text := '';
        Responses[N].Text := '';
      until N = 10;
      SIPEdit.Text := DataArray[Index].IPAddress;
      SPortEdit.Text := '23';
      S := DataArray[Index].CommandPrefix;
    end;
    N := 0;
    SCommandBox.ItemIndex := DefaultIndex;
    if S > '' then
    repeat
      if S = SCommandBox.Items[N] then Break;
      Inc(N);
    until N = SCommandBox.Items.Count;
    if N < SCommandBox.Items.Count then SCommandBox.ItemIndex := N;
  end;
end;

//Respond to change of Thomson model option
procedure Tmainform.ThomBoxChange(Sender: TObject);
begin
  if ThomBox.Checked then
  begin
    SecondStage.Enabled := False;
    Label7.Enabled := False;
    ShellCommand.Enabled := False;
    AdvancedTweaksTab.TabVisible := False;
    CommandGroup.ItemIndex := 2;
    DSsnr.Checked := False;
    DSsnr.Enabled := False;
    DShlog.Checked := False;
    DShlog.Enabled := False;
    DSqln.Checked := False;
    DSqln.Enabled := False;
  end
  else begin
    SecondStage.Enabled := True;
    Label7.Enabled := True;
    ShellCommand.Enabled := True;
    AdvancedTweaksTab.TabVisible := True;
    DoSaveData.Enabled := not IsRalink;
    DoUploadData.Enabled := not IsRalink;
    DSsnr.Enabled := not IsRalink;
    DShlog.Enabled := not IsRalink;
    DSqln.Enabled := not IsRalink;
  end;
  SetOptions;
end;

//Process change of HG622 override options
procedure Tmainform.IsHG622BoxChange(Sender: TObject);
begin
  TelnetForm.IsHG622 := IsHG622Box.Checked;
end;

//Respond to change of annex M option
procedure Tmainform.AnnexmChange(Sender: TObject);
begin
  Modechange;
end;

//Toggle password reveal
procedure Tmainform.RevealBoxChange(Sender: TObject);
begin
  if RevealBox.Checked then PasswordEdit.EchoMode := emNormal else PasswordEdit.EchoMode := emPassword;
end;

//Toggle special login password reveal
procedure Tmainform.RevealBoxSChange(Sender: TObject);
begin
  if RevealBoxS.Checked then Responses[2].EchoMode := emNormal else Responses[2].EchoMode := emPassword;
end;

//Limit IP address text to permissible characters
procedure Tmainform.IPeditKeyPress(Sender: TObject; var Key: char);
begin
  if not (Key in ['0'..'9', '.', Chr(8), Chr(127)]) then Key := Chr(0);
end;

//Limit port number text to numerals
procedure Tmainform.PorteditKeyPress(Sender: TObject; var Key: char);
begin
  if not (Key in ['0'..'9', chr(8), chr(127)]) then Key := chr(0);
end;

//Process change of special login status
procedure Tmainform.SpecialLoginChange(Sender: TObject);
begin
  SpecialLabel.Visible := SpecialLogin.Checked;
  DisableSpecial.Enabled := SpecialLogin.Checked;
  DisableSpecial.Checked := not SpecialLogin.Checked;
end;

//Disable special login from normal login tab
procedure Tmainform.DisableSpecialChange(Sender: TObject);
begin
  if DisableSpecial.Checked then SpecialLogin.Checked := False;
end;

//Reset the login configuration to default values
procedure Tmainform.ResetLoginButtonClick(Sender: TObject);
begin
  if RouterBox.Items.Count > 0 then RouterBox.ItemIndex := 0 else RouterBox.ItemIndex := -1;
  IPEdit.Text := '';
  NameEdit.Text := '';
  PasswordEdit.Text := '';
  PortEdit.Text := '23';
  SecondStage.Checked := False;
  ShellCommand.Text := '';
  CommandBox.ItemIndex := DefaultIndex;
  ThomBox.Checked := False;
  RevealBox.Checked := False;
  IsHG622Box.Checked := False;
  SpecialLogin.Checked := False;
end;

// **** SAMPLING ****

//Initiate a new sample
procedure Tmainform.Timer1Timer(Sender: TObject);
var
  Snapped: Boolean;
  Dir, Thisday, S: string;
  Minutes, Seconds, N: Word;
  T: TDateTime;
  FileList: TStringList;

  procedure TakeSnapshots;
  var
    Loop: Word;
  begin
    Snapped := False;
    Auto := True;
    Dir := IncludeTrailingPathDelimiter(SnapshotsPath);
    TestOK := TestWrite(Dir);
    if TestOK then
    begin
      LastSnapshot := Now;
      if SNRsnap.Checked then
      begin
        MsnapbuttonClick(Msnapbutton);
        Snapped := True;
      end;
      if SNRMpbsnap.Checked then
      begin
        MsnapbuttonClick(SpbSnapButtonDown);
        MsnapbuttonClick(SpbSnapButtonUp);
        Snapped := True;
      end;
      if Speedsnap.Checked then
      begin
        MsnapbuttonClick(Ssnapbutton);
        Snapped := True;
      end;
      if Bitsnap.Checked then
      begin
        MsnapbuttonClick(Bsnapbutton);
        Snapped := True;
      end;
      if Swapptsnap.Checked then
      begin
        MsnapbuttonClick(SWsnapbutton);
        Snapped := True;
      end;
      if Swappmsnap.Checked then
      begin
        MsnapbuttonClick(PMsnapbutton);
        Snapped := True;
      end;
      if SNRptsnap.Checked then
      begin
        MsnapbuttonClick(SNRsnapbutton);
        Snapped := True;
      end;
      if CRCsnap.Checked then
      begin
        MsnapbuttonClick(Csnapbutton);
        Snapped := True;
      end;
      if SESsnap.Checked then
      begin
        MsnapbuttonClick(SESsnapbutton);
        Snapped := True;
      end;
      if FECsnap.Checked then
      begin
        MsnapbuttonClick(Fsnapbutton);
        Snapped := True;
      end;
      if QLNsnap.Checked then
      begin
        MsnapbuttonClick(Qsnapbutton);
        Snapped := True;
      end;
      if HLogsnap.Checked then
      begin
        MsnapbuttonClick(Hsnapbutton);
        Snapped := True;
      end;
      if TrafficSnap.Checked then
      begin
        if PerMinute.Checked then TrafficPerMin.Checked := True
        else if PerDay.Checked then DailyButton.Checked := True
        else MonthlyButton.Checked := True;
        for Loop := 1 to 50 do   //Give it half a second to stabilise
        begin
          Sleep(10);
          Application.ProcessMessages;
        end;
        MSnapButtonClick(USnapButton);
      end;
      if ESsnap.Checked then
      begin
         MsnapbuttonClick(ESnapButton);
         Snapped := True;
      end;
      if CustomCommandsText.Checked then
      begin
        ManualSaveTextPages(SaveTextButton);
        Snapped := True;
      end;
      if AttenuationLogText.Checked then
      begin
        ManualSaveTextPages(SaveLogButton1);
        Snapped := True;
      end;
      if AverageErrorsText.Checked and DoAverages.Checked then
      begin
        ManualSaveTextPages(SaveAveragesButton);
        Snapped := True;
      end;
      if StatsText1.Checked then
      begin
        ManualSaveTextPages(SaveStatsButton);
        Snapped := True;
      end;
      if ConnectionStatsText.Checked then
      begin
        ManualSaveTextPages(SaveTDStats);
        Snapped := True;
      end;
      if SNRText.Checked then
      begin
        ManualSaveTextPages(SaveTDSNR);
        Snapped := True;
      end;
      if QLNText.Checked then
      begin
        ManualSaveTextPages(SaveTDQLN);
        Snapped := True;
      end;
      if BitloadingText.Checked then
      begin
        ManualSaveTextPages(SaveTDBits);
        Snapped := True;
      end;
      if pbParamsText.Checked then
      begin
        ManualSaveTextPages(SaveTDParams);
        Snapped := True;
      end;
      if HLogText.Checked then
      begin
        ManualSaveTextPages(SaveTDHLog);
        Snapped := True;
      end;
      if TrafficText.Checked then
      begin
        ManualSaveTextPages(SaveTDTraffic);
        Snapped := True;
      end;
      if EventLogText.Checked then
      begin
        ManualSaveTextPages(SaveLogButton);
        Snapped := True;
      end;
      //SaveTextPages;
      if (Snapped or EventLogText.Checked)
      and PlannedEventsBox.Checked then LogEvent(leAutosnapshot); //Log this event before snapping the event log
      if EventlogText.Checked then ManualSaveTextPages(SavelogButton);
    end;
    Auto := False;
    TestOK := False;
  end;

begin
  T := Now;
  if T < SampleTime then Exit;   //Not yet time for next sample
  {$IFNDEF Windows}
  if MinutesBetween(T, LastTimeCheck) >= 30 then //Do a time check every 30 minutes
  begin
    LastTimeCheck := T;
    ReadTimezoneFile(GetTimezoneFile);
    GetLocalTimezone(fpTime);
  end;
  {$ENDIF}
  Snapped := False;
  Seconds := SampleRateEdit.Value;
  if Seconds >= 60 then
  begin
    N := Seconds;
    Seconds := N mod 60;
    Minutes := N div 60;
  end
  else Minutes := 0;
  repeat      //If necessary, bypass sample times missed during PC sleep or hibernation
    SampleTime := SampleTime + EncodeTimeInterval(0, Minutes, Seconds, 0);
  until SampleTime > T;
  if MinutesBetween(T, LastUpdateCheck) > 5 then //Hide response to manual update check after 5 minutes
  begin
    VersionResult.Hide;
    NoResult.Hide;
  end;
  FillGap;   //Check if time since previous sample > 2 sample periods, and if so, fill gap with dotted lines
  for N := 1 to 4 do
    if (CommandNumbers[N, 5].Checked and (MinutesBetween(T, LastCustom) >= EveryMinutes)) then  //Check if it's time to issue custom commands
    begin
      TransferCommands;
      TelnetForm.DoCustom[5] := True;
      LastCustom := Now;
      Break;
    end;
  for N := 1 to 4 do          //Has user pressed "Send now"?
    if CommandNumbers[N, 6].Checked then
    begin
      TransferCommands;
      TelnetForm.DoCustom[6] := True;
      Break;
    end;
  Activitylight.Show;
  FailedLabel.Hide;
  StartDateTime.Show;
  AdjustErrorArray1;         //Shift items up in error array if date has changed
  Application.ProcessMessages;
  DelayedClose := False;
  DelayedStop := False;
  DelayedPause := False;
  ProcessOperation(Self, toCollectstats);   //Get the next sample
  Collecting := False;
  if DelayedClose then Close;
  if DelayedStop then
  begin
    StopButtonClick(StopButton);
    Exit;
  end;
  if DelayedPause then
  begin
    StopButtonClick(PauseButton);
    Exit;
  end;
  if TelnetForm.LE <> leSuccess then LogEvent(TelnetForm.LE);
  Activitylight.Hide;
  if StartofRecording.Checked then
  begin
    if MinutesBetween(Now, LastSnapshot) >= Minutespersnapshot then //Take snapshots if configured to do so
      TakeSnapshots;
  end
  else begin
    if Date > SnapshotDate then DoSnapTimes;
    N := 0;
    repeat
      Inc(N);
      if SnapshotTimes[N] = 1 then Break;
      if (Time >= SnapshotTimes[N]) and not SnapshotsDone[N] then
      begin
        TakeSnapshots;
        SnapshotsDone[N] := True;
        Break;
      end;
    until N = 24;
  end;
  if DoUploadSS.Checked and Snapped then    //Upload the snapshots to remote server
  begin
    Thisday := FormatDateTime('yyyy"-"mm"-"dd', Now);
    Dir := IncludeTrailingPathDelimiter(SnapshotsPath) + 'Uploads';
    S := SSUploadDir.Text;
    if S[Length(S)] = '\' then S[Length(s)] := '/'
    else if S[Length(S)] <> '/' then S := S + '/';
     try
      FileList := FindAllFiles(Dir, '*.png;*.txt', False);
      if FileList.Count > 0 then
      begin
        for N := 0 to FileList.Count - 1 do
          FileList[N] := ExtractFilename(FileList[N]);
        if not CurlUploadFiles(usSnapshots, Dir, SSUploadURL.Text, S + Thisday,
          SSUploadUsername.Text, SSUploadPassword.Text, SSFTPPort.Text, FileList) then
          begin
            Inc(SSUploadFailed);
            if SSUploadFailed > 3 then
            begin
              MiscString := 'FTP upload of snapshots failed four times in succession. Uploading disabled';
              LogEvent(leMisc);
              if FTPfailBox.Checked then
              begin
                MiscString := 'Snapshots';
                SendAlert(atFTPFail, False, 0);
              end;
              DoUploadSS.Checked := False;
            end
            else begin
              MiscString := 'FTP upload of snapshots failed';
              LogEvent(leMisc);
            end;
          end;
      end;
    finally
      FileList.Free;
    end;
    DeleteDirectory(Dir, True);
  end;
  if DaysBetween(Now, LastUpdateCheck) >= FrequencyBox.Value then  //Check for updates as configured
  begin
    CheckVersion(False);
  end;
  if MinutesBetween(Now, LastTrafficSave) > 10 then  //Save traffic data and config settings every 10 minutes
  begin
    WriteTrafficData;
    LastTrafficSave := Now;
    WriteIniFile;
  end;
  if MinutesBetween(Now, LastESSave) >= 60 then
  begin
    if ESChart.SaveHourlyES(Homepath + 'es_data.txt') then
    begin
      {$IFDEF Windows}
      if FileExists(Homepath + 'es.data') then Deletefile(PChar(Homepath + 'es.data'));
      {$ELSE}
      if FileExists(Homepath + 'es.data') then Deletefile(Homepath + 'es.data');
      {$ENDIF}
    end;
    LastESSave := Now;
  end;
  if (DeleteOlderBox.Checked) and (DaysBetween(Now, LastDelete) >= 1) then DeleteOlder;    //Check once a day
end;

// Reset all values to establish a clean start to recording
procedure Tmainform.ResetValues;
begin
  LogEvent(leStart);
  Marginchart.ClearArray;
  SNRMpbChartUp.ClearArray;
  SNRMpbChartDown.ClearArray;
  Speedchart1.ClearArray;
  CRCchart.ClearArray;
  FECchart.ClearArray;
  LEFTRSchart.ClearArray;
  rtxtxchart.ClearArray;
  rtxcchart.ClearArray;
  rtxucchart.ClearArray;
  SESchart.ClearArray;
  Swapsperminchart.ClearArray;
  TrafficperminChart.ClearArray;
  Swapsperminchart.MaxYValue := 20;
  MarginChart.MaxYValue := 10;
  SNRMpbChartUp.MaxYValue := 10;
  SNRMpbChartDown.MaxYValue := 10;
  SpeedChart1.MaxYValue := 10000;
  CRCChart.MaxYValue := 10;
  FECChart.MaxYValue := 10;
  LEFTRSChart.MaxYValue := 10;
  SESChart.MaxYValue := 10;
  rtxtxChart.MaxYValue := 10;
  rtxcChart.MaxYValue := 10;
  rtxucChart.MaxYValue := 10;
  TrafficperminChart.MaxYValue := 10;
  ResetButtonClick(Self);
  SNRchart.YMax := 70;
  SNRchart.YMin := -20;
  StartTime := Now;
  OldT := Now;
  TelnetForm.Bandsdone := False;
  Modefound := False;
  Unconnected := True;
  with SNRMrecord do
  begin
    MaxUpTime := 0;
    MinUpTime := 0;
    MaxDownTime := 0;
    MinDownTime := 0;
  end;
  LastSample := 0;
  BitsSaved := False;
  BitswapChart.StartTime := Now;
end;

//Transfer the login values to TelnetForm
procedure Tmainform.TransferLoginValues;
begin
  if SpecialLogin.Checked then
  begin
    TelnetForm.DoInsertDelay := InsertDelayBox.Checked;
    TelnetForm.DelayTime := DelayTimeEdit.Value;
    TelnetForm.HostName := SIPEdit.Text;
    TelnetUnit1.PortNumber := StrToInt(SPortEdit.Text);
    TelnetForm.UserName := Responses[1].Text;
    TelnetForm.Password := Responses[2].Text;
    TelnetForm.Special := True;
  end
  else begin
    TelnetForm.HostName := IPEdit.Text;
    TelnetUnit1.PortNumber := StrToInt(PortEdit.Text);
    TelnetForm.UserName := NameEdit.Text;
    TelnetForm.Password := PasswordEdit.Text;
    TelnetForm.UseSecondStage := Secondstage.Checked;
    TelnetForm.Shelltext := Shellcommand.Text;
    TelnetForm.Special := False;
  end;
  TelnetForm.IsThomson := ThomBox.Checked;
end;

//Synchronise sampling to 35-40 seconds after the minute
procedure TMainform.DoSynchronise;
begin
  ActivityLight.Caption := 'Synchronising...';
  ActivityLight.Show;
  repeat
    Application.ProcessMessages;
    Sleep(50);
  until SecondOfTheMinute(Now) in [35..40];
  SampleTime := Now;
end;

// Start or resume recording data
procedure Tmainform.StartButtonClick(Sender: TObject);
var
  N: Integer;
  NUp, NDown, CUp, CDown, FUp, FDown, ValtxUp, ValtxDown, ValcUp, ValcDown, ValucUp, ValucDown, ValLEFTRSup, ValLEFTRSdown: Single;
  Minutes, Seconds: Word;
begin
  TelnetForm.IsHG622 := IsHG622Box.Checked;
  Timer8.Enabled := False;
  if SpecialLogin.Checked then
  begin
    if (SIPEdit.Text = '') or (SCommandBox.ItemIndex = DefaultIndex) then
    begin
      ShowMessage('Please enter the modem/router IP address and other login details as needed');
      Exit;
    end;
  end
  else begin
    if (IPEdit.Text = '') or (CommandBox.ItemIndex = DefaultIndex) then
    begin
      ShowMessage('Please enter the modem/router IP address and other login details as needed');
      Exit;
    end;
  end;
  WriteInifile;     //Save the configuration
  JustStarted := True;
  Timeouts := 0;
  if Stopped then    //Recording was stopped (or not previously started)
  begin
    ResetValues;
    ResetTriggers;
    FirstSample := True;
    ResyncBaseTime := Now;
    StatsArray[1, 5] := '0 (since ' + FormatDateTime('dd mmm yyyy hh:nn:ss', ResyncBaseTime) + ')';
    AdjustErrorArray1;         //Shift items up in error array if date has changed
    IPAddress := '';
    WasStopped := True;
    LastCustom := Now;
    TransferCommands;
    TelnetForm.DoCustom[1] := True;
    if not AutoscaleBox.Checked then
    begin
      MarginChart.MaxYValue := MaxYEdit.Value;
      MarginChart.MinYValue := MinYEdit.Value;
    end;
    PreviousLAttLine := '';
    PreviousSAttLine := '';
  end
  else if not SyncOptionChanged then
  begin    //Recording was paused
    LogEvent(leResume);
    N := Swapsperminchart.NumSamples - 1;     //Special action for 'per minute' graphs (needs to be done before getting next sample)
    NUp := SwapsperminChart.SampleArray[N].UpYValue;
    NDown := SwapsperminChart.SampleArray[N].DownYValue;
    N := CRCchart.NumSamples - 1;
    CUp := CRCChart.SampleArray[N].UpYValue;
    CDown := CRCChart.SampleArray[N].DownYValue;
    N := FECchart.NumSamples - 1;
    FUp := FECChart.SampleArray[N].UpYValue;
    FDown := FECChart.SampleArray[N].DownYValue;
    N := LEFTRSChart.NumSamples - 1;
    ValLEFTRSUp := LEFTRSChart.SampleArray[N].UpYValue;
    ValLEFTRSDown := LEFTRSChart.SampleArray[N].DownYValue;
    N := rtxtxChart.NumSamples - 1;
    ValtxUp := rtxtxChart.SampleArray[N].UpYValue;
    ValtxDown := rtxtxChart.SampleArray[N].DownYValue;
    N := rtxcChart.NumSamples - 1;
    ValcUp := rtxcChart.SampleArray[N].UpYValue;
    ValcDown := rtxcChart.SampleArray[N].DownYValue;
    N := rtxucChart.NumSamples - 1;
    ValucUp := rtxucChart.SampleArray[N].UpYValue;
    ValucDown := rtxucChart.SampleArray[N].DownYValue;
    FillGap;
    Swapsperminchart.AddXY(Now, NUp, NDown, True);
    CRCchart.AddXY(Now, CUp, CDown, True);
    FECchart.AddXY(Now, FUp, FDown, True);
    LEFTRSchart.AddXY(Now, ValLEFTRSUp, ValLEFTRSDown, True);
    rtxtxChart.AddXY(Now, ValtxUp, ValtxDown, True);
    rtxcChart.AddXY(Now, ValcUp, ValcDown, True);
    rtxucChart.AddXY(Now, ValucUp, ValucDown, True);
  end;
  SyncOptionChanged := False;
  StopButton.Enabled := True;
  PauseButton.Enabled := True;
  StartButton.Enabled := False;
  FailedLabel.Hide;
  if Synchronise.Checked then DoSynchronise;     //Set sample time to 35-40 seconds after the minute
  Seconds := SampleRateEdit.Value;
  if Seconds >= 60 then
  begin
    N := Seconds;
    Seconds := N mod 60;
    Minutes := N div 60;
  end
  else Minutes := 0;
  SampleTime := Now + EncodeTimeInterval(0, Minutes, Seconds, 0);
  if Stopped then StartDateTime.Caption := 'Recording started ' + FormatDateTime('dd mmm yyyy hh:mm:ss', Now)
  else StartDateTime.Caption := 'Recording resumed ' + FormatDateTime('dd mmm yyyy hh:mm:ss', Now);
  StartDateTime.Show;
  Samplerate := 1000 * SampleRateEdit.Value;
  TransferLoginValues;
  LoginTab.Enabled := False;   //Disable changing of login details while recording is in process
  SpecialTab.Enabled := False;
  DisabledLabel.Visible := True;
  DisabledLabel1.Visible := True;
  Timer1.Enabled := True;
  LastSnapshot := StartTime;
  ActivityLight.Caption := 'Sampling';
  ActivityLight.Show;
  DelayedClose := False;
  DelayedPause := False;
  DelayedStop := False;
  ProcessOperation(Self, toCollectstats);    //Get first sample after start or restart
  Collecting := False;
  FirstSample := False;
  JustStarted := False;
  if DelayedClose then Close;
  if TelnetForm.LE <> leSuccess then
  begin
    if TelnetForm.LE <> leMisc then LogEvent(TelnetForm.LE);    //Telnet form logs leMisc events directly
    if DiagnosticsBox.Checked then
    begin
      EventList.Lines.Add(TelnetForm.ConnectResult);
      if TelnetForm.LE = leLoginexception then
      begin
        EventList.Lines.Add('Exception during telnet login');
        ManualSaveTextPages(SavelogButton);
      end
      else if TelnetForm.LE = leNoresponse then
      begin
        EventList.Lines.Add('Modem/router failed to respond to telnet request');
        ManualSaveTextPages(SavelogButton);
      end
      else if TelnetForm.LE = leNospeciallogin then
      begin
        EventList.Lines.Add(TelnetForm.LogText);
        ManualSaveTextPages(SavelogButton);
      end
      else if TelnetForm.LE = leNosecondstage then
      begin
        EventList.Lines.Add(TelnetForm.CommandResult);
        ManualSaveTextPages(SavelogButton);
      end
      else if TelnetForm.LE = leBadPassword then
      begin
        EventList.Lines.Add('An incorrect login name or password was supplied');
        ManualSaveTextPages(SavelogButton);
      end;
    end;
  end
  else if DiagnosticsBox.Checked then EventList.Lines.Add(TelnetForm.DiagsText);
  if DelayedStop then
  begin
    StopButtonClick(StopButton);
    Exit;
  end;
  if DelayedPause then
  begin
    StopButtonClick(PauseButton);
    Exit;
  end;
  ActivityLight.Hide;
  if Stopped then
  begin
    MarginChart.SetPageStart(Now);
    Speedchart1.SetPageStart(Now);
    CRCchart.SetPageStart(Now);
    FECchart.SetPageStart(Now);
    SESchart.SetPageStart(Now);
    SwapsperminChart.SetPageStart(Now);
    rtxtxChart.SetPageStart(Now);
    rtxcChart.SetPageStart(Now);
    rtxucChart.SetPageStart(Now);
    SNRMpbChartUp.SetPageStart(Now);
    SNRMpbChartDown.SetPageStart(Now);
  end;
  Marginchart.Hoursperpage := Hoursperpage;
  Speedchart1.Hoursperpage := Hoursperpage;
  CRCchart.Hoursperpage := Hoursperpage;
  FECchart.Hoursperpage := Hoursperpage;
  SESchart.Hoursperpage := Hoursperpage;
  Swapsperminchart.Hoursperpage := Hoursperpage;
  Trafficperminchart.Hoursperpage := Hoursperpage;
  SNRMpbChartup.Hoursperpage := Hoursperpage;
  SNRMpbChartdown.Hoursperpage := Hoursperpage;
  Stopped := False;
end;

// Stop or pause recording data
procedure Tmainform.StopbuttonClick(Sender: TObject);
begin
  Timer1.Enabled := False;
  OldUpswaps := 0;        //Reset counting of swaps per minute
  OldDownswaps := 0;
  if Collecting then   //Delay stop/pause until after sampling is completed
  begin
    if Sender = StopButton then
    begin
      ActivityLight.Caption := 'Stopping...';
      DelayedStop := True;
    end
    else begin
      ActivityLight.Caption := 'Pausing...';
      DelayedPause := True;
    end;
    PauseButton.Enabled := False;
    StopButton.Enabled := False;
    Exit;
  end;
  StopPulsing;
  NoLoginCount := 0;
  LoginTab.Enabled := True;    //Re-enable changing of login details
  SpecialTab.Enabled := True;
  DisabledLabel.Hide;
  DisabledLabel1.Hide;
  Collecting := False;
  TelnetForm.QLNDone := False;
  TelnetForm.HLogDone := False;
  QGraphDone := False;
  HGraphDone := False;
  if Sender = Stopbutton then
  begin
    LogEvent(leStop);
    StartDateTime.Caption := 'Not recording';
    Stopped := True;
    TrayIcon1.Hint := 'Not recording';
    ResyncCount := 0;
    ResyncBaseTime := Now;
    StatsArray[1, 5] := '0 (since ' + FormatDateTime('dd mmm yyyy hh:nn:ss', ResyncBaseTime) + ')';
    if SNRSaveBox.Checked then MsnapButtonClick(SNRSaveButton);   //Save selected graphs as XML files
    if SpeedSaveBox.Checked then MsnapButtonClick(SpeedSaveButton);
    if CRCSaveBox.Checked then MsnapButtonClick(CRCSaveButton);
    if FECSaveBox.Checked then MsnapButtonClick(FECSaveButton);
    if BitswapSaveBox.Checked then MsnapButtonClick(BitswapSaveButton);
  end
  else begin
    LogEvent(lePause);
    StartDateTime.Caption := 'Recording paused';
    PauseTime := Now;
    Stopped := False;
    TrayIcon1.Hint := 'Recording paused';
    if RestartBox.Checked then
    begin
      Timer8.Interval := 1000 * 60 * RestartEdit.Value;
      Timer8.Enabled := True;
    end;
  end;
  ActivityLight.Hide;
  StartButton.Enabled := True;
  PauseButton.Enabled := False;
  StopButton.Enabled := False;
end;

//Restart recording automatically after a pause
procedure Tmainform.Timer8Timer(Sender: TObject);
begin
  Timer8.Enabled := False;
  StartButtonClick(StartButton);
end;

//Tidy up the pbParams tabulation
procedure Tmainform.TidypbParams;
var
  Linenum, N, N1, Count, Tabs, Len: Word;
  S, S1, Prefix, Number: string;

  procedure FormatLine;
  begin
    S := ParamsList.Lines[Linenum];
    Len := Length(S);
    S1 := Prefix;
    N := Pos(':', S);  //Find the numbers and N/A, and space with single tabs
    Count := 0;
    repeat
      repeat
        Inc(N);
      until (S[N] in ['0'..'9', 'N', '-']);
      N1 := N;
      repeat
        Inc(N1);
      until (N1 > Len) or (S[N1] in [' ', Chr(9), Chr(10), Chr(13)]);            //Space, tab or CR
      Number := S[N..N1 - 1];
      S1 := S1 + Chr(9) + Number;
      Inc(Count);
      N := N1;
    until (Count = 8) or (N1 >= Len - 2);
    ParamsList.Lines[Linenum] := S1;
  end;

begin
  with ParamsList do
  begin
    if Lines.Count < 12 then Exit;
    Linenum := 1;
    Tabs := 0;
    repeat      //Find 'VDSL Port Details' line
      Inc(LineNum);
      S := Lines[Linenum];
    until (Pos('port', LowerCase(S)) > 0) or (Linenum = Lines.Count);
    if Linenum = Lines.Count then Exit;
    N := 0;
    repeat
      Inc(N);
      if S[N] = Chr(9) then Inc(Tabs);
    until (S[N] = 'V') or (N = Length(S));
    if N = Length(S) then Exit;
    if Tabs > 1 then    //Delete one or more tabs
    repeat
      S := S[2..Length(S)];
      Dec(Tabs);
    until Tabs = 1;
    Lines[Linenum] := S;
    repeat      //Find 'VDSL Band Status' line and note which items are included
      Inc(Linenum);
      S := Lines[LineNum];
    until (Pos('band status', LowerCase(S)) > 0) or (Linenum = Lines.Count);
    if Linenum = Lines.Count then Exit;
    for N := 0 to 4 do
    begin
      if Pos('U' + IntToStr(N), S) > 0 then BandsReported[0, N] := True else BandsReported[0, N] := False;
      if Pos('D' + IntToStr(N), S) > 0 then BandsReported[1, N] := True else BandsReported[1, N] := False;
    end;
    S := '  VDSL Band Status';   //Recreate title line with single tab spacings
    for N := 0 to 4 do
      if BandsReported[0, N] then S := S + Chr(9) + 'U' + IntToStr(N);
    for N := 0 to 4 do
      if BandsReported[1, N] then S := S + Chr(9) + 'D' + IntToStr(N);
    Lines[LineNum] := S;
    Inc(LineNum);         //Format the last four data lines with only single tabs between numbers
    if Linenum = Lines.Count then Exit;
    Prefix := '  Line Attenuation(dB):';
    FormatLine;
    Inc(LineNum);
    if Linenum = Lines.Count then Exit;
    Prefix := 'Signal Attenuation(dB):';
    FormatLine;
    Inc(LineNum);
    if Linenum = Lines.Count then Exit;
    Prefix := '        SNR Margin(dB):';
    FormatLine;
    Inc(LineNum);
    if Linenum = Lines.Count then Exit;
    Prefix := '         TX Power(dBm):';
    FormatLine;
  end;
end;

{$IFDEF Windows}
//Check if a named process is running
function TMainForm.ProcessExists(Filename: string; var Num: Word): Boolean;
var
  ContinueLoop: Boolean;
  SnapshotHandle: THandle;
  ProcessEntry32: TProcessEntry32;
begin
  SnapshotHandle := CreateToolHelp32Snapshot(TH32CS_SNAPPROCESS, 0);
  ProcessEntry32.dwSize := SizeOf(ProcessEntry32);
  ContinueLoop := Process32First(SnapshotHandle, ProcessEntry32);
  Result := False;
  while ContinueLoop do
  begin
    if ((UpperCase(ExtractFilename(ProcessEntry32.szExeFile)) = UpperCase(Filename))
    or (UpperCase(ProcessEntry32.szExeFile) = UpperCase(Filename))) then
    begin
      Result := True;
      Inc(Num);
    end;
    ContinueLoop := Process32Next(SnapshotHandle, ProcessEntry32);
  end;
  CloseHandle(SnapshotHandle);
end;
{$ENDIF}

{$IFDEF Windows}
//Check if HG612-Modem-Stats programs are running
function Tmainform.CheckModemStats: Boolean;
var
  Num: Word;
begin
  Result := False;
  try
    Num := 0;
    if ProcessExists('HG612_run.exe', Num) then
    begin
      Result := True;
      IsRunning := IntToStr(Num) + ' instance(s) of HG612_Run.exe running, sample missed';
      LogEvent(leSamplemissed);
    end;
    Num := 0;
    if ProcessExists('HG612_stats.exe', Num) then
    begin
      Result := True;
      IsRunning := IntToStr(Num) + ' instance(s) of HG612_stats.exe running, sample missed';
      LogEvent(leSamplemissed);
    end;
    Num := 0;
    if ProcessExists('HG612_current_stats.exe', Num) then
    begin
      Result := True;
      IsRunning := IntToStr(Num) + ' instance(s) of HG612_current_stats.exe running, sample missed';
      LogEvent(leSamplemissed);
    end;
  except
    on E: Exception do LogEvent(leIsrunningfailed);
  end;
  if Result then StartPulsing else if Timer9.Enabled then StopPulsing;
end;
{$ENDIF}

// Login and process the requested operation
function Tmainform.ProcessOperation(Sender: TObject; TelnetOp: TTelnetOperation): TLogEvent;
var
  Dataline, Bitstring, Bitline, SNRline, SNRstring, QLNLine, QLNstring,
     HLogLine, HLogString, VendorLine, Upstring, Downstring, Statusstring, Uptimestring, Up, Down,
     WSString, BitsLogString, HLogLogString, QLNLogString, SNRLogString,
     SNRMlatestUp, SNRMlatestDown, DailyFolder, BitsZip, SNRZip, QLNZip, HlogZip: Ansistring;
  C: Char;
  S, S1, Decimal1, Decimal2, Stamp: string;
  CommaFormat: Byte;
  Linenumber, Tonenumber, N, N1, Y, Len, Increments: Integer;
  Num1, Num2: Integer;
  UpDiff, DownDiff: Int64;
  Loop, Index, WSIndex, PLineNumber: Word;
  Speedfound, SNRfound, Attenfound, Powerfound, BFound, Interleave0found, Uptimefound, Statusfound,
     Maxfound, Swapsfound, WasPaused, SNROK, RXFirst, SATNFound,
     PowerStateFound, ProfileFound, TPSTCFound, TrellisFound, LineStatusFound, OHF0Found,
     OHFErr0Found, RS0Found, RSCorr0Found, RSUncorr0Found,
     RxQ0Found, HECFound, OCDFound, LCDFound, ESFound, TotalcellsFound,
     DatacellsFound, DropcellsFound, BiterrorsFound, SESFound, UASFound, ASFound, INPFound, PERFound,
     DelayFound, ORFound, CRCfound, FECfound, DoDS, SNRLogDone, Resync, rtxtxFound, LEFTRSFound, SinceLinkTimeFound: Boolean;
  {TNow,} OldDate, ADateTime: TDateTime;
  SNRValue, QLNValue, HLogValue, SNRMpbValue, Multiplier, DeltaUp, DeltaDown: Extended;
  TrafficValue: Int64;
  AYear, AMonth, ADay, AHour, AMinute, ASecond, AMillisecond, CurrentESUpValue,
     CurrentESDownValue, NewESUpValue, NewESDownValue, ElapsedDays, Days1, Hours, Minutes, Seconds, Milliseconds: Word;
  MemoPointer: TMemo;  //Used as a pointer, either to CommandOutput or to EventList
  SR: TSearchRec;

  //Convert float to a string, replacing commas with periods as required
  function FloatToStrFDot(FPValue: Single; Places: Word): string;
  begin
    Result := '.';
    case CommaFormat of
      0: begin    //Not yet tested
           Result := FloatToStrF(FPValue, ffFixed, 0, Places);
           if Pos(',', Result) > 0 then   //Locale uses comma
           begin
             Result := ReplaceStr(Result, ',', '.');
             CommaFormat := 1;
           end
           else CommaFormat := 2;
         end;

      1: begin    //Locale uses comma as decimal divider
           Result := ReplaceStr(FloatToStrF(FPValue, ffFixed, 0, Places), ',', '.');
         end;

      2: begin    //Locale uses period as decimal divider
           Result := FloatToStrF(FPValue, ffFixed, 0, Places);
         end;
    end;
  end;

  //Pad number strings with spaces to make them 16 chars long
  procedure PadStrings;
  var
    Len: Byte;
  begin
    Len := Length(Number1String);
    if Len < 16 then
    repeat
      Number1String := Number1String + ' ';
      Inc(Len);
    until Len = 16;
    Len := Length(Number2String);
    if Len < 16 then
    repeat
      Number2String := Number2String + ' ';
      Inc(Len);
    until Len = 16;
  end;

  //Add a time stamp to a filename
  function AddTimeStamp(InFilename: string): string;
  begin
    Result := InFilename + '-' + FormatDateTime('hh"-"nn', TNow);
  end;

  //Save a string as a file
  procedure SaveString(InString, OutFilename: string; IsStats: Boolean);
  var
    F: TextFile;
    Thisday: string;
  begin
    Thisday := FormatDateTime('yyyy"-"mm"-"dd', TNow);
    DailyFolder := IncludeTrailingPathDelimiter(DataStorePath) + IncludeTrailingPathDelimiter(Thisday);
    if ForceDirectories(DailyFolder) then
    begin
      AssignFile(F, DailyFolder + OutFilename);
    end
    else begin
      MiscString := 'Unable to create directory ' + DailyFolder + '. Saving files in ' + DataStorePath;
      LogEvent(leMisc);
      AssignFile(F, DataStorePath + OutFilename);
    end;
    if IsStats and SingleButton.Checked then
    begin
      AssignFile(F, DailyFolder + 'Stats.log');
      if FileExists(DailyFolder + 'Stats.log') then
      try
        Append(F);
        Writeln(F, InString);
        CloseFile(F);
      except
        on E: Exception do
        begin
          MiscString := 'Unable to save stats in Data Store';
          LogEvent(leMisc);
          CloseFile(F);
        end;
      end
      else try
        ReWrite(F);
        Writeln(F, InString);
        CloseFile(F);
      except
        on E: Exception do
        begin
          MiscString := 'Unable to save stats in Data Store';
          LogEvent(leMisc);
          CloseFile(F);
        end;
      end;
    end
    else try
      try
        ReWrite(F);
        if IsStats then Writeln(F, Instring)
        else Write(F, InString);
      finally
        CloseFile(F);
      end;
    except
      on E: Exception do
      begin
        MiscString := 'Unable to save ' + OutFilename + ' in DataStore';
        LogEvent(leMisc);
      end;
    end;
  end;

  //Zip a text file
  procedure ZipFile(InFilename, OutFilename: string);
  var
    Zipper: TZipper;
    InFilePath: string;
  begin
    Zipper := TZipper.Create;
    try
      if ForceDirectories(DailyFolder) then
      begin
        Zipper.Filename := DailyFolder + OutFilename;
        InFilePath := DailyFolder + InFilename;
      end
      else begin
        Zipper.Filename := DataStorePath + OutFilename;
        InFilePath := DataStorePath + InFilename;
      end;
      Zipper.Entries.AddFileEntry(InFilePath, ExtractFilename(InFilePath));
      Zipper.ZipAllFiles;
    except
      on E: Exception do
      begin
        MiscString := 'Unable to save ' + OutFilename + ' for DataStore';
        LogEvent(leMisc);
      end;
    end;
    Zipper.Free;
  end;

  //Extract the value from a line of text for SNR, QLN or HLog
  procedure ParseALine(LineType: Byte);    //1=SNR, 2=QLN, 3=HLog
  var
    ValueOK: Boolean;
    ALine, AString: string;
    AList: TMemo;
    AValue: Single;
  begin
    case LineType of
      1: AList := SNRList;
      2: AList := QLNList;
      3: AList := HLogList;
    end;
    N := 0;
    ValueOK := False;
    Inc(Linenumber);
    try
      ALine := AList.Lines[Linenumber];
      repeat
        Inc(N);
        if N > Length(ALine) then Exit;
        C := ALine[N];
      until (C in ['0'..'9']);
      repeat
        Inc(N);
        if N > Length(ALine) then Exit;
        C := ALine[N];
      until not (C in ['0'..'9']);
      repeat
        Inc(N);
        if N > Length(ALine) then Exit;
        C := ALine[N];
      until (C in ['-', '0'..'9']);
      AString := C;
      while N < Length(ALine) do
      begin
        Inc(N);
        C := ALine[N];
        if (C in ['0'..'9', '.', ',']) then
        begin
          if (C = '.') or (C = ',') then C := DSeparator;
          AString := AString + C;
        end
        else Break;
      end;
      AValue := StrToFloat(AString);

      if DoDS then       //Check if locale uses comma as decimal divider and replace with period if it does
      case CommaFormat of
        0: begin    //Not yet tested
             if Pos(',', AString) > 0 then
             begin
               CommaFormat := 1;
               AString := ReplaceStr(AString, ',', '.');
             end
             else CommaFormat := 2;
           end;

        1: begin    //Locale uses comma as decimal divider
             AString := ReplaceStr(AString, ',', '.');
           end;

        2: begin    //Locale uses period as decimal divider
             //Don't change
           end;
      end;

      N := Pos('.', AString);
      if N > 0 then AString := AString[1..N + 1];  //One decimal place only
      ValueOK := True;
    except
      on E: Exception do
      begin
        AValue := 0;
        AString := '0.0';
        MiscString := 'Unknown error. Zero ';
        case LineType of
          1: MiscString := MiscString + 'dB SNR returned';
          2: MiscString := MiscString + 'QLN value returned';
          3: MiscString := MiscString + 'dB HLog returned';
        end;
        LogEvent(leMisc);
      end;
    end;
    case LineType of
      1: begin
           SNROK := ValueOK;
           SNRString := AString;
           SNRValue := AValue;
         end;
      2: begin
           QLNString := AString;
           QLNValue := AValue;
         end;
      3: begin
           HLogString := AString;
           HLogValue := AValue;
         end;
    end;
  end;

  //Update the max/min form
  procedure UpdateMaxMin;
  begin
    with SNRMrecord do
    begin
      if MaxUpTime = 0 then
      begin
        MaxUpValue := Number2;
        MinUpValue := Number2;
        MaxDownValue := Number1;
        MinDownValue := Number1;
        MaxUpTime := TNow;
        MinUpTime := TNow;
        MaxDownTime := TNow;
        MinDownTime := TNow;
      end
      else begin
        if Number2 >= MaxUpValue then
        begin
          MaxUpValue := Number2;
          MaxUpTime := TNow;
        end
        else if Number2 <= MinUpValue then
        begin
          MinUpValue := Number2;
          MinUpTime := TNow;
        end;
        if Number1 >= MaxDownValue then
        begin
          MaxDownValue := Number1;
          MaxDownTime := TNow;
        end
        else if Number1 <= MinDownValue then
        begin
          MinDownValue := Number1;
          MinDownTime := TNow;
        end;
      end;
      Maxminform.Maxup.Caption := FloatToStrF(MaxUpValue, ffFixed, 1, 1) + ' dB';
      Maxminform.Minup.Caption := FloatToStrF(MinUpValue, ffFixed, 1, 1) + ' dB';
      Maxminform.Maxdown.Caption := FloatToStrF(MaxDownValue, ffFixed, 1, 1) + ' dB';
      Maxminform.Mindown.Caption := FloatToStrF(MinDownValue, ffFixed, 1, 1) + ' dB';
      Maxminform.MaxUpTime.Caption := FormatDateTime('dd mmm yyyy hh:mm:ss', MaxUpTime);
      Maxminform.MinUpTime.Caption := FormatDateTime('dd mmm yyyy hh:mm:ss', MinUpTime);
      Maxminform.MaxDownTime.Caption := FormatDateTime('dd mmm yyyy hh:mm:ss', MaxDownTime);
      Maxminform.MinDownTime.Caption := FormatDateTime('dd mmm yyyy hh:mm:ss', MinDownTime);
    end;
  end;

  //Get delta values from old and new totals
  procedure GetDeltaValues(OldUp, OldDown, NewUp, NewDown: Int64);
  begin
    DeltaUp := 0;      //Set both values to zero initially
    DeltaDown := 0;
    if ((OldUp > 0) or (OldDown > 0))        //Don't process if both previous values were zero
    or ((NewUp = 0) and (NewDown = 0)) then  //unless both new values are also zero
    begin
      if (NewDown >= OldDown) and (NewUp >= OldUp) then  //Normal situation
      begin
        DeltaUp := NewUp - OldUp;
        DeltaDown := NewDown - OldDown;
      end
      else if (NewDown = 0) and (NewUp = 0) then  //Treat as missed sample
      begin
        WasPaused := True;
        DeltaUp := 0;
        DeltaDown := 0;
      end
      else if (OldDown < NewDown) or (OldUp < NewUp) then  //Treat as a blip or a value wraparound
      begin
        if NewUp < OldUp then     //Up value has blipped or wrapped around
        begin
          DeltaUp := 0;
          DeltaDown := NewDown - OldDown;
        end
        else begin      //Down value has blipped or wrapped around
          DeltaUp := NewUp - OldUp;
          DeltaDown := 0;
        end;
      end
      else begin       //Treat as resync/reboot
        DeltaUp := 0;
        DeltaDown := 0;
      end;
    end;
  end;

  //Process CRC values
  procedure ProcessCRC(Up, Down: Extended);
  begin
    GetDeltaValues(CRCUp, CRCDown, Round(Up), Round(Down));
    DailyCRCDown := DailyCRCDown + Round(DeltaDown);
    DailyCRCUp := DailyCRCUp + Round(DeltaUp);
    ErrorArray1[0].CRCdown := Round(DailyCRCDown);
    ErrorArray1[0].CRCup := Round(DailyCRCUp);
    CRCchart.AddXY(TNow, Round(DeltaUp), Round(DeltaDown), WasPaused);
    if DoDS then
    begin
      DSstats[20] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
      if ThomBox.Checked then
      begin
        DSstats[32] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
        DSstats[128] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
      end;
    end;
    if CRCpm.Checked then     //Base alerts on per-minute equivalents
    try
      DeltaUp := DeltaUp * 60 / SampleRateEdit.Value;
      DeltaDown := DeltaDown * 60 / SampleRateEdit.Value;
    except
      on E: EDivByZero do
      begin
        MiscString := '***Sample rate value equals zero (should be impossible) - 1';
        LogEvent(leMisc);
        DeltaUp := 0;
        DeltaDown := 0;
      end;
    end;
    CheckAlerts(atCRC, DeltaDown, DeltaUp);
    CRCUp := Round(Up);
    CRCDown := Round(Down);
  end;

  //Process ES values
  procedure ProcessES(Up, Down: Extended);
  begin
    GetDeltaValues(ESUp, ESDown, Round(Up), Round(Down));
    DailyESDown := DailyESDown + Round(DeltaDown);
    DailyESUp := DailyESUp + Round(DeltaUp);
    ErrorArray1[0].ESdown := Round(DailyESDown);
    ErrorArray1[0].ESup := Round(DailyESUp);
    ESup := Round(Up);
    ESdown := Round(Down);
  end;

  //Process SES values
  procedure ProcessSES(Up, Down: Extended);
  begin
    GetDeltaValues(SESUp, SESDown, Round(Up), Round(Down));
    if DoDS then
    begin
      DSstats[48] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
      DSstats[49] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
    end;
    DailySESDown := DailySESDown + Round(DeltaDown);
    DailySESUp := DailySESUp + Round(DeltaUp);
    ErrorArray1[0].SESdown := Round(DailySESDown);
    ErrorArray1[0].SESup := Round(DailySESUp);
    SESchart.AddXY(TNow, Round(DeltaUp), Round(DeltaDown), WasPaused);
    SESup := Round(Up);
    SESdown := Round(Down);
  end;

  //Process FEC values
  procedure ProcessFEC(Up, Down: Extended);
  begin
    GetDeltaValues(FECUp, FECDown, Round(Up), Round(Down));
    DailyFECDown := DailyFECDown + Round(DeltaDown);
    DailyFECUp := DailyFECUp + Round(DeltaUp);
    ErrorArray1[0].FECdown := Round(DailyFECDown);
    ErrorArray1[0].FECup := Round(DailyFECUp);
    FECchart.AddXY(TNow, Round(DeltaUp), Round(DeltaDown), WasPaused);
    if DoDS then
    begin
      DSstats[21] := FloatToStrF(DeltaDown, ffFixed, 0, 0);     //No corresponding up value in DS
      if ThomBox.Checked then
      begin
        DSstats[26] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
        DSstats[47] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
      end;
    end;
    if FECpm.Checked then     //Base alerts on per-minute equivalents
    try
      DeltaUp := DeltaUp * 60 / SampleRateEdit.Value;
      DeltaDown := DeltaDown * 60 / SampleRateEdit.Value;
    except
      on E: EDivByZero do
      begin
        MiscString := '***Sample rate equals zero (should be impossible) - 2';
        LogEvent(leMisc);
        DeltaUp := 0;
        DeltaDown := 0;
      end;
    end;
    CheckAlerts(atFEC, DeltaDown, DeltaUp);
    FECup := Round(Up);
    FECdown := Round(Down);
  end;

  //Process RSCorr values
  procedure ProcessRSCorr(Up, Down: Extended);
  begin
    GetDeltaValues(RSCorr0Up, RSCorr0Down, Round(Up), Round(Down));
    if DoDS then
    begin
      DSstats[26] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
      DSstats[47] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
    end;
    RSCorr0up := Round(Up);
    RSCorr0Down := Round(Down);
  end;

  //Process SFErr or OHFErr values
  procedure ProcessOHFErr(Up, Down: Extended);
  begin
    GetDeltaValues(OHFErr0Up, OHFErr0Down, Round(Up), Round(Down));
    if DoDS then
    begin
      DSstats[32] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
      DSstats[128] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
    end;
    OHFErr0up := Round(Up);
    OHFErr0down := Round(Down);
  end;

  //Process HEC values
  procedure ProcessHEC(Up, Down: Extended);
  begin
    GetDeltaValues(HECUp, HECDown, Round(Up), Round(Down));
    if DoDS then
    begin
      DSstats[22] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
      DSstats[129] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
    end;
    DailyHECDown := DailyHECDown + Round(DeltaDown);
    DailyHECUp := DailyHECUp + Round(DeltaUp);
    ErrorArray1[0].HECdown := Round(DailyHECDown);
    ErrorArray1[0].HECup := Round(DailyHECUp);
    HECup := Round(Up);
    HECdown := Round(Down);
  end;

//Start of main procedure
begin
  Result := leUnknown;
  Resync := False;
  if Collecting then       //Don't start new sample if previous one is not yet completed
  begin
    Result := leBusy;
    Exit;
  end
  else Collecting := True;
  DoDS := (DoSaveData.Checked and DoSaveData.Enabled);
  if DoDS then
  begin
    for N := 1 to 150 do DSstats[N] := '0';  //Clear the array before filling in new values
    DSstatsChange(Self);  //Enable graphs as required
    if Now >= IncMinute(LastZipTime, 59) then
    begin
      DoZips := True;
      if LastZipTime = 0 then LastZipTime := Now
      else LastZipTime := IncHour(LastZipTime);
    end
    else DoZips := False;
  end;
  BitsLogString := '';
  HLogLogString := '';
  QLNLogString := '';
  SNRLogString := '';
  WasPaused := (PauseTime > 0);
  PauseTime := 0;
  Speedfound := False;
  Maxfound := False;
  SNRfound := False;
  AttenFound := False;
  VendorWritten := False;
  CommaFormat := 0;
  if TelnetForm.Mode = -1 then     //First sample
  begin
    StatsArray[1, 8] := 'Not yet determined';
    StatsArray[2, 8] := '';
  end;
  ModeFound := False;
  Powerfound := False;
  PowerStateFound := False;
  ProfileFound := False;
  Profile := '';
  TPSTCFound := False;
  TrellisFound := False;
  LineStatusFound := False;
  BFound := False;
  Interleave0Found := False;
  Uptimefound := False;
  Statusfound := False;
  Swapsfound := False;
  CRCFound := False;
  FECFound := False;
  OHF0Found := False;
  OHFErr0Found := False;
  RS0Found := False;
  RSCorr0Found := False;
  RSUncorr0Found := False;
  RxQ0Found := False;
  HECFound := False;
  OCDFound := False;
  LCDFound := False;
  ESFound := False;
  TotalcellsFound := False;
  DatacellsFound := False;
  DropcellsFound := False;
  BiterrorsFound := False;
  SESFound := False;
  UASFound := False;
  ASFound := False;
  INPFound := False;
  PERFound := False;
  DelayFound := False;
  ORFound := False;
  rtxtxFound := False;
  LEFTRSFound := False;
  SinceLinkTimeFound := False;
  SNRLogDone := False;
  Terminated := False;
  if Stopped then Statslist.Clear;
  if SpecialLogin.Checked then S := SCommandBox.Items[SCommandBox.ItemIndex]
  else S := CommandBox.Items[CommandBox.ItemIndex];
  if S = 'Custom' then S := CustomPrefixEdit.Text;
  if S = '' then
  begin
    MiscString := 'No custom prefix has been entered';
    Result := leMisc;
    ShowMessage('No custom prefix has been entered');
    Exit;
  end;
  if ThomBox.Checked then
  begin
    TelnetForm.Statscommand := S + ' info expand=enabled';
    TelnetForm.Bitscommand := S + ' debug bitloadinginfo';
    TelnetForm.TrafficCommand := 'ip iflist';
    TelnetForm.GInpCommand := S + ' info ginp=yes';
    DSsnr.Checked := False;
    DSsnr.Enabled := False;
    DShlog.Checked := False;
    DShlog.Enabled := False;
    DSqln.Checked := False;
    DSqln.Enabled := False;
  end
  else begin
    TelnetForm.Statscommand := S + ' info --stats';
    TelnetForm.Bitscommand := S + ' info --Bits';
    TelnetForm.TrafficCommand := CommandGroup.Items[CommandGroup.ItemIndex];
    DSsnr.Enabled := True;
    DShlog.Enabled := True;
    DSqln.Enabled := True;
  end;
  TelnetForm.Paramscommand := S + ' info --pbParams';
  TelnetForm.SNRcommand := S + ' info --SNR';
  TelnetForm.QLNcommand := S + ' info --QLN';
  TelnetForm.Hlogcommand := S + ' info --Hlog';
  TelnetForm.VendorCommand := S + ' info --vendor';
  TelnetForm.LinediagCommand := S + ' info --linediag';
  TelnetForm.Uptimecommand := 'cat /proc/uptime';
  TelnetForm.VersionCommand := S + ' --version';
  TelnetForm.VectoringCommand := S + ' info --vectoring';
  TelnetForm.DoBitloading := DoBitloading.Checked;
  TelnetForm.DoSNRpertone := DoSNRpertone.Checked;
  TelnetForm.DoQLN := DoQLN.Checked;
  TelnetForm.DoHLog := DoHLog.Checked;
  TelnetForm.DoSignalAttn := DoSignalAttn.Checked;
  TelnetForm.DoFirmware := DoFirmware.Checked;
  TelnetForm.DoVendor := DoVendor.Checked;
  TelnetForm.DoVectoring := DoVectoring.Checked;
  TelnetForm.DoSysuptime := DoSysuptime.Checked;

  //Process the operation
  {$IFDEF Windows}
    if Synchronise.Checked then
    begin
      if (CheckCount < 5) and CheckModemStats then
      begin
        Inc(CheckCount);
        Result := leSuccess;
        StartDateTime.Hide;
        FailedLabel.Caption := 'HG612-Modem-Stats running';
        FailedLabel.Show;
        Collecting := False;
        Exit;
      end
      else if CheckCount = 5 then
      begin
        MiscString := 'Five missed samples, HG612-Modem-Stats checking suspended for 30 samples';
        LogEvent(leMisc);
        Inc(CheckCount);
      end
      else if CheckCount > 5 then
      begin
        Inc(CheckCount);
        if CheckCount > 34 then
        begin
          CheckCount := 0;
          MiscString := 'HG612-Modem-Stats checking resumed';
          LogEvent(leMisc);
        end;
      end;

      //Launch dslstatssampling.exe
      try
        AProcess := TProcess.Create(nil);
        AProcess.Executable := DataDir + 'dslstatssampling.exe';
        AProcess.Options := AProcess.Options + [poNoConsole];
        AProcess.Execute;
      except
        on E: Exception do
        begin
          MiscString := 'Unable to launch dslstatssampling.exe (file missing?';
          LogEvent(leMisc);
        end;
      end;
    end;
  {$ENDIF}
  TelnetForm.TelnetOperation := TelnetOp;
  TransferCommands;
  try
    TelnetForm.DoTelnet;
  except
    on E: Exception do
    begin
      Result := leUnknown;
      StartDateTime.Hide;
      FailedLabel.Caption := 'Unknown telnet error';
      FailedLabel.Show;
      Collecting := False;
      Exit;
    end;
  end;
  {$IFDEF Windows}
  if Synchronise.Checked then
  try
    AProcess.Terminate(0);
    AProcess.Free;
  except
    on E: Exception do;  //Do nothing
  end;
  {$ENDIF}
  for N1 := 1 to 6 do
    TelnetForm.DoCustom[N1] := False;
  TNow := Now;
  if DoDS then
  begin
    DSstats[1] := FormatDateTime('dd"/"mm"/"yyyy', TNow);
    DSstats[2] := FormatDateTime('hh:nn', TNow);
  end;
  Collecting := False;
  if TelnetForm.CustomDone then   //Get the output of the custom commands
  begin
    if DisplayHereButton.Checked then MemoPointer := CommandOutput else MemoPointer := EventList;
    if MemoPointer.Lines.Count > 0 then MemoPointer.Lines.Add('');
    MemoPointer.Lines.Add(FormatDateTime('dd mmm yyyy hh:nn:ss', TNow));
    for N := 1 to 4 do
      if TelnetForm.CustomOutputs[N] > '' then
      begin
        MemoPointer.Lines.Add(TelnetForm.CustomOutputs[N]);
        TelnetForm.CustomOutputs[N] := '';
      end;
    TelnetForm.CustomDone := False;
    for N1 := 1 to 6 do TelnetForm.DoCustom[N1] := False;
    DoCustomNow := False;
    SendNowButton.Enabled := True;
    for N := 1 to 4 do     //Remove the "Send now" options after use
    begin
      TelnetForm.CustomIncludes[N, 6] := False;
    end;
  end;
  TelnetForm.CustomDone := False;
  if TelnetForm.FinalCustom then Exit;
  ActivityLight.Hide;
  Result := TelnetForm.LE;
  if (Result <> leSuccess) and (Result <> leNoBits) then
  begin
    Inc(NoLoginCount);
    if NoLoginCount >= 3 then
    begin
      StartPulsing;
      if NoLoginCount = 3 then
      begin
        MiscString := 'Unable to collect stats 3 or more times in succession';
        LogEvent(leMisc);
      end;
    end;
    if TelnetForm.LE = leMisc then  //Various kinds of login failure
    begin
      StartDateTime.Hide;
      FailedLabel.Caption := MiscString;
      FailedLabel.Show;
      if NoLoginCount < 3 then TrayIcon1.Icon.LoadFromResourceName(HInstance, 'BROWN');
      IconColour := 'BROWN';
      Application.Icon.LoadFromResourceName(HInstance, 'BROWN');
      Collecting := False;
      Exit;
    end
    else if TelnetForm.LE = leNoshowtime then
    begin
      StartDateTime.Hide;
      FailedLabel.Caption := 'No DSL connection';
      FailedLabel.Show;
      if NoLoginCount < 3 then TrayIcon1.Icon.LoadFromResourceName(HInstance, 'BROWN');
      IconColour := 'BROWN';
      Application.Icon.LoadFromResourceName(HInstance, 'BROWN');
      TelnetForm.DoCustom[3] := True;
      Collecting := False;
      Exit;
    end
    else if TelnetForm.LE = leNosecondstage then
    begin
      StartDateTime.Hide;
      FailedLabel.Caption := 'Second stage login failure';
      FailedLabel.Show;
      MiscString := '*** Second stage login failure. Did you enable the option "Requires command to enter shell" by mistake?';
      LogEvent(leMisc);
      if PauseBox.Checked then StopButtonClick(PauseButton);
      Collecting := False;
      Exit;
    end
    else if (TelnetForm.LE = leLoginexception) or (TelnetForm.LE = leNologin) then
    begin
      StartDateTime.Hide;
      FailedLabel.Caption := 'Unable to login to modem/router';
      FailedLabel.Show;
      if PauseBox.Checked then StopButtonClick(PauseButton);
      if NoLoginCount < 3 then TrayIcon1.Icon.LoadFromResourceName(HInstance, 'TRANSPARENT');
      IconColour := 'TRANSPARENT';
      Application.Icon.LoadFromResourceName(HInstance, 'TRANSPARENT');
      Collecting := False;
      Exit;
    end
    else if TelnetForm.LE = leNoresponse then
    begin
      StartDateTime.Hide;
      FailedLabel.Caption := 'No response from modem/router';
      FailedLabel.Show;
      if PauseBox.Checked then StopButtonClick(PauseButton);
      if NoLoginCount < 3 then TrayIcon1.Icon.LoadFromResourceName(HInstance, 'TRANSPARENT');
      IconColour := 'TRANSPARENT';
      Application.Icon.LoadFromResourceName(HInstance, 'TRANSPARENT');
      Collecting := False;
      Exit;
    end
    else if TelnetForm.LE = leNospeciallogin then
    begin
      StartDateTime.Hide;
      FailedLabel.Caption := 'Special login failure';
      FailedLabel.Show;
      if PauseBox.Checked then StopButtonClick(PauseButton);
      Collecting := False;
      Exit;
    end
    else if TelnetForm.LE = leBadpassword then
    begin
      StartDateTime.Hide;
      FailedLabel.Caption := 'Incorrect login name or password';
      FailedLabel.Show;
      if PauseBox.Checked then StopButtonClick(PauseButton);
      Collecting := False;
      Exit;
    end
    else if TelnetForm.LE = leHG622Error then
    begin
      StartDateTime.Hide;
      FailedLabel.Caption := 'Error running HG622 workaround';
      FailedLabel.Show;
      if PauseBox.Checked then StopButtonClick(PauseButton);
      Collecting := False;
      Exit;
    end
    else if TelnetForm.LE = leNostats then
    begin
      StartDateTime.Hide;
      FailedLabel.Caption := 'No data returned in latest sample';
      FailedLabel.Show;

      //Special action for 'per minute' graphs
      N := Swapsperminchart.NumSamples - 1;
      DeltaUp := SwapsperminChart.SampleArray[N].UpYValue;
      DeltaDown := SwapsperminChart.SampleArray[N].DownYValue;
      Swapsperminchart.AddXY(TNow, DeltaUp, DeltaDown, True);
      N := CRCchart.NumSamples - 1;
      DeltaUp := CRCChart.SampleArray[N].UpYValue;
      DeltaDown := CRCChart.SampleArray[N].DownYValue;
      CRCchart.AddXY(TNow, DeltaUp, DeltaDown, True);
      N := SESchart.NumSamples - 1;
      DeltaUp := SESChart.SampleArray[N].UpYValue;
      DeltaDown := SESChart.SampleArray[N].DownYValue;
      SESchart.AddXY(TNow, DeltaUp, DeltaDown, True);
      N := FECchart.NumSamples - 1;
      DeltaUp := FECChart.SampleArray[N].UpYValue;
      DeltaDown := FECChart.SampleArray[N].DownYValue;
      FECchart.AddXY(TNow, DeltaUp, DeltaDown, True);
      N := Trafficperminchart.NumSamples - 1;
      DeltaUp := TrafficperminChart.SampleArray[N].UpYValue;
      DeltaDown := TrafficperminChart.SampleArray[N].DownYValue;
      Trafficperminchart.AddXY(TNow, DeltaUp, DeltaDown, True);

      Collecting := False;
      Exit;
    end;
  end
  else begin
    StopPulsing;
    NoLoginCount := 0;
    LastSample := TNow;
    TrayIcon1.Icon.LoadFromResourceName(HInstance, 'BLUE');
    IconColour := 'BLUE';
    Application.Icon.LoadFromResourceName(HInstance, 'BLUE');
  end;
  if TelnetOp <> toCollectStats then
  begin
    Collecting := False;
    Exit;
  end;

  //Process the data collected from the modem/router

  ErrorArray1[0].TD := CurrentErrorDay;
  ErrorArray1[0].Seconds := ErrorArray1[0].Seconds + SampleRateEdit.Value;
  if (SendEmailAlerts or SendEventLogBox.Checked) then
  begin
    if SNRMfallBox.Checked and not TriggerSNRMDone then TriggerSNRM := GetTriggerFP(atSNRM, SNRMTrigger.Text);
    if SNRMreturnBox.Checked and not TriggerSNRMreturnDone then TriggerSNRMreturn := GetTriggerFP(atSNRMreturn, SNRMreturnTrigger.Text);
    if CRCriseBox.Checked and not TriggerCRCDone then TriggerCRC := GetTriggerFP(atCRC, CRCTrigger.Text);
    if FECriseBox.Checked and not TriggerFECDone then TriggerFEC := GetTriggerFP(atFEC, FECTrigger.Text);
    if ESriseBox.Checked and not TriggerESDone then TriggerES := GetTriggerFP(atES, ESTrigger.Text);
  end;
  Unconnected := False;

  //Make sure that none of the lists is empty
  if TelnetForm.Stats > '' then StatsList.Text := TelnetForm.Stats else StatsList.Text := 'blank';
  if TelnetForm.Bits > '' then BitsList.Text := TelnetForm.Bits else BitsList.Text := 'blank';
  if TelnetForm.Params > '' then ParamsList.Text := TelnetForm.Params else ParamsList.Text := 'blank';
  if TelnetForm.SNR > '' then SNRList.Text := TelnetForm.SNR else SNRList.Text := 'blank';
  if TelnetForm.QLN > '' then QLNList.Text := TelnetForm.QLN else QLNList.Text := 'blank';
  if TelnetForm.HLog > '' then HLogList.Text := TelnetForm.HLog else HLogList.Text := 'blank';
  if TelnetForm.Vendor > '' then VendorList.Text := TelnetForm.Vendor else VendorList.Text := 'blank';
  if TelnetForm.Vectoring > '' then VectoringList.Text := TelnetForm.Vectoring else VectoringList.Text := 'blank';
  if TelnetForm.Linediags > '' then LinediagsList.Text := TelnetForm.Linediags else LinediagsList.Text := 'blank';
  if TelnetForm.Traffic > '' then TrafficList.Text := TelnetForm.Traffic else TrafficList.Text := 'blank';
  if TelnetForm.GInpStats > '' then GInpList.Text := TelnetForm.GInpStats else GInpList.Text := 'blank';
  Uptimestring := TelnetForm.Uptime;

  //Process pbParams
  TidypbParams;   //Tidy up the pbParams tabulation if necessary;
  if ThomBox.Checked or IsRalink then
  begin
    TelnetParamsTab.TabVisible := False;
    DoSNRMperband.Checked := False;
    DoSNRMperband.Enabled := False;
    SNRMpbSnap.Checked := False;
    SNRMpbSnap.Enabled := False;
  end
  else if not TelnetForm.Bandsdone then
  begin
    if Paramslist.Lines.Count < 12 then  //Disable pbParams features if data not available
    begin
      TelnetParamsTab.TabVisible := False;
      DoSNRMperband.Checked := False;
      DoSNRMperband.Enabled := False;
      SNRMpbSnap.Checked := False;
      SNRMpbSnap.Enabled := False;
      SNRMpbweb.Checked := False;
      SNRMpbweb.Enabled := False;
      VDSL2 := False;
    end
    else begin
      TelnetParamsTab.TabVisible := True;
      DoSNRMperband.Enabled := True;
      SNRMpbsnap.Enabled := True;
      SNRMpbweb.Enabled := True;
      VDSL2 := True;
      TelnetForm.Mode := 2;
      ExtractBands;
      TelnetForm.Bandsdone := True;
      LAttenuationLog.Clear;
      S := 'Line attenuation (dB)    ';
      for N := 0 to 4 do
        if BandsReported[0, N] then S := S + Chr(9) + 'U' + IntToStr(N);
      for N := 0 to 4 do
        if BandsReported[1, N] then S := S + Chr(9) + 'D' + IntToStr(N);
      LAttenuationLog.Lines.Add(S);
      SAttenuationLog.Clear;
      S := 'Signal attenuation (dB) ';
      for N := 0 to 4 do
        if BandsReported[0, N] then S := S + Chr(9) + 'U' + IntToStr(N);
      for N := 0 to 4 do
        if BandsReported[1, N] then S := S + Chr(9) + 'D' + IntToStr(N);
      SAttenuationLog.Lines.Add(S);
    end;
  end;

  if VDSL2 then     //Extract SNRM per band values
  begin
    N := ParamsList.Lines.Count;
    UpRecord.XValue := TNow;
    DownRecord.XValue := TNow;
    UpRecord.DrawBlank := False;
    DownRecord.DrawBlank := False;
    if N > 10 then
    repeat
      Dec(N);
      Dataline := ParamsList.Lines[N];
      if Pos('snr margin', LowerCase(Dataline)) > 0 then
      begin
        DataLine := DataLine + ' ';    //To ensure that the last value is recovered
        Index := 16;
        Len := Length(Dataline);
        WSIndex := 65;
        for Loop := 1 to 8 do    //Find the (up to) 8 numbers or N/A on the line
        begin
          S := '';
          repeat      //Find the next number
            Inc(Index);
            C := Dataline[Index];
          until (C in ['0'..'9', 'N']) or (Index = Len);
          if Index = Len then Break;    //No more data available
          repeat
            C := Dataline[Index];
            if (C in ['0'..'9', '.', ',', 'N', '/', 'A']) then
            begin
              if (C = '.') or (C = ',') then C := DSeparator;
              S := S + C;
            end
            else begin
              SNRMpb[Loop] := S;
              Break;
            end;
            Inc(Index);
          until Index > Len;
          if Index >= Len then Break;
        end;
        N := 0;         //Bands U or D 0-4
        N1 := 0;        //0 = upstream, 1 = downstream
        Loop := 0;      //Index into SNRMpb[1..8]
        repeat
          Inc(Loop);
          if SNRMpb[Loop] = '' then Break;    //No more bands reported
          if BandsReported[N1, N] then
          begin
            if SNRMpb[Loop] = 'N/A' then      //Band is reported but not used
            begin
              if N1 = 0 then UpRecord.YValues[N] := -255
              else DownRecord.YValues[N] := -255;
            end
            else begin                        //Band is used
              try
                SNRMpbValue := StrToFloat(SNRMpb[Loop]);
              except
                on E: Exception do SNRMpbValue := 0;
              end;
              if N1 = 0 then
              begin
                UpRecord.YValues[N] := SNRMpbValue;
                if SNRMpbRecord.MaxUTimes[N] = 0 then   //This is the first entry in the max/min table
                begin
                  SNRMpbRecord.MaxUValues[N] := UpRecord.YValues[N];
                  SNRMpbRecord.MaxUTimes[N] := TNow;
                  SNRMpbRecord.MinUValues[N] := UpRecord.YValues[N];
                  SNRMpbRecord.MinUTimes[N] := TNow;
                end
                else if UpRecord.YValues[N] >= SNRMpbRecord.MaxUValues[N] then  //New highest value
                begin
                  SNRMpbRecord.MaxUValues[N] := UpRecord.YValues[N];
                  SNRMpbRecord.MaxUTimes[N] := TNow;
                end
                else if UpRecord.YValues[N] <= SNRMpbRecord.MinUValues[N] then   //New lowest value
                begin
                  SNRMpbRecord.MinUValues[N] := UpRecord.YValues[N];
                  SNRMpbRecord.MinUTimes[N] := TNow;
                end;
              end
              else begin
                DownRecord.YValues[N] := SNRMpbValue;
                if SNRMpbRecord.MaxDTimes[N] = 0 then   //This is the first entry in the max/min table
                begin
                  SNRMpbRecord.MaxDValues[N] := DownRecord.YValues[N];
                  SNRMpbRecord.MaxDTimes[N] := TNow;
                  SNRMpbRecord.MinDValues[N] := DownRecord.YValues[N];
                  SNRMpbRecord.MinDTimes[N] := TNow;
                end
                else if DownRecord.YValues[N] >= SNRMpbRecord.MaxDValues[N] then  //New highest value
                begin
                  SNRMpbRecord.MaxDValues[N] := DownRecord.YValues[N];
                  SNRMpbRecord.MaxDTimes[N] := TNow;
                end
                else if DownRecord.YValues[N] <= SNRMpbRecord.MinDValues[N] then   //New lowest value
                begin
                  SNRMpbRecord.MinDValues[N] := DownRecord.YValues[N];
                  SNRMpbRecord.MinDTimes[N] := TNow;
                end;
              end;
            end;
            repeat                            //Find the next band reported
              Inc(N);
              if N = 5 then                   //No more in the upstream group
              begin
                N := 0;
                Inc(N1);
              end;
            until BandsReported[N1, N] or (N1 = 2);
          end
          else repeat                         //Find the next band reported
            Inc(N);
            if N = 5 then                     //No more in the upstream group
            begin
              N := 0;
              Inc(N1);
            end;
          until BandsReported[N1, N] or (N1 = 2);
        until (Loop = 8) or (N1 = 2);
        SNRMpbChartup.AddRecord(@UpRecord);
        SNRMpbChartdown.AddRecord(@DownRecord);
        N := 0;
      end;
    until N = 0;
    U0.Enabled := (UpRecord.YValues[0] <> -255);
    U1.Enabled := (UpRecord.YValues[1] <> -255);
    U2.Enabled := (UpRecord.YValues[2] <> -255);
    U3.Enabled := (UpRecord.YValues[3] <> -255);
    U4.Enabled := (UpRecord.YValues[4] <> -255);
    D0.Enabled := (DownRecord.YValues[0] <> -255);
    D1.Enabled := (DownRecord.YValues[1] <> -255);
    D2.Enabled := (DownRecord.YValues[2] <> -255);
    D3.Enabled := (DownRecord.YValues[3] <> -255);
    D4.Enabled := (DownRecord.YValues[4] <> -255);

    if DoDS then
    begin
      if U0.Enabled then DSstats[65] := FloatToStrFDot(UpRecord.YValues[0], 1);
      if U1.Enabled then DSstats[66] := FloatToStrFDot(UpRecord.YValues[1], 1);
      if U2.Enabled then DSstats[67] := FloatToStrFDot(UpRecord.YValues[2], 1);
      if U3.Enabled then DSstats[68] := FloatToStrFDot(UpRecord.YValues[3], 1);
      if D1.Enabled then DSstats[69] := FloatToStrFDot(DownRecord.YValues[1], 1);
      if D2.Enabled then DSstats[70] := FloatToStrFDot(DownRecord.YValues[2], 1);
      if D3.Enabled then DSstats[71] := FloatToStrFDot(DownRecord.YValues[3], 1);
    end;

    //Populate the max/min table
    with pbMaxMinForm do
    begin
      N1 := 0;
      for N := 0 to 4 do
      if UpRecord.YValues[N] <> -255 then
      begin
        Inc(N1);
        BandLabels[N1].Caption := 'U' + IntToStr(N);
        HValues[N1].Caption := FloatToStrF(SNRMpbRecord.MaxUValues[N], ffFixed, 1, 1);
        HTimes[N1].Caption := FormatDateTime('dd mmm yyyy hh:nn:ss', SNRMpbRecord.MaxUTimes[N]);
        LValues[N1].Caption := FloatToStrF(SNRMpbRecord.MinUValues[N], ffFixed, 1, 1);
        LTimes[N1].Caption := FormatDateTime('dd mmm yyyy hh:nn:ss', SNRMpbRecord.MinUTimes[N]);
        HTimeValues[N1] := SNRMpbRecord.MaxUTimes[N];
        LTimeValues[N1] := SNRMpbRecord.MinUTimes[N];
      end;
      for N := 0 to 4 do
      if DownRecord.YValues[N] <> -255 then
      begin
        Inc(N1);
        BandLabels[N1].Caption := 'D' + IntToStr(N);
        HValues[N1].Caption := FloatToStrF(SNRMpbRecord.MaxDValues[N], ffFixed, 1, 1);
        HTimes[N1].Caption := FormatDateTime('dd mmm yyyy hh:nn:ss', SNRMpbRecord.MaxDTimes[N]);
        LValues[N1].Caption := FloatToStrF(SNRMpbRecord.MinDValues[N], ffFixed, 1, 1);
        LTimes[N1].Caption := FormatDateTime('dd mmm yyyy hh:nn:ss', SNRMpbRecord.MinDTimes[N]);
        HTimeValues[N1] := SNRMpbRecord.MaxDTimes[N];
        LTimeValues[N1] := SNRMpbRecord.MinDTimes[N];
      end;
      if N1 < 8 then
      begin
        for N := 1 to 10 do
        begin
          BandLabels[N].Visible := (N <= N1);
          HValues[N].Visible := (N <= N1);
          HTimes[N].Visible := (N <= N1);
          LValues[N].Visible := (N <= N1);
          LTimes[N].Visible := (N <= N1);
          HGoButtons[N].Visible := (N <= N1);
          LGoButtons[N].Visible := (N <= N1);
        end;
        CloseButton.Top := 60 + (N1 * 25);
        Height := CloseButton.Top + 40;
      end;
    end;
  end;

  if VDSL2 then  //Extract per-band values
  begin
    PLineNumber := ParamsList.Lines.Count;
    if PLineNumber > 10 then
    repeat
      Dec(PLineNumber);
      Dataline := ParamsList.Lines[PLineNumber] + ' ';
      Len := Length(Dataline);
      if Pos('tx power', LowerCase(Dataline)) > 0 then
      begin
        Index := 16;
        for Loop := 1 to 8 do    //Find the (up to) 8 numbers or N/A on the line
        begin
          S := '';
          repeat      //Find the next number
            Inc(Index);
            C := Dataline[Index];
          until (C in ['0'..'9', 'N']) or (Index = Len);
          if Index = Len then Break;    //No more data available
          repeat
            C := Dataline[Index];
            if (C in ['0'..'9', '.', ',', 'N', '/', 'A']) then
            begin
              if (C = '.') or (C = ',') then C := DSeparator;
              S := S + C;
            end
            else begin
              SNRMpb[Loop] := S;
              if DoDS then
              begin
                if S = 'N/A' then S := '0';
                case Loop of
                  1: DSstats[72] := S;
                  2: DSstats[73] := S;
                  3: DSstats[74] := S;
                  4: DSstats[75] := S;
                  6: DSstats[76] := S;
                  7: DSstats[77] := S;
                  8: DSstats[78] := S;
                end;
              end;
              Break;
            end;
            Inc(Index);
          until Index > Len;
          if Index >= Len then Break;
        end;
      end;

      if Pos('signal attenuation', LowerCase(Dataline)) > 0 then
      begin
        Index := 16;
        WSIndex := 1;
        for Loop := 1 to 8 do    //Find the (up to) 8 numbers on the line
        begin
          S := '';
          repeat      //Find the next number
            Inc(Index);
            C := Dataline[Index];
            if C = 'N' then   //Bypass N/A
            begin
              Inc(WSIndex);
              Index := Index + 3;
            end;
          until (C in ['0'..'9']) or (Index >= Len);
          if Index >= Len then Break;    //No more data available
          repeat
            C := Dataline[Index];
            if (C in ['0'..'9', '.', ',']) then S := S + C
            else begin
              VDSLAtt[Loop] := S;
              if DoDS then
              begin
                case WSIndex of
                  1: DSstats[58] := S;
                  2: DSstats[59] := S;
                  3: DSstats[60] := S;
                  4: DSstats[61] := S;
                  6: DSstats[62] := S;
                  7: DSstats[63] := S;
                  8: DSstats[64] := S;
                end;
              end;
              Break;
            end;
            Inc(Index);
          until Index > Len;
          if Index >= Len then Break;
          Inc(WSIndex);
        end;
        if DoLogging and VDSL2 then
        begin
          N := Pos(':', Dataline);
          S := Dataline[N + 1..Length(Dataline)];
          if S <> PreviousSAttLine then
            SAttenuationLog.Lines.Add(FormatDateTime('dd mmm yyyy hh:mm:ss', TNow) + Chr(9) + S);
          PreviousSAttLine := S;
        end;
      end;

      if Pos('line attenuation', LowerCase(Dataline)) > 0 then
      begin
        Index := 16;
        WSIndex := 1;
        for Loop := 1 to 8 do    //Find the (up to) 8 numbers on the line
        begin
          S := '';
          repeat      //Find the next number
            Inc(Index);
            C := Dataline[Index];
            if C = 'N' then   //Bypass N/A
            begin
              Inc(WSIndex);
              Index := Index + 3;
            end;
          until (C in ['0'..'9']) or (Index >= Len);
          if Index >= Len then Break;    //No more data available
          repeat
            C := Dataline[Index];
            if (C in ['0'..'9', '.', ',']) then S := S + C
            else begin
              if DoDS then
              begin
                case WsIndex of
                  1: DSstats[51] := S;
                  2: DSstats[52] := S;
                  3: DSstats[53] := S;
                  4: DSstats[54] := S;
                  6: DSstats[55] := S;
                  7: DSstats[56] := S;
                  8: DSstats[57] := S;
                end;
              end;
              Break;
            end;
            Inc(Index);
          until Index > Len;
          if Index >= Len then Break;
          Inc(WSIndex);
        end;
        if DoLogging and VDSL2 then
        begin
          N := Pos(':', Dataline);
          S := Dataline[N + 1..Length(Dataline)];
          if S <> PreviousLAttLine then
            LAttenuationLog.Lines.Add(FormatDateTime('dd mmm yyyy hh:mm:ss', TNow) + Chr(9) + S);
          PreviousLAttLine := S;
        end;
        PLineNumber := 0;
      end;

    until PLineNumber = 0;
  end;

  //Process the output of the Stats command

  with Statslist do
  begin
    if Lines.Count < 20 then  //No proper stats received
    begin
      ActivityLight.Hide;
      Result := leNodata;
      Collecting := False;
      Exit;
    end;

    for N := 1 to 22 do GDotInpStrings[N] := '';
    GDotInpStrings[1] := GDotInpTitles[1];
    GDotInpStrings[8] := GDotInpTitles[8];
    GDotInpStrings[18] := GDotInpTitles[18];

    if not ThomBox.Checked then  //Check G.INP status
    begin
      if Pos('Bearer 1', StatsList.Text) > 0 then
      begin
        StatsArray[1, 15] := 'Enabled';
        GdotInp := True;
        DSstats[19] := '1';
      end
      else begin
        StatsArray[1, 15] := 'Not enabled';
        StatsArray[2, 15] := 'Not enabled';
        GdotInp := False;
        DSstats[19] := '0';
      end;
      GdotInpTab.TabVisible := GdotInp;
    end;

    if ThomBox.Checked then  //Router is a Thomson/Technicolor model
    begin
      Linenumber := -1;
      try     //Catchall exception handler
        while Linenumber < Lines.Count - 1 do
        begin
          Inc(Linenumber);
          Dataline := Lines[Linenumber];

          //System or DSL link uptime
          if not Uptimefound then
          begin
            if Pos('Up time', Dataline) > 0 then
            begin
              Len := Length(Dataline);
              N := 5;
              repeat
                Inc(N);
                if Dataline[N] in ['0'..'9'] then
                begin
                  S := Dataline[N..Len];
                  Uptime.Caption := 'DSL uptime: ' + S;
                  Uptimefound := True;
                  StatsArray[1, 4] := S;

                  Len := Length(S);  //Convert uptime to seconds
                  N := 0;
                  repeat
                    Inc(N);
                    C := S[N];
                  until (C in ['0'..'9']) or (N >= Len);
                  if N >= Len then Break;
                  S1 := C;
                  repeat
                    Inc(N);
                    C := S[N];
                    if (C in ['0'..'9']) then S1 := S1 + C
                    else Break;
                  until N >= Len;
                  Days1 := StrToInt(S1);
                  repeat
                    Inc(N);
                    C := S[N];
                  until (C in ['0'..'9']);
                  S1 := S[N..Len];
                  try
                    ADateTime := StrToTime(S1);
                    DecodeTime(ADateTime, Hours, Minutes, Seconds, Milliseconds);
                  except
                    on E: Exception do
                    begin
                      Hours := 0;
                      Minutes := 0;
                      Seconds := 0;
                    end;
                  end;
                  Break;
                end;
              until N = Len;
              Number1 := (Days1 * 3600 * 24) + (Hours * 3600) + (Minutes * 60) + Seconds;
              if Number1 < PreviousAS then Resync := True;
              PreviousAS := Round(Number1);
              DSstats[18] := FloatToStrF(Number1, ffFixed, 0, 0);
              DSstats[23] := FloatToStrFDot(Number1 / 3600, 3);
              Continue;
            end;
          end;

          //Connection mode
          if not Modefound then
          begin
            if Pos('Standard:', Dataline) > 0 then
            begin
              Modefound := True;
              N := Pos('G.', Dataline);
              Modestring := Dataline[N..N + 6];
              if Modestring = 'G.993.2' then
              begin
                TelnetForm.Mode := 2;       //VDSL2
                DSstats[125] := 'VDSL2';
              end
              else if Modestring = 'G.992.5' then
              begin
                TelnetForm.Mode := 1;       //ADSL2+
                DSstats[125] := 'ADSL2+';
              end
              else begin                    //ADSL or ADSL2
                TelnetForm.Mode := 0;
                DSstats[125] := 'ADSL';
              end;
              Dataline := Lines[Linenumber + 1];
              if Pos('Annex', Dataline) > 0 then    //Not with VDSL2
              begin
                N := Pos('Annex', Dataline[10..Length(Dataline)]) + 9;
                S := Dataline[N..N + 6];
                Modestring := Modestring + ' ' + S;
              end
              else if TelnetForm.Mode = 2 then Modestring := Modestring + ' (VDSL2)';

              StatsArray[1, 2] := Modestring;
              StatsArray[1, 3] := 'Showtime';     //Not available from the CLI, so assumed
              if DoDS then
              begin
                DSstats[79] := StatsArray[1, 3];
                DSstats[87] := StatsArray[1, 3]
              end;
              Modechange;
              Continue;
            end;
          end;

          //Connection speeds
          if not Speedfound then
          begin
            if Pos('Payload', Dataline) > 0 then
            begin
              Extractnumbers(Dataline, False);
              if Number1 <> PreviousSpeed then     //detect a re-sync
              begin
                Timer6.Enabled := True;   //Get the IP address after 2 seconds
              end;
              PreviousSpeed := Number1;
              Speedfound := True;
              Speedchart1.AddXY(TNow, Number2, Number1, WasPaused);
              if DoDS then
              begin
                DSstats[3] := Number1String;
                DSstats[4] := Number2String;
              end;
              Upstring := FloatToStr(Number2);
              Downstring := FloatToStr(Number1);
              Downspeed.Caption := 'Down speed: ' + Downstring + ' kbps';
              Upspeed.Caption := 'Up speed: ' + Upstring + ' kbps';
              StatsArray[1, 10] := Downstring;
              StatsArray[2, 10] := Upstring;
              Continue;
            end;
          end;

          //Attenuation values
          if not Attenfound then
          begin
            if Pos('Attenuation', Dataline) > 0 then
            begin
              Extractnumbers(Dataline, False);
              Attenfound := True;
              Down := FloatToStrF(Number1, ffFixed, 1, 1);
              Up := FloatToStrF(Number2, ffFixed, 1, 1);
              if DoDS then
              begin
                DSstats[88] := Number1String;
                DSstats[89] := Number2String;
                DSstats[130] := Number1String;
                DSstats[131] := Number2String;
              end;
              if (Number1 > 0) or (Number2 > 0) then
              begin
                if DoLogging and (TelnetForm.Mode in [0, 1]) and ((Down <> StatsArray[1, 8]) or (Up <> StatsArray[2, 8]))
                  then LogLAttenuation(Up, Down);
              end;
              StatsArray[1, 8] := Down;
              StatsArray[2, 8] := Up;
              Continue;
            end;
          end;

          //SNR margins
          if not SNRfound then
          begin
            if Pos('Margins', Dataline) > 0 then //extract SNR margins
            begin
              Extractnumbers(Dataline, False);
              SNRfound := True;
              if Number1 < -2 then Number1 := -2;    //Filter out unreal values
              if Number2 < -2 then Number2 := -2;
              if Number1 > 80 then Number1 := 0;
              if Number2 > 80 then Number2 := 0;
              CheckAlerts(atSNRM, Number1, Number2);
              CheckAlerts(atSNRMreturn, Number1, Number2);
              Marginchart.AddXY(TNow, Number2, Number1, WasPaused);
              SNRMlatestDown := Number1String;
              SNRMlatestUp := Number2String;
              if DoDS then
              begin
                DSstats[5] := Number1String;
                DSstats[6] := Number2String;
              end;
              UpdateMaxMin;
              StatsArray[1, 11] := FloatToStrF(Number1, ffFixed, 1, 1);
              StatsArray[2, 11] := FloatToStrF(Number2, ffFixed, 1, 1);
              DownSNRMargin.Caption := 'Down SNRM: ' + StatsArray[1, 11] + ' dB';
              UpSNRMargin.Caption := 'Up SNRM: ' + StatsArray[2, 11] + ' dB';
              Continue;
            end;
          end;

          //Power levels
          if not Powerfound then
          begin
            if Pos('Output power', Dataline) > 0 then
            begin
              Powerfound := True;
              Extractnumbers(Dataline, False);
              StatsArray[1, 12] := FloatToStrF(Number1, ffFixed, 1, 1);
              StatsArray[2, 12] := FloatToStrF(Number2, ffFixed, 1, 1);
              if DoDS then
              begin
                DSstats[7] := Number1String;
                DSstats[8] := Number2String;
              end;
              Continue;
            end;
          end;

          //INP
          if not INPfound then
          begin
            if Pos('INP', Dataline) > 0 then
            begin
              INPfound := True;
              ExtractNumbers(Dataline, False);
              if DoDS then
              begin
                DSstats[115] := Number1String;
                DSstats[116] := Number2String;
              end;
              PadStrings;
              GDotInpStrings[15] := Chr(9) + GDotInpTitles[15] + Chr(9) + Number1String + Chr(9) + Number2String;
              if Number1 > 0 then StatsArray[1, 14] := FloatToStrF(Number1, ffFixed, 0, 2)
              else StatsArray[1, 14] := '0';
              if Number2 > 0 then StatsArray[2, 14] := FloatToStrF(Number2, ffFixed, 0, 2)
              else StatsArray[2, 14] := '0';
              Continue;
            end;
          end;

          //Delay
          if not DelayFound then
          begin
            if Pos('Delay', Dataline) > 0 then
            begin
              DelayFound := True;
              ExtractNumbers(Dataline, False);
              if DoDS then
              begin
                DSstats[119] := Number1String;
                DSstats[120] := Number2String;
              end;
              PadStrings;
              GDotInpStrings[17] := Chr(9) + GDotInpTitles[17] + Chr(9) + Number1String + Chr(9) + Number2String;
              Continue;
            end;
          end;

          //Interleave status
          if not Interleave0found then
          begin
            if Pos('Depth', Dataline) > 0 then
            begin
              Extractnumbers(Dataline, False);
              Interleave0found := True;
              StatsArray[1, 13] := FloatToStr(Number1);
              StatsArray[2, 13] := FloatToStr(Number2);
              if DoDS then
              begin
                DSstats[39] := Number1String;
                DSstats[40] := Number2String;
              end;
              PadStrings;
              GDotInpStrings[14] := Chr(9) + GDotInpTitles[14] + Chr(9) + Number1String + Chr(9) + Number2String;
              Interleave.Caption := 'Interleave (Down/Up): ' + StatsArray[1, 13] + ' / ' + StatsArray[2, 13];
              Continue;
            end;
          end;

          //CRC, FEC and other error values
          if (not CRCFound) or (not FECFound) or (not ESFound) then
          begin

            //Errored seconds
            if Pos('(ES)', Dataline) > 0 then
            begin
              ESFound := True;
              ExtractNumbers(Dataline, False);
              if DoDS then
              begin
                DSstats[41] := Number1String;
                DSstats[42] := Number2String;
              end;
              NewESDown := Round(Number1);
              NewESUp := Round(Number2);
              ProcessES(Number2, Number1);

              //Code Violation, which equals CRC
              if LineNumber < Lines.Count - 1 then
              repeat
                Inc(LineNumber);
                Dataline := Lines[LineNumber];
                if (Pos('(CV)', Dataline) > 0) or (Pos('Code violation', Dataline) > 0) then
                begin
                  CRCFound := True;
                  ExtractNumbers(Dataline, False);
                  ProcessCRC(Number2, Number1);
                  Break;
                end;
              until LineNumber = Lines.Count - 1;    //Should never be reached

              //FEC
              if LineNumber < Lines.Count - 1 then
              repeat
                Inc(LineNumber);
                Dataline := Lines[LineNumber];
                if Pos('FEC', Dataline) > 0 then
                begin
                  FECFound := True;
                  ExtractNumbers(Dataline, False);
                  ProcessFEC(Number2, Number1);
                  Break;
                end;
              until LineNumber = Lines.Count - 1;    //Should never be reached

              //HEC
              if LineNumber < Lines.Count - 1 then
              repeat
                Inc(LineNumber);
                Dataline := Lines[LineNumber];
                if Pos('HEC', Dataline) = 1 then
                begin
                  ExtractNumbers(Dataline, False);
                  ProcessHEC(Number2, Number1);
                  Break;
                end;
              until LineNumber = Lines.Count - 1;    //Should never be reached

            end;
          end;

        end;
        OldT := TNow;
        TrayIcon1.Hint := 'Downstream speed: ' + StatsArray[1, 10] + ' kbps' + Chr(10) + 'Downstream SNRM: ' + StatsArray[1, 11] + ' dB';

      except
        on E: Exception do
        begin
          Result := leNodata;
          Collecting := False;
          Exit;
        end;
      end;

      //Process bitloading and bitswapping
      if (DoBitloading.Checked or DoBitswaps.Checked) and (Bitslist.Lines.Count > 20) then
      begin
        Bitchart1.Upcolour := Upcolour.Selected;
        Bitchart1.DownColour := Downcolour.Selected;
        Bitswapchart.Upcolour := Upcolour.Selected;
        Bitswapchart.DownColour := Downcolour.Selected;
        try
          ToneNumber := 0;
          for LineNumber := 2 to BitsList.Lines.Count - 3 do
          begin
            Bitline := BitsList.Lines[LineNumber];
            Len := Length(Bitline);
            N := Pos(':', Bitline) + 1;
            if N = 1 then
            begin
              LogEvent(leNobits);
              Break;
            end;
            repeat
              C := Bitline[N];
              if C in ['0'..'9'] then
              begin
                Bitstring := C;
                Inc(N);
                C := Bitline[N];
                if C in ['0'..'9'] then
                begin
                  Bitstring := Bitstring + C;
                  Inc(N);
                end;
                if AnnexM.Checked and (Tonenumber in [57..59]) then Bitstring := '0';
                if DoBitloading.Checked then Bitchart1.BitsArray[Tonenumber] := StrToInt(Bitstring);
                if DoBitswaps.Checked then
                begin
                  if BitsSaved then
                  begin
                    Num1 := OldBits[ToneNumber];
                    Num2 := StrToInt(Bitstring);
                    Bitswapchart.BitsArray[ToneNumber] := Bitswapchart.BitsArray[ToneNumber] + Abs(Num2 - Num1);
                  end;
                  OldBits[ToneNumber] := StrToInt(Bitstring);
                end;
                Inc(ToneNumber);
              end
              else Inc(N);
            until N > Len;
          end;
          if ToneNumber < ToneRange then
          begin
            Bitchart1.Maxreported := ToneNumber - 1;
            Bitswapchart.Maxreported := ToneNumber - 1;
          end
          else begin
            Bitchart1.Maxreported := LastToneNumber;
            Bitswapchart.Maxreported := LastToneNumber;
          end;
          if DoBitloading.Checked then Bitchart1.Drawbits(ToneRange);
          if DoBitswaps.Checked then
          begin
            if BitsSaved then Bitswapchart.DrawBits(ToneRange);
            BitsSaved := True;
          end;
        except
          on E: Exception do
          begin
            MiscString := 'Error processing bitloading';
            LogEvent(leMisc);
          end;
        end;
      end;

      //Process DSLAM/MSAN vendor
      if not VendorWritten then
      begin
        LineNumber := 5;
        repeat
          Inc(LineNumber);
          DataLine := StatsList.Lines[LineNumber];
          if Pos('Chipset', Dataline) > 0 then
          begin
            Inc(LineNumber);
            Inc(LineNumber);
            DataLine := StatsList.Lines[LineNumber];
            N := Pos(':', Dataline);
            repeat
              Inc(N);
              C := Dataline[N];
            until C in ['A'..'Z'];     //Start of local chipset vendor
            repeat
              Inc(N);
              C := Dataline[N];
            until not (C in ['A'..'Z']);
            repeat
              Inc(N);
              C := Dataline[N];
            until C in ['A'..'Z'];      //Start of remote chipset vendor
            Vendor := '';
            repeat
              Vendor := Vendor + C;
              Inc(N);
              C := Dataline[N];
            until not (C in ['A'..'Z']);
            VendorWritten := True;
          end;
        until LineNumber = 15;
        if LineNumber = 15 then VendorWritten := True;
        VendorWritten := True;
        StatsArray[1, 0] := Vendor;
      end;

      StatsArray[1, 1] := 'Not reported';   //Modem/router firmware version
      StatsArray[1, 17] := 'N/A';           //RSCorr/RS% (RS not reported)
      StatsArray[2, 17] := 'N/A';
    end

    else if IsRalink then    //Modem/router has Ralink SoC and uses 'adslphxcmd' command prefix
    begin
      Linenumber := -1;
      try     //Catchall exception handler
        StatsArray[1, 4] := 'Not reported';
        Uptime.Caption := 'DSL uptime: Not reported';
        SysUptime.Caption := 'System uptime: Not reported';
        TelnetForm.VersionText := 'Unknown';
        while Linenumber < Lines.Count - 1 do
        begin
          Inc(Linenumber);
          Dataline := Lines[Linenumber];

          //Connection speeds
          if not Speedfound then
          begin
            if Pos('Channel:', Dataline) > 0 then
            begin
              Extractnumbers(Dataline, False);
              TempNum := Number1;       //Reverse the up/down values to correspond with other values
              Number1 := Number2;
              Number2 := TempNum;
              if Number1 <> PreviousSpeed then     //detect a re-sync
              begin
                Timer6.Enabled := True;   //Get the IP address after 2 seconds
              end;
              PreviousSpeed := Number1;
              Speedfound := True;
              Speedchart1.AddXY(TNow, Number2, Number1, WasPaused);
              Upstring := FloatToStr(Number2);
              Downstring := FloatToStr(Number1);
              Downspeed.Caption := 'Down speed: ' + Downstring + ' kbps';
              Upspeed.Caption := 'Up speed: ' + Upstring + ' kbps';
              StatsArray[1, 10] := Downstring;
              StatsArray[2, 10] := Upstring;
              Continue;
            end;
          end;

          //Connection mode
          if not Modefound then   //Search for connection mode
          begin
            if Pos('Mode:', Dataline) = 1 then
            begin
              Modefound := True;
              N := 4;
              repeat
                Inc(N);
                C := Dataline[N];
              until (C in ['A'..'Z', 'a'..'z']);
              Modestring := LowerCase(Dataline[N..Length(Dataline)]);
              if Pos('vdsl2', Modestring) > 0 then TelnetForm.Mode := 2         //Tones up to LastToneNumber
              else if (Pos('adsl2+', Modestring) > 0)
              or (Pos('adsl2_plus', ModeString) > 0) then TelnetForm.Mode := 1  //Tones up to 511
              else TelnetForm.Mode := 0;                                        //Tones up tp 255
              StatsArray[1, 2] := Modestring;
              Modechange;
              Continue;
            end;
          end;

          //SNR margins
          if not SNRfound then
          begin
            if Pos('SNR', Dataline) > 0 then //extract SNR margins
            begin
              Extractnumbers(Dataline, False);
              SNRfound := True;
              if Number1 < -2 then Number1 := -2;      //Filter out unreal values
              if Number2 < -2 then Number2 := -2;
              if Number1 > 80 then Number1 := 0;
              if Number2 > 80 then Number2 := 0;
              CheckAlerts(atSNRM, Number1, Number2);
              CheckAlerts(atSNRMreturn, Number1, Number2);
              Marginchart.AddXY(TNow, Number2, Number1, WasPaused);
              SNRMlatestDown := Number1String;
              SNRMlatestUp := Number2String;
              UpdateMaxMin;
              StatsArray[1, 11] := FloatToStrF(Number1, ffFixed, 1, 1);
              StatsArray[2, 11] := FloatToStrF(Number2, ffFixed, 1, 1);
              DownSNRMargin.Caption := 'Down SNRM: ' + StatsArray[1, 11] + ' dB';
              UpSNRMargin.Caption := 'Up SNRM: ' + StatsArray[2, 11] + ' dB';
              Continue;
            end;
          end;

          //Attenuation values
          if not Attenfound then
          begin
            if Pos('Attn', Dataline) > 0 then
            begin
              Extractnumbers(Dataline, False);
              Attenfound := True;
              Down := FloatToStrF(Number1, ffFixed, 1, 1);
              Up := FloatToStrF(Number2, ffFixed, 1, 1);
              if (Number1 > 0) or (Number2 > 0) then
              begin
                if DoLogging and (TelnetForm.Mode in [0, 1]) and ((Down <> StatsArray[1, 8]) or (Up <> StatsArray[2, 8]))
                  then LogLAttenuation(Up, Down);
              end;
              StatsArray[1, 8] := Down;
              StatsArray[2, 8] := Up;
              Continue;
            end;
          end;

          //Power levels
          if not Powerfound then
          begin
            if Pos('Pwr', Dataline) > 0 then
            begin
              Extractnumbers(Dataline, False);
              Powerfound := True;
              StatsArray[1, 12] := FloatToStrF(Number1, ffFixed, 1, 1);
              StatsArray[2, 12] := FloatToStrF(Number2, ffFixed, 1, 1);
              Continue;
            end;
          end;

          //Max attainable speeds
          if not Maxfound then
          begin
            if Pos('Max', Dataline) = 1 then
            begin
              Extractnumbers(Dataline, False);
              Speedchart1.AddXYmax(TNow, Number2, Number1, WasPaused);
              MaxUpLabel.Caption := 'Max attainable now: ' + IntToStr(Round(Number1)) + ' kbps';
              MaxDownLabel.Caption := 'Max attainable now: ' + IntToStr(Round(Number2)) + ' kbps';
              Maxfound := True;
              Continue;
            end;
          end;

          //Connection speeds
          if not Speedfound then
          begin
            if Pos('Rate', Dataline) > 0 then
            begin
              Extractnumbers(Dataline, False);
              if Number1 <> PreviousSpeed then     //detect a re-sync
              begin
                Timer6.Enabled := True;   //Get the IP address after 2 seconds
              end;
              PreviousSpeed := Number1;
              Speedfound := True;
              Speedchart1.AddXY(TNow, Number2, Number1, WasPaused);
              Upstring := FloatToStr(Number2);
              Downstring := FloatToStr(Number1);
              Downspeed.Caption := 'Down speed: ' + Downstring + ' kbps';
              Upspeed.Caption := 'Up speed: ' + Upstring + ' kbps';
              StatsArray[1, 10] := Downstring;
              StatsArray[2, 10] := Upstring;
              Continue;
            end;
          end;

          //Interleave status
          if not Interleave0found then
          begin
            if Pos('Depth', Dataline) > 0 then
            begin
              Extractnumbers(Dataline, False);
              Interleave0found := True;
              StatsArray[1, 13] := FloatToStr(Number1);
              StatsArray[2, 13] := FloatToStr(Number2);
              Interleave.Caption := 'Interleave (Down/Up): ' + StatsArray[1, 13] + ' / ' + StatsArray[2, 13];
              Continue;
            end;
          end;

          //CRC, FEC and other error values
          if (not CRCFound) or (not FECFound) or (not ESFound) then
          begin

            //RS
            begin
              if Pos('RS:', Dataline) = 1 then
              begin
                ExtractNumbers(Dataline, False);
                RSDown := Round(Number1);
                RSup := Round(Number2);
              end
              else Continue;
            end;

            //HEC
            if LineNumber < Lines.Count - 1 then
            repeat
              Inc(LineNumber);
              Dataline := Lines[LineNumber];
              if Pos('HEC', Dataline) = 1 then
              begin
                ExtractNumbers(Dataline, False);
                ProcessHEC(Number2, Number1);
                Break;
              end;
            until LineNumber = Lines.Count - 1;    //Should never be reached

            //Errored seconds
            if LineNumber < Lines.Count - 1 then
            repeat
              Inc(LineNumber);
              Dataline := Lines[LineNumber];
              if Pos('ES', Dataline) > 0 then
              begin
                ESFound := True;
                ExtractNumbers(Dataline, False);
                NewESDown := Round(Number1);
                NewESUp := Round(Number2);
                ProcessES(Number2, Number1);
                Break;
              end;
            until LineNumber = Lines.Count - 1;  //Should never be reached

            //CRC
            if LineNumber < Lines.Count - 1 then
            repeat
              Inc(LineNumber);
              Dataline := Lines[LineNumber];
              if Pos('CRC', Dataline) > 0 then
              begin
                CRCFound := True;
                ExtractNumbers(Dataline, False);
                ProcessCRC(Number2, Number1);
                Break;
              end;
            until LineNumber = Lines.Count - 1;    //Should never be reached

            //FEC
            if LineNumber < Lines.Count - 1 then
            repeat
              Inc(LineNumber);
              Dataline := Lines[LineNumber];
              if Pos('FEC', Dataline) > 0 then
              begin
                FECFound := True;
                ExtractNumbers(Dataline, False);
                ProcessFEC(Number2, Number1);
                Break;
              end;
            until LineNumber = Lines.Count - 1;    //Should never be reached
          end;
        end;
      except
        on E: Exception do
        begin
          Result := leNodata;
          Collecting := False;
          Exit;
        end;
      end;

      StatsArray[1, 1] := 'Not reported';   //Modem/router firmware version
      if RSdown > 0 then StatsArray[1, 17] := FloatToStrF(RSCorr0down / RSdown * 100, ffFixed, 0, 4)    //RSCorr/RS%
      else StatsArray[1, 16] := 'N/A';
      if RSup > 0 then StatsArray[2, 17] := FloatToStrF(RSCorr0up / RSup * 100, ffFixed, 0, 4)
      else StatsArray[2, 17] := 'N/A';

    end

    else begin               //Normal Broadcom-based modem/router

      //Is section 'Since Link time =...' present near the end
      Linenumber := Lines.Count;  //Search back from last line
      while Linenumber >= Lines.Count - 16 do
      begin
        Dec(Linenumber);
        Dataline := Lines[Linenumber];
        Len := Length(Dataline);
        if Pos('Since Link', Dataline) > 0 then
        begin
          SinceLinkTimeFound := True;
          repeat       //Check for LOS etc.
            Inc(LineNumber);
            Dataline := Lines[LineNumber];
            Len := Length(Dataline);
            if Pos('LOS:', Dataline) = 1 then
            begin
              ExtractNumbers(Dataline, False);
              GetDeltaValues(LOSup, LOSdown, Round(Number2), Round(Number1));
              DSstats[136] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
              DSstats[137] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
              LOSup := Round(Number2);
              LOSdown := Round(Number1);
              Inc(LineNumber);
              Dataline := Lines[LineNumber];     //LOF
              ExtractNumbers(Dataline, False);
              GetDeltaValues(LOFup, LOFdown, Round(Number2), Round(Number1));
              DSstats[138] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
              DSstats[139] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
              LOFup := Round(Number2);
              LOFdown := Round(Number1);
              Inc(LineNumber);
              Dataline := Lines[LineNumber];     //LOM
              ExtractNumbers(Dataline, False);
              GetDeltaValues(LOMup, LOMdown, Round(Number2), Round(Number1));
              DSstats[140] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
              DSstats[141] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
              LOMup := Round(Number2);
              LOMdown := Round(Number1);
              LineNumber := Lines.Count;  //To break out of main loop
            end;
          until LineNumber >= Lines.Count;
          Break;
        end;
      end;

      //Extract the rest of the stats

      Linenumber := -1;
      try
        while Linenumber < Lines.Count - 1 do
        begin
          Inc(Linenumber);
          Dataline := Lines[Linenumber];

          //Connection status
          if not Statusfound then
          begin
            if Pos('Status:', Dataline) = 1 then
            begin
              Statusfound := True;
              N := 7;
              Len := Length(Dataline);
              repeat
                Inc(N);
                C := Dataline[N];
              until (C in ['A'..'Z', 'a'..'z']);
              Statusstring := C;
              while N < Len do
              begin
                Inc(N);
                C := Dataline[N];
                if C = ' ' then Break;
                Statusstring := Statusstring + C;
              end;
              StatsArray[1, 3] := Statusstring;
              if DoDS then
              begin
                DSstats[79] := StatusString;
                DSstats[87] := StatusString;
              end;
              Index := Pos('Retrain', Dataline);           //Retrain reason can be on the same line
              if Index = 0 then
              begin
                Dataline := Lines[LineNumber + 1];         //Or on the next line
                Len := Length(Dataline);
                Index := Pos('Retrain', Dataline);
              end;
              if Index > 0 then
              begin
                N := Index;
                repeat
                  Inc(N);
                  C := Dataline[N];
                  if (C in ['0'..'9']) then
                  begin
                    S := C;
                    repeat
                      Inc(N);
                      if N > Len then Break;
                      C := Dataline[N];
                      if (C in ['0'..'9']) then S := S + C
                      else N := Len;                           //Break from both loops
                    until N >= Len;
                  end;
                until N >= Len;
                if DoDS and (S > '') then DSstats[80] := S;
              end;
              Continue;
            end;
          end;

          //Max attainable speeds
          if not Maxfound then
          begin
            if Pos('Max', Dataline) = 1 then
            begin
              if Dataline[4] = ':' then  //Upstream reported first (newer modem/routers)
              begin
                Extractnumbers(Dataline, True);
                Speedchart1.AddXYmax(TNow, Number1, Number2, WasPaused);
                MaxUpLabel.Caption := 'Max attainable now: ' + IntToStr(Round(Number1)) + ' kbps';
                MaxDownLabel.Caption := 'Max attainable now: ' + IntToStr(Round(Number2)) + ' kbps';
                if DoDS then
                begin
                  DSstats[9] := Number2String;
                  DSstats[10] := Number1string;
                  DSstats[134] := Number2String;
                  DSstats[135] := Number1String;
                end;
              end
              else begin    //Downstream reported first (older modem/routers)
                Extractnumbers(Dataline, False);
                Speedchart1.AddXYmax(TNow, Number2, Number1, WasPaused);
                if DoDS then
                begin
                  DSstats[9] := Number1String;
                  DSstats[10] := Number2String;
                  DSstats[134] := Number1String;
                  DSstats[135] := Number2String;
                end;
              end;
              Maxfound := True;
              Continue;
            end;
          end;

          //Connection speeds
          if not Speedfound then
          begin
            if (Pos('Upstream rate', Dataline) > 0) and (Dataline[1..3] <> 'Max') then
            begin
              Speedfound := True;
              Extractnumbers(Dataline, True);
            end
            else if Pos('Rate', Dataline) = 1 then     //DSL-2640B for example
            begin
              Speedfound := True;
              Extractnumbers(Dataline, False);
              TempNum := Number1;     //Reverse the numbers to match the order in the "Upstream rate" line
              Number1 := Number2;
              Number2 := TempNum;
            end;
            if Speedfound then
            begin
              if Number2 <> PreviousSpeed then     //May have been a re-sync (not if SRA is enabled for the connection)
              begin
                TelnetForm.QLNDone := False;       //Refresh these in case it was a resync
                TelnetForm.HLogDone := False;
                QGraphDone := False;
                HGraphDone := False;
                //if not FirstSample then Resync := True;
                //Timer6.Enabled := True;   //Get the IP address after 2 seconds
              end;
              PreviousSpeed := Number2;
              Speedchart1.AddXY(TNow, Number1, Number2, WasPaused);
              if DoDS then
              begin
                DSstats[3] := Number2String;
                DSstats[4] := Number1String;
              end;
              Upstring := FloatToStr(Number1);
              Downstring := FloatToStr(Number2);
              Downspeed.Caption := 'Down speed: ' + Downstring + ' kbps';
              Upspeed.Caption := 'Up speed: ' + Upstring + ' kbps';
              StatsArray[1, 10] := Downstring;
              StatsArray[2, 10] := Upstring;
              if Pos('Status:', Dataline) = 1 then    //Status is on the same line
              begin
                Statusfound := True;
                N := 7;
                Len := Length(Dataline);
                repeat
                  Inc(N);
                  C := Dataline[N];
                until (C in ['A'..'Z', 'a'..'z']);
                StatusString := C;
                while N < Len do
                begin
                  Inc(N);
                  C := Dataline[N];
                  if C = ' ' then Break;
                  StatusString := StatusString + C;
                end;
                StatsArray[1, 3] := StatusString;
                if DoDS then
                begin
                  DSstats[79] := StatusString;
                  DSstats[87] := StatusString;
                end;
              end;
              Continue;
            end;
          end;

          //Link power state
          if not PowerStateFound then
          begin
            if Pos('Link', Dataline) = 1 then
            begin
              PowerStateFound := True;
              N := Pos(':', Dataline);
              S := '';
              Len := Length(Dataline);
              repeat
                Inc(N);
                C := Dataline[N];
                if (C in ['A'..'Z']) then
                repeat
                  S := S + C;
                  Inc(N);
                  if N > Len then Break;
                  C := Dataline[N];
                until not (C in ['A'..'Z', '0'..'9']);
              until N >= Len;
              if DoDS then DSstats[82] := S;
              Continue;
            end;
          end;

          //Connection mode
          if not Modefound then   //Search for connection mode
          begin
            if Pos('Mode', Dataline) = 1 then
            begin
              Modefound := True;
              N := 4;
              repeat
                Inc(N);
                C := Dataline[N];
              until (C in ['A'..'Z', 'a'..'z']);
              Modestring := C;
              repeat
                Inc(N);
                if N > Length(Dataline) then Break;
                C := Dataline[N];
                if not (C in ['A'..'Z', '0'..'9', '+', '(', ')']) then Break;
                Modestring := Modestring + C;
              until False;
              if Pos('VDSL2', Modestring) > 0 then TelnetForm.Mode := 2         //Tones up to LastToneNumber
              else if Pos('ADSL2+', Modestring) > 0 then TelnetForm.Mode := 1   //Tones up to 511
              else TelnetForm.Mode := 0;                                        //Tones up tp 255
              StatsArray[1, 2] := Modestring;
              if DoDS then DSstats[125] := Modestring;
              Modechange;
              Continue;
            end;
          end;

          //VDSL2 profile
          if not ProfileFound then
          begin
            if Pos('VDSL2 Profile:', Dataline) > 0 then
            begin
              Len := Length(Dataline);
              S := '';
              N := Pos(':', Dataline);
              repeat
                C := Dataline[N];
                if (C in ['0'..'9']) then
                begin
                  S := S + C;
                  repeat
                    Inc(N);
                    if N > Len then Break;
                    C := Dataline[N];
                    if C = ' ' then Break;
                    S := S + C;
                  until N >= Len
                end;
                Inc(N);
              until (N >= Len);
              if DoDS then DSstats[83] := S;
              if S > '' then StatsArray[1, 2] := ModeString + ' Profile ' + S;
              ProfileFound := True;
              Continue;
            end;
          end;

          //TPS-TC
          if not TPSTCFound then
          begin
            if Pos('TPS-TC', Dataline) > 0 then
            begin
              if Pos('ATM', Dataline) > 0 then S := 'ATM'
              else if Pos('PTM', Dataline) > 0 then S := 'PTM'
              else S := 'Unknown';
              if DoDS then DSstats[11] := S;
              TPSTCFound := True;
              Continue;
            end;
          end;

          //Trellis
          if not TrellisFound then
          begin
            if Pos('Trellis:', Dataline) > 0 then
            begin
              if DoDS then
              begin
                N := Pos('U:', Dataline);
                if Dataline[N + 3] = 'N' then DSstats[84] := 'ON' else DSstats[84] := 'OFF';
                N := Pos('D:', Dataline);
                if Dataline[N + 3] = 'N' then DSstats[85] := 'ON' else DSstats[85] := 'OFF';
              end;
              TrellisFound := True;
              Continue;
            end;
          end;

          //Line status
          if not LineStatusFound then
          begin
            Len := Length(Dataline);
            if Pos('Line Status:', Dataline) > 0 then
            begin
              S := '';
              N := 12;
              repeat
                Inc(N);
                C := Dataline[N];
                if (C in ['A'..'Z', 'a'..'z']) then
                repeat
                  S := S + C;
                  Inc(N);
                  if N > Len then Break;
                  C := Dataline[N];
                  if C = ' ' then N := Len + 1;
                until (N > Len) or not (C in ['A'..'Z', 'a'..'z']);
              until N >= Len;
              LineStatusFound := True;
              if DoDS and (S > '') then DSstats[86] := S;
              Continue;
            end;
          end;

          //SNR margins
          if not SNRfound then
          begin
            if Pos('SNR', Dataline) > 0 then //extract SNR margins
            begin
              Extractnumbers(Dataline, False);
              SNRfound := True;
              if Number1 < -2 then Number1 := -2;      //Filter out unreal values
              if Number2 < -2 then Number2 := -2;
              if Number1 > 80 then
              begin
                Number1 := 0;
                Number1String := '0';
              end;
              if Number2 > 80 then
              begin
                Number2 := 0;
                Number2String := '0';
              end;
              CheckAlerts(atSNRM, Number1, Number2);
              CheckAlerts(atSNRMreturn, Number1, Number2);
              Marginchart.AddXY(TNow, Number2, Number1, WasPaused);
              SNRMlatestDown := Number1String;
              SNRMlatestUp := Number2String;
              if DoDS then
              begin
                DSstats[5] := Number1String;
                DSstats[6] := Number2String;
              end;
              UpdateMaxMin;
              StatsArray[1, 11] := FloatToStrF(Number1, ffFixed, 1, 1);
              StatsArray[2, 11] := FloatToStrF(Number2, ffFixed, 1, 1);
              DownSNRMargin.Caption := 'Down SNRM: ' + StatsArray[1, 11] + ' dB';
              UpSNRMargin.Caption := 'Up SNRM: ' + StatsArray[2, 11] + ' dB';
              Continue;
            end;
          end;

          //Attenuation values
          if not Attenfound then
          begin
            if Pos('Attn', Dataline) > 0 then
            begin
              Extractnumbers(Dataline, False);
              Attenfound := True;
              Down := FloatToStrF(Number1, ffFixed, 1, 1);
              Up := FloatToStrF(Number2, ffFixed, 1, 1);
              if DoDS then
              begin
                DSstats[88] := Number1String;
                DSstats[89] := Number2String;
                DSstats[130] := Number1String;
                DSstats[131] := Number2String;
              end;
              if (Number1 > 0) or (Number2 > 0) then
              begin
                if DoLogging and (TelnetForm.Mode in [0, 1]) and ((Down <> StatsArray[1, 8]) or (Up <> StatsArray[2, 8]))
                  then LogLAttenuation(Up, Down);
              end;
              StatsArray[1, 8] := Down;
              StatsArray[2, 8] := Up;
              Continue;
            end;
          end;

          //Power levels
          if not Powerfound then
          begin
            if Pos('Pwr', Dataline) > 0 then
            begin
              Extractnumbers(Dataline, False);
              Powerfound := True;
              StatsArray[1, 12] := FloatToStrF(Number1, ffFixed, 1, 1);
              StatsArray[2, 12] := FloatToStrF(Number2, ffFixed, 1, 1);
              if DoDS then
              begin
                DSstats[7] := Number1String;
                DSstats[8] := Number2String;
              end;
              Continue;
            end;
          end;

          //**** Bearer 0 ****

          if not BFound then
          begin
            if Pos('B:', Dataline) > 0 then
            begin
              Extractnumbers(Dataline, False);
              BFound := True;
              if DoDS then
              begin
                DSstats[90] := Number1String;
                DSstats[91] := Number2String;
                Inc(LineNumber);     //M:
                Dataline := Lines[LineNumber];
                Extractnumbers(Dataline, False);
                DSstats[92] := Number1String;
                DSstats[93] := Number2String;
                Inc(LineNumber);     //T:
                Dataline := Lines[LineNumber];
                Extractnumbers(Dataline, False);
                DSstats[94] := Number1String;
                DSstats[95] := Number2String;
                Inc(LineNumber);     //R:
                Dataline := Lines[LineNumber];
                Extractnumbers(Dataline, False);
                DSstats[96] := Number1String;
                DSstats[97] := Number2String;
                Inc(LineNumber);     //S:
                Dataline := Lines[LineNumber];
                Extractnumbers(Dataline, False);
                DSstats[98] := Number1String;
                DSstats[99] := Number2String;
                Inc(LineNumber);     //L:
                Dataline := Lines[LineNumber];
                Extractnumbers(Dataline, False);
                DSstats[100] := Number1String;
                DSstats[101] := Number2String;
              end;
              Continue;
            end;
          end;

          //Interleave status
          if not Interleave0found then
          begin
            if Pos('D:', Dataline) = 1 then //extract interleave levels
            begin
              if not GDotInp then
              begin
                Increments := 1;    //First check forwards to see if there is a second framing section (e.g. DG834Nv1)
                repeat
                  S := Lines[LineNumber + Increments];
                  if Pos('D:', S) = 1 then
                  begin
                    LineNumber := LineNumber + Increments;
                    Dataline := Lines[LineNumber];
                    Break;
                  end;
                  Inc(Increments);
                until Increments = 12;
              end;
              Extractnumbers(Dataline, False);
              Interleave0found := True;
              StatsArray[1, 13] := FloatToStr(Number1);
              StatsArray[2, 13] := FloatToStr(Number2);
              if DoDS then
              begin
                DSstats[39] := Number1String;
                DSstats[40] := Number2String;
              end;
              PadStrings;
              GDotInpStrings[14] := Chr(9) + GDotInpTitles[14] + Chr(9) + Number1String + Chr(9) + Number2String;
              Interleave.Caption := 'Interleave (Down/Up): ' + StatsArray[1, 13] + ' / ' + StatsArray[2, 13];
              Continue;
            end;
          end;

          //**** General ****

          if GDotInp then
          begin
            if not RxQ0Found then
            begin
              if Pos('RxQ', Dataline) = 1 then  //Process RxQ0 values
              begin
                RxQ0Found := True;
                ExtractNumbers(Dataline, False);
                if DoDS then
                begin
                  DSstats[24] := Number1String;
                  DSstats[25] := Number2String;
                end;
                PadStrings;
                GDotInpStrings[9] := Chr(9) + GDotInpTitles[9] + Chr(9) + Number1String + Chr(9) + Number2String;
                RxQ0Up := Round(Number2);
                RxQ0Down := Round(Number1);

                Inc(Linenumber);
                Dataline := Lines[Linenumber];
                if Pos('TxQ', Dataline) = 1 then  //Process TxQ values
                begin
                  ExtractNumbers(Dataline, False);
                  if DoDS then
                  begin
                    DSstats[27] := Number1String;
                    DSstats[28] := Number2String;
                  end;
                  PadStrings;
                  GDotInpStrings[10] := Chr(9) + GDotInpTitles[10] + Chr(9) + Number1String + Chr(9) + Number2String;
                  TxQ0Up := Round(Number2);
                  TxQ0Down := Round(Number1);
                end;

                Inc(Linenumber);
                Dataline := Lines[Linenumber];
                if Pos('raming', Dataline) > 0 then  //Process G.INP Framing values
                begin
                  ExtractNumbers(Dataline, False);
                  if DoDS then
                  begin
                    DSstats[30] := Number1String;
                    DSstats[31] := Number2String;
                  end;
                  PadStrings;
                  GDotInpStrings[11] := Chr(9) + GDotInpTitles[11] + Chr(9) + Number1String + Chr(9) + Number2String;
                  Framing0Up := Round(Number2);
                  Framing0Down := Round(Number1);
                  if Framing0Up = 0 then StatsArray[2, 15] := 'Not enabled' else StatsArray[2, 15] := 'Enabled';
                end;

                Inc(Linenumber);
                Dataline := Lines[Linenumber];
                if Pos('ookback', Dataline) > 0 then  //Process G.INP Lookback values
                begin
                  ExtractNumbers(Dataline, False);
                  if DoDS then
                  begin
                    DSstats[33] := Number1String;
                    DSstats[34] := Number2String;
                  end;
                  PadStrings;
                  GDotInpStrings[12] := Chr(9) + GDotInpTitles[12] + Chr(9) + Number1String + Chr(9) + Number2String;
                  Lookback0Up := Round(Number2);
                  Lookback0Down := Round(Number1);
                end;

                Inc(Linenumber);
                Dataline := Lines[Linenumber];
                if Pos('RRC', Dataline) = 1 then  //Process RRC values
                begin
                  ExtractNumbers(Dataline, False);
                  PadStrings;
                  GDotInpStrings[13] := Chr(9) + GDotInpTitles[13] + Chr(9) + Number1String + Chr(9) + Number2String;
                  RRC0Up := Round(Number2);
                  RRC0Down := Round(Number1);
                end;

                Continue;
              end;
            end;
          end;

          //**** Bearer 1 ****

          if GDotInp then
          begin
            if Pos('D:', Dataline) = 1 then      //B1 Interleave depth
            begin
              ExtractNumbers(Dataline, False);
              if DoDS then
              begin
                DSstats[121] := Number1String;
                DSstats[122] := Number2String;
              end;
              PadStrings;
              GDotInpStrings[19] := Chr(9) + GDotInpTitles[19] + Chr(9) + Number1String + Chr(9) + Number2String;
            end;
          end;

          // **** Bearer 0 ****

          //SF or OHF
          if not OHF0Found then
          begin
            if (Pos('SF:', Dataline) = 1) or (Pos('OHF', Dataline) = 1) then
            begin
              ExtractNumbers(Dataline, False);
              OHF0Found := True;
              Continue;
            end;
          end;

          //SFErr or OHFErr
          if not OHFErr0Found then
          begin
            if (Pos('SFErr', Dataline) = 1) or (Pos('OHFErr', Dataline) = 1) then
            begin
              OHFErr0found := True;
              ExtractNumbers(Dataline, False);
              ProcessOHFErr(Number2, Number1);
              Continue;
            end;
          end;

          //RS
          if not RS0Found then
          begin
            if Pos('RS', Dataline) = 1 then
            begin
              ExtractNumbers(Dataline, False);
              RSDown := Round(Number1);
              RSup := Round(Number2);
              RS0Found := True;
              Continue;
            end;
          end;

          //RSCorr
          if not RSCorr0Found then
          begin
            if Pos('RSCorr', Dataline) = 1 then
            begin
              RSCorr0Found := True;
              ExtractNumbers(Dataline, False);
              ProcessRSCorr(Number2, Number1);
              Continue;
            end;
          end;

          //RSUncorr
          if not RSUncorr0Found then
          begin
            if Pos('RSUnCorr', Dataline) = 1 then
            begin
              RSUncorr0Found := True;
              ExtractNumbers(Dataline, False);
              RSUnCorr0Up := Round(Number2);
              RSUnCorr0Down := Round(Number1);
              if DoDS then
              begin
                DSstats[36] := Number1String;
                DSstats[37] := Number2String;
              end;
              if RSdown > 0 then StatsArray[1, 18] := FloatToStrF(RSUnCorr0Down / RSdown * 100, ffFixed, 0, 4)
              else StatsArray[1, 18] := 'N/A';
              if RSup > 0 then StatsArray[2, 18] := FloatToStrF(RSUnCorr0Up / RSup * 100, ffFixed, 0, 4)
              else StatsArray[2, 18] := 'N/A';
              Continue;
            end;
          end;

          //**** Bearer 1 ****

          if GDotInp then
          begin
            if Pos('OHFErr', Dataline) = 1 then
            begin
              ExtractNumbers(Dataline, False);
              GetDeltaValues(OHFErr1Up, OHFErr1Down, Round(Number2), Round(Number1));
             { if DoDS then
              begin
                DSstats[77] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
                DSstats[78] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
              end; }
              OHFErrchart.AddXY(TNow, Round(DeltaUp), Round(DeltaDown), WasPaused);
              OHFErr1Up := Round(Number2);
              OHFErr1Down := Round(Number1);

              Linenumber := Linenumber + 2;
              Dataline := Lines[Linenumber];
              if Pos('RSCorr', Dataline) = 1 then
              begin
                ExtractNumbers(Dataline, False);
                GetDeltaValues(RSCorr1Up, RSCorr1Down, Round(Number2), Round(Number1));
              {  if DoDS then
                begin
                  DSstats[73] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
                  DSstats[74] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
                end; }
                RSCorrchart.AddXY(TNow, Round(DeltaUp), Round(DeltaDown), WasPaused);
                RSCorr1Up := Round(Number2);
                RSCorr1Down := Round(Number1);
              end;

              Inc(Linenumber);
              Dataline := Lines[Linenumber];
              if Pos('RSUnCorr', Dataline) = 1 then
              begin
                ExtractNumbers(Dataline, False);
                GetDeltaValues(RSUnCorr1Up, RSUnCorr1Down, Round(Number2), Round(Number1));
               { if DoDS then
                begin
                  DSstats[75] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
                  DSstats[76] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
                end; }
                RSUnCorr1Up := Round(Number2);
                RSUnCorr1Down := Round(Number1);
              end;

              Continue;
            end;
          end;

          //**** General ****

          if GDotInp then
          begin
            if not rtxtxFound then    //Retransmit counters
            begin
              if Pos('rtx_tx', Dataline) = 1 then   //Process rtx_tx values
              begin
                rtxtxFound := True;
                ExtractNumbers(Dataline, False);
                GetDeltaValues(rtxtxUp, rtxtxDown, Round(Number2), Round(Number1));
                if DoDS then
                begin
                  DSstats[102] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
                  DSstats[103] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
                end;
                rtxtxChart.AddXY(TNow, Round(DeltaUp), Round(DeltaDown), WasPaused);
                rtxtxUp := Round(Number2);
                rtxtxDown := Round(Number1);
                PadStrings;
                GDotInpStrings[2] := Chr(9) + GDotInpTitles[2] + Chr(9) + Number1String + Chr(9) + Number2String;

                Inc(Linenumber);
                Dataline := Lines[Linenumber];
                if Pos('rtx_c', Dataline) = 1 then  //Process rtx_c values
                begin
                  ExtractNumbers(Dataline, False);
                  GetDeltaValues(rtxcUp, rtxcDown, Round(Number2), Round(Number1));
                  if DoDS then
                  begin
                    DSstats[104] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
                    DSstats[105] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
                  end;
                  rtxcChart.AddXY(TNow, Round(DeltaUp), Round(DeltaDown), WasPaused);
                  rtxcUp := Round(Number2);
                  rtxcDown := Round(Number1);
                  PadStrings;
                  GDotInpStrings[3] := Chr(9) + GDotInpTitles[3] + Chr(9) + Number1String + Chr(9) + Number2String;
                end;

                Inc(Linenumber);
                Dataline := Lines[Linenumber];
                if Pos('rtx_uc', Dataline) = 1 then    //Process rtx_uc values
                begin
                  ExtractNumbers(Dataline, False);
                  GetDeltaValues(rtxucUp, rtxucDown, Round(Number2), Round(Number1));
                  if DoDS then
                  begin
                    DSstats[106] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
                    DSstats[107] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
                  end;
                  rtxucChart.AddXY(TNow, Round(DeltaUp), Round(DeltaDown), WasPaused);
                  rtxucUp := Round(Number2);
                  rtxucDown := Round(Number1);
                  PadStrings;
                  GDotInpStrings[4] := Chr(9) + GDotInpTitles[4] + Chr(9) + Number1String + Chr(9) + Number2String;
                end;

              end;
              Continue;
            end;

            if not LEFTRSFound then   //G.INP counters
            begin
              if Pos('LEFTRS', Dataline) = 1 then     //Process LEFTRS values
              begin
                LEFTRSFound := True;
                ExtractNumbers(Dataline, False);
                GetDeltaValues(LEFTRSUp, LEFTRSDown, Round(Number2), Round(Number1));
                if DoDS then
                begin
                  DSstats[108] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
                  DSstats[109] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
                end;
                LEFTRSChart.AddXY(TNow, Round(DeltaUp), Round(DeltaDown), WasPaused);
                LEFTRSUp := Round(Number2);
                LEFTRSDown := Round(Number1);
                PadStrings;
                GDotInpStrings[5] := Chr(9) + GDotInpTitles[5] + Chr(9) + Number1String + Chr(9) + Number2String;
                LEFTRSUp := Round(Number2);
                LEFTRSDown := Round(Number1);

                Inc(Linenumber);
                Dataline := Lines[Linenumber];
                if Pos('minEFTR', Dataline) = 1 then  //Process min_EFTR values
                begin
                  ExtractNumbers(Dataline, False);
                  if DoDS then
                  begin
                    DSstats[110] := Number1String;
                    DSstats[111] := Number2String;
                  end;
                  PadStrings;
                  GDotInpStrings[6] := Chr(9) + GDotInpTitles[6] + Chr(9) + Number1String + Chr(9) + Number2String;
                  minEFTRUp := Round(Number2);
                  minEFTRDown := Round(Number1);
                end;

                Inc(Linenumber);
                Dataline := Lines[Linenumber];
                if Pos('errFreeBits', Dataline) = 1 then    //Process errFreeBits values
                begin
                  ExtractNumbers(Dataline, False);
                  PadStrings;
                  GDotInpStrings[7] := Chr(9) + GDotInpTitles[7] + Chr(9) + Number1String + Chr(9) + Number2String;
                  errFreeBitsUp := Round(Number2);
                  errFreeBitsDown := Round(Number1);
                end;

              end;
              Continue
            end;
          end;

          //**** Bearer 0 ****

          //HEC
          if not HECFound then
          begin
            if Pos('HEC', Dataline) = 1 then
            begin
              HECFound := True;
              ExtractNumbers(Dataline, False);
              ProcessHEC(Number2, Number1);
              Continue;
            end;
          end;

          //OCD
          if not OCDFound then
          begin
            if Pos('OCD', Dataline) = 1 then
            begin
              OCDFound := True;
              ExtractNumbers(Dataline, False);
              Continue;
            end;
          end;

          //LCD
          if not LCDFound then
          begin
            if Pos('LCD', Dataline) = 1 then
            begin
              LCDFound := True;
              ExtractNumbers(Dataline, False);
              Continue;
            end;
          end;

          //Total cells
          if not TotalcellsFound then
          begin
            if Pos('Total Cells', Dataline) = 1 then
            begin
              TotalcellsFound := True;
              ExtractNumbers(Dataline, False);
              Continue;
            end;
          end;

          //Data cells
          if not DatacellsFound then
          begin
            if Pos('Data Cells', Dataline) = 1 then
            begin
              DatacellsFound := True;
              ExtractNumbers(Dataline, False);
              Continue;
            end;
          end;

          //Drop cells
          if not DropcellsFound then
          begin
            if Pos('Drop Cells', Dataline) = 1 then
            begin
              DropcellsFound := True;
              ExtractNumbers(Dataline, False);
              Continue;
            end;
          end;

          //Bit errors
          if not BiterrorsFound then
          begin
            if Pos('Bit Errors', Dataline) = 1 then
            begin
              BiterrorsFound := True;
              ExtractNumbers(Dataline, False);
              Continue;
            end;
          end;

          //**** Bearer 1 ****

          //HEC etc, all zeroes?

          //**** General ****

          //Errored seconds
          if not ESFound then
          begin
            if Pos('ES', Dataline) = 1 then
            begin
              ESFound := True;
              ExtractNumbers(Dataline, False);
              NewESDown := Round(Number1);
              NewESUp := Round (Number2);
              if DoDS then
              begin
                DSstats[41] := Number1String;
                DSstats[42] := Number2String;
              end;
              ProcessES(Number2, Number1);
              Continue;
            end;
          end;

          //SES
          if not SESFound then
          begin
            if Pos('SES', Dataline) = 1 then
            begin
              SESFound := True;
              ExtractNumbers(Dataline, False);
              if DoDS then
              begin
                DSstats[43] := Number1String;
                DSstats[44] := Number2String;
              end;
              ProcessSES(Number2, Number1);
              Continue;
            end;
          end;

          //UAS
          if not UASFound then
          begin
            if Pos('UAS', Dataline) = 1 then
            begin
              UASFound := True;
              ExtractNumbers(Dataline, False);
              if DoDS then
              begin
                DSstats[113] := Number1String;
                DSstats[114] := Number2String;
              end;
              Continue;
            end;
          end;

          //**** Bearer 0 ****

          //INP
          if not INPFound then
          begin
            if Pos('INP', Dataline) = 1 then
            begin
              INPFound := True;
              ExtractNumbers(Dataline, False);
              if DoDS then
              begin
                DSstats[115] := Number1String;
                DSstats[116] := Number2String;
              end;
              PadStrings;
              GDotInpStrings[15] := Chr(9) + GDotInpTitles[15] + Chr(9) + Number1String + Chr(9) + Number2String;
              if Number1 > 0 then StatsArray[1, 14] := FloatToStrF(Number1, ffFixed, 0, 2)
              else StatsArray[1, 14] := '0';
              if Number2 > 0 then StatsArray[2, 14] := FloatToStrF(Number2, ffFixed, 0, 2)
              else StatsArray[2, 14] := '0';

              Inc(Linenumber);
              Dataline := Lines[Linenumber];
              if Pos('INPRein', Dataline) = 1 then
              begin
                ExtractNumbers(Dataline, False);
                if DoDS then
                begin
                  DSstats[12] := Number1String;
                  DSstats[13] := Number2String;
                end;
                PadStrings;
                GDotInpStrings[16] := Chr(9) + GDotInpTitles[16] + Chr(9) + Number1String + Chr(9) + Number2String;
              end;

              Continue;
            end;
          end;

          //Delay
          if not DelayFound then
          begin
            if Pos('delay', LowerCase(Dataline)) = 1 then
            begin
              DelayFound := True;
              ExtractNumbers(Dataline, False);
              if DoDS then
              begin
                DSstats[119] := Number1String;
                DSstats[120] := Number2String;
              end;
              if GDotInp then
              begin
                PadStrings;
                GDotInpStrings[17] := Chr(9) + GDotInpTitles[17] + Chr(9) + Number1String + Chr(9) + Number2String;
              end;
              Continue;
            end;
          end;

          //PER
          if not PERFound then
          begin
            if Pos('PER', Dataline) = 1 then
            begin
              PERFound := True;
              ExtractNumbers(Dataline, False);
              Continue;
            end;
          end;

          //OR
          if not ORFound then
          begin
            if Pos('OR:', Dataline) = 1 then
            begin
              ORFound := True;
              ExtractNumbers(Dataline, False);
              Continue;
            end;
          end;

          //**** Bearer 1 ****

          if GDotInp then
          begin
            if Pos('INP', Dataline) = 1 then
            begin
              ExtractNumbers(Dataline, False);
              if DoDS then
              begin
                DSstats[117] := Number1String;
                DSstats[118] := Number2String;
              end;
              PadStrings;
              GDotInpStrings[20] := Chr(9) + GDotInpTitles[20] + Chr(9) + Number1String + Chr(9) + Number2String;

              Inc(Linenumber);
              Dataline := Lines[Linenumber];
              if Pos('INPRein', Dataline) = 1 then
              begin
                ExtractNumbers(Dataline, False);
                if DoDS then
                begin
                  DSstats[14] := Number1String;
                  DSstats[15] := Number2String;
                end;
                PadStrings;
                GDotInpStrings[21] := Chr(9) + GDotInpTitles[21] + Chr(9) + Number1String + Chr(9) + Number2String;
              end;

              Inc(Linenumber);
              Dataline := Lines[Linenumber];
              if Pos('delay', Dataline) = 1 then
              begin
                ExtractNumbers(Dataline, False);
                if DoDS then
                begin
                  DSstats[16] := Number1String;
                  DSstats[17] := Number2String;
                end;
                PadStrings;
                GDotInpStrings[22] := Chr(9) + GDotInpTitles[22] + Chr(9) + Number1String + Chr(9) + Number2String;
              end;

              Continue;
            end;
          end;

          //Bitswaps per minute
          if not Swapsfound then
          begin
            if Pos('Bitswap', Dataline) = 1 then
            begin
              Swapsfound := True;
              Extractnumbers(Dataline, False);  //Get the up and down figures
              if DoDS then
              begin
                DSstats[123] := Number1String;
                DSstats[124] := Number2String;
              end;
              if (Number1 < OldDownswaps) or (Number2 < OldUpswaps) then  //Resync or incorrect report by modem/router
              begin
                Swapsperminchart.AddXY(TNow, 0, 0, WasPaused);  //Treat it as new start
                if DoDS then
                begin
                  DSstats[126] := '0';
                  DSstats[127] := '0';
                end;
              end
              else begin
                if (Oldupswaps > 0) or (Olddownswaps > 0) then
                begin
                  if TNow > OldT then Multiplier := 60 / SecondsBetween(TNow, OldT)
                  else Multiplier := 1;
                  Swapsperminchart.AddXY(TNow, (Number2 - OldUpswaps) * Multiplier, (Number1 - OldDownSwaps) * Multiplier, WasPaused);
                  if DoDS then
                  begin
                    DSstats[126] := IntToStr(Round(Number1 - OldDownSwaps));
                    DSstats[127] := IntToStr(Round(Number2 - OldUpSwaps));
                  end;
                end;
              end;
              Oldupswaps := Number2;
              Olddownswaps := Number1;
              Continue;
            end;
          end;

          if (not SinceLinkTimeFound) and (Pos('LOS:', Dataline) = 1) then    //LOS, LOF and LOM
          begin
            SinceLinkTimeFound := True;
            ExtractNumbers(Dataline, False);
            GetDeltaValues(LOSup, LOSdown, Round(Number2), Round(Number1));
            DSstats[136] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
            DSstats[137] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
            LOSup := Round(Number2);
            LOSdown := Round(Number1);
            Inc(LineNumber);
            Dataline := Lines[LineNumber];     //LOF
            ExtractNumbers(Dataline, False);
            GetDeltaValues(LOFup, LOFdown, Round(Number2), Round(Number1));
            DSstats[138] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
            DSstats[139] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
            LOFup := Round(Number2);
            LOFdown := Round(Number1);
            Inc(LineNumber);
            Dataline := Lines[LineNumber];     //LOM
            ExtractNumbers(Dataline, False);
            GetDeltaValues(LOMup, LOMdown, Round(Number2), Round(Number1));
            DSstats[140] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
            DSstats[141] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
            LOMup := Round(Number2);
            LOMdown := Round(Number1);
            Continue;
          end;

          if (not ASFound) and (Pos('AS', Dataline) = 1) then
          begin
            ASFound := True;
            Extractnumbers(Dataline, False);
            if Number1 < PreviousAS then Resync := True;
            PreviousAS := Round(Number1);
            if DoDS then
            begin
              DSstats[18] := Number1String;
              DSstats[23] := FloatToStrFDot(Number1 / 3600, 3);
            end;
            if not UptimeFound then
            begin
              Num1 := Round(Number1);
              S := Timestring(Num1);
              Uptime.Caption := 'DSL uptime: ' + S;
              StatsArray[1, 4] := S;
              UptimeFound := True;
            end;
            Continue;
          end;

          if (not CRCFound) or (not FECFound) then
          begin
            if Pos('Total time', Dataline) = 1 then
            for N := 1 to 5 do
            begin
              Inc(Linenumber);
              Dataline := Lines[LineNumber];

              //FEC
              if (not FECFound) and (Pos('FEC', Dataline) = 1) then
              begin
                FECFound := True;
                ExtractNumbers(Dataline, False);
                ProcessFEC(Number2, Number1);
              end;

              //CRC
              if (not CRCFound) and (Pos('CRC', Dataline) = 1) then
              begin
                CRCFound := True;
                ExtractNumbers(Dataline, False);
                ProcessCRC(Number2, Number1);
              end;

            end;
          end;

          if not FECFound then Continue;
          if SpeedFound and SNRFound and AttenFound and PowerFound and Interleave0Found and RSCorr0Found
            and StatusFound and MaxFound and SwapsFound and ESFound and CRCFound then
            if GDotInp then
            begin
              if RxQ0Found and rtxtxFound and LEFTRSFound then Break;
            end
            else Break;
          if (LineNumber > 30) and not SpeedFound then Break;
        end;

        if not FECFound then  //Fall back to RSCorr for FEC values
        begin
          ProcessFEC(RSCorr0up, RSCorr0down);
        end;

        if GDotInp then
        begin
          GDotInpList.Clear;
          GDotInpList.Lines.Add(Chr(9) + Chr(9) + Chr(9) + Chr(9) + 'Downstream' + Chr(9) + Chr(9) + 'Upstream');
          for N := 1 to 22 do
            GDotInpList.Lines.Add(GDotInpStrings[N]);
        end;

        OldT := TNow;
        TrayIcon1.Hint := 'Downstream speed: ' + StatsArray[1, 10] + ' kbps' + Chr(10) + 'Downstream SNRM: ' + StatsArray[1, 11] + ' dB';
      except
        on E: Exception do
        begin
          Result := leNodata;
          Collecting := False;
          Exit;
        end;
      end;
      if RSdown > 0 then StatsArray[1, 17] := FloatToStrF(RSCorr0down / RSdown * 100, ffFixed, 0, 4)    //RSCorr/RS%
      else StatsArray[1, 17] := 'N/A';
      if RSup > 0 then StatsArray[2, 17] := FloatToStrF(RSCorr0up / RSup * 100, ffFixed, 0, 4)
      else StatsArray[2, 17] := 'N/A';
    end;
    if not Speedfound then
    begin
      Result := leNodata;
      Collecting := False;
      Exit;
    end;
    if DoDS and not ProfileFound then DSstats[83] := 'N/A';   //Not a VDSL2 connection
  end;

  if ThomBox.Checked then
  begin
    Linenumber := 0;
    with GInpList do
    begin
      repeat
        Inc(Linenumber);
        if Linenumber = Lines.Count then Break;
        Dataline := Lines[Linenumber];
      until Pos('Status:', Dataline) > 0;
      if Linenumber < Lines.Count then
      begin
        ExtractNumbers(Dataline, False);
        if Number1 = 0 then GdotInp := False
        else begin
          StatsArray[1, 15] := 'Enabled';
          GdotInp := True;
          DSstats[19] := '1';
        end;
      end
      else GdotInp := False;
     if not GdotInp then
     begin
        StatsArray[1, 15] := 'Not enabled';
        DSstats[19] := '0';
      end;
    end;
    GdotInpTab.TabVisible := GdotInp;
    if GDotInp then
    begin
      with GInpList do
      try
        Linenumber := 0;
        repeat
          Inc(Linenumber);
          Dataline := Lines[Linenumber];

          if not RxQ0Found then
          begin
            if Pos('RxQ', Dataline) > 0 then  //Process RxQ values
            begin
              RxQ0Found := True;
              ExtractNumbers(Dataline, False);
              if DoDS then
              begin
                DSstats[24] := Number1String;
                DSstats[25] := Number2String;
              end;
              PadStrings;
              GDotInpStrings[9] := Chr(9) + GDotInpTitles[9] + Chr(9) + Number1String + Chr(9) + Number2String;
              RxQ0Up := Round(Number2);
              RxQ0Down := Round(Number1);

              Inc(Linenumber);
              Dataline := Lines[Linenumber];
              if Pos('TxQ', Dataline) > 0 then  //Process TxQ values
              begin
                ExtractNumbers(Dataline, False);
                if DoDS then
                begin
                  DSstats[27] := Number1String;
                  DSstats[28] := Number2String;
                end;
                PadStrings;
                GDotInpStrings[10] := Chr(9) + GDotInpTitles[10] + Chr(9) + Number1String + Chr(9) + Number2String;
                TxQ0Up := Round(Number2);
                TxQ0Down := Round(Number1);
              end;

              Inc(Linenumber);
              Dataline := Lines[Linenumber];
              if Pos('raming', Dataline) > 0 then  //Process G.INP Framing values
              begin
                ExtractNumbers(Dataline, False);
                if DoDS then
                begin
                  DSstats[30] := Number1String;
                  DSstats[31] := Number2String;
                end;
                PadStrings;
                GDotInpStrings[11] := Chr(9) + GDotInpTitles[11] + Chr(9) + Number1String + Chr(9) + Number2String;
                Framing0Up := Round(Number2);
                Framing0Down := Round(Number1);
              end;

              Inc(Linenumber);
              Dataline := Lines[Linenumber];
              if Pos('ookback', Dataline) > 0 then  //Process G.INP Lookback values
              begin
                ExtractNumbers(Dataline, False);
                if DoDS then
                begin
                  DSstats[33] := Number1String;
                  DSstats[34] := Number2String;
                end;
                PadStrings;
                GDotInpStrings[12] := Chr(9) + GDotInpTitles[12] + Chr(9) + Number1String + Chr(9) + Number2String;
                Lookback0Up := Round(Number2);
                Lookback0Down := Round(Number1);
              end;

              Inc(Linenumber);
              Dataline := Lines[Linenumber];
              if Pos('RRC', Dataline) > 0 then  //Process RRC values
              begin
                ExtractNumbers(Dataline, False);
                PadStrings;
                GDotInpStrings[13] := Chr(9) + GDotInpTitles[13] + Chr(9) + Number1String + Chr(9) + Number2String;
                RRC0Up := Round(Number2);
                RRC0Down := Round(Number1);
              end;

              Continue;
            end;
          end;

          if not rtxtxFound then    //Retransmit counters
          begin
            if Pos('rtx_tx', Dataline) > 0 then   //Process rtx_tx values
            begin
              rtxtxFound := True;
              ExtractNumbers(Dataline, False);
              GetDeltaValues(rtxtxUp, rtxtxDown, Round(Number2), Round(Number1));
              if DoDS then
              begin
                DSstats[102] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
                DSstats[103] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
              end;
              rtxtxChart.AddXY(TNow, Round(DeltaUp), Round(DeltaDown), WasPaused);
              rtxtxUp := Round(Number2);
              rtxtxDown := Round(Number1);
              PadStrings;
              GDotInpStrings[2] := Chr(9) + GDotInpTitles[2] + Chr(9) + Number1String + Chr(9) + Number2String;

              Inc(Linenumber);
              Dataline := Lines[Linenumber];
              if Pos('rtx_c', Dataline) > 0 then  //Process rtx_c values
              begin
                ExtractNumbers(Dataline, False);
                GetDeltaValues(rtxcUp, rtxcDown, Round(Number2), Round(Number1));
                if DoDS then
                begin
                  DSstats[104] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
                  DSstats[105] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
                end;
                rtxcChart.AddXY(TNow, Round(DeltaUp), Round(DeltaDown), WasPaused);
                rtxcUp := Round(Number2);
                rtxcDown := Round(Number1);
                PadStrings;
                GDotInpStrings[3] := Chr(9) + GDotInpTitles[3] + Chr(9) + Number1String + Chr(9) + Number2String;
              end;

              Inc(Linenumber);
              Dataline := Lines[Linenumber];
              if Pos('rtx_uc', Dataline) > 0 then    //Process rtx_uc values
              begin
                ExtractNumbers(Dataline, False);
                GetDeltaValues(rtxucUp, rtxucDown, Round(Number2), Round(Number1));
                if DoDS then
                begin
                  DSstats[106] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
                  DSstats[107] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
                end;
                rtxucChart.AddXY(TNow, Round(DeltaUp), Round(DeltaDown), WasPaused);
                rtxucUp := Round(Number2);
                rtxucDown := Round(Number1);
                PadStrings;
                GDotInpStrings[4] := Chr(9) + GDotInpTitles[4] + Chr(9) + Number1String + Chr(9) + Number2String;
              end;

            end;
            Continue;
          end;

          if not LEFTRSFound then   //G.INP counters
          begin
            if Pos('LEFTRS', Dataline) > 0 then     //Process LEFTRS values
            begin
              LEFTRSFound := True;
              ExtractNumbers(Dataline, False);
              GetDeltaValues(LEFTRSUp, LEFTRSDown, Round(Number2), Round(Number1));
              if DoDS then
              begin
                DSstats[108] := FloatToStrF(DeltaDown, ffFixed, 0, 0);
                DSstats[109] := FloatToStrF(DeltaUp, ffFixed, 0, 0);
              end;
              LEFTRSChart.AddXY(TNow, Round(DeltaUp), Round(DeltaDown), WasPaused);
              LEFTRSUp := Round(Number2);
              LEFTRSDown := Round(Number1);
              PadStrings;
              GDotInpStrings[5] := Chr(9) + GDotInpTitles[5] + Chr(9) + Number1String + Chr(9) + Number2String;
              LEFTRSUp := Round(Number2);
              LEFTRSDown := Round(Number1);

              Inc(Linenumber);
              Dataline := Lines[Linenumber];
              if Pos('minEFTR', Dataline) > 0 then  //Process min_EFTR values
              begin
                ExtractNumbers(Dataline, False);
                if DoDS then
                begin
                  DSstats[110] := Number1String;
                  DSstats[111] := Number2String;
                end;
                PadStrings;
                GDotInpStrings[6] := Chr(9) + GDotInpTitles[6] + Chr(9) + Number1String + Chr(9) + Number2String;
                minEFTRUp := Round(Number2);
                minEFTRDown := Round(Number1);
              end;

              Inc(Linenumber);
              Dataline := Lines[Linenumber];
              if Pos('errFreeBits', Dataline) > 0 then    //Process errFreeBits values
              begin
                ExtractNumbers(Dataline, False);
                PadStrings;
                GDotInpStrings[7] := Chr(9) + GDotInpTitles[7] + Chr(9) + Number1String + Chr(9) + Number2String;
                errFreeBitsUp := Round(Number2);
                errFreeBitsDown := Round(Number1);
              end;

            end;
            Continue
          end;

        until Linenumber >= Lines.Count;

        if GDotInp then
        begin
          GDotInpStrings[16] := Chr(9) + GDotInpTitles[16] + Chr(9) + 'Not available';
          for N := 19 to 22 do
            GDotInpStrings[N] := Chr(9) + GDotInpTitles[N] + Chr(9) + 'Not available';
          GDotInpList.Clear;
          GDotInpList.Lines.Add(Chr(9) + Chr(9) + Chr(9) + Chr(9) + 'Downstream' + Chr(9) + Chr(9) + 'Upstream');
          for N := 1 to 22 do
            GDotInpList.Lines.Add(GDotInpStrings[N]);
        end;
      except

      end;
    end;

  end;

  //Process ES values

  //Reminder: values of AHour are between 0 and 23, but ESchart hours run from 1 to 24

  DecodeDateTime(TNow, AYear, AMonth, ADay, AHour, AMinute, ASecond, AMillisecond);
  if LastDay = 0 then   //No data yet recorded
  begin
    OldESUp := NewESUp;
    OldESDown := NewESDown;
    LastYear := AYear;
    LastMonth := AMonth;
    LastDay := ADay;
    LastHour := AHour;
  end
  else begin
    if (AYear = LastYear) and (AMonth = LastMonth) and (ADay = LastDay) then  //Same day as previous
    begin
      if AHour = LastHour then   //Same hour
      begin
        CurrentESUpValue := 0;
        CurrentESDownValue := 0;
        ESChart.GetHourlyES(0, AHour + 1, CurrentESUpValue, CurrentESDownValue);
        if NewESUp >= OldESUp then  //Normal situation
        begin
          NewESUpValue := CurrentESUpValue + NewESUp - OldESUp;
        end
        else if NewESUp = 0 then  //Missed sample or no new data
        begin
          NewESUpValue := CurrentESUpValue;
        end
        else begin     //ESUp has wrapped round or been reset
          NewESUpValue := CurrentESUpValue + NewESUp;
        end;
        if NewESDown >= OldESDown then  //Normal situation
        begin
          NewESDownValue := CurrentESDownValue + NewESDown - OldESDown;
        end
        else if NewESDown = 0 then  //Missed sample or no new data
        begin
          NewESDownValue := CurrentESDownValue;
        end
        else begin     //ESDown has wrapped round or been reset
          NewESDownValue := CurrentESDownValue + NewESDown;
        end;
      end
      else begin       //Same day, different hour
        if AHour = LastHour + 1 then      //Next hour
        begin
          NewESUpValue := NewESUp - OldESUp;
          NewESDownValue := NewESDown - OldESDown;
        end
        else begin              //More than one hour ago
          NewESUpValue := 0;
          NewESDownValue := 0;
        end;
      end;
    end
    else if (AYear = LastYear) and (AMonth = LastMonth) then   //Same month, different day
    begin
      ESChart.Scrolldays(ADay - LastDay);
      ESChart.SetESDate(TNow);
      OldDate := EncodeDateTime(LastYear, LastMonth, LastDay, LastHour, 0, 0, 0);
      if HoursBetween(TNow, OldDate) = 1 then    //Next hour
      begin
        if NewESUp >= OldESUp then  //Normal situation
        begin
          NewESUpValue := NewESUp - OldESUp;
        end
        else if NewESUp = 0 then  //Missed sample or no new data
        begin
          NewESUpValue := 0;
        end
        else begin     //ESUp has wrapped round or been reset
          NewESUpValue := NewESUp;
        end;
        if NewESDown >= OldESDown then  //Normal situation
        begin
          NewESDownValue := NewESDown - OldESDown;
        end
        else if NewESDown = 0 then  //Missed sample or no new data
        begin
          NewESDownValue := 0;
        end
        else begin     //ESDown has wrapped round or been reset
          NewESDownValue := NewESDown;
        end;
      end
      else begin        //More than one hour ago
        NewESUpValue := 0;
        NewESDownValue := 0;
      end;
    end
    else begin        //Different month or year
      ESChart.SaveHourlyES('es_data-endofmonth.txt');   //Save data to end of previous month as a precaution
      ESChart.SetESDate(TNow);
      OldDate := EncodeDate(LastYear, LastMonth, LastDay);
      //ElapsedDays := DaysBetween(DateOf(Date), DateOf(OldDate));
      ElapsedDays := DaysBetween(Date, OldDate);
      if ElapsedDays > 99 then   //Clear all old data
      begin
        ESChart.ClearESArray;
        NewESUp := 0;
        NewESDown := 0;
        OldESUp := 0;
        OldESDown := 0;
        LastYear := 0;
        LastMonth := 0;
        LastDay := 0;
        LastHour := 0;
      end
      else begin
        N := ElapsedDays;
        //if AHour <= LastHour then Inc(N);
        ESChart.Scrolldays(N);
        if HoursBetween(TNow, OldDate) = 1 then    //Next hour
        begin
          NewESUpValue := NewESUp - OldESUp;
          NewESDownValue := NewESDown - OldESDown;
        end
        else begin        //More than one hour ago
          NewESUpValue := 0;
          NewESDownValue := 0;
        end;
      end;
    end;
    if NewESUpValue > 3600 then
    begin
      NewESUpValue := 0;
      MiscString := 'Impossible upstream ES value reported. Value set to zero.';
      LogEvent(leMisc);
    end;
    if NewESDownValue > 3600 then
    begin
      NewESDownValue := 0;
      MiscString := 'Impossible downstream ES value reported. Value set to zero.';
      LogEvent(leMisc);
    end;
    ESChart.SetHourlyES(0, AHour + 1, NewESUpValue, NewESDownValue);
    CheckAlerts(atES, NewESUpValue, NewESDownValue);
    if DoDS then
    begin
      UpDiff := NewESUp - OldESUp;
      DownDiff := NewESDown - OldESDown;
      if UpDiff > 60 then UpDiff := 0;
      if DownDiff > 60 then DownDiff := 0;
      if DownDiff > 0 then DSstats[45] := IntToStr(DownDiff) else DSstats[45] := '0';
      if UpDiff > 0 then DSstats[46] := IntToStr(UpDiff) else DSstats[46] := '0';
    end;
    OldESUp := NewESUp;
    OldESDown := NewESDown;
    LastYear := AYear;
    LastMonth := AMonth;
    LastDay := ADay;
    LastHour := AHour;
  end;

  //Process bitloading and bitswapping
  if (DoBitloading.Checked or DoBitswaps.Checked) and (Bitslist.Lines.Count > 20) then
  begin
    Bitchart1.Upcolour := Upcolour.Selected;
    Bitchart1.DownColour := Downcolour.Selected;
    Bitswapchart.Upcolour := Upcolour.Selected;
    Bitswapchart.DownColour := Downcolour.Selected;
    try
      if ThomBox.Checked then   // *** Thomson/Technicolor modem
      begin
        ToneNumber := 0;
        for LineNumber := 2 to BitsList.Lines.Count - 3 do
        begin
          Bitline := BitsList.Lines[LineNumber];
          Len := Length(Bitline);
          N := Pos(':', Bitline) + 1;
          if N = 1 then
          begin
            LogEvent(leNobits);
            Break;
          end;
          repeat
            C := Bitline[N];
            if C in ['0'..'9'] then
            begin
              Bitstring := C;
              Inc(N);
              C := Bitline[N];
              if C in ['0'..'9'] then
              begin
                Bitstring := Bitstring + C;
                Inc(N);
              end;
              if AnnexM.Checked and (Tonenumber in [57..59]) then Bitstring := '0';
              if DoBitloading.Checked then Bitchart1.BitsArray[Tonenumber] := StrToInt(Bitstring);
              if DoBitswaps.Checked then
              begin
                if BitsSaved then
                begin
                  Num1 := OldBits[ToneNumber];
                  Num2 := StrToInt(Bitstring);
                  Bitswapchart.BitsArray[ToneNumber] := Bitswapchart.BitsArray[ToneNumber] + Abs(Num2 - Num1);
                end;
                OldBits[ToneNumber] := StrToInt(Bitstring);
              end;
              Inc(ToneNumber);
            end
            else Inc(N);
          until N > Len;
        end;
        if DoDS and DoZips then
        begin
          for ToneNumber := 0 to ToneRange do
            BitsLogString := BitsLogString + IntToStr(Bitchart1.BitsArray[ToneNumber]) + ',';
          if Length(BitsLogString) > 500 then WSBitsOK := True else WSBitsOK := False;
          Stamp := AddTimeStamp('Bits');
          SaveString(BitsLogString, Stamp, False);
          BitsZip := Stamp + '.zip';
          ZipFile(Stamp, BitsZip);
          BitsZip := Stamp + '.zip';
          Sysutils.DeleteFile(DailyFolder + Stamp);
          LastZipTime := TNow;
        end;
        if ToneNumber < ToneRange then
        begin
          Bitchart1.Maxreported := ToneNumber - 1;
          Bitswapchart.Maxreported := ToneNumber - 1;
        end
        else begin
          Bitchart1.Maxreported := LastToneNumber;
          Bitswapchart.Maxreported := LastToneNumber;
        end;
        if DoBitloading.Checked then Bitchart1.Drawbits(ToneRange);
        if DoBitswaps.Checked then
        begin
          if BitsSaved then Bitswapchart.DrawBits(ToneRange);
          BitsSaved := True;
        end;
      end
      else begin    // *** Normal Broadcom modem
        if TelnetForm.LE = leNoBits then
        begin
          Miscstring := 'Unable to process bitloading data';
          LogEvent(leMisc);
        end
        else if BitsList.Lines.Count < 20 then
        begin
          MiscString := 'Bitloading data fewer than 20 lines';
          LogEvent(leMisc);
        end
        else begin
          Linenumber := 0;
          Tonenumber := 0;
          repeat
            Inc(Linenumber);
            Bitline := Bitslist.Lines[Linenumber];
          until (Pos('Tone', Bitline) > 0) or (Linenumber = 20);
          if Linenumber < 20 then
          begin
            repeat
              Inc(Linenumber);
              Bitline := Bitslist.Lines[Linenumber];
              Len := Length(Bitline);
              if N < 3 then Break;   //Was no Break
              N := Len - 3;
              repeat
                Inc(N);
                if N > Len then Break;
                C := Bitline[N];
              until (C in ['0'..'9']);
              if N > Len then Break;  //Was no Break
              Bitstring := C;
              while N < Length(Bitslist.Lines[Linenumber]) do
              begin
                Inc(N);
                C := Bitline[N];
                if (C in ['0'..'9']) then Bitstring := Bitstring + C
                else Break;
              end;
              if AnnexM.Checked and (Tonenumber in [57..59]) then Bitstring := '0';
              if DoBitloading.Checked then
                Bitchart1.BitsArray[Tonenumber] := StrToInt(Bitstring);
              if DoBitswaps.Checked then
              begin
                if BitsSaved then
                begin
                  Num1 := OldBits[ToneNumber];
                  Num2 := StrToInt(Bitstring);
                  Bitswapchart.BitsArray[ToneNumber] := Bitswapchart.BitsArray[ToneNumber] + Abs(Num2 - Num1);
                end;
                OldBits[ToneNumber] := StrToInt(Bitstring);
              end;
              Inc(Tonenumber);
            until Linenumber = Bitslist.Lines.Count - 2;
            if DoDS and DoZips then
            begin
              for ToneNumber := 0 to ToneRange do
                BitsLogString := BitsLogString + IntToStr(Bitchart1.BitsArray[ToneNumber]) + ',';
              if Length(BitsLogString) > 500 then WSBitsOK := True else WSBitsOK := False;
              Stamp := AddTimeStamp('Bits');
              SaveString(BitsLogString, Stamp, False);
              BitsZip := Stamp + '.zip';
              ZipFile(Stamp, BitsZip);
              Sysutils.DeleteFile(DailyFolder + Stamp);
              LastZipTime := TNow;
            end;
            if ToneNumber < ToneRange then
            begin
              Bitchart1.Maxreported := ToneNumber - 1;
              Bitswapchart.Maxreported := ToneNumber - 1;
            end
            else begin
              Bitchart1.Maxreported := LastToneNumber;
              Bitswapchart.Maxreported := LastToneNumber;
            end;
            if DoBitloading.Checked then Bitchart1.Drawbits(ToneRange);
            if DoBitswaps.Checked then
            begin
              if BitsSaved then Bitswapchart.DrawBits(ToneRange);
              BitsSaved := True;
            end;
          end;
        end;
      end;
    except
      on E: Exception do
      begin
        MiscString := 'Error processing bitloading';
        LogEvent(leMisc);
      end;
    end;
  end;

  //Process SNR per tone included with Bitloading
  if DoSNRpertone.Checked and SNRwithbitloading.Checked and (SNRlist.Lines.Count > 20) then
  begin
    try
      Linenumber := 0;
      Tonenumber := 0;
      repeat
        Inc(Linenumber);
        SNRline := SNRlist.Lines[Linenumber];
      until (Pos('Tone', SNRline) > 0) or (Linenumber = SNRlist.Lines.Count - 2);
      if Linenumber < SNRlist.Lines.Count - 2 then
      begin
        Y := 20;
        repeat
          ParseALine(1);
          if not SNROK then
          begin
            LogEvent(leNosnr);
            Break;
          end;
          if DoDS then SNRLogString := SNRLogString + SNRString + ',';
          if SNRValue > Y * 3 then
          begin
            Y := Round((SNRValue / 3) + 0.5);
            if Y shr 1 shl 1 < Y then Inc(Y);  //Y is rounded up to the next multiple of 2
          end;
          Bitchart1.Ymax := Y;
          if SNRValue < 0 then SNRValue := 0;
          Bitchart1.SNRArray[Tonenumber] := SNRValue;
          Inc(Tonenumber);
        until Tonenumber > Tonerange;
        if DoDS and DoZips then
        begin
          if Length(SNRLogString) > 500 then WSSNROK := True else WSSNROK := False;
          Stamp := AddTimeStamp('SNR');
          SaveString(SNRLogString, Stamp, False);
          SNRZip := Stamp + '.zip';
          ZipFile(Stamp, SNRZip);
          Sysutils.DeleteFile(DailyFolder + Stamp);
          SNRLogDone := True;
          LastZipTime := TNow;
        end;
        Bitchart1.DrawBits(ToneRange);  //This triggers the redraw
      end;
    except
      on E: Exception do
      begin
        MiscString := 'Error processing SNR per tone';
        LogEvent(leMisc);
      end;
    end;
  end;

  //Process SNR per tone on separate tab
  if DoSNRpertone.Checked and SNRseparate.Checked and (SNRlist.Lines.Count > 20) then
  try
    Linenumber := 0;
    Tonenumber := 0;
    repeat
      Inc(Linenumber);
      SNRline := SNRlist.Lines[Linenumber];
    until (Pos('Tone', SNRline) > 0) or (Linenumber = SNRlist.Lines.Count - 2);
    if Linenumber < SNRlist.Lines.Count - 2 then
    begin
      Y := 20;
      SNRLogString := '';
      repeat
        ParseALine(1);       //Extract the value
        if not SNROK then
        begin
          LogEvent(leNosnr);
          Break;
        end;
        if DoDS then SNRLogString := SNRLogString + SNRString + ',';
        SNRchart.GraphArray[ToneNumber] := SNRValue;
        Inc(Tonenumber);
      until Tonenumber > Tonerange;
      if DoDS and not SNRLogDone and DoZips then   //Only do this if it wasn't already done in SNR with Bitloading
      begin
        if Length(SNRLogString) > 500 then WSSNROK := True else WSSNROK := False;
        Stamp := AddTimeStamp('SNR');
        SaveString(SNRLogString, Stamp, False);
        SNRZip := Stamp + '.zip';
        ZipFile(Stamp, SNRZip);
        Sysutils.DeleteFile(DailyFolder + Stamp);
        LastZipTime := TNow;
      end;
      SNRchart.DrawGraph(Tonerange);   //This triggers the redraw
    end;
  except
    on E: Exception do
    begin
      MiscString := 'Error processing SNR per tone';
      LogEvent(leMisc);
    end;
  end;

  //Process DSLAM/MSAN vendor
  if not DoVendor.Checked then StatsArray[1, 0] := 'Not monitored'
  else if not VendorWritten then
  begin
    if Vendorlist.Count < 4 then   //--vendor command not accepted, or not monitored
    begin
      QLNchart.DSLAMType := dslNone;
      HLogchart.DSLAMType := dslNone;
      VendorWritten := True;
      if TelnetForm.Vendor = 'Not monitored' then Vendor := TelnetForm.Vendor
      else Vendor := 'Not reported by modem';
    end
    else begin
      LineNumber := 0;
      try
      repeat
        VendorLine := VendorList[LineNumber];
        if Pos('Vendor Id', VendorLine) > 0 then
        begin
          N := 0;
          repeat
            Inc(N);
          until VendorLine[N] = ':';
          repeat
            Inc(N);
            C := VendorLine[N];
          until (C in ['A'..'Z']) or (N = Length(VendorLine));
          if N = Length(Vendorline) then Break;
          Vendor := C;
          repeat
            Inc(N);
            C := VendorLine[N];
            Vendor := Vendor + C;
          until N = Length(VendorLine);  //Vendor = (for example) "BDCM:0xb12d" at this point
          try                            //Convert to decimal values
            N := Pos(':', Vendor);
            Decimal1 := IntToStr(Hex2Dec(Vendor[N + 3] + Vendor[N + 4]));
            Decimal2 := IntToStr(Hex2Dec(Vendor[N + 5] + Vendor[N + 6]));
            Vendor := Vendor + ' (' + Decimal1 + '.' + Decimal2 + ')';
          except
            on E: Exception do;          //Do nothing
          end;
          Inc(LineNumber);
          VendorLine := VendorList[LineNumber];
          if Pos('Version', VendorLine) > 0 then
          begin
            N := 0;
            repeat
              Inc(N);
            until VendorLine[N] = ':';
            repeat
              Inc(N);
              C := VendorLine[N];
            until (C in ['0'..'9']) or (N = Length(VendorLine));
            if N = Length(VendorLine) then Break;
            Vendor := Vendor + ' / v' + C;
            repeat
              Inc(N);
              C := VendorLine[N];
              Vendor := Vendor + C;
            until N = Length(VendorLine);
          end;
          if Vendor[1..4] = 'IFTN' then
          begin
            QLNchart.DSLAMType := dslNone;
            HLogchart.DSLAMType := dslNone;
          end
          else if Vendor[1..4] = 'BDCM' then
          begin
            QLNchart.DSLAMType := dslHuawei;
            HLogchart.DSLAMType := dslHuawei;
          end
          else begin
            QLNchart.DSLAMType := dslECI;
            HLogchart.DSLAMType := dslECI;
          end;
          Break;
        end;
        Inc(LineNumber);
      until LineNumber = VendorList.Count;
      except
        on E: Exception do
        begin
          MiscString := 'Error processing vendor name';
          LogEvent(leMisc);
        end;
      end;
    end;
    VendorWritten := True;
    StatsArray[1, 0] := Vendor;
  end;

  //Process QLN
  if DoQLN.Checked and (QLNlist.Lines.Count > 20) {and (not QGraphDone)} then
  try
    begin
      Linenumber := 0;
      Tonenumber := 0;
      repeat
        Inc(Linenumber);
        QLNline := QLNlist.Lines[Linenumber];
      until (Pos('Tone', QLNline) > 0) or (Linenumber = QLNlist.Lines.Count - 2);
      if Linenumber < QLNlist.Lines.Count - 2 then
      begin
        QLNString := '';
        repeat
          ParseALine(2);     //Extract the value
          QLNchart.GraphArray[Tonenumber] := QLNValue;
          if DoDS then QLNLogString := QLNLogString + QLNString + ',';
          Inc(Tonenumber);
        until Linenumber >= QLNList.Lines.Count - 1;
        if DoDS and DoZips then
        begin
          if Length(QLNLogString) > 500 then WSQLNOK := True else WSQLNOK := False;
          Stamp := AddTimeStamp('QLN');
          SaveString(QLNLogString, Stamp, False);
          QLNZip := Stamp + '.zip';
          ZipFile(Stamp, QLNZip);
          Sysutils.DeleteFile(DailyFolder + Stamp);
          LastZipTime := TNow;
        end;
        QLNChart.GraphType := gtQLN;
        QLNchart.YMax := -60;
        QLNchart.YMin := -170;
        QLNchart.DrawGraph(ToneRange);  //This triggers the redraw
        QGraphDone := True;
      end;
    end;
  except
    on E: Exception do
    begin
      MiscString := 'Error processing QLN';
      LogEvent(leMisc);
    end;
  end;

  //Process HLog
  if DoHLog.Checked and (HLoglist.Lines.Count > 20) {and (not HGraphDone)} then  //copy the HLog values
  try
    begin
      Linenumber := 0;
      Tonenumber := 0;
      repeat
        Inc(Linenumber);
        HLogline := HLoglist.Lines[Linenumber];
      until (Pos('Tone', HLogline) > 0) or (Linenumber = HLoglist.Lines.Count - 2);
      if LineNumber < HLoglist.Lines.Count - 2 then
      begin
        HLogLogString := '';
        repeat
          ParseALine(3);     //Extract the value
          HLogchart.GraphArray[Tonenumber] := HLogValue;
          if DoDS then HLogLogString := HLogLogString + HLogString + ',';
          Inc(ToneNumber);
        until LineNumber >= HLogList.Lines.Count - 1;
        if DoDS and DoZips then
        begin
          if Length(HLogLogString) > 500 then WSHLogOK := True else WSHLogOK := False;
          Stamp := AddTimeStamp('Hlog');
          SaveString(HLogLogString, Stamp, False);
          HlogZip := Stamp + '.zip';
          ZipFile(Stamp, HlogZip);
          Sysutils.DeleteFile(DailyFolder + Stamp);
          LastZipTime := TNow;
        end;
        HLogchart.GraphType := gtHLog;
        HLogchart.YMax := 0;
        if VDSL2 then HLogchart.YMin := -100 else HLogchart.YMin := -140;
        HLogchart.DrawGraph(ToneRange);    //This triggers the redraw
        HGraphDone := True;
      end;
    end;
  except
    on E: Exception do
    begin
      MiscString := 'Error processing HLog';
      LogEvent(leMisc);
    end;
  end;
  //DoZips := False;

  //Process signal attenuation
  if DoSignalAttn.Checked then
  begin
    if TelnetForm.Mode = -1 then
    begin
      StatsArray[1, 9] := 'Not yet available';
      StatsArray[2, 9] := '';
    end
    else if TelnetForm.Mode = 2 then  //VDSL2
    begin
      StatsArray[1, 9] := 'Not available on VDSL2';
      StatsArray[2, 9] := '';
    end
    else try
      SATNFound := False;
      for N := 0 to 20 do
      begin
        if Pos('SATN', LinediagsList[N]) = 1 then    //Data is available
        begin
          SATNFound := True;
          Extractnumbers(LinediagsList[N], False);
          Down := FloatToStrF(Number1, ffFixed, 1, 1);
          Up := FloatToStrF(Number2, ffFixed, 1, 1);
          if DoLogging and (TelnetForm.Mode in [0, 1]) and ((Down <> StatsArray[1, 9]) or (Up <> StatsArray[2, 9])) then
            LogSAttenuation(Up, Down);
          StatsArray[1, 9] := Down;
          StatsArray[2, 9] := Up;
          if DoDS then
          begin
            DSstats[132] := StatsArray[1, 9];
            DSstats[133] := StatsArray[2, 9];
          end;
        end;
      end;
      if not SATNFound then
      begin
        StatsArray[1, 9] := 'N/A';
        StatsArray[2, 9] := 'N/A';
      end;
    except
      on E: Exception do
      begin
        StatsArray[1, 9] := 'N/A';
        StatsArray[2, 9] := 'N/A';
      end;
    end;
  end
  else begin    //Not being monitored
    StatsArray[1, 9] := 'Not monitored';
    StatsArray[2, 9] := '';
  end;

  //Extract modem/router firmware version
  if not DoFirmware.Checked then StatsArray[1, 1] := 'Not monitored'
  else if (TelnetForm.VersionText <> 'Unknown') and (TelnetForm.VersionText <> 'Not reported') then
  begin
    VersionList.Text := TelnetForm.VersionText;
    N := 0;
    try
      repeat
        Inc(N);
        N1 := Pos('PHY: ', VersionList[N]);
      until (N1 > 0) or (N = VersionList.Count - 1);
      if N1 > 0 then TelnetForm.VersionText := VersionList[N][N1 + 5 .. VersionList.Count - 1];
    except
      on E: Exception do TelnetForm.VersionText := 'Not reported';
    end;
    StatsArray[1, 1] := TelnetForm.VersionText;
  end;

  //Process vectoring status
  if not DoVectoring.Checked then
  begin
    StatsArray[1, 16] := 'Not monitored';
  end
  else try
    StatsArray[1, 16] := 'Unknown';
    for N := 0 to VectoringList.Lines.Count - 1 do
    begin
      S := VectoringList.Lines[N];
      if Pos('Vectoring state', S) > 0 then
      begin
        ExtractNumbers(S, False);
        case Round(Number1) of
          0: StatsArray[1, 16] := '0 (VECT_WAIT_FOR_CONFIG)';
          1: StatsArray[1, 16] := '1 (VECT_FULL)';
          2: StatsArray[1, 16] := '2 (VECT_WAIT_FOR_TRIGGER)';
          3: StatsArray[1, 16] := '3 (VECT_RUNNING)';
          4: StatsArray[1, 16] := '4 (VECT_DISABLED)';
          5: StatsArray[1, 16] := '5 (VECT_UNCONFIGURED)';
        end;
        if DoDS then DSstats[112] := 'V' + Number1String;
      end;
    end;
  except
    on E: Exception do
    begin
      MiscString := 'Error checking vectoring status';
      LogEvent(leMisc);
    end;
  end;

  //Get system uptime
  if ThomBox.Checked then S := 'Not available'
  else if not DoSysuptime.Checked then S := 'Not monitored'
  else begin
    ExtractNumbers(Uptimestring, False);
    if Number1 = -1 then S := 'Not available'
    else S := Timestring(Round(Number1));
    RouterUptime := Round(Number1);
  end;
  SysUptime.Caption := 'Modem uptime: ' + S;

  //Process traffic
  if MonitorTrafficBox.Checked then
  begin
    if ThomBox.Checked and (TrafficList.Lines.Count > 3) then
    begin
      for LineNumber := 1 to TrafficList.Lines.Count - 1 do
      begin
        Dataline := TrafficList.Lines[LineNumber];
        if Pos('Interface', Dataline) > 0 then
        begin
          if Pos('RX', Dataline) < Pos('TX', Dataline) then RXFirst := True else RXFirst := False;
          Continue;
        end;
        if (Pos('Internet', Dataline) > 0) or (Pos('wan', Dataline) > 0) then
        begin
          Len := Length(Dataline);
          N := 5;
          repeat    //Find first number (MTU)
            C := Dataline[N];
            Inc(N);
          until (C in ['0'..'9']) or (N = Len);
          if N < Len then    //Bypass first number
          repeat
            C := Dataline[N];
            Inc(N);
          until (not (C in ['0'..'9', '.', ','])) or (N = Len);
          if N < Len then        //Find second number (up or down Traffic)
          repeat
            C := Dataline[N];
            Inc(N);
          until (C in ['0'..'9']) or (N = Len);
          if N < Len then    //Read second number
          begin
            S := C;
            repeat
              C := Dataline[N];
              if (C in ['0'..'9', '.', ',']) then
              begin
                if (C in ['0'..'9']) then S := S + C;    //Exclude non-numerical characters
                Inc(N);
              end;
            until (not (C in ['0'..'9', '.', ','])) or (N = Len);
            TrafficValue := StrToInt64(S);
            S := Dataline[N + 1..N + 2];
            if S = 'KB' then TrafficValue := TrafficValue * 1000
            else if S = 'MB' then TrafficValue := TrafficValue * 1000000
            else if S = 'GB' then TrafficValue := TrafficValue * 1000000000;
            if RXFirst then ReportedDownTraffic := TrafficValue else ReportedUpTraffic := TrafficValue;
            repeat               //Find third number (down Traffic)
              C := Dataline[N];
              Inc(N);
            until (C in ['0'..'9']) or (N = Len);
            if N < Len then     //Read third number
            begin
              S := C;
              repeat
                C := Dataline[N];
                if (C in ['0'..'9', '.', ',']) then
                begin
                  if (C in ['0'..'9']) then S := S + C;       //Exclude non-numerical characters
                  Inc(N);
                end;
              until (not (C in ['0'..'9', '.', ','])) or (N = Len);
              TrafficValue := StrToInt64(S);
              S := Dataline[N + 1..N + 2];
              if S = 'KB' then TrafficValue := TrafficValue * 1000
              else if S = 'MB' then TrafficValue := TrafficValue * 1000000
              else if S = 'GB' then TrafficValue := TrafficValue * 1000000000;
              if RXFirst then ReportedUpTraffic := TrafficValue else ReportedDownTraffic := TrafficValue;
            end;
          end;
          Break;
        end;
      end;
    end
    else if (not ThomBox.Checked) and (TrafficList.Lines.Count > 6) then
    begin
      S := InterfaceGroup.Items[InterfaceGroup.ItemIndex];
      if S = 'Other...' then S := OtherInterfaceEdit.Text;
      S := DelChars(S, '*');
      for LineNumber := 1 to TrafficList.Lines.Count - 1 do
      begin
        Dataline := TrafficList.Lines[LineNumber];
        if Pos(S, Dataline) in [1..5] then
        begin
          if CommandGroup.ItemIndex < 2 then   //ifconfig
          begin
            N1 := LineNumber;
            repeat
              Dataline := TrafficList.Lines[N1];
              N := Pos('RX bytes', Dataline);
              if N > 0 then
              begin
                Len := Length(Dataline);
                N := N + 8;  //Points at character before first digit of number
                S := '';
                repeat
                  Inc(N);
                  C := Dataline[N];
                  if (C in ['0'..'9']) then S := S + C
                  else Break;
                until N >= Len - 1;
                if ReverseBox.Checked then ReportedUpTraffic := StrToInt64(S)
                else ReportedDownTraffic := StrToInt64(S);
                N := Pos('TX bytes', Dataline) + 8;
                S := '';
                repeat
                  Inc(N);
                  C := Dataline[N];
                  if (C in ['0'..'9']) then S := S + C
                  else Break;
                until N >= Len - 1;
                if ReverseBox.Checked then ReportedDownTraffic := StrToInt64(S)
                else ReportedUpTraffic := StrToInt64(S);
                Break;
              end;
              Inc(N1);
            until N1 >= TrafficList.Lines.Count;
          end
          else begin   //cat /proc/net/dev
            N := Pos(':', Dataline);       //Get downstream value
            S := '';
            Len := Length(Dataline);
            repeat
              Inc(N);
              C := Dataline[N];
              if (C in ['0'..'9']) then S := S + C
              else if S > '' then Break;
            until N >= Len - 1;
            ReportedDownTraffic := StrToInt64(S);
            for N1 := 1 to 7 do   //Bypass the seven unwanted values
            begin
              repeat   //Bypass spaces
                Inc(N);
                C := Dataline[N];
              until (C in ['0'..'9']);
              repeat         //Bypass number
                Inc(N);
                C := Dataline[N];
              until not (C in ['0'..'9']);
            end;
            repeat   //Bypass last set of spaces
              Inc(N);
              C := Dataline[N];
            until (C in ['0'..'9']);
            S := '';       //Get upstream value
            repeat
              C := Dataline[N];
              if (C in ['0'..'9']) then S := S + C
              else if S > '' then Break;
              Inc(N);
            until N >= Len;
            ReportedUpTraffic := StrToInt64(S);
          end;
          Break;
        end;
      end;
    end;
    ProcessTrafficValues;
  end;

  DisplayErrors1;  //Display the average error data

  FillInStats;   //Fill in basic stats on Stats tab

  if DoDS then  //Process and save the DataStore data
  begin
    WSString := '';
    DSstats[81] := '-1';
    for N := 1 to NumDSstats do WSString := WSString + DSstats[N] + ' ';
    Stamp := AddTimeStamp('Stats') + '.log';
    SaveString(WSString, Stamp, True);

    if DoUploadData.Checked then   //Upload selected data to FTP server
    begin
      UploadList.Clear;
      UploadList.Add('Stats.log');
      if DoZips then
      begin
        if DSbits.Checked then UploadList.Add(BitsZip);
        if DSsnr.Checked then UploadList.Add(SNRZip);
        if DSqln.Checked then UploadList.Add(QLNZip);
        if DShlog.Checked then UploadList.Add(HlogZip);
      end;
      if not CurlUploadFiles(usDataStore, DataStorePath + FormatDateTime('yyyy"-"mm"-"dd', TNow), DSUploadURL.Text, DSUploadDir.Text,
        DSUploadUsername.Text, DSUploadPassword.Text, DSFTPPort.Text, UploadList) then
      begin
        Inc(DSUploadFailed);
        if DSUploadFailed > 3 then
        begin
          MiscString := 'Data Store FTP upload failed four times in succession. Uploading disabled';
          LogEvent(leMisc);
          if FTPfailBox.Checked then
          begin
            MiscString := 'Data Store';
            SendAlert(atFTPFail, False, 0);
          end;
          DoUploadData.Checked := False;
          DSUploadFailed := 0;
        end
        else begin
          MiscString := 'Data Store FTP upload failed';
          LogEvent(leMisc);
        end;
      end
      else DSUploadFailed := 0;
    end;
  end;
  DoZips := False;

  if DoUploadWS.Checked and (SecondsBetween(TNow, LastUploadWS) >= (WSUploadRate.Value * 60 - 5)) then  //Upload webserver files
  begin
    LastUploadWS := TNow;
    UploadList.Clear;
    if SysUtils.FindFirst(IncludeTrailingPathDelimiter(WebserverPath) + '*', faAnyfile, SR) = 0 then   //Get list of webserver files
    repeat
      if (SR.Attr and faDirectory) <> faDirectory then
      begin
        UploadList.Add(SR.Name);
      end;
    until SysUtils.FindNext(SR) <> 0;
    SysUtils.FindClose(SR);
    if not CurlUploadFiles(usWebserver, WebserverPath, WSUploadURL.Text, WSUploadDir.Text, WSUploadUsername.Text, WSUploadPassword.Text,
      WSFTPPort.Text, UploadList) then
    begin
      Inc(WSUploadFailed);
      if WSUploadFailed > 3 then
      begin
        MiscString := 'Webserver upload failed four times in succession. Uploading disabled';
        LogEvent(leMisc);
        if FTPfailBox.Checked then
        begin
          MiscString := 'Webserver';
          SendAlert(atFTPFail, False, 0);
        end;
        DoUploadWS.Checked := False;
      end
      else begin
        MiscString := 'Webserver FTP upload failed';
        LogEvent(leMisc);
      end;
    end
    else WSUploadFailed := 0;
  end;

  if Resync then
  begin
    MiscString := 'DSL connection restored. SNRMup = ' + SNRMlatestUp + ' dB, SNRMdown = ' + SNRMlatestDown + ' dB';
    LogEvent(leMisc);
    if (SendEmailAlerts or SendEventLogBox.Checked) and ResyncBox.Checked then
    begin
      SendAlert(atResync, False, 0);
    end;
    Inc(ResyncCount);
    StatsArray[1, 5] := IntToStr(ResyncCount) + ' (since ' + FormatDateTime('dd mmm yyyy hh:nn:ss', ResyncBaseTime) + ')';
    ResetTriggers;
  end;

  Result := leSuccess;
  Collecting := False;
  PopulateWebserver;
  if WasStopped then Timer6.Enabled := True;  //Get IP address in 2 seconds
  WasStopped := False;
end;

//Fill in basic stats on Stats tab
procedure TMainform.FillInStats;
var
  S: string;
  N: Word;
begin
  StatsText.Lines.Clear;
  StatsText.Lines.Add('Stats recorded ' + FormatDateTime('dd mmm yyyy hh:nn:ss', TNow));
  StatsText.Lines.Add('');
  for N := 0 to 16 do
  begin
    if N < 6 then S := StatsArray[0, N] + Chr(9) + StatsArray[1, N]
    else if N = 7 then S := Chr(9) + Chr(9) + Chr(9) + Chr(9) + StatsArray[1, N] + Chr(9) + StatsArray[2, N]
    else S := StatsArray[0, N] + Chr(9) + StatsArray[1, N] + Chr(9) + Chr(9) + StatsArray[2, N];
    StatsText.Lines.Add(S);
  end;
  StatsText.Lines.Add('');
  for N := 17 to 19 do
  begin
    if ThomBox.Checked and (N <> 15) then  //Only ES values available for Thomson
    begin
      StatsArray[1, N] := '';
      StatsArray[2, N] := '';
    end;
    StatsText.Lines.Add(StatsArray[0, N] + Chr(9) + StatsArray[1, N] + Chr(9) + Chr(9) + StatsArray[2, N]);
  end;
end;

// ***** TRAFFIC MONITORING *****

//Process the reported traffic values and convert to values for the graphs
procedure Tmainform.ProcessTrafficValues;
var
  UpIncrease, DownIncrease: Int64;
  AYear, AMonth, ADay, AHour, AMinute, ASecond, AMillisecond, FirstMonth: Word;
  T: TDateTime;
  Multiplier: Single;
begin
  T := TNow;
  DecodeDateTime(T, AYear, AMonth, ADay, AHour, AMinute, ASecond, AMillisecond);
  if (UpTraffic = 0) and (DownTraffic = 0) then  //Fresh start
  begin
    UpTraffic := ReportedUpTraffic;
    DownTraffic := ReportedDownTraffic;
    DailyUpValue := 0;
    DailyDownValue := 0;
    MonthlyUpValue := 0;
    MonthlyDownValue := 0;
    CurrentDay := ADay;
    CurrentMonth := AMonth;
    CurrentYear := AYear;
    Exit;
  end;
  if (ADay <> CurrentDay) or (AMonth <> CurrentMonth) then    //Day has changed
  begin
    if (ADay > CurrentDay) and (AMonth = CurrentMonth) then   //Same month, new day
    begin
      CurrentDay := ADay;
      DailyUpValue := 0;
      DailyDownValue := 0;
    end
    else begin       //New month
      CurrentMonth := AMonth;
      if CurrentMonth = 12 then FirstMonth := 1 else FirstMonth := CurrentMonth + 1;
      repeat
        TrafficChart.IncrementMonth;
      until TrafficChart.FirstMonth = TMonths(FirstMonth - 1);
      if FirstMonth = 1 then CurrentYear := TrafficChart.FirstYear else CurrentYear := TrafficChart.FirstYear + 1;
      TrafficChart.ClearDaily;
      CurrentDay := ADay;
      DailyUpValue := 0;
      DailyDownValue := 0;
      MonthlyUpValue := 0;
      MonthlyDownValue := 0;
    end;
  end;
  if ReportedUpTraffic >= UpTraffic then      //Normal situation
  begin
    UpIncrease := ReportedUpTraffic - UpTraffic;
    UpTraffic := ReportedUpTraffic;
  end
  else begin    //Resync, missing sample or wrapround etc.
    UpIncrease := 0;
    if ReportedUpTraffic > 0 then UpTraffic := ReportedUpTraffic;
  end;
  if ReportedDownTraffic >= DownTraffic then   //Normal situation
  begin
    DownIncrease := ReportedDownTraffic - DownTraffic;
    DownTraffic := ReportedDownTraffic;
  end
  else begin      //Resync, missing sample or wrapround etc.
    DownIncrease := 0;
    if ReportedDownTraffic > 0 then DownTraffic := ReportedDownTraffic;
  end;
  DailyUpValue := DailyUpValue + UpIncrease;
  DailyDownValue := DailyDownValue + DownIncrease;
  MonthlyUpValue := MonthlyUpValue + UpIncrease;
  MonthlyDownValue := MonthlyDownValue + DownIncrease;
  TrafficChart.SetDailyTraffic(0, CurrentDay, Round(DailyUpValue / 1000000), Round(DailyDownValue / 1000000));
  TrafficChart.SetMonthlyTraffic(12, Round(MonthlyUpValue / 1000000), Round(MonthlyDownValue / 1000000));
  UpDay.Caption := ConvertToLabel(DailyUpValue);
  DownDay.Caption := ConvertToLabel(DailyDownValue);
  UpMonth.Caption := ConvertToLabel(MonthlyUpValue);
  DownMonth.Caption := ConvertToLabel(MonthlyDownValue);
  try
    Multiplier := 60 / SampleRateEdit.Value / 1000;
  except
    on E: EDivByZero do
    begin
      MiscString := '***Sample rate value equals zero (should be impossible) - 3';
      LogEvent(leMisc);
      Multiplier := 0;
    end;
  end;
  TrafficperminChart.AddXY(T, UpIncrease * Multiplier, DownIncrease * Multiplier, False);
  CalculateUsage;
end;

//Convert an Int64 byte value to MB/MiB or GB/GiB strings
function Tmainform.ConvertToLabel(Value: Int64): string;
var
  Temp: Single;
  S: string;
begin
  Temp := Value / 1000000;
  if Temp >= 1000 then
  begin
    Temp := Temp / 1000;
    S := 'GB';
  end
  else S := 'MB';
  Result := FloatToStrF(Temp, ffFixed, 0, 2) + ' ' + S;
  if S = 'GB' then
  begin
    S := 'GiB';
    Temp := Temp / 1.0737418;
  end
  else begin
    S := 'MiB';
    Temp := Temp / 1.048576;
  end;
  Result := Result + ' (' + FloatToStrF(Temp, ffFixed, 0, 2) + ' ' + S + ')';
end;

//Toggle traffic limit warning
procedure Tmainform.LimitWarningBoxChange(Sender: TObject);
begin
  DownonlyButton.Enabled := LimitWarningBox.Checked;
  UpPlusDownButton.Enabled := LimitWarningBox.Checked;
  LimitLabel.Enabled := LimitWarningBox.Checked;
  GBLabel.Enabled := LimitWarningBox.Checked;
  LimitEdit.Enabled := LimitWarningBox.Checked;
  SetTrafficLimits;
end;

//Restrict traffic limit setting to whole numbers
procedure Tmainform.LimitEditKeyPress(Sender: TObject; var Key: char);
begin
  if not (Key in ['0'..'9', Chr(8), Chr(127)]) then Key := #0;
end;

//Process change of download only option
procedure Tmainform.DownonlyButtonChange(Sender: TObject);
begin
  SetTrafficLimits;
end;

//Process change of limiting value
procedure Tmainform.LimitEditChange(Sender: TObject);
begin
  SetTrafficLimits;
end;

//Set traffic limit values
procedure Tmainform.SetTrafficLimits;
begin
  if LimitWarningBox.Checked then
  begin
    if LimitEdit.Text > '' then TrafficChart.MonthlyLimit := StrToInt(LimitEdit.Text)
    else TrafficChart.MonthlyLimit := 0;
    TrafficChart.MonthlyLimitDownOnly := DownOnlyButton.Checked;
  end
  else TrafficChart.MonthlyLimit := 0;
end;

//Process change of billing day
procedure Tmainform.DayEditChange(Sender: TObject);
begin
  TrafficChart.BillingDay := DayEdit.Value;
  CalculateUsage;
end;

//Toggle display of billing month data
procedure Tmainform.ShowBillingBoxChange(Sender: TObject);
begin
  TrafficChart.ShowBilling := ShowBillingBox.Checked;
end;

//Calculate and display billing month usage
procedure Tmainform.CalculateUsage;
var
  AYear, AMonth, ADay, AHour, AMinute, ASecond, AMillisecond: Word;
  N: Byte;
  TrafficUp, TrafficDown, Up, Down: Int64;
  TUp, TDown, TTotal: string;
begin
  DecodeDateTime(TNow, AYear, AMonth, ADay, AHour, AMinute, ASecond, AMillisecond);
  TrafficUp := 0;
  TrafficDown := 0;
  if ADay >= DayEdit.Value then       //All in current calendar month
  begin
    for N := DayEdit.Value to ADay do
    begin
      TrafficChart.GetDailyTraffic(0, N, Up, Down);
      TrafficUp := TrafficUp + Up;
      TrafficDown := TrafficDown + Down;
    end;
  end
  else begin                          //Billing period covers part of last month and this
    for N := DayEdit.Value to 31 do   //Last calendar month
    begin
      TrafficChart.GetDailyTraffic(1, N, Up, Down);
      TrafficUp := TrafficUp + Up;
      TrafficDown := TrafficDown + Down;
    end;
    for N := 1 to ADay do             //This calendar month
    begin
      TrafficChart.GetDailyTraffic(0, N, Up, Down);
      TrafficUp := TrafficUp + Up;
      TrafficDown := TrafficDown + Down;
    end;
  end;
  TrafficChart.BillingDay := DayEdit.Value;
  TUp := FloatToStrF(TrafficUp / 1000, ffFixed, 0, 2) + ' GB';
  TDown := FloatToStrF(TrafficDown / 1000, ffFixed, 0, 2) + ' GB';
  TTotal := FloatToStrF((TrafficUp + TrafficDown) / 1000, ffFixed, 0, 2) + ' GB';
  TrafficChart.UpString := TUp;
  TrafficChart.DownString := TDown;
  TrafficChart.TotalString := TTotal;
end;

//Reset the current day's traffic
procedure Tmainform.ResetDayButtonClick(Sender: TObject);
var
  N: Byte;
  Up, Down: Int64;
begin
  TrafficChart.SetDailyTraffic(0, CurrentDay, 0, 0);
  MonthlyUpValue := 0;
  MonthlyDownValue := 0;
  Up := 0;
  Down := 0;
  for N := 1 to CurrentDay do     //Calculate the new monthly total
  begin
    TrafficChart.GetDailyTraffic(0, N, Up, Down);
    MonthlyUpValue := MonthlyUpValue + Up;
    MonthlyDownValue := MonthlyDownValue + Down;
  end;
  TrafficChart.SetMonthlyTraffic(12, MonthlyUpValue, MonthlyDownValue);
  MonthlyUpValue := MonthlyUpValue * 1000000;
  MonthlyDownValue := MonthlyDownValue * 1000000;
  UpDay.Caption := '';
  DownDay.Caption := '';
  UpMonth.Caption := ConvertToLabel(MonthlyUpValue);
  DownMonth.Caption := ConvertToLabel(MonthlyDownValue);
  DailyUpValue := 0;
  DailyDownValue := 0;
  LogEvent(leDayreset);
end;

//Reset the current month's traffic
procedure Tmainform.ResetMonthButtonClick(Sender: TObject);
begin
  TrafficChart.SetMonthlyTraffic(12, 0, 0);
  UpDay.Caption := '';
  DownDay.Caption := '';
  UpMonth.Caption := '';
  DownMonth.Caption := '';
  DailyUpValue := 0;
  DailyDownValue := 0;
  MonthlyUpValue := 0;
  MonthlyDownValue := 0;
  LogEvent(leMonthreset);
end;

// *** VARIOUS TEXT FUNCTIONS *** //

//Toggle attenuation logging
procedure Tmainform.LoggingButtonClick(Sender: TObject);
begin
  DoLogging := not DoLogging;
  if DoLogging then LoggingButton.Caption := 'Stop logging' else LoggingButton.Caption := 'Start logging';
end;

//Copy the basic stats to the clipboard
procedure Tmainform.ClipButton2Click(Sender: TObject);
begin
  ClipBoard.AsText := StatsText.Text;
end;

//Copy the full stats to the clipboard
procedure Tmainform.ClipButton3Click(Sender: TObject);
begin
  ClipBoard.AsText := StatsList.Text;
end;

//Copy the bitloading data to the clipboard
procedure Tmainform.ClipButton4Click(Sender: TObject);
begin
  ClipBoard.AsText := BitsList.Text;
end;

//Copy the pbParams data to the clipboard
procedure Tmainform.ClipButton5Click(Sender: TObject);
begin
  ClipBoard.AsText := ParamsList.Text;
end;

//Copy the QLN data to the clipboard
procedure Tmainform.ClipButton6Click(Sender: TObject);
begin
  ClipBoard.AsText := QLNList.Text;
end;

//Copy the HLog data to the clipboard
procedure Tmainform.ClipButton7Click(Sender: TObject);
begin
  ClipBoard.AsText := HLogList.Text;
end;

//Copy the SNR per tone data to the clipboard
procedure Tmainform.ClipButton8Click(Sender: TObject);
begin
  ClipBoard.AsText := SNRList.Text;
end;

//Copy the line attenuation log to the clipboard
procedure Tmainform.ClipButton9Click(Sender: TObject);
begin
  ClipBoard.AsText := LAttenuationLog.Text;
end;

//Copy the signal attenuation log to the clipboard
procedure Tmainform.ClipButton13Click(Sender: TObject);
begin
  ClipBoard.AsText := SAttenuationLog.Text;
end;

//Copy the Traffic data to the clipboard
procedure Tmainform.ClipButton10Click(Sender: TObject);
begin
  ClipBoard.AsText := TrafficList.Text;
end;

//Copy the vectoring data to the clipboard
procedure Tmainform.ClipButton16Click(Sender: TObject);
begin
  ClipBoard.AsText := VectoringList.Text;
end;

//Copy the event log to the clipboard
procedure Tmainform.ClipButton11Click(Sender: TObject);
begin
  ClipBoard.AsText := EventList.Text;
end;

//Save the event log and clear it
procedure Tmainform.SaveClearButtonClick(Sender: TObject);
begin
  ManualSaveTextPages(SaveLogButton);
  EventList.Clear;
end;

//Copy the G.INP data to the clipboard
procedure Tmainform.ClipButton12Click(Sender: TObject);
begin
  ClipBoard.AsText := GDotInpList.Text;
end;

//Copy the G.INP telnet data to the clipboard
procedure Tmainform.ClipButton14Click(Sender: TObject);
begin
  ClipBoard.AsText := GInpList.Text;
end;

//Reset the resync count for the stats summary
procedure Tmainform.ResetResyncButtonClick(Sender: TObject);
begin
  ResyncCount := 0;
  ResyncBaseTime := Now;
  FirstSample := True;
  StatsArray[1, 5] := '0 (since ' + FormatDateTime('dd mmm yyyy hh:nn:ss', ResyncBaseTime) + ')';
  FillInStats;
end;

// **** PROCESS CHANGES IN GRAPH OPTIONS ****

//Process various graph option changes
procedure Tmainform.DoMarginChange(Sender: TObject);
begin
  SetTabs;
end;

//Set the font used in graphs
procedure Tmainform.FontButtonClick(Sender: TObject);
begin
  with FontDialog1 do
  begin
    Font.Name := FontName;
    if Execute then
    begin
      FontName := Font.Name;
      DoGraphFonts;
    end;
  end;
end;

//Apply the selected graph font
procedure TMainform.DoGraphFonts;
begin
  FontButton.Caption := FontName;
  MarginChart.Font.Name := FontName;
  SNRMpbChartup.Font.Name := FontName;
  SNRMpbChartdown.Font.Name := FontName;
  Speedchart1.Font.Name := FontName;
  Bitchart1.Font.Name := FontName;
  BitswapChart.Font.Name := FontName;
  SwapsperminChart.Font.Name := FontName;
  SNRChart.Font.Name := FontName;
  CRCChart.Font.Name := FontName;
  FECChart.Font.Name := FontName;
  SESChart.Font.Name := FontName;
  ESChart.Font.Name := FontName;
  QLNChart.Font.Name := FontName;
  HLogChart.Font.Name := FontName;
  TrafficperminChart.Font.Name := FontName;
  TrafficChart.Font.Name := FontName;
  MarginChart.Font.Name := FontName;
  rtxtxChart.Font.Name := FontName;
  rtxcChart.Font.Name := FontName;
  rtxucChart.Font.Name := FontName;
  LEFTRSChart.Font.Name := FontName;
  OHFErrChart.Font.Name := FontName;
  RSCorrChart.Font.Name := FontName;
end;

//Toggle extended tone range for HLog or QLN
procedure Tmainform.ExtendedBoxChange(Sender: TObject);
begin
  if Sender = ExtendedBox then QLNchart.ExtendedToneRange := ExtendedBox.Checked
  else HLogchart.ExtendedToneRange := ExtendedBox1.Checked
end;

//Toggle plotting of baseline values for HLog or QLN
procedure Tmainform.BaselineBoxChange(Sender: TObject);
begin
  if Sender = BaselineBox then QLNchart.PlotBaseline := BaselineBox.Checked
  else HLogchart.PlotBaseline := BaselineBox1.Checked;
end;

//Select font used in text pages
procedure Tmainform.FontSelectButtonClick(Sender: TObject);
begin
  with FontDialog2 do
  begin
    Font.Name := TextFontName;
    if Execute then
    begin
      TextFontName := Font.Name;
      DoTextFonts;
    end;
  end;
end;

procedure Tmainform.DoTextFonts;
begin
  StatsText.Font.Name := TextFontName;
  StatsList.Font.Name := TextFontName;
  BitsList.Font.Name := TextFontName;
  ParamsList.Font.Name := TextFontName;
  QLNList.Font.Name := TextFontName;
  HLogList.Font.Name := TextFontName;
  SNRList.Font.Name := TextFontName;
  LAttenuationLog.Font.Name := TextFontName;
  SAttenuationLog.Font.Name := TextFontName;
  GDotInpList.Font.Name := TextFontName;
  VectoringList.Font.Name := TextFontName;
  AveragesTab1.Font.Name := TextFontName;
end;

//Hide the tone range setting form when leaving the graph which called it
procedure Tmainform.SNRTabExit(Sender: TObject);
begin
  ToneRangeForm.Hide;
end;

//Process change of graph tones per pixel

procedure Tmainform.Tonesperpx(Value: Smallint);
begin
  TonesperPixel := Value;
  BitswapChart.Tonesperpixel := -Value;
  BitChart1.Tonesperpixel := -Value;
  HLogChart.Tonesperpixel := -Value;
  QLNChart.Tonesperpixel := -Value;
  SNRChart.Tonesperpixel := -Value;
end;

procedure Tmainform.ToneRangeClick(Sender: TObject);
begin
  with ToneRangeForm do
  begin
    RangeSetting := TonesperPixel;
    TonesperpxEdit.Position := TonesperPixel;
    case TonesperPixel of
       1: ADSLButton.Checked := True;
      -1: ADSL2plusButton.Checked := True;
      -7: VDSL2Button.Checked := True;
    else
      CustomButton.Checked := True;
    end;
    Show;
    Passive := False;
  end;
end;

//Process change of bitloading option
procedure Tmainform.DoBitloadingChange(Sender: TObject);
begin
  if DSbits.Checked and not DoBitloading.Checked then //Don't disable this option if DSbits is enabled
  begin
    ShowMessage('DataStore option "Bitloading" is enabled, so cannot disable this option');
    DoBitloading.Checked := True;
    Exit;
  end;
  SetTabs;
end;

//Process change of SNR per tone option
procedure Tmainform.DoSNRpertoneChange(Sender: TObject);
begin
  if DSsnr.Checked and not DoSNRpertone.Checked then //Don't disable this option if DSsnr is enabled
  begin
    ShowMessage('DataStore option "SNR per tone" is enabled, so cannot disable this option');
    DoSNRpertone.Checked := True;
    Exit;
  end;
  Bitchart1.DoSNRpertone := (DoSNRpertone.Checked and SNRwithbitloading.Checked);
  SNRwithbitloading.Enabled := DoSNRpertone.Checked;
  SNRseparate.Enabled := DoSNRpertone.Checked;
  if not DoSNRpertone.Checked then
  begin
    SNRseparate.Checked := False;
    SNRwithbitloading.Checked := False;
  end;
  if DoSNRpertone.Checked and not (SNRwithbitloading.Checked or SNRseparate.Checked) then SNRseparate.Checked := True;
  SetTabs;
end;

//Process change of SNR with bitloading option
procedure Tmainform.SNRwithbitloadingChange(Sender: TObject);
begin
  SNRTab.TabVisible := DoSNRpertone.Checked and SNRseparate.Checked;
  Bitchart1.DoSNRpertone := DoSNRpertone.Checked and SNRwithbitloading.Checked;
  DoSNRpertone.Checked := SNRwithbitloading.Checked or SNRseparate.Checked;
  SetTabs;
end;

//Process change of SNR as separate graph option
procedure Tmainform.SNRseparateChange(Sender: TObject);
begin
  SNRTab.TabVisible := DoSNRpertone.Checked and SNRseparate.Checked;
  DoSNRpertone.Checked := SNRwithbitloading.Checked or SNRseparate.Checked;
  SetTabs;
end;

//Process change of SNR margin clipping level
procedure Tmainform.MarginClipChange(Sender: TObject);
begin
  Marginchart.ClipValue := MarginClip.Value;
end;

//Process change of CRC clipping level
procedure Tmainform.CRCClipChange(Sender: TObject);
begin
  CRCchart.ClipValue := CRCClip.Value;
end;

//Process change of FEC clipping level
procedure Tmainform.FECClipChange(Sender: TObject);
begin
  FECchart.ClipValue := FECClip.Value;
end;

//Process change of OHFErr clipping level
procedure Tmainform.OHFErrClipChange(Sender: TObject);
begin
  OHFErrchart.ClipValue := OHFErrClip.Value;
end;

//Process change of RSCorr clipping level
procedure Tmainform.RSCorrClipChange(Sender: TObject);
begin
  RSCorrchart.ClipValue := RSCorrClip.Value;
end;

//Process change of SES clipping level
procedure Tmainform.SESClipChange(Sender: TObject);
begin
  SESchart.ClipValue := SESClip.Value;
end;

//Process change of swaps per minute clipping level
procedure Tmainform.PMClipChange(Sender: TObject);
begin
  SwapsperminChart.ClipValue := PMClip.Value;
end;

//Process change of LEFTRS per minute clipping level
procedure Tmainform.LEFTRSClipChange(Sender: TObject);
begin
  LEFTRSChart.ClipValue := LEFTRSClip.Value;
end;

//Process change of retransmissions clipping levels
procedure Tmainform.rtxtxClipChange(Sender: TObject);
begin
  rtxtxChart.ClipValue := rtxtxClip.Value;
end;

procedure Tmainform.rtxcClipChange(Sender: TObject);
begin
  rtxcChart.ClipValue := rtxcClip.Value;
end;

procedure Tmainform.rtxucClipChange(Sender: TObject);
begin
  rtxucChart.ClipValue := rtxucClip.Value;
end;

//Process toggle of CRC/min - CRC/sample
procedure Tmainform.PerSampleBoxCChange(Sender: TObject);
begin
  if PerSampleBoxC.Checked then
  begin
    CPerminLabel.Caption := 'CRC/sample';
    CRCChart.PerMinuteMultiplier := 1;
  end
  else begin
    CPerminLabel.Caption := 'CRC/min';
    try
      CRCChart.PerMinuteMultiplier := 60 / SampleRateEdit.Value;
    except
      on E: EDivByZero do
      begin
        MiscString := '***Sample rate equals zero (should be impossible) - 4';
        LogEvent(leMisc);
        CRCChart.PerMinuteMultiplier := 1;
      end;
    end;
  end;
  CRCchart.PerSample := PerSampleBoxC.Checked;
end;

//Process toggle of SES/min - SES/sample
procedure Tmainform.PerSampleBoxSESChange(Sender: TObject);
begin
  if PerSampleBoxSES.Checked then
  begin
    SESPerminLabel.Caption := 'SES/sample';
    SESChart.PerMinuteMultiplier := 1;
  end
  else begin
    SESPerminLabel.Caption := 'SES/min';
    try
      SESChart.PerMinuteMultiplier := 60 / SampleRateEdit.Value;
    except
      on E: EDivByZero do
      begin
        MiscString := '***Sample rate equals zero (should be impossible) - 5';
        LogEvent(leMisc);
        SESChart.PerMinuteMultiplier := 1;
      end;
    end;
  end;
  SESchart.PerSample := PerSampleBoxSES.Checked;
end;

//Process toggle of FEC/min - FEC/sample
procedure Tmainform.PerSampleBoxFChange(Sender: TObject);
begin
  if PerSampleBoxF.Checked then
  begin
    FPerminLabel.Caption := 'FEC/sample';
    FECChart.PerMinuteMultiplier := 1;
  end
  else begin
    FPerminLabel.Caption := 'FEC/min';
    try
      FECChart.PerMinuteMultiplier := 60 / SampleRateEdit.Value;
    except
      on E: EDivByZero do
      begin
        MiscString := '***Sample rate equals zero (should be impossible) - 6';
        LogEvent(leMisc);
        FECChart.PerMinuteMultiplier := 1;
      end;
    end;
  end;
  FECchart.PerSample := PerSampleBoxF.Checked;
end;

//Process toggle of OHFErr/min - OHFErr/sample
procedure Tmainform.PerSampleBoxOChange(Sender: TObject);
begin
  if PerSampleBoxO.Checked then
  begin
    OPerminLabel.Caption := 'OHFErr/sample';
    OHFErrChart.PerMinuteMultiplier := 1;
  end
  else begin
    OPerminLabel.Caption := 'OHFErr/min';
    try
      OHFErrChart.PerMinuteMultiplier := 60 / SampleRateEdit.Value;
    except
      on E: EDivByZero do
      begin
        MiscString := '***Sample rate equals zero (should be impossible) - 7';
        LogEvent(leMisc);
        OHFErrChart.PerMinuteMultiplier := 1;
      end;
    end;
  end;
  OHFErrchart.PerSample := PerSampleBoxO.Checked;
end;

//Process toggle of RSCorr/min - RSCorr/sample
procedure Tmainform.PerSampleBoxRChange(Sender: TObject);
begin
  if PerSampleBoxR.Checked then
  begin
    RPerminLabel.Caption := 'RSCorr/sample';
    RSCorrChart.PerMinuteMultiplier := 1;
  end
  else begin
    RPerminLabel.Caption := 'RSCorr/min';
    try
      RSCorrChart.PerMinuteMultiplier := 60 / SampleRateEdit.Value;
    except
      on E: EDivByZero do
      begin
        MiscString := '***Sample rate equals zero (should be impossible) - 8';
        LogEvent(leMisc);
        RSCorrChart.PerMinuteMultiplier := 1;
      end;
    end;
  end;
  RSCorrchart.PerSample := PerSampleBoxO.Checked;
end;

//Process toggle of rtx/min - rtx/sample
procedure Tmainform.PerSampleBoxrtxtxChange(Sender: TObject);
begin
  if PerSampleBoxrtxtx.Checked then
  begin
    rtxtxPerminLabel.Caption := 'rtx_tx/sample';
    rtxtxChart.PerMinuteMultiplier := 1;
  end
  else begin
    rtxtxPerminLabel.Caption := 'rtx_tx/min';
    try
      rtxtxChart.PerMinuteMultiplier := 60 / SampleRateEdit.Value;
    except
      on E: EDivByZero do
      begin
        MiscString := '***Sample rate equals zero (should be impossible) - 9';
        LogEvent(leMisc);
        rtxtxChart.PerMinuteMultiplier := 1;
      end;
    end;
  end;
  rtxtxchart.PerSample := PerSampleBoxrtxtx.Checked;
end;

//Process toggle of LEFTRS/min - LEFTRS/sample
procedure Tmainform.PerSampleBoxLEFTRSChange(Sender: TObject);
begin
  if PerSampleBoxLEFTRS.Checked then
  begin
    LEFTRSPerminLabel.Caption := 'LEFTRS/sample';
    LEFTRSChart.PerMinuteMultiplier := 1;
  end
  else begin
    LEFTRSPerminLabel.Caption := 'LEFTRS/min';
    try
      LEFTRSChart.PerMinuteMultiplier := 60 / SampleRateEdit.Value;
    except
      on E: EDivByZero do
      begin
        MiscString := '***Sample rate equals zero (should be impossible) - 10';
        LogEvent(leMisc);
        LEFTRSChart.PerMinuteMultiplier := 1;
      end;
    end;
  end;
  LEFTRSchart.PerSample := PerSampleBoxLEFTRS.Checked;
end;

procedure Tmainform.PerSampleBoxrtxcChange(Sender: TObject);
begin
  if PerSampleBoxrtxc.Checked then
  begin
    rtxcPerminLabel.Caption := 'rtx_c/sample';
    rtxcChart.PerMinuteMultiplier := 1;
  end
  else begin
    rtxcPerminLabel.Caption := 'rtx_c/min';
    try
      rtxcChart.PerMinuteMultiplier := 60 / SampleRateEdit.Value;
    except
      on E: EDivByZero do
      begin
        MiscString := '***Sample rate equals zero (should be impossible) - 11';
        LogEvent(leMisc);
        rtxcChart.PerMinuteMultiplier := 1;
      end;
    end;
  end;
  rtxcchart.PerSample := PerSampleBoxrtxc.Checked;
end;

procedure Tmainform.PerSampleBoxrtxucChange(Sender: TObject);
begin
  if PerSampleBoxrtxuc.Checked then
  begin
    rtxucPerminLabel.Caption := 'rtx_uc/sample';
    rtxucChart.PerMinuteMultiplier := 1;
  end
  else begin
    rtxucPerminLabel.Caption := 'rtx_uc/min';
    try
      rtxucChart.PerMinuteMultiplier := 60 / SampleRateEdit.Value;
    except
      on E: EDivByZero do
      begin
        MiscString := '***Sample rate equals zero (should be impossible) - 12';
        LogEvent(leMisc);
        rtxucChart.PerMinuteMultiplier := 1;
      end;
    end;
  end;
  rtxucchart.PerSample := PerSampleBoxrtxuc.Checked;
end;

//Enable/disable clipping on CRC chart
procedure Tmainform.CEnableClippingBoxChange(Sender: TObject);
begin
  CRCchart.UseClipping := CEnableClippingBox.Checked;
end;

//Enable/disable clipping on SES chart
procedure Tmainform.SESEnableClippingBoxChange(Sender: TObject);
begin
  SESchart.UseClipping := SESEnableClippingBox.Checked;
end;

//Enable/disable clipping on FEC chart
procedure Tmainform.FEnableClippingBoxChange(Sender: TObject);
begin
  FECchart.UseClipping := FEnableClippingBox.Checked;
end;

//Enable/disable clipping on OHFErr chart
procedure Tmainform.OEnableClippingBoxChange(Sender: TObject);
begin
  OHFErrchart.UseClipping := OEnableClippingBox.Checked;
end;

//Enable/disable clipping on RSCorr chart
procedure Tmainform.REnableClippingBoxChange(Sender: TObject);
begin
  RSCorrchart.UseClipping := REnableClippingBox.Checked;
end;

//Enable/disable clipping on LEFTRS chart
procedure Tmainform.LEFTRSEnableClippingBoxChange(Sender: TObject);
begin
  LEFTRSchart.UseClipping := LEFTRSEnableClippingBox.Checked;
end;

//Enable/disable clipping on rtx_tx chart
procedure Tmainform.rtxtxEnableClippingBoxChange(Sender: TObject);
begin
  rtxtxchart.UseClipping := rtxtxEnableClippingBox.Checked;
end;

//Enable/disable clipping on rtx_c chart
procedure Tmainform.rtxcEnableClippingBoxChange(Sender: TObject);
begin
  rtxcchart.UseClipping := rtxcEnableClippingBox.Checked;
end;

//Enable/disable clipping on rtx_uc chart
procedure Tmainform.rtxucEnableClippingBoxChange(Sender: TObject);
begin
  rtxucchart.UseClipping := rtxucEnableClippingBox.Checked;
end;

//Process change of CRC graph log Y axis
procedure Tmainform.CRCLogYChange(Sender: TObject);
begin
  CRCchart.LogYScale := CRCLogY.Checked;
  CEnableClippingBox.Enabled := not CRCLogY.Checked;
  CClipLabel.Enabled := not CRCLogY.Checked;
  CRCClip.Enabled := not CRCLogY.Checked;
  CperminLabel.Enabled := not CRCLogY.Checked;
end;

//Process change of SES graph log Y axis
procedure Tmainform.SESLogYChange(Sender: TObject);
begin
  SESchart.LogYScale := SESLogY.Checked;
  SESEnableClippingBox.Enabled := not SESLogY.Checked;
  SESClipLabel.Enabled := not SESLogY.Checked;
  SESClip.Enabled := not SESLogY.Checked;
  SESperminLabel.Enabled := not SESLogY.Checked;
end;

//Process change of FEC graph log Y axis
procedure Tmainform.FECLogYChange(Sender: TObject);
begin
  FECchart.LogYScale := FECLogY.Checked;
  FEnableClippingBox.Enabled := not FECLogY.Checked;
  FClipLabel.Enabled := not FECLogY.Checked;
  FECClip.Enabled := not FECLogY.Checked;
  FperminLabel.Enabled := not FECLogY.Checked;
end;

//Process change of OHFErr graph log Y axis
procedure Tmainform.OHFErrLogYChange(Sender: TObject);
begin
  OHFErrchart.LogYScale := OHFErrLogY.Checked;
  OEnableClippingBox.Enabled := not OHFErrLogY.Checked;
  OClipLabel.Enabled := not OHFErrLogY.Checked;
  OHFErrClip.Enabled := not OHFErrLogY.Checked;
  OperminLabel.Enabled := not OHFErrLogY.Checked;
end;

//Process change of RSCorr graph log Y axis
procedure Tmainform.RSCorrLogYChange(Sender: TObject);
begin
  RSCorrchart.LogYScale := RSCorrLogY.Checked;
  REnableClippingBox.Enabled := not RSCorrLogY.Checked;
  RClipLabel.Enabled := not RSCorrLogY.Checked;
  RSCorrClip.Enabled := not RSCorrLogY.Checked;
  RperminLabel.Enabled := not RSCorrLogY.Checked;
end;

//Process change of LEFTRS graph log Y axis
procedure Tmainform.LEFTRSLogYChange(Sender: TObject);
begin
  LEFTRSchart.LogYScale := LEFTRSLogY.Checked;
  LEFTRSEnableClippingBox.Enabled := not LEFTRSLogY.Checked;
  LEFTRSClipLabel.Enabled := not LEFTRSLogY.Checked;
  LEFTRSClip.Enabled := not LEFTRSLogY.Checked;
  LEFTRSperminLabel.Enabled := not LEFTRSLogY.Checked;
end;

//Process change of rtx_tx graph log Y axis
procedure Tmainform.rtxtxLogYChange(Sender: TObject);
begin
  rtxtxchart.LogYScale := rtxtxLogY.Checked;
  rtxtxEnableClippingBox.Enabled := not rtxtxLogY.Checked;
  rtxtxClipLabel.Enabled := not rtxtxLogY.Checked;
  rtxtxClip.Enabled := not rtxtxLogY.Checked;
  rtxtxperminLabel.Enabled := not rtxtxLogY.Checked;
end;

//Process change of rtx_c graph log Y axis
procedure Tmainform.rtxcLogYChange(Sender: TObject);
begin
  rtxcchart.LogYScale := rtxcLogY.Checked;
  rtxcEnableClippingBox.Enabled := not rtxcLogY.Checked;
  rtxcClipLabel.Enabled := not rtxcLogY.Checked;
  rtxcClip.Enabled := not rtxcLogY.Checked;
  rtxcperminLabel.Enabled := not rtxcLogY.Checked;
end;

//Process change of rtx_uc graph log Y axis
procedure Tmainform.rtxucLogYChange(Sender: TObject);
begin
  rtxucchart.LogYScale := rtxucLogY.Checked;
  rtxucEnableClippingBox.Enabled := not rtxucLogY.Checked;
  rtxucClipLabel.Enabled := not rtxucLogY.Checked;
  rtxucClip.Enabled := not rtxucLogY.Checked;
  rtxucperminLabel.Enabled := not rtxucLogY.Checked;
end;

//Process change of Connection Speed graph log Y axis
procedure Tmainform.ConnspeedLogYChange(Sender: TObject);
begin
  SpeedChart1.LogYScale := ConnspeedLogY.Checked;
end;

//Process change of margin chart autoscale option
procedure Tmainform.AutoscaleBoxChange(Sender: TObject);
begin
  Label79.Enabled := not AutoscaleBox.Checked;
  Label80.Enabled := not AutoscaleBox.Checked;
  MClipLabel.Enabled := AutoscaleBox.Checked;
  MdbLabel.Enabled := AutoscaleBox.Checked;
  MaxYEdit.Enabled := not AutoscaleBox.Checked;
  MinYEdit.Enabled := not AutoscaleBox.Checked;
  MarginClip.Enabled := AutoscaleBox.Checked;
  MarginChart.Autoscale := AutoscaleBox.Checked;
  MarginChart.UseClipping := AutoscaleBox.Checked;
  if not AutoscaleBox.Checked then
  begin
    MarginChart.MaxYValue := MaxYEdit.Value;
    MarginChart.MinYValue := MinYEdit.Value;
  end
  else Marginchart.MinYValue := 0;
end;

//Process change of margin chart max or min Y value
procedure Tmainform.MaxYEditChange(Sender: TObject);
begin
  if MaxYEdit.Value = MinYEdit.Value + 1 then
  begin
    MaxYEdit.MinValue := MaxYEdit.Value;
    MinYEdit.MaxValue := MinYEdit.Value;
  end
  else begin
    MaxYEdit.MinValue := 0;
    MinYEdit.MaxValue := 50;
  end;
  MarginChart.MaxYValue := MaxYEdit.Value;
  MarginChart.MinYValue := MinYEdit.Value;
end;

//Disable key presses in max/min spinedits
procedure Tmainform.MaxYEditKeyPress(Sender: TObject; var Key: char);
begin
  if not (Key = Chr(9)) then Key := Chr(0);
end;

//Toggle position of legend
procedure Tmainform.LegendLeftChange(Sender: TObject);
begin
  if LegendLeft.Checked then
  begin
    MarginChart.LegendPosition := lplLeft;
    SpeedChart1.LegendPosition := lplLeft;
    SwapsperminChart.LegendPosition := lplLeft;
    CRCChart.LegendPosition := lplLeft;
    SESChart.LegendPosition := lplLeft;
    FECChart.LegendPosition := lplLeft;
    Bitchart1.LegendPosition := lpbLeft;
    Bitswapchart.LegendPosition := lpbLeft;
    SNRchart.LegendPosition := lpldLeft;
    QLNchart.LegendPosition := lpldLeft;
    HLogchart.LegendPosition := lpldLeft;
    TrafficChart.LegendPosition := lptLeft;
    TrafficperminChart.LegendPosition := lplLeft;
    ESChart.LegendPosition := lptLeft;
    SNRMpbChartup.LegendPosition := lppLeft;
    SNRMpbChartdown.LegendPosition := lppLeft;
  end
  else begin
    MarginChart.LegendPosition := lplRight;
    SpeedChart1.LegendPosition := lplRight;
    SwapsperminChart.LegendPosition := lplRight;
    CRCChart.LegendPosition := lplRight;
    SESChart.LegendPosition := lplRight;
    FECChart.LegendPosition := lplRight;
    Bitchart1.LegendPosition := lpbRight;
    Bitswapchart.LegendPosition := lpbRight;
    SNRchart.LegendPosition := lpldRight;
    QLNchart.LegendPosition := lpldRight;
    HLogchart.LegendPosition := lpldRight;
    TrafficChart.LegendPosition := lptRight;
    TrafficperminChart.LegendPosition := lplRight;
    ESChart.LegendPosition := lptRight;
    SNRMpbChartup.LegendPosition := lppRight;
    SNRMpbChartdown.LegendPosition := lppRight;
  end;
end;

//Hide or show the graph legend
procedure Tmainform.MHideLegendChange(Sender: TObject);
begin
  MarginChart.ShowLegend := not MHideLegend.Checked;
  SpeedChart1.ShowLegend := not SHideLegend.Checked;
  Bitchart1.ShowLegend := not BHideLegend.Checked;
  BitswapChart.ShowLegend := not BsHideLegend.Checked;
  SwapsperminChart.ShowLegend := not PMHideLegend.Checked;
  SNRChart.ShowLegend := not SNRHideLegend.Checked;
  CRCChart.ShowLegend := not CHideLegend.Checked;
  SESChart.ShowLegend := not SESHideLegend.Checked;
  FECChart.ShowLegend := not FHideLegend.Checked;
  TrafficChart.ShowLegend := not UHideLegend.Checked;
  TrafficperminChart.ShowLegend := not UHideLegend.Checked;
  QLNChart.ShowLegend := not QLNHideLegend.Checked;
  HLogChart.ShowLegend := not HLogHideLegend.Checked;
  ESChart.ShowLegend := not EHideLegend.Checked;
  SNRMpbChartup.ShowLegend := not SpBHideLegendUp.Checked;
  SNRMpbChartdown.ShowLegend := not SpBHideLegendDown.Checked;
  LEFTRSChart.ShowLegend := not LEFTRSHideLegend.Checked;
  rtxtxChart.ShowLegend := not rtxtxHideLegend.Checked;
  rtxcChart.ShowLegend := not rtxcHideLegend.Checked;
  rtxucChart.ShowLegend := not rtxucHideLegend.Checked;
  OHFErrChart.ShowLegend := not OHideLegend.Checked;
  RSCorrChart.ShowLegend := not RHideLegend.Checked;
end;

//Set hint on Exit button according to status
procedure Tmainform.MinimiseoncloseBoxChange(Sender: TObject);
begin
  if MinimiseoncloseBox.Checked then ExitButton.Hint := 'Minimise'
  else ExitButton.Hint := 'Exit';
end;

//Toggle display of max attainable speed values
procedure Tmainform.ShowMaxUpBoxChange(Sender: TObject);
begin
  Speedchart1.ShowMaxUp := ShowMaxUpBox.Checked;
  //MaxUpLabel.Visible := ShowMaxUpBox.Checked;
end;

procedure Tmainform.ShowMaxDownBoxChange(Sender: TObject);
begin
  Speedchart1.ShowMaxDown := ShowMaxDownBox.Checked;
  //MaxDownLabel.Visible := ShowMaxDownBox.Checked;
end;

//Show or hide tabs, and enable/disable webserver pages
procedure Tmainform.SetTabs;
const
  Suffix = ' graph is set to autosave. The corresponding tab must remain visible while this option is in place.';
begin
  if (SNRsnap.Checked and not DoMargin.Checked) then
  begin
    ShowMessage('The SNR margin' + Suffix);
    DoMargin.Checked := True;
  end
  else SNRMTab.TabVisible := DoMargin.Checked;

  if (Speedsnap.Checked and not DoSpeed.Checked) then
  begin
    ShowMessage('The Connection speed' + Suffix);
    DoSpeed.Checked := True;
  end
  else SpeedTab.TabVisible := DoSpeed.Checked;

  if (Bitsnap.Checked and not DoBitloading.Checked) then
  begin
    ShowMessage('The Bitloading' + Suffix);
    DoBitloading.Checked := True;
  end
  else BitloadingTab.TabVisible := (DoBitloading.Checked or DoSNRpertone.Checked);

  if (CRCsnap.Checked and not DoCRC.Checked) then
  begin
    ShowMessage('The CRC' + Suffix);
    DoCRC.Checked := True;
  end
  else CRCtab.TabVisible := DoCRC.Checked;

  if (SESsnap.Checked and not DoSES.Checked) then
  begin
    ShowMessage('The SES' + Suffix);
    DoSES.Checked := True;
  end
  else SEStab.TabVisible := DoSES.Checked;

  if (FECsnap.Checked and not DoFEC.Checked) then
  begin
    ShowMessage('The FEC' + Suffix);
    DoFEC.Checked := True;
  end
  else FECtab.TabVisible := DoFEC.Checked;

  if (ESsnap.Checked and not DoES.Checked) then
  begin
    ShowMessage('The ES per hour' + Suffix);
    DoES.Checked := True;
  end
  else EStab.TabVisible := DoES.Checked;

  if (SNRptsnap.Checked and not DoSNRpertone.Checked) then
  begin
    ShowMessage('The SNR' + Suffix);
    DoSNRpertone.Checked := True;
  end
  else SNRtab.TabVisible := DoSNRpertone.Checked;

  if (Swapptsnap.Checked and not DoBitswaps.Checked) then
  begin
    ShowMessage('The Bitswaps per tone' + Suffix);
    DoBitswaps.Checked := True;
  end
  else Bitswaptab.TabVisible := DoBitswaps.Checked;

  if (Swappmsnap.Checked and not DoSwapspermin.Checked) then
  begin
    ShowMessage('The Bitswaps per min' + Suffix);
    DoSwapspermin.Checked := True;
  end
  else Swapspermintab.TabVisible := DoSwapspermin.Checked;

  if (QLNsnap.Checked and not DoQLN.Checked) then
  begin
    ShowMessage('The QLN' + Suffix);
    DoQLN.Checked := True;
  end
  else QLNtab.TabVisible := DoQLN.Checked;

  if (HLogsnap.Checked and not DoHLog.Checked) then
  begin
    ShowMessage('The HLog' + Suffix);
    DoHLog.Checked := True;
  end
  else HLogtab.TabVisible := DoHLog.Checked;

  TonesTab.TabVisible := (DoSNRpertone.Checked or DoBitloading.Checked or DoBitswaps.Checked or DoSwapspermin.Checked);
  ErrorTab.TabVisible := (DoCRC.Checked or DoFEC.Checked or DoES.Checked or DoSES.Checked or DoAverages.Checked);
  if DoCRC.Checked then
  begin
    CRCTab.TabVisible := True;
    PageControl5.ActivePage := CRCTab;
  end
  else CRCTab.TabVisible := False;
  if DoFEC.Checked then
  begin
    FECTab.TabVisible := True;
    if not DoCRC.Checked then PageControl5.ActivePage := FECTab;
  end
  else FECTab.TabVisible := False;
  if DoSES.Checked then
  begin
    SESTab.TabVisible := True;
    if not DoCRC.Checked and (not DoFEC.Checked) then PageControl5.ActivePage := SESTab;
  end
  else SESTab.TabVisible := False;
  if DoES.Checked then
  begin
    ESTab.TabVisible := True;
    if (not DoCRC.Checked) and (not DoFEC.Checked) and (not DoSES.Checked) then PageControl5.ActivePage := ESTab;
  end;
  if DoAverages.Checked then
  begin
    AveragesTab1.TabVisible := True;
    if (not DoCRC.Checked) and (not DoFEC.Checked) and (not DoSES.Checked) and (not DoES.Checked) then PageControl5.ActivePage := AveragesTab1;
  end
  else begin
    AveragesTab1.TabVisible := False;
  end;
  if DoBitLoading.Checked then
  begin
    BitloadingTab.TabVisible := True;
    PageControl4.ActivePage := BitloadingTab;
  end
  else BitloadingTab.TabVisible := False;
  if DoBitswaps.Checked then
  begin
    BitswapTab.TabVisible := True;
    if not DoBitloading.Checked then PageControl4.ActivePage := BitswapTab;
  end
  else BitswapTab.TabVisible := False;
  if DoSwapspermin.Checked then
  begin
    SwapsperminTab.TabVisible := True;
    if (not DoBitloading.Checked) and (not DoBitswaps.Checked) then PageControl4.ActivePage := SwapsperminTab;
  end
  else SwapsperminTab.TabVisible := False;
  TelnetVectoringTab.TabVisible := DoVectoring.Checked;
  if SNRseparate.Checked then
  begin
    SNRTab.TabVisible := True;
    if (not DoBitloading.Checked) and (not DoBitswaps.Checked) and (not Doswapspermin.Checked) then PageControl4.ActivePage := SNRTab;
  end
  else SNRTab.TabVisible := False;
  SNRMperbandTab.TabVisible := DoSNRMperband.Checked;
  GDotInpTab.TabVisible := GDotInp;
  TelnetBitTab.TabVisible := DoBitloading.Checked;
  TelnetQLNTab.TabVisible := DoQLN.Checked;
  TelnetHlogTab.TabVisible := DoHlog.Checked;
  TelnetSNRTab.TabVisible := DoSNRpertone.Checked;
  EventlogTab.TabVisible := Eventlogbox.Checked;
  if MonitorTrafficBox.Checked then
  begin
    TrafficTab.TabVisible := True;
    TelnetTrafficTab.TabVisible := True;
  end
  else begin
    TrafficTab.TabVisible := False;
    if PageControl3.ActivePage = TelnetTrafficTab then PageControl3.ActivePage := AttenuationTab;
    TelnetTrafficTab.TabVisible := False;
  end;
  if ThomBox.Checked then
  begin
    AdvancedTweaksTab.TabVisible := False;
    SwapsperminTab.TabVisible := False;
    SNRTab.TabVisible := False;
    QLNTab.TabVisible := False;
    HLogTab.TabVisible := False;
    TelnetParamsTab.TabVisible := False;
    TelnetSNRTab.TabVisible := False;
    TelnetQLNTab.TabVisible := False;
    TelnetHLogTab.TabVisible := False;
  end
  else begin
    AdvancedTweaksTab.TabVisible := True;
  end;

  //Enable/disable webserver pages
  SNRMweb.Enabled := SNRMTab.TabVisible;
  SNRMpbweb.Enabled := SNRMperbandTab.TabVisible;
  Bitloadingweb.Enabled := BitloadingTab.TabVisible;
  CRCweb.Enabled := CRCTab.TabVisible;
  ESweb.Enabled := ESTab.TabVisible;
  SESweb.Enabled := SESTab.TabVisible;
  FECweb.Enabled := FECTab.TabVisible;
end;

//Enable/disable graph options
procedure Tmainform.SetOptions;
begin
  DoSwapspermin.Enabled := not ThomBox.Checked;
  DoSNRMperband.Enabled := VDSL2 and not ThomBox.Checked;
  DoSNRpertone.Enabled := not ThomBox.Checked;
  SNRMpbSnap.Enabled := VDSL2 and not ThomBox.Checked;
  SNRwithbitloading.Enabled := not ThomBox.Checked;
  DoQLN.Enabled := not ThomBox.Checked;
  DoHLog.Enabled := not ThomBox.Checked;
  SNRptsnap.Enabled := not ThomBox.Checked;
  Swappmsnap.Enabled := not ThomBox.Checked;
  QLNsnap.Enabled := not ThomBox.Checked;
  HLogsnap.Enabled := not ThomBox.Checked;
  ShowMaxUpBox.Enabled := not ThomBox.Checked;
  ShowMaxDownBox.Enabled := not ThomBox.Checked;
  Label161.Enabled := not ThomBox.Checked;
  SNR.Enabled := not ThomBox.Checked;
  BitswapsperminLabel.Enabled := not ThomBox.Checked;
  BitswapSaveBox.Enabled := not ThomBox.Checked;
  BitswapSaveEBox.Enabled := not ThomBox.Checked;
  BitswapSaveButton.Enabled := not ThomBox.Checked;
  Save5.Enabled := not ThomBox.Checked;
  TelnetGInpTab.TabVisible := ThomBox.Checked;
  if ThomBox.Checked then
  begin
    ShowMaxUpBox.Checked := False;
    ShowMaxDownBox.Checked := False;
    BitswapSaveBox.Checked := False;
    BitswapSaveEBox.Checked := False;
  end
  else if CommandGroup.ItemIndex = 3 then CommandGroup.ItemIndex := 0;   //Can't be ip iflist if modem isn't Thomson
end;

//Process change of option "HG612-Modem-Stats co-operation"
procedure Tmainform.SynchroniseChange(Sender: TObject);
begin
  if Synchronise.Checked then
  begin
    SampleRateEdit.Value := 60;
    SampleRateEdit.ReadOnly := True;
    LogEvent(leSynchronise);
    if Timer1.Enabled then
    begin
      SyncOptionChanged := True;
      StartButtonClick(PauseButton);    //This performs a "synchronise" and then samples
    end;
  end
  else begin
    LogEvent(leUnsync);
    SampleRateEdit.ReadOnly := DoSaveData.Checked;
  end;
end;

//Process change in sample rate
procedure Tmainform.SampleRateEditChange(Sender: TObject);
begin
  if Synchronise.Checked and (SampleRateEdit.Value <> 60) then
  begin
    SampleRateEdit.Value := 60;
    ShowMessage('Option "Synchronise with HG612-Modem-Stats" is set, so sample rate must remain at 60 seconds');
  end
  else if DoUploadData.Checked and (SampleRateEdit.Value <> 60) then
  begin
    SampleRateEdit.Value := 60;
    ShowMessage('Upload data option is set, so sample rate must remain at 60 seconds');
  end;
  Samplerate := SamplerateEdit.Value * 1000;
  if SameButton.Checked then  //Sample period has changed, so switch webpage refresh period to Custom
  begin
    CustomButton.Checked := True;
    RefreshEdit.Enabled := True;
  end;
end;

//Process change of CommandHours per page
procedure Tmainform.TimeperpageEditChange(Sender: TObject);
begin
  if DaysperpageEdit.Value > 0 then TimeperpageEdit.MinValue := 0
  else TimeperpageEdit.MinValue := 1;
  Hoursperpage := TimeperpageEdit.Value + (DaysPerPageEdit.Value * 24);
  if SameAsBox.Checked then TimepersnapshotEdit.Value := Hoursperpage;
  Minutesperpage := Hoursperpage * 60;
  MarginChart.Hoursperpage := Hoursperpage;
  Speedchart1.Hoursperpage := Hoursperpage;
  CRCchart.Hoursperpage := Hoursperpage;
  SESchart.Hoursperpage := Hoursperpage;
  FECchart.Hoursperpage := Hoursperpage;
  Swapsperminchart.Hoursperpage := Hoursperpage;
  TrafficperminChart.Hoursperpage := Hoursperpage;
  SNRMpbChartup.Hoursperpage := Hoursperpage;
  SNRMpbChartdown.Hoursperpage := Hoursperpage;
  LEFTRSChart.Hoursperpage := Hoursperpage;
  rtxtxChart.Hoursperpage := Hoursperpage;
  rtxcChart.Hoursperpage := Hoursperpage;
  rtxucChart.Hoursperpage := Hoursperpage;
  OHFErrChart.Hoursperpage := Hoursperpage;
  RSCorrChart.Hoursperpage := Hoursperpage;
end;

//Set the graph line thickness
procedure Tmainform.DoThickness;
begin
  Marginchart.GraphThickness := ThicknessEdit.Value;
  Speedchart1.GraphThickness := ThicknessEdit.Value;
  QLNchart.GraphThickness := ThicknessEdit.Value;
  SNRchart.GraphThickness := ThicknessEdit.Value;
  SNRMpbchartup.GraphThickness := ThicknessEdit.Value;
  SNRMpbchartdown.GraphThickness := ThicknessEdit.Value;
  HLogchart.GraphThickness := ThicknessEdit.Value;
  CRCchart.GraphThickness := ThicknessEdit.Value;
  SESchart.GraphThickness := ThicknessEdit.Value;
  FECchart.GraphThickness := ThicknessEdit.Value;
  Swapsperminchart.GraphThickness := ThicknessEdit.Value;
  Trafficperminchart.GraphThickness := ThicknessEdit.Value;
  LEFTRSChart.GraphThickness := ThicknessEdit.Value;
  rtxtxChart.GraphThickness := ThicknessEdit.Value;
  rtxcChart.GraphThickness := ThicknessEdit.Value;
  rtxucChart.GraphThickness := ThicknessEdit.Value;
  OHFErrChart.GraphThickness := ThicknessEdit.Value;
  RSCorrChart.GraphThickness := ThicknessEdit.Value;
end;

procedure Tmainform.ThicknessEditChange(Sender: TObject);
begin
  DoThickness;
end;

//Process change of graph colour
procedure Tmainform.GraphcolourChange(Sender: TObject);
begin
  Setcolours;
end;

//Set the graph colours
procedure Tmainform.Setcolours;
begin
  Marginchart.UpColour := Upcolour.Selected;
  Marginchart.DownColour := Downcolour.Selected;
  Marginchart.GraphColour := Graphcolour.Selected;
  Speedchart1.UpColour := Upcolour.Selected;
  Speedchart1.DownColour := Downcolour.Selected;
  Speedchart1.GraphColour := Graphcolour.Selected;
  CRCchart.UpColour := Upcolour.Selected;
  CRCchart.DownColour := Downcolour.Selected;
  CRCchart.GraphColour := Graphcolour.Selected;
  SESchart.UpColour := Upcolour.Selected;
  SESchart.DownColour := Downcolour.Selected;
  SESchart.GraphColour := Graphcolour.Selected;
  FECchart.UpColour := Upcolour.Selected;
  FECchart.DownColour := Downcolour.Selected;
  FECchart.GraphColour := Graphcolour.Selected;
  Bitchart1.GraphColour := GraphColour.Selected;
  Bitchart1.UpColour := UpColour.Selected;
  Bitchart1.DownColour := DownColour.Selected;
  Bitchart1.OtherColour := OtherColour.Selected;
  Bitchart1.SNRColour := SNRColour.Selected;
  Bitswapchart.GraphColour := GraphColour.Selected;
  Bitswapchart.OtherColour := OtherColour.Selected;
  Bitswapchart.SNRColour := SNRColour.Selected;
  Swapsperminchart.UpColour := Upcolour.Selected;
  Swapsperminchart.DownColour := Downcolour.Selected;
  Swapsperminchart.GraphColour := GraphColour.Selected;
  SNRchart.GraphColour := GraphColour.Selected;
  SNRchart.UpColour := Upcolour.Selected;
  SNRchart.DownColour := DownColour.Selected;
  SNRchart.OtherColour := OtherColour.Selected;
  ESChart.GraphColour := GraphColour.Selected;
  ESChart.UpColour := UpColour.Selected;
  ESChart.DownColour := DownColour.Selected;
  QLNchart.GraphColour := GraphColour.Selected;
  QLNchart.UpColour := UpColour.Selected;
  QLNchart.DownColour := DownColour.Selected;
  QLNchart.OtherColour := OtherColour.Selected;
  HLogchart.GraphColour := GraphColour.Selected;
  HLogchart.UpColour := UpColour.Selected;
  HLogchart.DownColour := DownColour.Selected;
  HLogchart.OtherColour := OtherColour.Selected;
  TrafficChart.GraphColour := GraphColour.Selected;
  TrafficChart.UpColour := UpColour.Selected;
  TrafficChart.DownColour := DownColour.Selected;
  TrafficChart.BothColour := UpPlusDownColour.Selected;
  TrafficChart.LimitColour := LimitColour.Selected;
  TrafficperminChart.UpColour := Upcolour.Selected;
  TrafficperminChart.DownColour := Downcolour.Selected;
  TrafficperminChart.GraphColour := Graphcolour.Selected;
  SNRMpbChartup.GraphColour := GraphColour.Selected;
  SNRMpbChartdown.GraphColour := GraphColour.Selected;
  SNRMpbChartup.BandColours[0] := Colour0.Selected;
  SNRMpbChartup.BandColours[1] := Colour1.Selected;
  SNRMpbChartup.BandColours[2] := Colour2.Selected;
  SNRMpbChartup.BandColours[3] := Colour3.Selected;
  SNRMpbChartup.BandColours[4] := Colour4.Selected;
  SNRMpbChartdown.BandColours[0] := Colour0.Selected;
  SNRMpbChartdown.BandColours[1] := Colour1.Selected;
  SNRMpbChartdown.BandColours[2] := Colour2.Selected;
  SNRMpbChartdown.BandColours[3] := Colour3.Selected;
  SNRMpbChartdown.BandColours[4] := Colour4.Selected;
  LEFTRSchart.UpColour := UpColour.Selected;
  LEFTRSchart.DownColour := DownColour.Selected;
  LEFTRSchart.GraphColour := GraphColour.Selected;
  rtxtxchart.UpColour := UpColour.Selected;
  rtxtxchart.DownColour := DownColour.Selected;
  rtxtxchart.GraphColour := GraphColour.Selected;
  rtxcchart.UpColour := UpColour.Selected;
  rtxcchart.DownColour := DownColour.Selected;
  rtxcchart.GraphColour := GraphColour.Selected;
  rtxucchart.UpColour := UpColour.Selected;
  rtxucchart.DownColour := DownColour.Selected;
  rtxucchart.GraphColour := GraphColour.Selected;
  OHFErrChart.UpColour := UpColour.Selected;
  OHFErrChart.DownColour := DownColour.Selected;
  OHFErrchart.GraphColour := GraphColour.Selected;
  RSCorrChart.UpColour := UpColour.Selected;
  RSCorrChart.DownColour := DownColour.Selected;
  RSCorrchart.GraphColour := GraphColour.Selected;
end;

//Toggle snapshot scaling
procedure Tmainform.ScaleBoxChange(Sender: TObject);
begin
  if ScaleBox.Checked then
  begin
    HeightLabel.Enabled := True;
    WidthLabel.Enabled := True;
    HeightEdit.Enabled := True;
    WidthEdit.Enabled := True;
    PixelLabel1.Enabled := True;
    PixelLabel2.Enabled := True;
  end
  else begin
    HeightLabel.Enabled := False;
    WidthLabel.Enabled := False;
    HeightEdit.Enabled := False;
    WidthEdit.Enabled := False;
    PixelLabel1.Enabled := False;
    PixelLabel2.Enabled := False;
  end;
  SetSnapSize;
end;

//Respond to change of snapshot scaling
procedure Tmainform.HeightEditChange(Sender: TObject);
begin
  SetSnapSize;
end;

//Set snapshot size
procedure TMainform.SetSnapSize;
begin
  MarginChart.SnapHeight := HeightEdit.value;
  MarginChart.SnapWidth := WidthEdit.value;
  MarginChart.ScaleSnap := ScaleBox.Checked;
  SpeedChart1.SnapHeight := HeightEdit.value;
  SpeedChart1.SnapWidth := WidthEdit.value;
  SpeedChart1.ScaleSnap := ScaleBox.Checked;
  CRCChart.SnapHeight := HeightEdit.value;
  CRCChart.SnapWidth := WidthEdit.value;
  CRCChart.ScaleSnap := ScaleBox.Checked;
  SESChart.SnapHeight := HeightEdit.value;
  SESChart.SnapWidth := WidthEdit.value;
  SESChart.ScaleSnap := ScaleBox.Checked;
  FECChart.SnapHeight := HeightEdit.value;
  FECChart.SnapWidth := WidthEdit.value;
  FECChart.ScaleSnap := ScaleBox.Checked;
  Bitchart1.SnapHeight := HeightEdit.value;
  Bitchart1.SnapWidth := WidthEdit.value;
  Bitchart1.ScaleSnap := ScaleBox.Checked;
  BitswapChart.SnapHeight := HeightEdit.value;
  BitswapChart.SnapWidth := WidthEdit.value;
  BitswapChart.ScaleSnap := ScaleBox.Checked;
  SwapsperminChart.SnapHeight := HeightEdit.value;
  SwapsperminChart.SnapWidth := WidthEdit.value;
  SwapsperminChart.ScaleSnap := ScaleBox.Checked;
  SNRChart.SnapHeight := HeightEdit.value;
  SNRChart.SnapWidth := WidthEdit.value;
  SNRChart.ScaleSnap := ScaleBox.Checked;
  ESChart.SnapHeight := HeightEdit.value;
  ESChart.SnapWidth := WidthEdit.value;
  ESChart.ScaleSnap := ScaleBox.Checked;
  QLNChart.SnapHeight := HeightEdit.value;
  QLNChart.SnapWidth := WidthEdit.value;
  QLNChart.ScaleSnap := ScaleBox.Checked;
  HlogChart.SnapHeight := HeightEdit.value;
  HlogChart.SnapWidth := WidthEdit.value;
  HlogChart.ScaleSnap := ScaleBox.Checked;
  TrafficChart.SnapHeight := HeightEdit.value;
  TrafficChart.SnapWidth := WidthEdit.value;
  TrafficChart.ScaleSnap := ScaleBox.Checked;
  TrafficperminChart.SnapHeight := HeightEdit.value;
  TrafficperminChart.SnapWidth := WidthEdit.value;
  TrafficperminChart.ScaleSnap := ScaleBox.Checked;
  SNRMpbChartUp.SnapHeight := HeightEdit.value;
  SNRMpbChartUp.SnapWidth := WidthEdit.value;
  SNRMpbChartUp.ScaleSnap := ScaleBox.Checked;
  SNRMpbChartDown.SnapHeight := HeightEdit.value;
  SNRMpbChartDown.SnapWidth := WidthEdit.value;
  SNRMpbChartDown.ScaleSnap := ScaleBox.Checked;
  LEFTRSChart.SnapHeight := HeightEdit.value;
  LEFTRSChart.SnapWidth := WidthEdit.value;
  LEFTRSChart.ScaleSnap := ScaleBox.Checked;
  rtxtxChart.SnapHeight := HeightEdit.value;
  rtxtxChart.SnapWidth := WidthEdit.value;
  rtxtxChart.ScaleSnap := ScaleBox.Checked;
  rtxcChart.SnapHeight := HeightEdit.value;
  rtxcChart.SnapWidth := WidthEdit.value;
  rtxcChart.ScaleSnap := ScaleBox.Checked;
  rtxucChart.SnapHeight := HeightEdit.value;
  rtxucChart.SnapWidth := WidthEdit.value;
  rtxucChart.ScaleSnap := ScaleBox.Checked;
  OHFErrChart.SnapHeight := HeightEdit.value;
  OHFErrChart.SnapWidth := WidthEdit.value;
  OHFErrChart.ScaleSnap := ScaleBox.Checked;
  RSCorrChart.SnapHeight := HeightEdit.value;
  RSCorrChart.SnapWidth := WidthEdit.value;
  RSCorrChart.ScaleSnap := ScaleBox.Checked;
end;

//Enable/disable swapspermin upstream/downstream
procedure Tmainform.UpstreamBoxTChange(Sender: TObject);
begin
  Swapsperminchart.PlotUpstream := UpstreamBoxT.Checked;;
end;

procedure Tmainform.DownstreamBoxTChange(Sender: TObject);
begin
  Swapsperminchart.PlotDownstream := DownstreamBoxT.Checked;;
end;

//Enable/disable SNR margin upstream/downstream
procedure Tmainform.UpstreamBoxMChange(Sender: TObject);
begin
  MarginChart.PlotUpstream := UpstreamBoxM.Checked;
end;

procedure Tmainform.DownstreamBoxMChange(Sender: TObject);
begin
  MarginChart.PlotDownstream := DownstreamBoxM.Checked;
end;

//Enable/disable LEFTRS upstream/downstream
procedure Tmainform.UpstreamBoxLEFTRSChange(Sender: TObject);
begin
  LEFTRSChart.PlotUpstream := UpstreamBoxLEFTRS.Checked;
end;

procedure Tmainform.DownstreamBoxLEFTRSChange(Sender: TObject);
begin
  LEFTRSChart.PlotDownstream := DownstreamBoxLEFTRS.Checked;
end;

//Enable/disable rtx_tx upstream/downstream
procedure Tmainform.UpstreamBoxrtxtxChange(Sender: TObject);
begin
  rtxtxChart.PlotUpstream := UpstreamBoxrtxtx.Checked;
end;

procedure Tmainform.DownstreamBoxrtxtxChange(Sender: TObject);
begin
  rtxtxChart.PlotDownstream := DownstreamBoxrtxtx.Checked;
end;

//Enable/disable rtx_c upstream/downstream
procedure Tmainform.UpstreamBoxrtxcChange(Sender: TObject);
begin
  rtxcChart.PlotUpstream := UpstreamBoxrtxc.Checked;
end;

procedure Tmainform.DownstreamBoxrtxcChange(Sender: TObject);
begin
  rtxcChart.PlotDownstream := DownstreamBoxrtxc.Checked;
end;

//Enable/disable rtx_uc upstream/downstream
procedure Tmainform.UpstreamBoxrtxucChange(Sender: TObject);
begin
  rtxucChart.PlotUpstream := UpstreamBoxrtxuc.Checked;
end;

procedure Tmainform.DownstreamBoxrtxucChange(Sender: TObject);
begin
  rtxucChart.PlotDownstream := DownstreamBoxrtxuc.Checked;
end;

//Enable/disable CRC upstream/downstream
procedure Tmainform.UpstreamBoxCChange(Sender: TObject);
begin
  CRCChart.PlotUpstream := UpstreamBoxC.Checked;
end;

procedure Tmainform.DownstreamBoxCChange(Sender: TObject);
begin
  CRCChart.PlotDownstream := DownstreamBoxC.Checked;
end;

//Enable/disable SES upstream/downstream
procedure Tmainform.UpstreamBoxSESChange(Sender: TObject);
begin
  SESChart.PlotUpstream := UpstreamBoxSES.Checked;
end;

procedure Tmainform.DownstreamBoxSESChange(Sender: TObject);
begin
  SESChart.PlotDownstream := DownstreamBoxSES.Checked;
end;

//Enable/disable FEC upstream/downstream
procedure Tmainform.UpstreamBoxFChange(Sender: TObject);
begin
  FECChart.PlotUpstream := UpstreamBoxF.Checked;
end;

procedure Tmainform.DownstreamBoxFChange(Sender: TObject);
begin
  FECChart.PlotDownstream := DownstreamBoxF.Checked;
end;

//Enable/disable OHFErr1 upstream/downstream
procedure Tmainform.UpstreamBoxOChange(Sender: TObject);
begin
  OHFErrChart.PlotUpstream := UpstreamBoxO.Checked;
end;

procedure Tmainform.DownstreamBoxOChange(Sender: TObject);
begin
  OHFErrChart.PlotDownstream := DownstreamBoxO.Checked;
end;

//Enable/disable RSCorr11 upstream/downstream
procedure Tmainform.UpstreamBoxRChange(Sender: TObject);
begin
  RSCorrChart.PlotUpstream := UpstreamBoxR.Checked;
end;

procedure Tmainform.DownstreamBoxRChange(Sender: TObject);
begin
  RSCorrChart.PlotDownstream := DownstreamBoxR.Checked;
end;

//Enable/disable each upstream or downstream SNRM per band plot
procedure Tmainform.U0Change(Sender: TObject);
begin
  if Sender = U0 then SNRMpbChartup.SetBandsIncluded(0, U0.Checked)
  else if Sender = U1 then SNRMpbChartup.SetBandsIncluded(1, U1.Checked)
  else if Sender = U2 then SNRMpbChartup.SetBandsIncluded(2, U2.Checked)
  else if Sender = U3 then SNRMpbChartup.SetBandsIncluded(3, U3.Checked)
  else if Sender = U4 then SNRMpbChartup.SetBandsIncluded(4, U4.Checked)
  else if Sender = D0 then SNRMpbChartdown.SetBandsIncluded(0, D0.Checked)
  else if Sender = D1 then SNRMpbChartdown.SetBandsIncluded(1, D1.Checked)
  else if Sender = D2 then SNRMpbChartdown.SetBandsIncluded(2, D2.Checked)
  else if Sender = D3 then SNRMpbChartdown.SetBandsIncluded(3, D3.Checked)
  else if Sender = D4 then SNRMpbChartdown.SetBandsIncluded(4, D4.Checked);
end;

//Enable any graphs which are set to auto snapshot
procedure Tmainform.SNRsnapChange(Sender: TObject);
begin
  if SNRsnap.Checked then DoMargin.Checked := True;
  if SNRMpbsnap.Checked then DoSNRMperband.Checked := True;
  if Speedsnap.Checked then DoSpeed.Checked := True;
  if CRCsnap.Checked then DoCRC.Checked := True;
  if SESsnap.Checked then DoSES.Checked := True;
  if FECsnap.Checked then DoFEC.Checked := True;
  if ESSnap.Checked then DoES.Checked := True;
  if Bitsnap.Checked then DoBitloading.Checked := True;
  if SNRptsnap.Checked then DoSNRpertone.Checked := True;
  if Swapptsnap.Checked then DoBitswaps.Checked := True;
  if Swappmsnap.Checked then DoSwapspermin.Checked := True;
  if QLNsnap.Checked then DoQLN.Checked := True;
  if HLogsnap.Checked then DoHLog.Checked := True;
end;

//Reset the bitswaps graph to zero values
procedure Tmainform.ResetButtonClick(Sender: TObject);
var
  N: Integer;
begin
  Bitswapchart.YMax := 20;
  BitswapChart.YIncrement := 2;
  for N := 0 to Tonerange do
    Bitswapchart.BitsArray[N] := 0;
  Bitswapchart.StartTime := Now;;
  Bitswapchart.Drawbits(Tonerange);
  for N := 0 to LastToneNumber do
    OldBits[N] := 0;
  BitsSaved := False;
end;

//Check if time since last sample exceeds sample time, and fill gap with blanks
procedure Tmainform.FillGap;
var
  Seconds: Longint;
  N: Integer;
  NUp, NDown: Single;
  T: TDateTime;
begin
  Seconds := SecondsBetween(Now, LastSample);
  if Seconds > SampleRateEdit.Value shl 1 then //Fill gap
  begin
    T := IncMinute(LastSample, 1);
    repeat
      if DoMargin.Checked then
      begin
        N := MarginChart.NumSamples - 1;
        NUp := MarginChart.SampleArray[N].UpYValue;
        NDown := MarginChart.SampleArray[N].DownYValue;
        MarginChart.AddXY(T, NUp, NDown, True);
      end;
      if DoSpeed.Checked then
      begin
        N := Speedchart1.NumSamples - 1;
        NUp := Speedchart1.MaxSampleArray[N].UpYValue;
        NDown := Speedchart1.MaxSampleArray[N].DownYValue;
        SpeedChart1.AddXYmax(T, NUp, NDown, True);
        NUp := Speedchart1.SampleArray[N].UpYValue;
        NDown := Speedchart1.SampleArray[N].DownYValue;
        SpeedChart1.AddXY(T, NUp, NDown, True);
      end;
      if DoCRC.Checked then
      begin
        N := CRCChart.NumSamples - 1;
        NUp := CRCChart.SampleArray[N].UpYValue;
        NDown := CRCChart.SampleArray[N].DownYValue;
        CRCChart.AddXY(T, NUp, NDown, True);
      end;
      if DoSES.Checked then
      begin
        N := SESChart.NumSamples - 1;
        NUp := SESChart.SampleArray[N].UpYValue;
        NDown := SESChart.SampleArray[N].DownYValue;
        SESChart.AddXY(T, NUp, NDown, True);
      end;
      if DoFEC.Checked then
      begin
        N := FECChart.NumSamples - 1;
        NUp := FECChart.SampleArray[N].UpYValue;
        NDown := FECChart.SampleArray[N].DownYValue;
        FECChart.AddXY(T, NUp, NDown, True);
      end;
      if DoSwapspermin.Checked then
      begin
        N := SwapsperminChart.NumSamples - 1;
        NUp := SwapsperminChart.SampleArray[N].UpYValue;
        NDown := SwapsperminChart.SampleArray[N].DownYValue;
        SwapsperminChart.AddXY(T, NUp, NDown, True);
      end;
      if GDotInp then
      begin
        N := LEFTRSChart.NumSamples - 1;
        NUp := LEFTRSChart.SampleArray[N].UpYValue;
        NDown := LEFTRSChart.SampleArray[N].DownYValue;
        LEFTRSChart.AddXY(T, NUp, NDown, True);
        N := rtxtxChart.NumSamples - 1;
        NUp := rtxtxChart.SampleArray[N].UpYValue;
        NDown := rtxtxChart.SampleArray[N].DownYValue;
        rtxtxChart.AddXY(T, NUp, NDown, True);
        N := rtxcChart.NumSamples - 1;
        NUp := rtxcChart.SampleArray[N].UpYValue;
        NDown := rtxcChart.SampleArray[N].DownYValue;
        rtxcChart.AddXY(T, NUp, NDown, True);
        N := rtxucChart.NumSamples - 1;
        NUp := rtxucChart.SampleArray[N].UpYValue;
        NDown := rtxucChart.SampleArray[N].DownYValue;
        rtxucChart.AddXY(T, NUp, NDown, True);
        N := OHFErrChart.NumSamples - 1;
        NUp := OHFErrChart.SampleArray[N].UpYValue;
        NDown := OHFErrChart.SampleArray[N].DownYValue;
        OHFErrChart.AddXY(T, NUp, NDown, True);
        N := RSCorrChart.NumSamples - 1;
        NUp := RSCorrChart.SampleArray[N].UpYValue;
        NDown := RSCorrChart.SampleArray[N].DownYValue;
        RSCorrChart.AddXY(T, NUp, NDown, True);
      end;
      OldT := T;
      LastSample := T;
      T := IncMinute(T, 1);
    until T >= Now;
    OldUpswaps := 0;    //Restart counting of swaps per minute
    OldDownswaps := 0;
    CRCup := 0;         //and CRCs
    CRCdown := 0;
    FECup := 0;         //and FECs
    FECdown := 0;
    LEFTRSUp := 0;      //and LEFTRS
    LEFTRSDown := 0;
    rtxtxUp := 0;       //and retransmissions
    rtxtxDown := 0;
    rtxcUp := 0;
    rtxcDown := 0;
    rtxucUp := 0;
    rtxucDown := 0;
    OHFErrUp := 0;      //and OHFErr
    OHFErrDown := 0;
    RSCorrUp := 0;      //and RSCorr
    RSCorrDown := 0;
  end;
end;

// **** GRAPH SCROLLING ****

// **** Scrolling by dragging ****

//Scrolling the time indexed charts
procedure Tmainform.MarginchartMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Unconnected then Exit;  //disallow scrolling when first starting the program
  if Button = mbRight then
  begin
    if Sender = MarginChart then Maxminform.Show
    else if (Sender = SNRMpbChartup) or (Sender = SNRMpbChartdown) then pbMaxMinForm.Show;
  end
  else begin
    Marginscrolling := True;
    StartX := X;
  end;
end;

procedure Tmainform.MarginchartMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
var
  Scrollamount, Pageamount, GraphStart, GraphEnd: TDateTime;
begin
  if not Marginscrolling then Exit;
  GraphStart := StartTime;
  GraphEnd := Now;
  if XMLSave.SNRM and (Sender = MarginChart) then
  begin
    GraphStart := XMLSave.SNRMStart;
    GraphEnd := XMLSave.SNRMEnd;
  end
  else if XMLSave.ConSpeed and (Sender = Speedchart1) then
  begin
    GraphStart := XMLSave.ConSpeedStart;
    GraphEnd := XMLSave.ConSpeedEnd;
  end
  else if XMLSave.CRC and (Sender = CRCChart) then
  begin
    GraphStart := XMLSave.CRCStart;
    GraphEnd := XMLSave.CRCEnd;
  end
  else if XMLSave.FEC and (Sender = FECChart) then
  begin
    GraphStart := XMLSave.FECStart;
    GraphEnd := XMLSave.FECEnd;
  end
  else if XMLSave.Bitswaps and (Sender = SwapsperminChart) then
  begin
    GraphStart := XMLSave.BitswapsStart;
    GraphEnd := XMLSave.BitswapsEnd;
  end;
  Pageamount := EncodeTimeInterval(Hoursperpage, 0, 0, 0);
  Scrollamount := Pageamount / 50;
  if Sender is TLinechart then
  begin
    if (X > StartX) and ((Sender as TLinechart).PageStart > GraphStart) then   //dragging right
    begin
      StartX := X;
      (Sender as TLinechart).SetPageStart((Sender as TLinechart).PageStart - Scrollamount);
    end
    else if (X < StartX) and ((Sender as TLinechart).PageStart < GraphEnd - Pageamount) then  //dragging left
    begin
      StartX := X;
      (Sender as TLinechart).SetPageStart((Sender as TLinechart).PageStart + Scrollamount);
    end;
  end
  else if Sender is TSNRMperband then
  begin
    if (X > StartX) and ((Sender as TSNRMperband).PageStart > GraphStart) then   //dragging right
    begin
      StartX := X;
      (Sender as TSNRMperband).SetPageStart((Sender as TSNRMperband).PageStart - Scrollamount);
    end
    else if (X < StartX) and ((Sender as TSNRMperband).PageStart < GraphEnd - Pageamount) then  //dragging left
    begin
      StartX := X;
      (Sender as TSNRMperband).SetPageStart((Sender as TSNRMperband).PageStart + Scrollamount);
    end;
  end;
end;

procedure Tmainform.MarginchartMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  Marginscrolling := False;
end;

procedure Tmainform.MarginchartMouseLeave(Sender: TObject);
begin
  Marginscrolling := False;
end;

//Scroll SNR margin chart to point in time from max/min form
procedure Tmainform.Gototime(ATime: TDateTime);
var
  T: TDateTime;
  Hours, Minutes: Word;
begin
  Minutes := Minutesperpage shr 1;
  if Minutes > 59 then
  begin
    Hours := Minutes div 60;
    Minutes := Minutes mod 60;
  end
  else Hours := 0;
  T := ATime - EncodeTimeInterval(Hours, Minutes, 0, 0);
  if T < MarginChart.SampleArray[0].XValue then T := MarginChart.SampleArray[0].XValue;
  MarginChart.SetPageStart(T);
end;

//Scroll the SNRM per band charts to point in time from highest/lowest form
procedure Tmainform.GototimePB(ATime: TDateTime);
var
  T: TDateTime;
  Hours, Minutes: Word;
begin
  Minutes := Minutesperpage shr 1;
  if Minutes > 59 then
  begin
    Hours := Minutes div 60;
    Minutes := Minutes mod 60;
  end
  else Hours := 0;
  T := ATime - EncodeTimeInterval(Hours, Minutes, 0, 0);
  if T < SNRMpbChartup.SampleArray[0].XValue then T := SNRMpbChartup.SampleArray[0].XValue;
  SNRMpbChartup.SetPageStart(T);
  SNRMpbChartdown.SetPageStart(T);
end;

//Scroll the tone indexed charts
procedure Tmainform.QLNchartMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  TPP: Smallint;
  Tone, TonesPer50: Word;
  XMultiplier, FValue, SNRValue: Single;
begin
  if Button = mbRight then
  begin
    if Sender = SNRchart then
    begin
      if X < SNRchart.XOffset then Exit;
      TPP := SNRchart.Tonesperpixel;
      if TPP > 0 then XMultiplier := TPP
      else begin
        case TPP of
          0:  XMultiplier := 0.5;
          -1: XMultiplier := 0.33333;
          -2: XMultiplier := 0.25;
          -3: XMultiplier := 0.2;
        else
          XMultiplier := 1;
        end;
      end;
      Tonesper50 := Round(XMultiplier * 50);
      Tone := SNRchart.FirstTone + Round((X - SNRchart.XOffset) * Tonesper50 / 50);
      ToneNumber2.Caption := IntToStr(Tone);
      FValue := Tone * 4.3125;
      Frequency2.Caption := FloatToStr(FValue) + ' kHz';
      SNRValue := SNRChart.GraphArray[Tone];
      SNR2.Caption := FloatToStr(SNRValue) + ' dB';
    end;
    Exit;
  end;
  if Sender = QLNchart then QScrolling := True
  else if Sender = SNRchart then SNRscrolling := True
  else HScrolling := True;
  StartX := X;
end;

procedure Tmainform.QLNchartMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
const
  ScrollIncrement = 5;
begin
  if not (QScrolling or HScrolling or SNRscrolling) then Exit;
  if Sender = QLNchart then
  begin
    if (X > StartX) and (QLNchart.FirstTone > 0) then QLNchart.FirstTone := QLNchart.FirstTone - ScrollIncrement
    else if (X < StartX) and (QLNchart.MaxTone < QLNchart.LastTone) then QLNchart.FirstTone := QLNchart.FirstTone + ScrollIncrement;
  end
  else if Sender = SNRchart then
  begin
    if (X > StartX) and (SNRchart.FirstTone > 0) then SNRchart.FirstTone := SNRchart.FirstTone - ScrollIncrement
    else if (X < StartX) and (SNRchart.MaxTone < SNRchart.LastTone) then SNRchart.FirstTone := SNRchart.FirstTone + ScrollIncrement;
  end
  else begin
    if (X > StartX) and (HLogchart.FirstTone > 0) then HLogchart.FirstTone := HLogchart.FirstTone - ScrollIncrement
    else if (X < StartX) and (HLogchart.MaxTone < HLogchart.LastTone) then HLogchart.FirstTone := HLogchart.FirstTone + ScrollIncrement;
  end;
  StartX := X;
end;

procedure Tmainform.QLNchartMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  QScrolling := False;
  HScrolling := False;
  SNRscrolling := False;
end;

procedure Tmainform.QLNchartMouseLeave(Sender: TObject);
begin
  QScrolling := False;
  HScrolling := False;
  SNRscrolling := False;
end;

//Scroll the bitloading chart or the bitswap per tone chart, or display bit data
procedure Tmainform.Bitchart1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  Tone, Bits, Tonesper50: Word;
  TPP: Smallint;
  FValue, SNRValue, XMultiplier: Single;
begin
  if Button = mbLeft then
  begin
    if Unconnected then Exit;  //disallow scrolling when first starting the program
    if Sender = Bitchart1 then Bitscrolling := True
    else Swapscrolling := True;
    StartX := X;
  end
  else if (Button = mbRight) and (Sender = Bitchart1) then
  begin
    if X < Bitchart1.XOffset then Exit;
    TPP := Bitchart1.Tonesperpixel;
    if TPP > 0 then XMultiplier := TPP
    else begin
      case TPP of
        0:  XMultiplier := 0.5;
        -1: XMultiplier := 0.33333;
        -2: XMultiplier := 0.25;
        -3: XMultiplier := 0.2;
      else
        XMultiplier := 1;
      end;
    end;
    Tonesper50 := Round(XMultiplier * 50);
    Tone := Bitchart1.FirstTone + Round((X - Bitchart1.XOffset) * Tonesper50 / 50);
    Tonenumber1.Caption := IntToStr(Tone);
    FValue := Tone * 4.3125;
    Frequency.Caption := FloatToStr(FValue) + ' kHz';
    Bits := Bitchart1.GetBits(Tone);
    Numberofbits.Caption := IntToStr(Bits);
    if DoSNRpertone.Checked then
    begin
      if SNRwithbitloading.Checked then SNRValue := Bitchart1.GetSNR(Tone)
      else SNRValue := SNRChart.GraphArray[Tone];
      SNR.Caption := FloatToStr(SNRValue) + ' dB';
    end
    else SNR.Caption := 'N/A';
    Panel6.Refresh;
  end;
end;

procedure Tmainform.Bitchart1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var
  Scrollincrement: Byte;
begin
  if not (Bitscrolling or Swapscrolling) then Exit;
  if Sender = Bitchart1 then Scrollincrement := 5 else Scrollincrement := 20;
  if (X < StartX) then
  begin
    (Sender as TBitchart).FirstTone := (Sender as TBitchart).FirstTone + ScrollIncrement;
    StartX := X;
  end
  else if (X > StartX) and ((Sender as TBitchart).FirstTone > 0) then
  begin
    if (Sender as TBitchart).FirstTone < ScrollIncrement then (Sender as TBitchart).FirstTone := 0
    else (Sender as TBitchart).FirstTone := (Sender as TBitchart).FirstTone - ScrollIncrement;
    StartX := X;
  end;
end;

procedure Tmainform.Bitchart1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  Bitscrolling := False;
  Swapscrolling := False;
end;

procedure Tmainform.Bitchart1MouseLeave(Sender: TObject);
begin
  Bitscrolling := False;
  Swapscrolling := False;
end;

//Clear displayed values for a selected tone on bitloading graph
procedure Tmainform.BitloadingTabMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbRight then
  begin
    Tonenumber1.Caption := '';
    Frequency.Caption := '';
    Numberofbits.Caption := '';
    SNR.Caption := '';
  end;
end;

//Clear displayed values for a selected tone on SNR per tone graph
procedure Tmainform.SNRTabMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbRight then
  begin
    Tonenumber2.Caption := '';
    Frequency2.Caption := '';
    SNR2.Caption := '';
  end;
end;

// **** Scrolling by scrollwheel ****

//Scrolling linechart graphs using the scroll wheel
procedure TMainform.DoLinechartScroll(Sender: TObject; Down: Boolean);
var
  PageAmount, ScrollAmount: TDateTime;
begin
  PageAmount := EncodeTimeInterval(HoursperPage, 0, 0, 0);
  ScrollAmount := PageAmount / 10;
  with (Sender as TLinechart) do
  begin
    if Down then
    begin
      if PageStart - ScrollAmount < StartTime then ScrollAmount := PageStart - StartTime;
      ScrollAmount := -ScrollAmount;
    end
    else if PageStart + PageAmount >= Now then ScrollAmount := 0;
    SetPageStart(PageStart + ScrollAmount);
  end;
end;

procedure Tmainform.MarginchartMouseWheelDown(Sender: TObject;
  Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
begin
  DoLinechartScroll(Sender, True);
end;

procedure Tmainform.MarginchartMouseWheelUp(Sender: TObject;
  Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
begin
  DoLinechartScroll(Sender, False);
end;

//Scrolling the SNRM per band chart using the scrollwheel
procedure TMainform.DoSNRMpbchartScroll(Sender: TObject; Down: Boolean);
var
  PageAmount, ScrollAmount: TDateTime;
begin
  PageAmount := EncodeTimeInterval(HoursperPage, 0, 0, 0);
  ScrollAmount := PageAmount / 10;
  with (Sender as TSNRMperband) do
  begin
    if Down then
    begin
      if PageStart - ScrollAmount < StartTime then ScrollAmount := PageStart - StartTime;
      ScrollAmount := -ScrollAmount;
    end
    else if PageStart + PageAmount >= Now then ScrollAmount := 0;
    SetPageStart(PageStart + ScrollAmount);
  end;
end;

procedure Tmainform.SNRMpbChartupMouseWheelDown(Sender: TObject;
  Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
begin
  DoSNRMpbchartScroll(Sender, True);
end;

procedure Tmainform.SNRMpbChartupMouseWheelUp(Sender: TObject;
  Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
begin
  DoSNRMpbchartScroll(Sender, False);
end;

//Scrolling tone indexed graphs using the scrollwheel
procedure TMainform.DoToneScroll(Sender: TObject; Down: Boolean);
var
  ScrollAmount: SmallInt;
begin
  with (Sender as TLinediagchart) do
  begin
    ScrollAmount := (Width - XOffset) * ToneIncrement div 500;
    if Down then
    begin
      if FirstTone < ScrollAmount then ScrollAmount := FirstTone;
      ScrollAmount := -ScrollAmount;
    end
    else if (Sender as TLinediagchart).MaxTone >= (Sender as TLinediagchart).LastTone then ScrollAmount := 0;
    FirstTone := FirstTone + ScrollAmount;
  end;
end;

procedure Tmainform.SNRchartMouseWheelDown(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
  DoToneScroll(Sender, True);
end;

procedure Tmainform.SNRchartMouseWheelUp(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
  DoToneScroll(Sender, False);
end;

//Scrolling bitloading/bitswaps per tone graphs using the scrollwheel
procedure TMainform.DoBitScroll(Sender: TObject; Down: Boolean);
var
  ScrollAmount: SmallInt;
begin
  with (Sender as TBitchart) do
  begin
    ScrollAmount := (Width - XOffset) * ToneIncrement div 500;
    if Down then
    begin
      if FirstTone < ScrollAmount then ScrollAmount := FirstTone;
      ScrollAmount := -ScrollAmount;
    end;
    FirstTone := FirstTone + ScrollAmount;
  end;
end;

procedure Tmainform.Bitchart1MouseWheelDown(Sender: TObject;
  Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
begin
  DoBitScroll(Sender, True);
end;

procedure Tmainform.Bitchart1MouseWheelUp(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
  DoBitScroll(Sender, False);
end;

//**** DSL MODES AND BAND PLANS ****

//Extract tone bands from output of pbParams command
procedure Tmainform.Extractbands;
var
  Dataline: string;
  N, Index, Val: Integer;
  C: Char;
begin
  for N := 0 to 7 do //Zero the tone band values initially
  begin
    ToneBands[0, N] := 0;
    ToneBands[1, N] := 0;
  end;
  if Paramslist.Lines.Count < 10 then //not VDSL2
  begin
    TelnetForm.Mode := 1;
    ExtendedBox.Checked := False;
    ExtendedBox.Enabled := False;
    Exit;
  end;
  ExtendedBox.Enabled := True;

  //First, check if band U0 is used
  N := 0;
  repeat
    Inc(N);
    Dataline := LowerCase(Paramslist.Lines[N]);
  until (Pos('margin', Dataline) > 0) or (N = ParamsList.Lines.Count - 1);    //SNR Margin line
  if N = ParamsList.Lines.Count - 1 then   //Bad data
  begin
    TelnetForm.Mode := 1;
    ExtendedBox.Checked := False;
    ExtendedBox.Enabled := False;
    MiscString := 'Bad pbParams data received from modem';
    LogEvent(leMisc);
    Exit;
  end;
  Index := Pos(':', Dataline);
  repeat
    Inc(Index);
    C := Dataline[Index];
  until (C in ['0'..'9','n']);   //Find numeral or 'n'
  if Pos('n/a', Dataline) = Index then U0notUsed := True     //Entry is N/A and not something else beginning with N
  else U0NotUsed := False;

  //Now extract the bands
  N := 0;
  repeat
    Inc(N);
    Dataline := LowerCase(Paramslist.Lines[N]);
  until Pos('discovery', Dataline) = 1;    //was 'medley' up to v6.0.12 (inc)
  ToneRange := 511;   //Will be increased, depending on parsing of Discovery values
  Inc(N);
  Dobandline(ParamsList.Lines[N], False);  //US bands
  Inc(N);
  Dobandline(Paramslist.Lines[N], True);   //DS bands

  //Check for overlapping bands
  N := 0;
  repeat
    if ToneBands[0, N + 1] > ToneBands[1, N] then
    begin
      Val := ToneBands[0, N + 1];
      ToneBands[0, N + 1] := ToneBands[1, N] - 1;
      ToneBands[1, N] := Val + 1;
    end;
    if N + 2 = LastIndex then Break;
    if ToneBands[1, N + 1] > ToneBands[0, N + 2] then
    begin
      Val := ToneBands[1, N + 1];
      ToneBands[1, N + 1] := ToneBands[0, N + 2] - 1;
      ToneBands[0, N + 2] := Val + 1;
    end;
    N := N + 2;
  until N = 8;

  //Send the band plan to the charts which use it
  for N := 0 to 7 do
  begin
    Bitchart1.UpTones[N] := ToneBands[0, N];
    Bitchart1.DownTones[N] := ToneBands[1, N];
    Bitchart1.VDSL2 := VDSL2;
    Bitswapchart.UpTones[N] := ToneBands[0, N];
    Bitswapchart.DownTones[N] := ToneBands[1, N];
    Bitswapchart.VDSL2 := VDSL2;
    SNRchart.UpTones[N] := ToneBands[0, N];
    SNRchart.DownTones[N] := ToneBands[1, N];
    SNRchart.VDSL2 := VDSL2;
    QLNchart.UpTones[N] := ToneBands[0, N];
    QLNchart.DownTones[N] := ToneBands[1, N];
    QLNchart.VDSL2 := VDSL2;
    HLogchart.UpTones[N] := ToneBands[0, N];
    HLogchart.DownTones[N] := ToneBands[1, N];
    HLogchart.VDSL2 := VDSL2;
  end;
end;

//Extract tone bands from one line of pbParams
procedure Tmainform.Dobandline(Dataline: string; Isdownstream: Boolean);
var
  C: Char;
  N, Len, Index, Num: Integer;
  Numstring: string;
begin
  Len := Length(Dataline);
  Index := 0;
  LastIndex := 0;
  N := 1;
  while N <= Len do
  begin
    C := Dataline[N];
    if (C in ['0'..'9']) then
    begin
      Numstring := C;
      Inc(N);
      C := Dataline[N];
      while (C in ['0'..'9']) do
      begin
        Numstring := Numstring + C;
        Inc(N);
        C := Dataline[N];
      end;
      Num := StrToInt(Numstring);
      if U0notUsed and (not IsDownstream) and (Index < 2) then //Band U0 isn't being used, so bypass Index 0 and 1 and ignore first set of values
      begin
        Index := 2;
        repeat        //Find start of next set of values (band U1)
          Inc(N);
        until Dataline[N] = '(';
        //Numstring := C;
        Numstring := '';
        Inc(N);
        C := Dataline[N];
        while (C in ['0'..'9']) do
        begin
          Numstring := Numstring + C;
          Inc(N);
          C := Dataline[N];
        end;
        Num := StrToInt(Numstring);
      end;
      if IsDownstream then Tonebands[1, Index] := Num
      else ToneBands[0, Index] := Num;
      if Num > ToneRange then ToneRange := Num;
      Inc(Index);
    end;
    Inc(N);
  end;
  LastIndex := Index;
  //if ToneRange > 4095 then ToneRange := 4095;  //Temporary fudge to stop crashing with profile 35b
end;

//Process change of DSL mode
procedure Tmainform.ModeChange;
begin
  case TelnetForm.Mode of
  0: begin  //ADSL, ADSL2
       ExtendedBox.Checked := False;
       ExtendedBox.Enabled := False;
       if AnnexM.Checked then
       begin
         Bitchart1.UpTones[0] := 0;
         Bitchart1.UpTones[1] := 56;
         Bitchart1.DownTones[0] := 60;
         Bitchart1.Downtones[1] := 255;
         Bitswapchart.UpTones[0] := 0;
         Bitswapchart.UpTones[1] := 56;
         Bitswapchart.DownTones[0] := 60;
         Bitswapchart.Downtones[1] := 255;
         Tonerange := 255;
       end
       else begin
         Bitchart1.UpTones[0] := 0;
         Bitchart1.UpTones[1] := 31;
         Bitchart1.DownTones[0] := 32;
         Bitchart1.Downtones[1] := 255;
         Bitswapchart.UpTones[0] := 0;
         Bitswapchart.UpTones[1] := 31;
         Bitswapchart.DownTones[0] := 32;
         Bitswapchart.Downtones[1] := 255;
         Tonerange := 255;
       end;
     end;
  1: begin   //ADSL2+
       ExtendedBox.Checked := False;
       ExtendedBox.Enabled := False;
       if AnnexM.Checked then
       begin
         Bitchart1.UpTones[0] := 0;
         Bitchart1.UpTones[1] := 56;
         Bitchart1.DownTones[0] := 60;
         Bitchart1.Downtones[1] := 511;
         Bitswapchart.UpTones[0] := 0;
         Bitswapchart.UpTones[1] := 56;
         Bitswapchart.DownTones[0] := 60;
         Bitswapchart.Downtones[1] := 511;
         Tonerange := 511;
       end
       else begin
         Bitchart1.UpTones[0] := 0;
         Bitchart1.UpTones[1] := 31;
         Bitchart1.DownTones[0] := 32;
         Bitchart1.Downtones[1] := 511;
         Bitswapchart.UpTones[0] := 0;
         Bitswapchart.UpTones[1] := 31;
         Bitswapchart.DownTones[0] := 32;
         Bitswapchart.Downtones[1] := 511;
         Tonerange := 511;
       end;
     end;
  2: begin
       if TelnetForm.Bandsdone then Extractbands;
     end;
   end;
end;

//Extract numbers from line of text
procedure Tmainform.Extractnumbers(Dataline: string; Pathline: Boolean);
var
  N, Len: Integer;
  C: Char;
  Numberstring: string;
  Number1done, Number2done: Boolean;
begin
  Number1done := False;
  Number2done := False;
  Len := Length(Dataline);
  N := 1;
  if Pathline and not ThomBox.Checked then repeat
    Inc(N);
    C := Dataline[N];
  until C = '=';
  repeat
    C := Dataline[N];
    if C in ['0'..'9', '-'] then
    begin
      Numberstring := C;
      while N < Len do
      begin
        Inc(N);
        C := Dataline[N];
        if (C in ['0'..'9', '.', ',']) then
        begin
          if (C = ',') or (C = '.') then C := DSeparator;
          Numberstring := Numberstring + C;
        end
        else begin
          if (C = '/') and not Number1done then   //Bypass second number in xxxx/yyyy situation
          repeat
            Inc(N);
            C := DataLine[N];
          until (not (C in ['0'..'9'])) or (N = Len);
          Break;
        end;
      end;
      if not Number1done then
      begin
        Number1 := Strtofloat(Numberstring);
        Number1String := Numberstring;
        Number1done := True;
      end
      else begin
        Number2 := Strtofloat(Numberstring);
        Number2String := Numberstring;
        Number2done := True;
        Break;
      end;
    end;
    Inc(N);
  until N > Len;
  if not Number1done then
  begin
    Number1 := -1;
    Number1String := '0';
  end;
  if not Number2done then
  begin
    Number2 := -1;
    Number2String := '0'
  end;
end;

// **** TWEAKING ****

//Set progress bar positions
procedure Tmainform.Timer2Timer(Sender: TObject);
begin
  Inc(SecondsPassed);
  if UtilityOp = AdvancedTweaking then ProgressBar2.Position := SecondsPassed;
end;

//Enable or abort tweaking operation
procedure Tmainform.Timer3Timer(Sender: TObject);
begin
  if not Collecting then
  begin
    Progressbar2.Position := SecondsPassed;
    Timer3.Enabled := False;
    case UtilityOp of
      AdvancedTweaking: DoAdvancedTweak(True);
    end;
    Exit;
  end;
  Inc(T3seconds);
  if T3seconds = 30 then    //Abort operation after 30 seconds
  begin
    Timer3.Enabled := False;
    case UtilityOp of
      AdvancedTweaking: DoAdvancedTweak(False);
    end;
  end;
end;

//Adjust SNRM tweaking value
procedure Tmainform.SNRMbarChange(Sender: TObject);
var
  N: Single;
  S: string;
begin
  SuccessLabel.Hide;
  TweakValue := 100 + (SNRMbar.Position * 5 div 3);
  if TweakValue = 0 then TweakValue := 1; //Zero does a reset to the default value
  if TweakValue < 0 then TweakValue := 65536 + TweakValue;
  N := SNRMbar.Position / 10;
  if N > 0 then OffsetLabel.Font.Color := clGreen
  else if N < 0 then OffsetLabel.Font.Color := clRed
  else OffsetLabel.Font.Color := clBlack;
  S := FloatToStrF(N, ffFixed, 0, 1);
  if S = '0.0' then S := '0';
  if N > 0 then S := '+' + S;
  OffsetLabel.Caption := S + ' dB';
  if DoSNRM.Checked then DoBitswapChange(Self);
end;

//Toggle custom CLI command
procedure Tmainform.CustomBoxChange(Sender: TObject);
begin
  SNRMbox.Enabled := not CustomBox.Checked;
  BitswapToggle.Enabled := not CustomBox.Checked;
  CominmgnToggle.Enabled := not CustomBox.Checked;
  i24kToggle.Enabled := not CustomBox.Checked;
  SesdropToggle.Enabled := not CustomBox.Checked;
  SraToggle.Enabled := not CustomBox.Checked;
  TrellisToggle.Enabled := not CustomBox.Checked;
  CustomEdit.Enabled := CustomBox.Checked;
  DoSNRM.Enabled := not CustomBox.Checked;
  DoBitswap.Enabled := not CustomBox.Checked;
  DoCoMinMgn.Enabled := not CustomBox.Checked;
  Doi24k.Enabled := not CustomBox.Checked;
  DoSesdrop.Enabled := not CustomBox.Checked;
  DoSra.Enabled := not CustomBox.Checked;
  DoTrellis.Enabled := not CustomBox.Checked;
end;

procedure Tmainform.DailyButtonChange(Sender: TObject);
begin
  if Trafficpermin.Checked then
  begin
    TrafficperminChart.Visible := True;
    TrafficChart.Visible := False;
    UpdownPanel.Visible := True;
    ShowBillingBox.Enabled := False;
  end
  else begin
    TrafficChart.Visible := True;
    TrafficperminChart.Visible := False;
    UpdownPanel.Visible := False;
    ShowBillingBox.Enabled := True;
    if DailyButton.Checked then TrafficChart.GraphType := utDaily else TrafficChart.GraphType := utMonthly;
  end;
end;

//Generate advanced tweak command line
procedure Tmainform.DobitswapChange(Sender: TObject);
var
  S: string;
  OptionsSet: Boolean;
begin
  TweakingCommand.Enabled := True;
  OptionsSet := False;
  if SpecialLogin.Checked then
  begin
    S := SCommandBox.Items[SCommandBox.ItemIndex] + ' configure';
    if SAnnexm.Checked then S := S + ' --mod m';
  end
  else begin
    S := CommandBox.Items[CommandBox.ItemIndex] + ' configure';
    if Annexm.Checked then S := S + ' --mod m';
  end;
  if DoSNRM.Checked then
  begin
    S := S + ' --snr ' + IntToStr(TweakValue);
    OptionsSet := True;
  end;
  if Dobitswap.Checked then
  begin
    if BitswapToggle.ItemIndex = 0 then S := S + ' --bitswap on'
    else S := S + ' --bitswap off';
    OptionsSet := True;
  end;
  if DoCoMinMgn.Checked then
  begin
    if CoMinMgnToggle.ItemIndex = 0 then S := S + ' --CoMinMgn on'
    else S := S + ' --CoMinMgn off';
    OptionsSet := True;
  end;
  if Doi24k.Checked then
  begin
    if i24kToggle.ItemIndex = 0 then S := S + ' --i24k on'
    else S := S + ' --i24k off';
    OptionsSet := True;
  end;
  if Dosesdrop.Checked then
  begin
    if SesdropToggle.ItemIndex = 0 then S := S + ' --sesdrop on'
    else S := S + ' --sesdrop off';
    OptionsSet := True;
    OptionsSet := True;
  end;
  if Dosra.Checked then
  begin
    if SraToggle.ItemIndex = 0 then S := S + ' --sra on'
    else S := S + ' --sra off';
    OptionsSet := True;
  end;
  if DoTrellis.Checked then
  begin
    if TrellisToggle.ItemIndex = 0 then S := S + ' --trellis on'
    else S := S + ' --trellis off';
    OptionsSet := True;
  end;
  TweakingCommand.Caption := S;
  if OptionsSet then FailLabel.Hide;
end;

//Apply advanced tweaks
procedure Tmainform.ApplyButtonClick(Sender: TObject);
begin
  if TweakingCommand.Caption = '(Tweaking command)' then
  begin
    FailLabel.Caption := 'No options have been set';
    FailLabel.Show;
    Exit;
  end
  else if not (DoSNRM.Checked or Dobitswap.Checked or DoCoMinMgn.Checked or Doi24k.Checked
    or Dosesdrop.Checked or Dosra.Checked or DoTrellis.Checked) then
  begin
    FailLabel.Caption := 'No options have been included';
    FailLabel.Show;
    Exit;
  end;
  LogEvent(leAdvancedTweaking);
  FailLabel.Hide;
  SuccessLabel.Hide;
  UtilityOp := AdvancedTweaking;
  ApplyButton.Enabled := False;
  StartButton.Enabled := False;
  PauseButton.Enabled := False;
  StopButton.Enabled := False;
  SNRMbar.Enabled := False;
  ProgressBar2.Position := 0;
  ProgressBar2.Visible := True;
  WaitLabel1.Show;
  if Collecting then  //Wait for sampling to complete
  begin
    T3seconds := 0;
    Timer3.Enabled := True;
  end
  else DoAdvancedTweak(True);
end;

procedure TMainform.DoAdvancedTweak(TweakOK: Boolean);
var
  Recording: Boolean;
begin
  if TweakOK then
  begin
    Recording := Timer1.Enabled;
    if Recording then StopButtonClick(PauseButton);
    StartButton.Enabled := False;
    SecondsPassed := 0;
    Timer2.Enabled := True;
    if CustomBox.Checked then TelnetForm.AdvancedTweakCommand := CustomEdit.Text
    else TelnetForm.AdvancedTweakCommand := TweakingCommand.Caption;
    if SpecialLogin.Checked then
    begin
      TelnetForm.Special := True;
      TelnetForm.HostName := SIPEdit.Text;
      TelnetUnit1.PortNumber := StrToInt(SPortEdit.Text);
      TelnetForm.UserName := Responses[1].Text;
      TelnetForm.Password := Responses[2].Text;
    end
    else begin
      TelnetForm.Special := False;
      TelnetForm.HostName := IPEdit.Text;
      TelnetUnit1.PortNumber := StrToInt(PortEdit.Text);
      TelnetForm.UserName := NameEdit.Text;
      TelnetForm.Password := PasswordEdit.Text;
    end;
    TelnetForm.LE := ProcessOperation(Self, toAdvancedTweak);
    if TelnetForm.LE <> leSuccess then
    begin
      LogEvent(TelnetForm.LE);
      FailLabel.Caption := 'CLI command failed';
      FailLabel.Show;
    end
    else begin
      repeat
        Sleep(50);
        Application.ProcessMessages;
      until SecondsPassed = 60;
      Inc(ResyncCount);
      StatsArray[1, 5] := IntToStr(ResyncCount) + ' (since ' + FormatDateTime('dd mmm yyyy hh:nn:ss', ResyncBaseTime) + ')';
    end;
    Timer2.Enabled := False;
  end
  else begin
    LogEvent(leNorouterresponse);
    FailLabel.Caption := 'Modem/router failed to respond within 30 seconds';
    FailLabel.Show;
  end;
  ProgressBar2.Hide;
  WaitLabel1.Hide;
  ApplyButton.Enabled := True;
  SNRMbar.Enabled := True;
  if TweakOK and (TelnetForm.LE = leSuccess) then
  begin
    SuccessLabel.Show;
  end;
  if Recording then
  begin
    StartButtonClick(Self);
    PauseButton.Enabled := True;
    StopButton.Enabled := True;
  end
  else StartButton.Enabled := True;
end;

// **** CUSTOM COMMANDS ****

//Calculate minutes between 'Every' custom commands
procedure Tmainform.EveryChange(Sender: TObject);
begin
  EveryMinutes := (CommandHours.Value * 60) + CommandMinutes.Value;
end;

//Transfer custom command details to TelnetForm
procedure Tmainform.TransferCommands;
var
  N, N1: Byte;
begin
  for N := 1 to 4 do
  begin
    TelnetForm.CustomCommands[N] := Commands[N].Text;
    for N1 := 1 to 5 do
    begin
      TelnetForm.CustomIncludes[N, N1] := CommandNumbers[N, N1].Checked;
      if DoCustomNow then TelnetForm.CustomIncludes[N, 6] := CommandNumbers[N, 6].Checked
      else TelnetForm.CustomIncludes[N, 6] := False;
    end;
  end;
end;

//Clear the custom commands output box
procedure Tmainform.ClearTextButtonClick(Sender: TObject);
begin
  CommandOutput.Clear;
end;

//Send custom command with next sample
procedure Tmainform.SendNowButtonClick(Sender: TObject);
begin
  SendNowButton.Enabled := False;
  DoCustomNow := True;
end;

//Toggle custom commands output location
procedure Tmainform.DisplayHereButtonChange(Sender: TObject);
begin
  CommandOutput.Visible := DisplayHereButton.Checked;
  SaveTextButton.Visible := DisplayHereButton.Checked;
  ClearTextButton.Visible := DisplayHereButton.Checked;
  CustomCommandsText.Enabled := DisplayHereButton.Checked;
end;

// **** ERROR REPORTING ****

//Read the contents of errors1.dat into the error array
procedure Tmainform.ReadErrorFile1;
var
  F: file of TErrors1;
  N: Integer;
  S: string;
begin
  S := HomePath + 'errors1.dat';
  if not FileExists(S) then
  begin
    DailyCRCup := 0;
    DailyCRCdown := 0;
    DailyFECup := 0;
    DailyFECdown := 0;
    DailyHECup := 0;
    DailyHECdown := 0;
    DailyESup := 0;
    DailyESdown := 0;
    DailySESup := 0;
    DailySESdown := 0;
    Exit;
  end;
  AssignFile(F, S);
  try
    Reset(F);
    for N := 0 to 10 do Read(F, ErrorArray1[N]);
    Closefile(F);
    DailyCRCup := ErrorArray1[0].CRCup;
    DailyCRCdown := ErrorArray1[0].CRCdown;
    DailyFECup := ErrorArray1[0].FECup;
    DailyFECdown := ErrorArray1[0].FECdown;
    DailyHECup := ErrorArray1[0].HECup;
    DailyHECdown := ErrorArray1[0].HECdown;
    DailyESup := ErrorArray1[0].ESup;
    DailyESdown := ErrorArray1[0].ESdown;
    DailySESup := ErrorArray1[0].SESup;
    DailySESdown := ErrorArray1[0].SESdown;
    AdjustErrorArray1;
  except
    on E: Exception do
    begin
      ResetErrors1;
      MiscString := 'Error reading average errors file. Average errors reset.';
      LogEvent(leMisc);
    end;
  end;
end;

//Save the contents of the error array
procedure Tmainform.WriteErrorFile1;
var
  F: file of TErrors1;
  N: Integer;
begin
  AssignFile(F, HomePath + 'errors1.dat');
  try
    Rewrite(F);
    for N := 0 to 10 do
      Write(F, ErrorArray1[N]);
    Closefile(F);
  except
    on E: Exception do;
       //Do nothing
  end;
end;

//Check error array for new date and shift as appropriate
procedure TMainform.AdjustErrorArray1;
var
  Days, N: Integer;
  Index: Byte;
begin
  if ErrorArray1[0].TD = 0 then Exit;
  try
    Days := DaysBetween(Date, DateOf(ErrorArray1[0].TD));
    if Days > 0 then
    begin
      CurrentErrorDay := StartOfTheDay(Now);
      for N := 1 to Days do
      begin
        for Index := 9 downto 0 do ErrorArray1[Index + 1] := ErrorArray1[Index];
        ResetErrorElement(0);
        ErrorArray1[0].TD :=  IncDay(ErrorArray1[1].TD, 1);
      end;
      ErrorArray1[0].TD := CurrentErrorDay;
      AverageDateBox.ItemIndex := 0;
    end;
    for Index := 0 to 10 do
      if ErrorArray1[Index].Seconds = 0 then
        AverageDateBox.Items[Index] := FormatDateTime('dd mmm yyyy', ErrorArray1[0].TD - Index) + ' (No data)'
      else AverageDateBox.Items[Index] := FormatDateTime('dd mmm yyyy', ErrorArray1[Index].TD);
    AverageDateBox.Items[0] := 'Today';
  except
    on E: Exception do
    begin
      MiscString := 'Average error data corrupted, values reset';
      LogEvent(leMisc);
      ResetErrors1;
    end;
  end;
end;

//Fill in average error rates
procedure TMainform.DisplayErrors1;
var
  Value, Divider: Double;
  IntValueUp, IntValueDown: Cardinal;
  ESUp1, ESDown1: Word;
  Index, D: Byte;
  SValue: string;

  procedure SetPrecision;
  begin
    if Divider = 0 then Value := 0
    else Value := Value / Divider;
    if Value < 0 then Value := 0;
    if Value < 0.006 then D := 0
    else if Value < 10 then D := 2
    else if Value < 100 then D := 1
    else D := 0;
    SValue := FloatToStrF(Value, ffFixed, 0, D);
  end;

begin
  Index := AverageDateBox.ItemIndex;
  StartingLabel1.Caption := AverageDateBox.Items[Index];
  if ErrorArray1[Index].Seconds = 0 then
  begin
    CRCAverageDown.Caption := '0';
    CRCAverageUp.Caption := '0';
    FECAverageDown.Caption := '0';
    FECAverageUp.Caption := '0';
    HECAverageDown.Caption := '0';
    HECAverageUp.Caption := '0';
    ESAverageDown.Caption := '0';
    ESAverageUp.Caption := '0';
    SESAverageDown.Caption := '0';
    SESAverageUp.Caption := '0';
    Exit;
  end;
  Divider := ErrorArray1[Index].Seconds / 3600;
  Value := ErrorArray1[Index].CRCdown;
  SetPrecision;
  CRCAverageDown.Caption := SValue;
  Value := ErrorArray1[Index].CRCup;
  SetPrecision;
  CRCAverageUp.Caption := SValue;
  Value := ErrorArray1[Index].FECdown;
  SetPrecision;
  FECAverageDown.Caption := SValue;
  Value := ErrorArray1[Index].FECup;
  SetPrecision;
  FECAverageUp.Caption := SValue;
  Value := ErrorArray1[Index].HECdown;
  SetPrecision;
  HECAverageDown.Caption := SValue;
  Value := ErrorArray1[Index].HECup;
  SetPrecision;
  HECAverageUp.Caption := SValue;
  Value := ErrorArray1[Index].ESdown;
  SetPrecision;
  ESAverageDown.Caption := SValue;
  StatsArray[1, 19] := SValue;
  Value := ErrorArray1[Index].ESup;
  SetPrecision;
  ESAverageUp.Caption := SValue;
  StatsArray[2, 19] := SValue;
  Value := ErrorArray1[Index].SESdown;
  SetPrecision;
  SESAverageDown.Caption := SValue;
  Value := ErrorArray1[Index].SESup;
  SetPrecision;
  SESAverageUp.Caption := SValue;
  IntValueUp := 0;
  IntValueDown := 0;
  ESUp1 := 0;
  ESDown1 := 0;
  for D := 1 to 24 do
  begin
    ESChart.GetHourlyES(AverageDateBox.ItemIndex, D, ESUp1, ESDown1);
    IntValueDown := IntValueDown + ESDown1;
    IntValueUp := IntValueUp + ESUp1;
  end;
  ESTotalDown.Caption := IntToStr(IntValueDown);
  ESTotalUp.Caption := IntToStr(IntValueUp);

  //Fill in today's ES values on the status bar
  if ErrorArray1[0].Seconds = 0 then Exit;
  Divider := ErrorArray1[0].Seconds / 3600;
  Value := ErrorArray1[0].ESdown;
  SetPrecision;
  ES.Caption := 'ES/hour (Down/Up): ' + SValue + ' / ';
  Value := ErrorArray1[0].ESup;
  SetPrecision;
  ES.Caption := ES.Caption + SValue;
end;

//Reset the error array
procedure Tmainform.ResetErrors1;
var
  N: Byte;
begin
  for N := 0 to 10 do ResetErrorElement(N);
  ErrorArray1[0].TD := CurrentErrorDay;
  AverageDateBox.ItemIndex := 0;
  DailyCRCUp := 0;
  DailyCRCDown := 0;
  DailyFECUp := 0;
  DailyFECDown := 0;
  DailyHECUp := 0;
  DailyHECDown := 0;
  DailyESUp := 0;
  DailyESDown := 0;
  DailySESUp := 0;
  DailySESDown := 0;
end;

//Reset one element of error array
procedure TMainform.ResetErrorElement(Element: Byte);
begin
  ErrorArray1[Element].CRCup := 0;
  ErrorArray1[Element].CRCdown := 0;
  ErrorArray1[Element].FECup := 0;
  ErrorArray1[Element].FECdown := 0;
  ErrorArray1[Element].HECup := 0;
  ErrorArray1[Element].HECdown := 0;
  ErrorArray1[Element].ESup := 0;
  ErrorArray1[Element].ESdown := 0;
  ErrorArray1[Element].SESup := 0;
  ErrorArray1[Element].SESdown := 0;
  ErrorArray1[Element].TD := 0;
  ErrorArray1[Element].Seconds := 0;
  if Element = 0 then
  begin
    AverageDateBox.Items[Element] := 'Today';
    DailyCRCUp := 0;
    DailyCRCDown := 0;
    DailyFECUp := 0;
    DailyFECDown := 0;
    DailyHECUp := 0;
    DailyHECDown := 0;
    DailyESUp := 0;
    DailyESDown := 0;
    DailySESUp := 0;
    DailySESDown := 0;
  end
  else AverageDateBox.Items[Element] := FormatDateTime('dd mmm yyyy', CurrentErrorDay - Element) + ' (No data)';
end;

//Reset all error data and redisplay
procedure Tmainform.ErrorResetButton1Click(Sender: TObject);
begin
  ResetErrors1;
  LogEvent(leErrorsReset);
  DisplayErrors1;
end;

//Reset today's error values and redisplay
procedure Tmainform.ResetTodayButtonClick(Sender: TObject);
begin
  ResetErrorElement(0);
  DisplayErrors1;
end;

//Toggle autoscaling of ES per hour chart
procedure Tmainform.ESAutoscaleChange(Sender: TObject);
begin
  ESChart.Autoscale := (not ESSetMax.Checked);
  ESMaxChange(Self);
end;

//Process change of max ES per hour value
procedure Tmainform.ESMaxChange(Sender: TObject);
begin
  if not ESAutoscale.Checked then ESChart.HourlyMaxY := StrToInt(ESMax.Text);
end;

//Process change of day for error averages
procedure Tmainform.AverageDateBoxChange(Sender: TObject);
begin
  if AverageDateBox.ItemIndex = 11 then AverageDateBox.ItemIndex := 0;
  DisplayErrors1;
end;

//Copy the average error data to the clipboard
procedure Tmainform.ClipButton15Click(Sender: TObject);
var
  ErrorList: TStringList;
  Str: string;
begin
  ErrorList := TStringList.Create;
  try
    Str := 'Average error rates for ';
    if StartingLabel1.Caption = 'Today' then Str := Str + FormatDateTime('dd mmm yyyy', CurrentErrorDay)
    else Str := Str + StartingLabel1.Caption;
    ErrorList.Add(Str);
    ErrorList.Add('');
    ErrorList.Add('CRC erors per hour:  ' + CRCAverageDown.Caption + ' Down,  ' + CRCAverageUp.Caption + ' Up');
    ErrorList.Add('FEC erors per hour:  ' + FECAverageDown.Caption + ' Down,  ' + FECAverageUp.Caption + ' Up');
    ErrorList.Add('HEC erors per hour:  ' + HECAverageDown.Caption + ' Down,  ' + HECAverageUp.Caption + ' Up');
    ErrorList.Add('ES per hour:         ' + ESAverageDown.Caption + ' Down,  ' + ESAverageUp.Caption + ' Up');
    ErrorList.Add('SES per hour:        ' + SESAverageDown.Caption + ' Down,  ' + SESAverageUp.Caption + ' Up');
    ErrorList.Add('Total ES in the day: ' + ESTotalDown.Caption + ' Down, ' + ESTotalUp.Caption + ' Up');
    Clipboard.AsText := ErrorList.Text;
  except
    MiscString := 'Copy to clipboard failed';
    LogEvent(leMisc);
  end;
  ErrorList.Free;
end;

//Set number of days ago for ES chart
procedure Tmainform.DaysAgoEditChange(Sender: TObject);
begin
  ESChart.DaysAgo := DaysAgoEdit.Value;
  ESChart.SetESDate(Today - DaysAgoEdit.Value);
end;

// ***** XML SAVE AND RESTORE *****

//Save graph data as XML file
procedure TMainform.SaveXML(Chart: TLinechart; XMLName, Filename: string);
var
  XMLList: TStringList;
  N, Digits: Integer;
  S: string;
begin
  if XMLName = 'ConSpeed' then Digits := 0 else Digits := 1;
  XMLList := TStringList.Create;
  try
    XMLList.Add('<?xml version="1.0" encoding="utf-8"?>');
    XMLList.Add('<' + XMLName + '>');
    for N := 0 to Chart.NumSamples - 1 do
    begin
      S := '  <Sample Number="' + IntToStr(N) + '">';
      XMLList.Add(S);
      S := '    <Time>' + DateTimeToStr(Chart.SampleArray[N].XValue) + '</Time>';
      XMLList.Add(S);
      S := '    <Downstream>' + FloatToStrF(Chart.SampleArray[N].DownYValue, ffFixed, 1, Digits) + '</Downstream>';
      XMLList.Add(S);
      S := '    <Upstream>' + FloatToStrF(Chart.SampleArray[N].UpYValue, ffFixed, 1, Digits) + '</Upstream>';
      XMLList.Add(S);
      if Chart.SampleArray[N].DrawBlank then S := 'True' else S := 'False';
      S := '    <DrawBlank>' + S + '</DrawBlank>';
      XMLList.Add(S);
      XMLList.Add('  </Sample>');
    end;
    XMLList.Add('</' + XMLName + '>');
    XMLList.SaveToFile(Filename);
  except
    on E: Exception do LogEvent(leXMLsavefailed);
  end;
  XMLList.Free;
end;

//Select a saved XML file to restore
procedure Tmainform.SelectXMLButtonClick(Sender: TObject);
begin
  with OpenDialog1 do
  begin
    InitialDir := SnapDirEdit.Text;
    if Execute then XMLFileEdit.Text := Filename;
  end;
end;

//Restore a graph from a saved XML file
procedure Tmainform.RestoreGraphButtonClick(Sender: TObject);
var
  S, Dir, SaveFile: string;
  Success: Boolean;
begin
  Success := False;
  if Timer1.Enabled then  //Pause the recording and save the relevant data
  begin
    WasRecording := True;
    StopButtonClick(PauseButton);
    if Collecting then  //Wait for sampling to complete
    begin
      Timer7.Enabled := True;
      Exit;
    end;
  end;
  Dir := IncludeTrailingPathDelimiter(SnapshotsPath);
  S := XMLFileEdit.Text;
  if Pos('SNRM', S) > 0 then
  begin
    SaveFile := Dir + 'SNRM-temp.xml';
    SaveXML(MarginChart, 'SNRM', SaveFile);
    XMLSave.SNRM := True;
    XMLSave.SNRMFile := SaveFile;
    MarginChart.ClearArray;
    MarginChart.MaxYValue := 10;
    MarginChart.Autoscale := True;
    if not LoadXML(MarginChart, XMLFileEdit.Text) then ShowMessage('Unable to restore data')
    else Success := True;
  end
  else if Pos('ConSpeed', S) > 0 then
  begin
    SaveFile := Dir + 'ConSpeed-temp.xml';
    SaveXML(Speedchart1, 'ConSpeed', SaveFile);
    XMLSave.ConSpeed := True;
    XMLSave.ConSpeedFile := SaveFile;
    Speedchart1.ClearArray;
    Speedchart1.MaxYValue := 10000;
    if not LoadXML(Speedchart1, XMLFileEdit.Text) then ShowMessage('Unable to restore data')
    else Success := True;
  end
  else if Pos('CRC', S) > 0 then
  begin
    SaveFile := Dir + 'CRC-temp.xml';
    SaveXML(CRCChart, 'CRC', SaveFile);
    XMLSave.CRC := True;
    XMLSave.CRCFile := SaveFile;
    CRCChart.ClearArray;
    CRCChart.MaxYValue := 10;
    if not LoadXML(CRCChart, XMLFileEdit.Text) then ShowMessage('Unable to restore data')
    else Success := True;
  end
  else if Pos('FEC', S) > 0 then
  begin
    SaveFile := Dir + 'FEC-temp.xml';
    SaveXML(FECChart, 'FEC', SaveFile);
    XMLSave.FEC := True;
    XMLSave.FECFile := SaveFile;
    FECChart.ClearArray;
    FECChart.MaxYValue := 10;
    if not LoadXML(FECChart, XMLFileEdit.Text) then ShowMessage('Unable to restore data')
    else Success := True;
  end
  else if Pos('Bitswaps', S) > 0 then
  begin
    SaveFile := Dir + 'Bitswaps-temp.xml';
    SaveXML(SwapsperminChart, 'Bitswaps', SaveFile);
    XMLSave.Bitswaps := True;
    XMLSave.BitswapsFile := SaveFile;
    SwapsperminChart.ClearArray;
    SwapsperminChart.MaxYValue := 10;
    if not LoadXML(SwapsperminChart, XMLFileEdit.Text) then ShowMessage('Unable to restore data')
    else Success := True;
  end
  else begin
    ShowMessage('Invalid XML file selected');
    LogEvent(leNographrestore);
    Exit;
  end;
  if Success then
  begin
    LogEvent(leGraphrestore);
    StartButton.Enabled := False;
    NormalButton.Enabled := True;
  end
  else LogEvent(leNographrestore);
end;

//Load XML data into graph
function TMainform.LoadXML(Graph: TLinechart; Filename: string): Boolean;
var
  XMLList: TStringList;
  N, N1, N2, Len: Integer;
  S, Data: string;
  XValue: TDateTime;
  UpYValue, DownYValue: Single;
  DrawBlank: Boolean;

  procedure ExtractData;
  begin
    S := XMLList[N];
    N1 := Pos('>', S) + 1;
    N2 := N1;
    repeat
      Inc(N2);
    until S[N2] = '<';
    Data := S[N1..N2 - 1];
  end;

begin
  Result := False;
  XMLList := TStringList.Create;
  try
    XMLList.LoadFromFile(Filename);
    Len := XMLList.Count;
    N := 3;
    repeat
      ExtractData;
      XValue := StrToDateTime(Data);
      if N = 3 then
      begin
        if Graph = MarginChart then XMLSave.SNRMStart := XValue
        else if Graph = Speedchart1 then XMLSave.ConSpeedStart := XValue
        else if Graph = CRCChart then XMLSave.CRCStart := XValue
        else if Graph = FECChart then XMLSave.FECStart := XValue
        else if Graph = SwapsperminChart then XMLSave.BitswapsStart := XValue;
      end;
      Inc(N);
      ExtractData;
      DownYValue := StrToFloat(Data);
      Inc(N);
      ExtractData;
      UpYValue := StrToFloat(Data);
      Inc(N);
      ExtractData;
      DrawBlank := (Data = 'True');
      Graph.AddXY(XValue, UpYValue, DownYValue, DrawBlank);
      N := N + 3;
      if N >= Len then
      begin
        if Graph = MarginChart then XMLSave.SNRMEnd := XValue
        else if Graph = Speedchart1 then XMLSave.ConSpeedEnd := XValue
        else if Graph = CRCChart then XMLSave.CRCEnd := XValue
        else if Graph = FECChart then XMLSave.FECEnd := XValue
        else if Graph = SwapsperminChart then XMLSave.BitswapsEnd := XValue;
      end;
    until N >= Len;
    Result := True;
    RestoredLabel.Caption := 'Graph restored';
  except
    on E: Exception do LogEvent(leXMLoadfailed);
  end;
  XMLList.Free;
  if not Result then
  begin
    RestoredLabel.Caption := 'Restore failed';
    LogEvent(leXMLoadfailed);
  end;
  RestoredLabel.Show;
end;

//Return to normal operation after restoring saved graph data
procedure Tmainform.NormalButtonClick(Sender: TObject);
begin
  RestoredLabel.Hide;
  try
    if XMLSave.SNRM then
    begin
      MarginChart.ClearArray;
      MarginChart.MaxYValue := 10;
      MarginChart.Autoscale := True;
      LoadXML(MarginChart, XMLSave.SNRMFile);
      Sysutils.Deletefile(XMLSave.SNRMFile);
      XMLSave.SNRM := False;
    end;
    if XMLSave.ConSpeed then
    begin
      Speedchart1.ClearArray;
      Speedchart1.MaxYValue := 10000;
      Speedchart1.Autoscale := True;
      LoadXML(Speedchart1, XMLSave.ConSpeedFile);
      Sysutils.Deletefile(XMLSave.ConSpeedFile);
      XMLSave.ConSpeed := False;
    end;
    if XMLSave.CRC then
    begin
      CRCChart.ClearArray;
      CRCChart.MaxYValue := 10;
      CRCChart.Autoscale := True;
      LoadXML(CRCChart, XMLSave.CRCFile);
      Sysutils.Deletefile(XMLSave.CRCFile);
      XMLSave.CRC := False;
    end;
    if XMLSave.FEC then
    begin
      FECChart.ClearArray;
      FECChart.MaxYValue := 10;
      FECChart.Autoscale := True;
      LoadXML(FECChart, XMLSave.FECFile);
      Sysutils.Deletefile(XMLSave.FECFile);
      XMLSave.FEC := False;
    end;
    if XMLSave.Bitswaps then
    begin
      SwapsperminChart.ClearArray;
      SwapsperminChart.MaxYValue := 10;
      SwapsperminChart.Autoscale := True;
      LoadXML(SwapsperminChart, XMLSave.BitswapsFile);
      Sysutils.Deletefile(XMLSave.BitswapsFile);
      XMLSave.Bitswaps := False;
    end;
  except
    on E: Exception do
    begin
      NormalLabel.Caption := 'Failed to restore normal operation';
      NormalLabel.Show;
      RestoredLabel.Hide;
      LogEvent(leNoreturntonormal);
      Exit;
    end;
  end;
  NormalLabel.Caption := 'Normal operation restored';
  NormalLabel.Show;
  RestoredLabel.Hide;
  NormalButton.Enabled := False;
  LogEvent(leReturntonormal);
  if WasRecording then
  begin
    StartButtonClick(Self);
    WasRecording := False;
  end;
end;

//Wait for sampling to complete
procedure Tmainform.Timer7Timer(Sender: TObject);
begin
  if not Collecting then   //Sampling is complete
  begin
    Timer7.Enabled := False;
    RestoreGraphButtonClick(Self);
  end;
end;


//**** TRAFFIC ****

//Process change of Traffic command selection
procedure Tmainform.CommandGroupSelectionChanged(Sender: TObject);
begin
  if (ThomBox.Checked) or (CommandGroup.ItemIndex = 3) then
  begin
    CommandGroup.ItemIndex := 3;
    InterfaceGroup.Enabled := False;
    OtherInterfaceLabel.Enabled := False;
    OtherInterfaceEdit.Enabled := False;
  end
  else begin
    InterfaceGroup.Enabled := True;
    OtherInterfaceLabel.Enabled := True;
    OtherInterfaceEdit.Enabled := True;
  end;
end;

//Process change of Traffic interface selection
procedure Tmainform.InterfaceGroupSelectionChanged(Sender: TObject);
var
  Other: Boolean;
begin
  Other := (InterfaceGroup.Items[InterfaceGroup.ItemIndex] = 'Other...');
  OtherInterfaceLabel.Visible := Other;
  OtherInterfaceEdit.Visible := Other;
end;

//Toggle Traffic monitor
procedure Tmainform.MonitorTrafficBoxChange(Sender: TObject);
begin
  TelnetUnit1.DoTraffic := MonitorTrafficBox.Checked;
  SetTabs;
end;

//Toggle up/down traffic per minute on/off
procedure Tmainform.TUpBoxChange(Sender: TObject);
begin
  TrafficperminChart.PlotUpstream := TUpBox.Checked;
  TrafficperminChart.PlotDownstream := TDownBox.Checked;
end;

//Disallow manual editing of spinedit control
procedure Tmainform.DayEditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key <> VK_TAB then Key := 0;
end;

//Set up popup timer
procedure Tmainform.TrafficChartMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  PopupTimer.Enabled := False;
  PopupTimer.Enabled := True;
  PopupForm.Hide;
end;

//Display details of traffic data when mouse is hovered over the chart
procedure Tmainform.PopupTimerTimer(Sender: TObject);
var
  APoint: TPoint;
  X, Y: Integer;
  S: string;
  N, Month1: Byte;
  Year: Word;
  UpTraf, DownTraf: Int64;
begin
  PopupForm.Enabled := False;
  APoint := TrafficChart.ScreenToClient(Mouse.CursorPos);
  X := APoint.X;
  Y := APoint.Y;
  if (X < TrafficChart.XOffset) or (X >= TrafficChart.Width) or (Y < 1) or (Y >= TrafficChart.Height - TrafficChart.YOffset) then
  begin
    PopupTimer.Enabled := False;
    PopupForm.Hide;
    Exit;
  end;
  with PopupForm do
  begin
    Left := X + Mainform.Left + 15;
    if Left > Screen.Width - Width then Left := Left - Width - 20;
    Top := Y + Height + Mainform.Top - 20;
    PopupMemo.Clear;
    N := 0;
    repeat
      Inc(N);
    until (N > 30) or (TrafficChart.XPosition[N + 1] > X);
    if MonthlyButton.Checked and (N > 12) then N := 12;
    if DailyButton.Checked then
    begin
      S := IntToStr(N) + ' ' + FormatDateTime('mmm yyyy', Now);
      PopupMemo.Lines.Add(S);
      TrafficChart.GetDailyTraffic(0, N, UpTraf, DownTraf);
      S := 'Upstream: ' + IntToStr(UpTraf) + ' MB (' + IntToStr(Round( UpTraf / 1.024 / 1.024)) + ' MiB)';
      PopupMemo.Lines.Add(S);
      S := 'Downstream: ' + IntToStr(DownTraf) + ' MB (' + IntToStr(Round( DownTraf / 1.024 / 1.024)) + ' MiB)';
      PopupMemo.Lines.Add(S);
    end
    else if MonthlyButton.Checked then
    begin
      Month1 := Ord(TrafficChart.FirstMonth) + N;
      Year := TrafficChart.FirstYear;
      if Month1 > 12 then
      begin
        Month1 := Month1 - 12;
        Inc(Year);
      end;
      S := FormatSettings.ShortMonthNames[Month1] + ' ' + IntToStr(Year);
      PopupMemo.Lines.Add(S);
      TrafficChart.GetMonthlyTraffic(N, UpTraf, DownTraf);
      S := 'Upstream: ' + IntToStr(UpTraf) + ' MB (' + IntToStr(Round( UpTraf / 1.024 / 1.024)) + ' MiB)';
      PopupMemo.Lines.Add(S);
      S := 'Downstream: ' + IntToStr(DownTraf) + ' MB (' + IntToStr(Round( DownTraf / 1.024 / 1.024)) + ' MiB)';
      PopupMemo.Lines.Add(S);
    end
    else Exit;
    PopupMemo.SelStart := 0;
    Show;
  end;
end;

procedure Tmainform.TrafficChartMouseLeave(Sender: TObject);
begin
  PopupForm.Hide;
  PopupTimer.Enabled := False;
  PopupTimer.Enabled := True;
end;

// ***** PROGRAM UPDATES *****

//Check for later version

{Format of VERSION file held at http://<website address>/files/:

Version number
32-bit Linux filename
64-bit Linux filename
32-bit Windows filename
Raspberry Pi filename
32-bit BSD filename
64-bit BSD filename
}

function Tmainform.CheckVersion(DoNow: Boolean): TVersionResult;
var
  HTTPClient: THTTPSend;
  OldMajor, NewMajor, OldMinor, NewMinor, OldSub, NewSub: Word;

  procedure ExtractParts(InVersion: string; var Maj, Min, Sub: Word);
  var
    N, Len: Byte;
    S: string;
    C: Char;
  begin
    Maj := 0;
    Min := 0;
    Sub := 0;
    N := 1;
    S := '';
    InVersion := TrimLeft(InVersion);
    Len := Length(InVersion);
    while N <= Len do  //Get major version
    begin
      C := InVersion[N];
      if (C in ['0'..'9']) then
      begin
        S := S + C;
        if N = Len then
        begin
          Maj := StrToInt(S);
          Break;
        end;
      end
      else begin
        if S > '' then Maj := StrToInt(S) else Maj := 0;
        S := '';
        while N < Len do   //Get minor version
        begin
          Inc(N);
          C := InVersion[N];
          if (C in ['0'..'9']) then
          begin
            S := S + C;
            if N = Len then
            begin
              Min := StrToInt(S);
              Break;
            end;
          end
          else begin
            if S > '' then Min := StrToInt(S) else Min := 0;
            S := '';
            while N < Len do   //Get subversion
            begin
              Inc(N);
              C := InVersion[N];
              if (C in ['0'..'9']) then
              begin
                S := S + C;
                if N = Len then
                begin
                  Sub := StrToInt(S);
                  Break;
                end;
              end
              else begin
                if S > '' then Sub := StrToInt(S) else Sub := 0;
                N := Len + 1;   //Just to force breaks from all loops
              end;
            end;
          end;
        end;
      end;
      Inc(N);
    end;
  end;

begin
  Result := vrNoResult;
  if (not DoNow) and (not CheckupdateBox.Checked) then Exit;
  LastUpdateCheck := Now;
  HTTPClient := THTTPSend.Create;
  HTTPClient.Timeout := 5000;
  try
    if HTTPClient.HTTPMethod('GET', FilesURL + 'VERSION') then
    begin
      UpdateList.LoadFromStream(HTTPClient.Document);
    end;
  except
    on E: Exception do
    begin
      Exit;
    end;
  end;
  HTTPClient.Free;
  if UpdateList.Count < 7 then Exit;  //No result
  ExtractParts(Version, OldMajor, OldMinor, OldSub);      //Extract major, minor and sub version numbers
  ExtractParts(UpdateList[0], NewMajor, NewMinor, NewSub);
  if (NewMajor > OldMajor)
  or ((NewMajor = OldMajor) and (NewMinor > OldMinor))
  or ((NewMajor = OldMajor) and (NewMinor = OldMinor) and (NewSub > OldSub)) then  //Upgrade is available
  begin
    HTTPClient := THTTPSend.Create;
    try
      if HTTPClient.HTTPMethod('GET', FilesURL + 'CHANGELOG') then
        Changelog.Lines.LoadFromStream(HTTPClient.Document);
    except
      on E: Exception do Changelog.Text := 'Unable to download';
    end;
    HTTPClient.Free;
    UpgradeTab.TabVisible := True;
    PageControl1.ActivePage := UpgradeTab;
    WindowState := wsNormal;
    Result := vrUpdateYes;
  end
  else Result := vrUpdateNo;
end;

//Check for program update
procedure Tmainform.CheckUpdateButtonClick(Sender: TObject);
var
  VR: TVersionResult;
begin
  VR := CheckVersion(True);
  VersionResult.Hide;
  NoResult.Hide;
  if VR = vrUpdateNo then VersionResult.Show
  else if VR = vrNoResult then NoResult.Show;
end;

//Do nothing in response to update notification
procedure Tmainform.DonothingButtonClick(Sender: TObject);
begin
  CheckConfig;
  UpgradeTab.TabVisible := False;
end;

//Toggle the automatic update check
procedure Tmainform.DontAskButtonChange(Sender: TObject);
begin
  CheckUpdateBox.Checked := not DontAskButton.Checked;
end;

//Download an updated version
procedure Tmainform.DownloadButtonClick(Sender: TObject);
begin
  DownloadYes;
end;

procedure Tmainform.DownloadYes;
var
  Filename, DownloadDir, S: string;
  HTTPClient: THTTPSend;
  HTTPResult, DownloadOK: Boolean;
  RetryAttempt: Word;
const
  MaxRetries = 3;
begin
  {$IFDEF Windows}        //Windows
  Filename := UpdateList[3];
  {$ELSE}
    {$IFDEF Cpu32}      //32 bit OS
      {$IFDEF CpuArm}   //RPi
      Filename := UpdateList[4];
      {$ELSE}
        {$IFDEF Linux}  //Linux 32 bit
        Filename := UpdateList[1];
        {$ELSE}
          {$IFDEF Unix} //BSD 32 bit
          Filename := UpdateList[5];
          {$ENDIF}
        {$ENDIF}
      {$ENDIF}
    {$ELSE}             //64 bit OS
      {$IFDEF Linux}    //Linux 64 bit
      Filename := UpdateList[2];
      {$ELSE}
        {$IFDEF Unix}   //BSD 64 bit
        Filename := UpdateList[6];
        {$ENDIF}
      {$ENDIF}
    {$ENDIF}
  {$ENDIF}

  DownloadOK := False;
  DownloadDir := IncludeTrailingPathDelimiter(DownloadDirEdit.Directory);
  DownloadingLabel.Show;
  Application.ProcessMessages;
  HTTPClient := THTTPSend.Create;
  try
    RetryAttempt := 1;
    try
      HTTPResult := HTTPClient.HTTPMethod('GET', FilesURL + Filename);
      while (HTTPResult = False) and (RetryAttempt < MaxRetries) do
      begin
        sleep(500*RetryAttempt);
        HTTPResult:=HTTPClient.HTTPMethod('GET', FilesURL + Filename);
        Inc(RetryAttempt);
      end;
      case HTTPClient.ResultCode of
        100..299: begin
                    HTTPClient.Document.SaveToFile(DownloadDir + Filename);
                    DownloadingLabel.Hide;
                    S := 'Download succeeded. File saved to ' + DownloadDir + Filename;
                    EventList.Lines.Add(S);
                    ShowMessage(S);
                    DownloadOK := True;
                    Version := UpdateList[0];
                  end;
      else
        DownloadingLabel.Hide;
      end;
    except
      on E: Exception do DownloadingLabel.Hide;
    end;
  except
    on E: Exception do LogEvent(leDownloadFailed);
  end;
  HTTPClient.Free;
  if DownloadOK then LogEvent(leDownloadSucceeded)
  else LogEvent(leDownloadFailed);
  CheckConfig;
  UpgradeTab.TabVisible := False;
end;

//Change frequency of program update checks
procedure Tmainform.FrequencyBoxChange(Sender: TObject);
begin
  if FrequencyBox.Value = 1 then Label73.Caption := 'day' else Label73.caption := 'days';
end;

// ***** SNAPSHOTS *****

//Toggle snapshot timing same as time per page
procedure Tmainform.SameAsBoxChange(Sender: TObject);
begin
  TakeEveryLabel.Enabled := not SameAsBox.Checked;
  EveryHoursLabel.Enabled := not SameAsBox.Checked;
  TimepersnapshotEdit.Enabled := not SameAsBox.Checked;
  if SameAsBox.Checked then TimepersnapshotEdit.Value := Hoursperpage;
end;

//Process change of time per snapshot
procedure Tmainform.TimepersnapshotEditChange(Sender: TObject);
begin
  Hourspersnapshot := TimepersnapshotEdit.Value;
  Minutespersnapshot := Hourspersnapshot * 60;
end;

//Process change of snapshot start time mode
procedure Tmainform.StartofRecordingChange(Sender: TObject);
begin
  SnapshotHours.Enabled := FirstAtTime.Checked;
  SnapshotMinutes.Enabled := FirstAtTime.Checked;
  hrLabel.Enabled := FirstAtTime.Checked;
  minLabel.Enabled := FirstAtTime.Checked;
  DoSnapTimes;
  if StartofRecording.Checked then LastSnapshot := StartTime;
end;

//Process change of snapshot start time
procedure Tmainform.SnapshotHoursChange(Sender: TObject);
begin
  DoSnapTimes;
end;

//Calculate snapshot times
procedure TMainform.DoSnapTimes;
var
  N: Byte;
  ATime: TDateTime;
  Temp: Double;
begin
  if FirstAtTime.Checked then   //Snapshot times defined by user
  begin
    SnapshotTimes[1] := EncodeTime(SnapshotHours.Value, SnapshotMinutes.Value, 0, 0);
    for N := 2 to 24 do
    begin
      if TimeperSnapshotEdit.Value = 24 then ATime := 1
      else begin
        Temp := SnapshotTimes[N - 1] + EncodeTime(TimeperSnapshotEdit.Value, 0, 0, 0);
        if Temp >= EncodeTime(23, 59, 59, 999) then ATime := 1 else ATime := Temp;
      end;
      SnapshotTimes[N] := ATime;
    end;
    for N := 1 to 24 do        //Mark earlier schedules as done
    begin
      SnapshotsDone[N] := ((Time - EncodeTime(0, 1, 0, 0)) > SnapshotTimes[N]);
    end;
    SnapshotDate := Date;
  end;
end;

//DailyBox and CategoryBox can't be checked at the same time
procedure Tmainform.DailyBoxChange(Sender: TObject);
begin
  if Sender = DailyBox then
    if DailyBox.Checked then CategoryBox.Checked := False;
  if Sender = CategoryBox then
    if CategoryBox.Checked then DailyBox.Checked := False;
end;

//SnapallBox and SnapallConfiguredBox can't be checked at the same time
procedure Tmainform.SnapallBoxChange(Sender: TObject);
begin
  if Sender = SnapallBox then
    if SnapallBox.Checked then SnapallConfiguredBox.Checked := False;
  if Sender = SnapallConfiguredBox then
    if SnapallConfiguredBox.Checked then SnapallBox.Checked := False;
end;

//AllActiveWeb and AllConfiguredWeb can't be checked at the same time
procedure TMainform.AllActiveWebChange(Sender: TObject);
begin
  if Sender = AllActiveWeb then
    if AllActiveWeb.Checked then AllConfiguredWeb.Checked := False;
  if Sender = AllConfiguredWeb then
    if AllConfiguredWeb.Checked then AllActiveWeb.Checked := False;
end;

//Test read/write status of snapshot directory
function Tmainform.Testwrite(Dir: string): Boolean;
var
  F: file of Byte;
begin
  Result := True;
  AssignFile(F, Dir + 'test');
  try
    Rewrite(F);
    CloseFile(F);
    Erase(F);
  except
    on E: Exception do
    begin
      LogEvent(leReadonlydir);
      Result := False;
    end;
  end;
end;

//Take a snapshot of all active graphs
procedure Tmainform.SnapallButtonClick(Sender: TObject);
begin
  MSnapButtonClick(SnapallBox);
end;

//Determine snapshot prefix and suffix
procedure Tmainform.PrefixSuffix(var Prefix, Suffix, Snapdir: string; Extension, Category: string);
var
  S, Thisday, CatDir, Dir: string;
begin
  Dir := IncludeTrailingPathDelimiter(SnapshotsPath);
  if DailyBox.Checked then  //Save snapshots in daily folders
  begin
    Thisday := FormatDateTime('yyyy"-"mm"-"dd', Now);
    Dir := IncludeTrailingPathDelimiter(Dir + Thisday);
    if not ForceDirectories(Dir) then
    begin
      MiscString := 'Unable to create daily folder ' + Dir;
      LogEvent(leMisc);
      Dir := IncludeTrailingPathDelimiter(SnapshotsPath);
    end;
    SnapDir := Dir;
  end
  else if CategoryBox.Checked then
  begin
    CatDir := IncludeTrailingPathDelimiter(Dir + Category);
    if not ForceDirectories(CatDir) then
    begin
      MiscString := 'Unable to create category folder ' + CatDir;
      LogEvent(leMisc);
      CatDir := Dir;
    end;
    SnapDir := CatDir;
  end
  else SnapDir := Dir;
  S := FormatDateTime('yyyy"-"mm"-"dd"-"hh"."nn"."ss', Now);
  if NameOrder.Checked then
  begin
    Prefix := SnapDir;
    Suffix := '-' + S + Extension;
  end
  else begin
    Prefix := SnapDir + S + '-';
    Suffix := Extension;
  end;
end;

//Save snapshots
procedure Tmainform.MsnapbuttonClick(Sender: TObject);
begin
  DoSnapshots(Sender, False);
end;

//Save snapshots plus "All active" and "All configured" webserver images
procedure TMainform.DoSnapshots(Sender: TObject; IsWebserver: Boolean);
var
  Prefix, Suffix, SnapDir, UploadDir, TimeStamp, S: string;
begin
  Prefix := '';
  Suffix := '';
  TimeStamp := FormatDateTime('yyyy"-"mm"-"dd"-"hh"."nn"."ss', Now);
  SnapDir := IncludeTrailingPathDelimiter(SnapshotsPath);
  if not TestOK then if not TestWrite(SnapDir) then
  begin
    MiscString := 'Unable to write to snapshots directory';
    LogEvent(leMisc);
    Exit;
  end;
  if DoUploadSS.Checked then
  begin
    UploadDir := IncludeTrailingPathDelimiter(SnapDir + 'Uploads');
    if not ForceDirectories(UploadDir) then
    begin
      MiscString := 'Unable to create Uploads directory ' + UploadDir + '. Snapshot uploads disabled.';
      LogEvent(leMisc);
      UploadDir := IncludeTrailingPathDelimiter(SnapshotsPath);
      DoUploadSS.Checked := False;
    end;
  end;
  if not CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', '');
  try
    if Sender = ESnapButton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'ErroredSeconds');
      if not ESChart.SaveToPNGFile(Prefix + 'ES' + Suffix) then
      begin
        GraphType := 'ES per hour';
        LogEvent(leSnapshotFailed);
      end;
      if DoUploadSS.Checked then
      begin
        if not ESChart.SaveToPNGFile(UploadDir + 'ES-' + TimeStamp + '.png') then
        begin
          GraphType := 'ES per hour';
          LogEvent(leSnapshotFailed);
        end;
      end;
    end
    else if Sender = MsnapButton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'SNRM');
      if not Marginchart.SaveToPNGFile(Prefix + 'SNRMargin' + Suffix) then
      begin
        GraphType := 'SNR margin';
        LogEvent(leSnapshotFailed);
      end;
      if DoUploadSS.Checked then
      begin
        if not MarginChart.SaveToPNGFile(UploadDir + 'SNRM-' + TimeStamp + '.png') then
        begin
          GraphType := 'SNR margin';
          LogEvent(leSnapshotFailed);
        end;
      end;
    end
    else if Sender = OsnapButton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'OHFErr');
      if not OHFErrchart.SaveToPNGFile(Prefix + 'OHFErr' + Suffix) then
      begin
        GraphType := 'OHFErr';
        LogEvent(leSnapshotFailed);
      end;
      if DoUploadSS.Checked then
      begin
        if not OHFErrChart.SaveToPNGFile(UploadDir + 'OHFErr-' + TimeStamp + '.png') then
        begin
          GraphType := 'OHFErr';
          LogEvent(leSnapshotFailed);
        end;
      end;
    end
    else if Sender = RsnapButton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'RSCorr');
      if not RSCorrchart.SaveToPNGFile(Prefix + 'RSCorr' + Suffix) then
      begin
        GraphType := 'RSCorr';
        LogEvent(leSnapshotFailed);
      end;
      if DoUploadSS.Checked then
      begin
        if not RSCorrChart.SaveToPNGFile(UploadDir + 'RSCorr-' + TimeStamp + '.png') then
        begin
          GraphType := 'RSCorr';
          LogEvent(leSnapshotFailed);
        end;
      end;
    end
    else if Sender = SpBSnapButtonUp then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'SNRMperband');
      if not SNRMpbChartup.SaveToPNGFile(Prefix + 'SNRMperbandUpstream' + Suffix) then
      begin
        GraphType := 'SNRM per band upstream';
        LogEvent(leSnapshotFailed);
      end;
      if DoUploadSS.Checked then
      begin
        if not SNRMpbChartup.SaveToPNGFile(UploadDir + 'SNRMperbandUpstream-' + TimeStamp + '.png') then
        begin
          GraphType := 'SNRM per band upstream';
          LogEvent(leSnapshotFailed);
        end;
      end;
    end
    else if Sender = SpBSnapButtonDown then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'SNRMperband');
      if not SNRMpbChartdown.SaveToPNGFile(Prefix + 'SNRMperbandDownstream' + Suffix) then
      begin
        GraphType := 'SNRM per band downstream';
        LogEvent(leSnapshotFailed);
      end;
      if DoUploadSS.Checked then
      begin
        if not SNRMpbChartdown.SaveToPNGFile(UploadDir + 'SNRMperbandDownstream-' + TimeStamp + '.png') then
        begin
          GraphType := 'SNRM per band downstream';
          LogEvent(leSnapshotFailed);
        end;
      end;
    end
    else if Sender = SSnapbutton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'ConnSpeed');
      if not Speedchart1.SaveToPNGFile(Prefix + 'Speed' + Suffix) then
      begin
        GraphType := 'Connection speed';
        LogEvent(leSnapshotFailed);
      end;
      if DoUploadSS.Checked then
      begin
        if not SpeedChart1.SaveToPNGFile(UploadDir + 'Speed-' + TimeStamp + '.png') then
        begin
          GraphType := 'Connection speed';
          LogEvent(leSnapshotFailed);
        end;
      end;
    end
    else if Sender = USnapbutton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'Traffic');
      if Trafficpermin.Checked then
      begin
        S := 'TrafficPerMin';
        GraphType := 'Traffic per minute';
      end
      else if DailyButton.Checked then
      begin
        S := 'DailyTraffic';
        GraphType := 'Daily traffic';
      end
      else begin
        S := 'MonthlyTraffic';
        GraphType := 'Monthly traffic';
      end;
      if Trafficpermin.Checked then
      begin
        if not TrafficperminChart.SaveToPNGFile(Prefix + S + Suffix) then LogEvent(leSnapshotFailed);
        if DoUploadSS.Checked then
          if not Trafficperminchart.SaveToPNGFile(UploadDir + 'TrafficPerMin-' + TimeStamp + '.png') then LogEvent(leSnapshotFailed);
      end
      else begin
        if not TrafficChart.SaveToPNGFile(Prefix + S + Suffix) then LogEvent(leSnapshotFailed);
        if DoUploadSS.Checked then
          if not TrafficChart.SaveToPNGFile(UploadDir + S +'-' + TimeStamp + '.png') then LogEvent(leSnapshotFailed);
      end;
    end
    else if Sender = BsnapButton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'Bitloading');
      if not Bitchart1.SaveToPNGFile(Prefix + 'Bitloading' + Suffix) then
      begin
        GraphType := 'Bitloading';
        LogEvent(leSnapshotFailed);
      end;
      if DoUploadSS.Checked then
      begin
        if not BitChart1.SaveToPNGFile(UploadDir + 'Bitloading-' + TimeStamp + '.png') then
        begin
          GraphType := 'Bitloading';
          LogEvent(leSnapshotFailed);
        end;
      end;
    end
    else if Sender = SWSnapbutton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'Bitswaps');
      if not Bitswapchart.SaveToPNGFile(Prefix + 'Bitswap' + Suffix) then
      begin
        GraphType := 'Bitswaps';
        LogEvent(leSnapshotFailed);
      end;
      if DoUploadSS.Checked then
      begin
        if not BitswapChart.SaveToPNGFile(UploadDir + 'Bitswap-' + TimeStamp + '.png') then
        begin
          GraphType := 'Bitswaps';
          LogEvent(leSnapshotFailed);
        end;
      end;
    end
    else if Sender = PMSnapButton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'SwapsPerMinute');
      if not Swapsperminchart.SaveToPNGFile(Prefix + 'Swapspermin' + Suffix) then
      begin
        GraphType := 'Bitswaps per min';
        LogEvent(leSnapshotFailed);
      end;
      if DoUploadSS.Checked then
      begin
        if not SwapsperminChart.SaveToPNGFile(UploadDir + 'Swapspermin-' + TimeStamp + '.png') then
        begin
          GraphType := 'Bitswaps per min';
          LogEvent(leSnapshotFailed);
        end;
      end;
    end
    else if Sender = CSnapButton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'CRC');
      if not CRCchart.SaveToPNGFile(Prefix + 'CRC' + Suffix) then
      begin
        GraphType := 'CRC';
        LogEvent(leSnapshotFailed);
      end;
      if DoUploadSS.Checked then
      begin
        if not CRCChart.SaveToPNGFile(UploadDir + 'CRC-' + TimeStamp + '.png') then
        begin
          GraphType := 'CRC';
          LogEvent(leSnapshotFailed);
        end;
      end;
    end
    else if Sender = SESSnapButton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'SES');
      if not SESchart.SaveToPNGFile(Prefix + 'SES' + Suffix) then
      begin
        GraphType := 'SES';
        LogEvent(leSnapshotFailed);
      end;
      if DoUploadSS.Checked then
      begin
        if not SESChart.SaveToPNGFile(UploadDir + 'SES-' + TimeStamp + '.png') then
        begin
          GraphType := 'SES';
          LogEvent(leSnapshotFailed);
        end;
      end;
    end
    else if Sender = FSnapButton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'FEC');
      if not FECchart.SaveToPNGFile(Prefix + 'FEC' + Suffix) then
      begin
        GraphType := 'FEC';
        LogEvent(leSnapshotFailed);
      end;
      if DoUploadSS.Checked then
      begin
        if not FECChart.SaveToPNGFile(UploadDir + 'FEC-' + TimeStamp + '.png') then
        begin
          GraphType := 'FEC';
          LogEvent(leSnapshotFailed);
        end;
      end;
    end
    else if Sender = QSnapButton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'QLN');
      if not QLNchart.SaveToPNGFile(Prefix + 'QLN' + Suffix) then
      begin
        GraphType := 'QLN';
        LogEvent(leSnapshotFailed);
      end;
      if DoUploadSS.Checked then
      begin
        if not QLNChart.SaveToPNGFile(UploadDir + 'QLN-' + TimeStamp + '.png') then
        begin
          GraphType := 'QLN';
          LogEvent(leSnapshotFailed);
        end;
      end;
    end
    else if Sender = HSnapButton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'HLog');
      if not HLogchart.SaveToPNGFile(Prefix + 'HLog' + Suffix) then
      begin
        GraphType := 'HLog';
        LogEvent(leSnapshotFailed);
      end;
      if DoUploadSS.Checked then
      begin
        if not HLogChart.SaveToPNGFile(UploadDir + 'HLog-' + TimeStamp + '.png') then
        begin
          GraphType := 'HLog';
          LogEvent(leSnapshotFailed);
        end;
      end;
    end
    else if Sender = SNRSnapbutton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'SNRperTone');
      if not SNRchart.SaveToPNGFile(Prefix + 'SNRpertone' + Suffix) then
      begin
        GraphType := 'SNR';
        LogEvent(leSnapshotFailed);
      end;
      if DoUploadSS.Checked then
      begin
        if not SNRChart.SaveToPNGFile(UploadDir + 'SNRpertone-' + TimeStamp + '.png') then
        begin
          GraphType := 'SNR';
          LogEvent(leSnapshotFailed);
        end;
      end;
    end
    else if (Sender = SnapallBox) or (Sender = SnapallConfiguredBox) then //Snap all active graphs or all snapshot-configured graphs
    begin
      if ((DoMargin.Checked) and (Sender = SnapallBox))
      or ((SNRsnap.Checked) and (Sender = SnapallConfiguredBox)) then
      begin
        if IsWebserver then MarginChart.SaveToPNGFile(WebserverPath + 'snrm.png')
        else begin
          if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'SNRM');
          Marginchart.SaveToPNGFile(Prefix + 'SNRMargin' + Suffix);
        end;
      end;
      if ((DoSNRMperband.Checked) and (Sender = SnapallBox))
      or ((SNRMpbsnap.Checked) and (Sender = SnapallConfiguredBox)) then
      begin
        if IsWebserver then
        begin
          SNRMpbChartup.SaveToPNGFile(WebserverPath + 'snrmpbup.png');
          SNRMpbChartdown.SaveToPNGFile(WebserverPath + 'snrmpbdown.png');
        end
        else begin
          if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'SNRMperband');
          SNRMpbChartup.SaveToPNGFile(Prefix + 'SNRMperbandUpstream' + Suffix);
          SNRMpbChartdown.SaveToPNGFile(Prefix + 'SNRMperbandDownstream' + Suffix);
        end;
      end;
      if ((DoSpeed.Checked) and (Sender = SnapallBox))
      or ((Speedsnap.Checked) and (Sender = SnapallConfiguredBox)) then
      begin
        if IsWebserver then Speedchart1.SaveToPNGFile(WebserverPath + 'connspeed.png')
        else begin
          if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'ConnSpeed');
          Speedchart1.SaveToPNGFile(Prefix + 'Speed' + Suffix);
        end;
      end;
      if ((DoCRC.Checked) and (Sender = SnapallBox))
      or ((CRCsnap.Checked) and (Sender = SnapallConfiguredBox)) then
      begin
        if IsWebserver then CRCchart.SaveToPNGFile(WebserverPath + 'crc.png')
        else begin
          if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'CRC');
          CRCchart.SaveToPNGFile(Prefix + 'CRC' + Suffix);
        end;
      end;
      if ((DoSES.Checked) and (Sender = SnapallBox))
      or ((SESsnap.Checked) and (Sender = SnapallConfiguredBox)) then
      begin
        if IsWebserver then SESchart.SaveToPNGFile(WebserverPath + 'ses.png')
        else begin
          if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'SES');
          SESchart.SaveToPNGFile(Prefix + 'SES' + Suffix);
        end;
      end;
      if ((DoFEC.Checked) and (Sender = SnapallBox))
      or ((FECsnap.Checked) and (Sender = SnapallConfiguredBox)) then
      begin
        if IsWebserver then FECchart.SaveToPNGFile(WebserverPath + 'fec.png')
        else begin
          if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'FEC');
          FECchart.SaveToPNGFile(Prefix + 'FEC' + Suffix);
        end;
      end;
      if ((DoSwapspermin.Checked) and (Sender = SnapallBox))
      or ((Swappmsnap.Checked) and (Sender = SnapallConfiguredBox)) then
      begin
        if IsWebserver then Swapsperminchart.SaveToPNGFile(WebserverPath + 'swapspermin.png')
        else begin
          if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'SwapsPerMinute');
          Swapsperminchart.SaveToPNGFile(Prefix + 'Swapspermin' + Suffix);
        end;
      end;
      if ((DoBitloading.Checked) and (Sender = SnapallBox))
      or ((Bitsnap.Checked) and (Sender = SnapallConfiguredBox)) then
      begin
        if IsWebserver then Bitchart1.SaveToPNGFile(WebserverPath + 'bitloading.png')
        else begin
          if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'Bitloading');
          Bitchart1.SaveToPNGFile(Prefix + 'Bitloading' + Suffix);
        end;
      end;
      if ((DoBitswaps.Checked) and (Sender = SnapallBox))
      or ((Swapptsnap.Checked) and (Sender = SnapallConfiguredBox)) then
      begin
        if IsWebserver then Bitswapchart.SaveToPNGFile(WebserverPath + 'bitswap.png')
        else begin
          if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'Bitswaps');
          Bitswapchart.SaveToPNGFile(Prefix + 'Bitswap' + Suffix);
        end;
      end;
      if ((DoQLN.Checked) and (Sender = SnapallBox))
      or ((QLNsnap.Checked) and (Sender = SnapallConfiguredBox)) then
      begin
        if IsWebserver then QLNchart.SaveToPNGFile(WebserverPath + 'qln.png')
        else begin
          if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'QLN');
          QLNchart.SaveToPNGFile(Prefix + 'QLN' + Suffix);
        end;
      end;
      if ((DoHLog.Checked) and (Sender = SnapallBox))
      or ((HLogsnap.Checked) and (Sender = SnapallConfiguredBox)) then
      begin
        if IsWebserver then HLogchart.SaveToPNGFile(WebserverPath + 'hlog.png')
        else begin
          if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'HLog');
          HLogchart.SaveToPNGFile(Prefix + 'HLog' + Suffix);
        end;
      end;
      if not SNRwithbitloading.Checked
      and (((DoSNRpertone.Checked) and (Sender = SnapallBox))
      or ((SNRptsnap.Checked) and (Sender = SnapallConfiguredBox))) then
      begin
        if IsWebserver then SNRchart.SaveToPNGFile(WebserverPath + 'snrpertone.png')
        else begin
          if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'SNRperTone');
          SNRchart.SaveToPNGFile(Prefix + 'SNRpertone' + Suffix);
        end;
      end;
      if ((DoES.Checked) and (Sender = SnapallBox))
      or ((ESsnap.Checked) and (Sender = SnapallConfiguredBox)) then
      begin
        if IsWebserver then ESchart.SaveToPNGFile(WebserverPath + 'es.png')
        else begin
          if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'ErroredSeconds');
          ESChart.SaveToPNGFile(Prefix + 'ES' + Suffix);
        end;
      end;
      if ((MonitorTrafficBox.Checked) and (Sender = SnapallBox)) then
      begin
        if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'Traffic');
        if Trafficpermin.Checked then S := 'TrafficPerMin'
        else if DailyButton.Checked then S := 'DailyTraffic'
        else S := 'MonthlyTraffic';
        if IsWebserver then TrafficChart.SaveToPNGFile(WebserverPath + 'traffic.png')
        else TrafficChart.SaveToPNGFile(Prefix + S + Suffix);
      end;
    end
    else if Sender = LEFTRSSnapButton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'LEFTRSpermin');
      if not rtxtxchart.SaveToPNGFile(Prefix + 'LEFTRSpermin' + Suffix) then
      begin
        GraphType := 'LEFTRS';
        LogEvent(leSnapshotFailed);
      end;
      if DoUploadSS.Checked then
      begin
        if not LEFTRSChart.SaveToPNGFile(UploadDir + 'LEFTRSpermin-' + TimeStamp + '.png') then
        begin
          GraphType := 'LEFTRS';
          LogEvent(leSnapshotFailed);
        end;
      end;
    end
    else if Sender = rtxtxSnapButton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'rtx_txPerMin');
      if not rtxtxchart.SaveToPNGFile(Prefix + 'rtx_txPerMin' + Suffix) then
      begin
        GraphType := 'rtx_tx';
        LogEvent(leSnapshotFailed);
      end;
      if DoUploadSS.Checked then
      begin
        if not rtxtxChart.SaveToPNGFile(UploadDir + 'rtx_txPerMin-' + TimeStamp + '.png') then
        begin
          GraphType := 'rtx_tx';
          LogEvent(leSnapshotFailed);
        end;
      end;
    end
    else if Sender = rtxcSnapButton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'rtx_cPerMin');
      if not rtxcchart.SaveToPNGFile(Prefix + 'rtx_cPerMin' + Suffix) then
      begin
        GraphType := 'rtx_c';
        LogEvent(leSnapshotFailed);
      end;
      if DoUploadSS.Checked then
      begin
        if not rtxcChart.SaveToPNGFile(UploadDir + 'rtx_cPerMin-' + TimeStamp + '.png') then
        begin
          GraphType := 'rtx_c';
          LogEvent(leSnapshotFailed);
        end;
      end;
    end
    else if Sender = rtxucSnapButton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.png', 'rtx_ucPerMin');
      if not rtxucchart.SaveToPNGFile(Prefix + 'rtx_ucPerMin' + Suffix) then
      begin
        GraphType := 'rtx_uc';
        LogEvent(leSnapshotFailed);
      end;
      if DoUploadSS.Checked then
      begin
        if not rtxucChart.SaveToPNGFile(UploadDir + 'rtx_ucPerMin-' + TimeStamp + '.png') then
        begin
          GraphType := 'rtx_uc';
          LogEvent(leSnapshotFailed);
        end;
      end;
    end
    else begin  //One of the XML save buttons was clicked
      if not CategoryBox.Checked then Suffix := Suffix[1..Length(Suffix) - 3] + 'xml';
      S := 'Saved ' + FormatDateTime('yyyy-mm-dd hh:nn:ss', Now);
      if Sender = SNRSaveButton then
      begin
        if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.xml', 'SNRM');
        SaveXML(MarginChart, 'SNRM', Prefix + 'SNRM' + Suffix);
        Save1.Caption := S;
        Save1.Visible := True;
      end
      else if Sender = SpeedSaveButton then
      begin
        if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.xml', 'ConSpeed');
        SaveXML(Speedchart1, 'ConSpeed', Prefix + 'ConSpeed' + Suffix);
        Save2.Caption := S;
        Save2.Visible := True;
      end
      else if Sender = CRCSaveButton then
      begin
        if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.xml', 'CRC');
        SaveXML(CRCchart, 'CRC', Prefix + 'CRC' + Suffix);
        Save3.Caption := S;
        Save3.Visible := True;
      end
      else if Sender = FECSaveButton then
      begin
        if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.xml', 'FEC');
        SaveXML(FECchart, 'FEC', Prefix + 'FEC' + Suffix);
        Save4.Caption := S;
        Save4.Visible := True;
      end
      else if Sender = BitswapSaveButton then begin
        SaveXML(Swapsperminchart, 'Bitswaps', Prefix + 'Bitswaps' + Suffix);
        if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.xml', 'Bitswaps');
        Save5.Caption := S;
        Save5.Visible := True;
      end;
    end;
  except
    on E: Exception do
    begin
      MiscString := 'Some snapshots could not be saved';
      LogEvent(leMisc);
    end;
  end;
end;

//Save text pages
{procedure Tmainform.SaveTextPages;
var
  Prefix, Suffix, SnapDir: string;
begin
  Prefix := '';
  Suffix := '';
  SnapDir := '';
  if not CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', '');
  try
    if ConnectionStatsText.Checked then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'StatsText');
      StatsList.Lines.SaveToFile(Prefix + 'ConnectionStats' + Suffix);
    end;
    if SNRText.Checked then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'SNRText');
      SNRList.Lines.SaveToFile(Prefix + 'SNRperTone' + Suffix);
    end;
    if QLNText.Checked then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'QLNText');
      QLNList.Lines.SaveToFile(Prefix + 'QLN' + Suffix);
    end;
    if BitloadingText.Checked then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'BitloadingText');
      BitsList.Lines.SaveToFile(Prefix + 'Bitloading' + Suffix);
    end;
    if pbParamsText.Checked then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'pbParamsText');
      ParamsList.Lines.SaveToFile(Prefix + 'pbParams' + Suffix);
    end;
    if HLogText.Checked then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'HLogText');
      HLogList.Lines.SaveToFile(Prefix + 'HLog' + Suffix);
    end;
    if TrafficText.Checked then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'TrafficText');
      TrafficList.Lines.SaveToFile(Prefix + 'Traffic' + Suffix);
    end;
  except
    on E: Exception do
    begin
      MiscString := 'Some text page snapshots could not be saved';
      LogEvent(leMisc);
    end;
  end;
end; }

//Respond to save text button press
procedure Tmainform.SaveTextButtonClick(Sender: TObject);
begin
  ManualSaveTextPages(Sender);
end;

//Manual save of text pages
procedure Tmainform.ManualSaveTextPages(Sender: TObject);
var
  Prefix, Suffix, SnapDir, UploadDir, TimeStamp, SValue, DString, UString: string;
  ErrorList: TStringList;
  N, D: Byte;
  TempDT: TDateTime;
  Value, Divider: Double;
  ValueUp, ValueDown, ESUp1, ESDown1: Word;

  procedure SetPrecision;
  begin
    try
      if Divider = 0 then Value := 0
      else Value := Value / Divider;
      if Value < 0 then Value := 0;
      if Value < 0.006 then D := 0
      else if Value < 10 then D := 2
      else if Value < 100 then D := 1
      else D := 0;
      SValue := FloatToStrF(Value, ffFixed, 0, D);
    except
      on E: Exception do
        SValue := '0.0';
    end;
  end;

begin
  Prefix := '';
  Suffix := '';
  TimeStamp := FormatDateTime('yyyy"-"mm"-"dd"-"hh"."nn"."ss', Now);
  SnapDir := IncludeTrailingPathDelimiter(SnapshotsPath);
  if not TestOK then if not TestWrite(SnapDir) then
  begin
    MiscString := 'Unable to write to snapshots directory';
    LogEvent(leMisc);
    Exit;
  end;
  if DoUploadSS.Checked then
  begin
    UploadDir := IncludeTrailingPathDelimiter(SnapDir + 'Uploads');
    if not ForceDirectories(UploadDir) then
    begin
      MiscString := 'Unable to create uploads directory ' + UploadDir + '. Snapshot uploads disabled.';
      LogEvent(leMisc);
      UploadDir := SnapDir;
      DoUploadSS.Checked := False;
    end;
  end;
  if not CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', '');
  try
    if Sender = SaveLogButton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'EventLog');
      SnapDir := SnapDir + 'Eventlog' + IntToStr(Lognum) + '.txt';
      if FileExists(SnapDir) then Sysutils.DeleteFile(SnapDir);
      Eventlist.Lines.SaveToFile(SnapDir);
      MiscString := SnapDir;
      Inc(LogNum);
      if DoUploadSS.Checked then
      begin
        EventList.Lines.SaveToFile(UploadDir + 'Eventlog-' + TimeStamp + '.txt');
      end;
    end
    else if DisplayHereButton.Checked and (Sender = SaveTextButton) then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'Custom');
      CommandOutput.Lines.SaveToFile(Prefix + 'Custom' + Suffix);
    end
    else if Sender = SaveAveragesButton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'ErrorText');
      ErrorList := TStringList.Create;
      TempDT := CurrentErrorDay;
      try
        ErrorList.Add('Average error rates by day');
        ErrorList.Add('');
        for N := 0 to 10 do
        begin
          AverageDateBox.ItemIndex := N;
          ErrorList.Add(FormatDateTime('dd mmm yyyy', TempDT));
          if (ErrorArray1[N].TD = 0) or (ErrorArray1[N].Seconds = 0) then
          begin
            ErrorList.Add('No Data');
            ErrorList.Add('');
          end
          else begin
            Divider := ErrorArray1[N].Seconds / 3600;
            Value := ErrorArray1[N].CRCdown;
            SetPrecision;
            DString := SValue;
            Value := ErrorArray1[N].CRCup;
            SetPrecision;
            UString := SValue;
            ErrorList.Add('CRC errors per hour: ' + DString + ' Down, ' + UString + ' Up');
            Value := ErrorArray1[N].FECdown;
            SetPrecision;
            DString := SValue;
            Value := ErrorArray1[N].FECup;
            SetPrecision;
            UString := SValue;
            ErrorList.Add('FEC errors per hour: ' + DString + ' Down, ' + UString + ' Up');
            Value := ErrorArray1[N].HECdown;
            SetPrecision;
            DString := SValue;
            Value := ErrorArray1[N].HECup;
            SetPrecision;
            UString := SValue;
            ErrorList.Add('HEC errors per hour: ' + DString + ' Down, ' + UString + ' Up');
            Value := ErrorArray1[N].ESdown;
            SetPrecision;
            DString := SValue;
            Value := ErrorArray1[N].ESup;
            SetPrecision;
            UString := SValue;
            ErrorList.Add('ES per hour: ' + DString + ' Down, ' + UString + ' Up');
            Value := ErrorArray1[N].SESdown;
            SetPrecision;
            DString := SValue;
            Value := ErrorArray1[N].SESup;
            SetPrecision;
            UString := SValue;
            ErrorList.Add('SES per hour: ' + DString + ' Down, ' + UString + ' Up');
            Divider := 1;
            ValueUp := 0;
            ValueDown := 0;
            ESUp1 := 0;
            ESDown1 := 0;
            for D := 1 to 24 do
            begin
              ESChart.GetHourlyES(N, D, ESUp1, ESDown1);
              ValueDown := ValueDown + ESDown1;
              ValueUp := ValueUp + ESUp1;
            end;
            DString := IntToStr(ValueDown);
            UString := IntToStr(ValueUp);
            ErrorList.Add('Total ES in the day: ' + DString + ' Down, ' + Ustring + ' Up');
            ErrorList.Add('');
          end;
          TempDT := TempDT - 1;
        end;
        AverageDateBox.ItemIndex := 0;
        ErrorList.SaveToFile(Prefix + 'AverageErrors' + Suffix);
        Inc(ErrorNumber);
        if DoUploadSS.Checked then
        begin
          ErrorList.SaveToFile(UploadDir + 'AverageErrors-' + TimeStamp + '.txt');
        end;
      except
        on E: Exception do logEvent(leErrorsnotsaved);
      end;
      ErrorList.Free;
    end
    else if Sender = SaveTDStats then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'StatsText');
      StatsList.Lines.SaveToFile(Prefix + 'ConnectionStats' + Suffix);
      if DoUploadSS.Checked then
      begin
        StatsList.Lines.SaveToFile(UploadDir + 'ConnectionStats-' + TimeStamp + '.txt');
      end;
    end
    else if Sender = SaveTDBits then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'BitloadingText');
      BitsList.Lines.SaveToFile(Prefix + 'Bits' + Suffix);
      if DoUploadSS.Checked then
      begin
        BitsList.Lines.SaveToFile(UploadDir + 'Bits-' + TimeStamp + '.txt');
      end;
    end
    else if Sender = SaveTDParams then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'pbParamsText');
      ParamsList.Lines.SaveToFile(Prefix + 'pbParams' + Suffix);
      if DoUploadSS.Checked then
      begin
        ParamsList.Lines.SaveToFile(UploadDir + 'pbParams-' + TimeStamp + '.txt');
      end;
    end
    else if Sender = SaveTDQLN then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'QLNText');
      QLNList.Lines.SaveToFile(Prefix + 'QLN' + Suffix);
      if DoUploadSS.Checked then
      begin
        QLNList.Lines.SaveToFile(UploadDir + 'QLN-' + TimeStamp + '.txt');
      end;
    end
    else if Sender = SaveTDHLog then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'HLogText');
      HLogList.Lines.SaveToFile(Prefix + 'HLog' + Suffix);
      if DoUploadSS.Checked then
      begin
        HLogList.Lines.SaveToFile(UploadDir + 'HLog-' + TimeStamp + '.txt');
      end;
    end
    else if Sender = SaveTDSNR then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'SNRText');
      SNRList.Lines.SaveToFile(Prefix + 'SNR' + Suffix);
      if DoUploadSS.Checked then
      begin
        SNRList.Lines.SaveToFile(UploadDir + 'SNR-' + TimeStamp + '.txt');
      end;
    end
    else if Sender = SaveTDTraffic then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'TrafficText');
      TrafficList.Lines.SaveToFile(Prefix + 'Traffic' + Suffix);
      if DoUploadSS.Checked then
      begin
        TrafficList.Lines.SaveToFile(UploadDir + 'Traffic-' + TimeStamp + '.txt');
      end;
    end
    else if Sender = SaveTDVectoring then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'VectoringText');
      VectoringList.Lines.SaveToFile(Prefix + 'Vectoring' + Suffix);
    end
    else if Sender = SaveGInpButton then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'GInpData');
      GInpList.Lines.SaveToFile(Prefix + 'GInpData' + Suffix);
    end
    else if Sender = EventList then
    begin
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'EventLog');
      EventList.Lines.SaveToFile(Prefix + 'EventLog' + Suffix);
    end
    else if Sender = SaveLogButton1 then
    try                     //Save both line and signal attenuation logs
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'LAttenuationLog');
      AttenFile := Prefix + 'LAttenuationLog' + Suffix;
      LAttenuationLog.Lines.SaveToFile(AttenFile);
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'SAttenuationLog');
      AttenFile := Prefix + 'SAttenuationLog' + Suffix;
      SAttenuationLog.Lines.SaveToFile(AttenFile);
    except
      on E:Exception do;    //Do nothing, just trap exception
    end
    else if Sender = SaveStatsButton then
    try
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'Stats');
      StatsText.Lines.SaveToFile(Prefix + 'Stats' + Suffix);
      if DoUploadSS.Checked then
      begin
        StatsText.Lines.SaveToFile(UploadDir + 'Stats-' + TimeStamp + '.txt');
      end;
    except
      on E: Exception do;   //Do nothing, just trap exception
    end
    else if Sender = SaveGDotInpData then
    try
      if CategoryBox.Checked then PrefixSuffix(Prefix, Suffix, SnapDir, '.txt', 'GDotInpData');
      GDotInpList.Lines.SaveToFile(Prefix + 'GDotInpData' + Suffix);
    except
      on E: Exception do;   //Do nothing, just trap exception
    end;
  except
    on E: Exception do
    begin
      MiscString := 'Not all text pages were saved';
      LogEvent(leMisc);
    end;
  end;
end;

//Delete snapshots older than N days
procedure TMainform.DeleteOlder;
var
  Dir: string;
  Rec: TSearchRec;
  FormattedDate: string;
  TD: TDateTime;
  Index: Word;
  Deleted: Boolean;
const
  Folders: array[0..21] of string = ('erroredseconds', 'snrm', 'snrmperband', 'connspeed', 'traffic', 'bitloading',
      'bitswaps', 'swapsperminute', 'crc', 'fec', 'qln', 'hlog', 'snrpertone', 'eventlog', 'statstext', 'bitstext',
      'pbparamstext', 'qlntext', 'hlogtext', 'snrtext', 'traffictext', 'attenuationlog');
begin
  Deleted := False;
  LastDelete := Now;
  DeleteAge := Now - OlderDays.Value;   //EncodeTimeInterval(OlderDays.Value * 24, 0, 0, 0);
  Dir := IncludeTrailingPathDelimiter(SnapshotsPath);
  if DeleteFiles(Dir) then Deleted := True;  //Start with main snapshot directory
  try
    if Sysutils.FindFirst(Dir + '*', faDirectory, Rec) = 0 then  //Delete daily folders dated earlier than DeleteAge
    begin
      TD := FileDateToDateTime(Rec.Time);
      FormattedDate := FormatDateTime('yyyy-mm-dd', TD);
      if (Rec.Name = FormattedDate) and (TD < DeleteAge) then
      begin
        DeletingDir := Dir + Rec.Name;
        if DeleteDirectory(Dir + Rec.Name, False) then
        begin
          LogEvent(leDeletesucceeded);
          Deleted := True;
        end
        else LogEvent(leDeletefailed);
      end;
      while Sysutils.FindNext(Rec) = 0 do
      begin
        TD := FileDateToDateTime(Rec.Time);
        FormattedDate := FormatDateTime('yyyy-mm-dd', TD);
        if (Rec.Name = FormattedDate) and (TD < DeleteAge) then    //Don't delete unless the folder name matches its timestamp
        begin
          DeletingDir := Dir + Rec.Name;
          if DeleteDirectory(Dir + Rec.Name, False) then
          begin
            LogEvent(leDeletesucceeded);
            Deleted := True;
          end
          else LogEvent(leDeletefailed);
        end;
      end;
    end;
    if Sysutils.FindFirst(Dir + '*', faDirectory, Rec) = 0 then  //Delete older files in category folders
    begin
      for Index := 0 to 21 do if LowerCase(Rec.Name) = Folders[Index] then
      begin
        if DeleteFiles(Dir + Rec.Name) then Deleted := True;
        Break;
      end;
      while Sysutils.FindNext(Rec) = 0 do
        for Index := 0 to 13 do if LowerCase(Rec.Name) = Folders[Index] then
        begin
          if DeleteFiles(Dir + Rec.Name) then Deleted := True;
          Break;
        end;
    end;
  except
    on E: Exception do
    begin
      DeletingDir := Dir;
      LogEvent(leDeletefailed);
    end;
  end;
  if Deleted then
  begin
    MiscString := 'Snapshots older than ' + IntToStr(OlderDays.Value) + ' days deleted';
    LogEvent(leMisc);
  end;
end;

//Delete files older than specified number of days in directory
function TMainform.DeleteFiles(Dir: string): Boolean;
var
  Rec: TSearchRec;
  List: TStringList;
  N: Word;
  TD: TDateTime;
  S: string;
const
  FileList: array[0..9] of string = ('statstext.txt', 'snrtext.txt', 'qlntext.txt', 'bitloadingtext.txt', 'pbparamstext.txt', 'hlogtext.txt', 'TrafficText.txt',
             'custom.txt', 'errortext.txt', 'attenuationlog.txt');

  procedure CheckFile;  //Check a file for age and qualification for deleting
  var
    Index: Word;
  begin
    TD := FileDateToDateTime(Rec.Time);
    if TD < DeleteAge then
    begin
      S := ExtractFileExt(Rec.Name);
      if (S = '.png') or (S = '.xml') then  //Delete all files with .png or .xml extension
      begin
        List.Add(Rec.Name);
        Inc(N);
      end
      else for Index := 0 to 9 do
      if LowerCase(Rec.Name) = FileList[Index] then   //Delete files matching names in FileList
      begin
        List.Add(Rec.Name);
        Inc(N);
      end;
      if Pos('eventlog', LowerCase(Rec.Name)) = 1 then   //Delete files named 'eventlog*.txt'
      begin
        List.Add(Rec.Name);
        Inc(N);
      end;
    end;
  end;

begin
  Result := False;
  List := TStringList.Create;
  Dir := IncludeTrailingPathDelimiter(Dir);
  N := 0;
  try
    if Sysutils.FindFirst(Dir + '*.*', 0, Rec) = 0 then
    begin
      CheckFile;
      while Sysutils.FindNext(Rec) = 0 do CheckFile;
    end;
    Sysutils.FindClose(Rec);
    while N > 0 do     //N contains number of files in list for deletion
    begin
      Result := True;
      Dec(N);
      Sysutils.DeleteFile(Dir + List[N]);
    end;
  except
    on E: Exception do
    begin
      DeletingDir := Dir;
      LogEvent(leDeletefailed);
    end;
  end;
  List.Free;
end;

// **** ALERTS ****

//Restrict trigger value to numerals or decimal separator
procedure Tmainform.SNRMTriggerKeyPress(Sender: TObject; var Key: char);
begin
  if not (Key in ['0'..'9', DefaultFormatSettings.DecimalSeparator, Chr(8), Chr(127)]) then Key := Chr(0)
  else begin   //Value has changed, so needs to be reprocessed
    if Sender = SNRMTrigger then TriggerSNRMDone := False
    else if Sender = SNRMreturnTrigger then TriggerSNRMreturnDone := False
    else if Sender = CRCTrigger then TriggerCRCDone := False
    else if Sender = FECTrigger then TriggerFECDone := False
    else TriggerESDone := False;
  end;
end;

//Restrict port number to numerals
procedure Tmainform.SMTPPortKeyPress(Sender: TObject; var Key: char);
begin
  if not (Key in ['0'..'9', Chr(8), Chr(127)]) then Key := Chr(0);
end;

//Enable/disable SSL/TLS
procedure Tmainform.SMTPPortChange(Sender: TObject);
begin
  if SMTPPort.Text = '465' then
  begin
    SSLButton.Enabled := True;
    SSLButton.Checked := True;
    PlainButton.Enabled := False;
    StartTLSButton.Enabled := False;
  end
  else begin
    StartTLSButton.Enabled := True;
    PlainButton.Enabled := True;
    StartTLSButton.Checked := True;
    SSLButton.Enabled := False;
  end;
end;

//Check if one or more of the email alerts is enabled
procedure Tmainform.SNRMemailChange(Sender: TObject);
begin
  SendEmailAlerts := SNRMemail.Checked or CRCemail.Checked or FECemail.Checked or ESemail.Checked
    or Resyncemail.Checked;
end;

//Reveal/hide password
procedure Tmainform.RevealEPBoxChange(Sender: TObject);
begin
  if RevealEPBox.Checked then EmailPassword.EchoMode := emNormal else EmailPassword.EchoMode := emPassword;
end;

//Copy 'Email from' to 'Login username' if the latter is blank
procedure Tmainform.EmailFromExit(Sender: TObject);
begin
  if EmailUsername.Text = '' then EmailUsername.Text := EmailFrom.Text;
end;

//Test the email notification setup
procedure Tmainform.TestButtonClick(Sender: TObject);
begin
  if EMailSubject.Text = '' then
  begin
    ShowMessage('The subject field must not be blank');
  end
  else SendAlert(atTest, False, 0);
end;

//Convert the text value of an alert to FP
function TMainform.GetTriggerFP(AlertType: TAlertType; Value: string): Extended;
var
  StringBase: string;
  CheckBox: TCheckBox;
  Success: Boolean;
begin
  Result := -999;
  case AlertType of
    atSNRM: begin
              StringBase := 'SNRM';
              CheckBox := SNRMfallBox;
            end;
    atSNRMreturn: begin
              StringBase := 'SNRMreturn';
              CheckBox := SNRMreturnBox;
            end;
    atCRC:  begin
              StringBase := 'CRC';
              CheckBox := CRCriseBox;
            end;
    atFEC:  begin
              StringBase := 'FEC';
              CheckBox := FECriseBox;
            end;
    atES:   begin
              StringBase := 'ES';
              CheckBox := ESriseBox;
            end;
  end;
  Success := False;
  try
    Result := StrToFloat(Value);
    Success := True;
  except
    on E: Exception do
    begin
      MiscString := 'Invalid ' + StringBase + ' alert value, alert disabled.';
      CheckBox.Checked := False;
      Result := -999;
    end;
  end;
  case AlertType of
    atSNRM: TriggerSNRMDone := Success;
    atSNRMreturn: TriggerSNRMreturnDone := Success;
    atCRC: TriggerCRCDone := Success;
    atFEC: TriggerFECDone := Success;
    atES: TriggerESDone := Success;
  end;
end;

//Reset the alert triggers
procedure TMainform.ResetTriggers;
begin
  TriggerSNRM := GetTriggerFP(atSNRM, SNRMTrigger.Text);
  TriggerSNRMreturn := GetTriggerFP(atSNRMreturn, SNRMreturnTrigger.Text);
  TriggerCRC := GetTriggerFP(atCRC, CRCTrigger.Text);
  TriggerFEC := GetTriggerFP(atFEC, FECTrigger.Text);
  TriggerES := GetTriggerFP(atES, ESTrigger.Text);
end;

//Check data for alert triggers
procedure TMainform.CheckAlerts(AlertType: TAlertType; Number1, Number2: Extended);
begin
  if SendEmailAlerts or SendEventLogBox.Checked then
  begin

    //SNRM fall
    if SNRMfallBox.Checked and (AlertType = atSNRM) then  //Check if SNRM has gone below alert threshold
    begin
      if UpSNRM.Checked and (Number2 < TriggerSNRM) then SendAlert(atSNRM, True, Number2)
      else if DownSNRM.Checked and (Number1 < TriggerSNRM) then SendAlert(atSNRM, False, Number1)
      else if EitherSNRM.Checked then
      begin
        if (Number1 < TriggerSNRM) or (Number2 < TriggerSNRM) then
        begin
          if Number1 < Number2 then SendAlert(atSNRM, False, Number1)
          else SendAlert(atSNRM, True, Number2);
        end;
      end;
    end;

    //SNRM return
    if SNRMreturnBox.Checked and SNRMAlertDone and not SNRMReturnDone
      and (AlertType = atSNRMreturn) then  //Check if SNRM has returned to higher trigger value after a fall
    begin
      if UpstreamIsLower and (Number2 > TriggerSNRMreturn) then SendAlert(atSNRMreturn, True, Number2)
      else if (not UpstreamIsLower) and (Number1 > TriggerSNRMreturn) then SendAlert(atSNRMreturn, False, Number1);
    end;

    //CRC/min rise
    if CRCriseBox.Checked and (AlertType = atCRC) then  //Check if CRC/min has gone over alert threshold
    begin
      if (Number1 > TriggerCRC) or (Number2 > TriggerCRC) then
      begin
        if Number1 > Number2 then SendAlert(atCRC, False, Number1)
        else SendAlert(atCRC, True, Number2);
      end;
    end;

    //FEC/min rise
    if FECriseBox.Checked and (AlertType = atFEC) then  //Check if FEC/min has gone over alert threshold
    begin
      if (Number1 > TriggerFEC) or (Number2 > TriggerFEC) then
      begin
        if Number1 > Number2 then SendAlert(atFEC, False, Number1)
        else SendAlert(atFEC, True, Number2);
      end;
    end;

    //ES/hour rise
    if ESriseBox.Checked and (AlertType = atES) then  //Check if ES/hour has gone over alert threshold
    begin
      if (Number1 > TriggerES) or (Number2 > TriggerES) then
      begin
        if Number1 > Number2 then SendAlert(atES, True, Number1)
        else SendAlert(atES, False, Number2);
      end;
    end;

  end;
end;

//Send an alert message
procedure Tmainform.SendAlert(AlertType: TAlertType; Up: Boolean; Value: Extended);
var
  AlertMessage, S: string;
  SMTP: TSMTPSend;
  MessageList: TStringList;
  Success, DoEmail: Boolean;
  Offset, N: Integer;
begin
  DoEMail := False;
  case AlertType of
    atSNRM:       begin
                    if Up then
                    begin
                      AlertMessage := 'Upstream';
                      UpstreamIsLower := True;
                    end
                    else begin
                      AlertMessage := 'Downstream';
                      UpstreamIsLower := False;
                    end;
                    AlertMessage := AlertMessage + ' SNRM fell to ' + FloatToStrF(Value, ffFixed, 0, 2) + ' dB';
                    if not KeepLevelsBox.Checked then TriggerSNRM := Value;         //New lower trigger
                    SNRMAlertDone := True;
                    SNRMreturnDone := False;
                    if SNRMemail.Checked then DoEMail := True;
                  end;
    atSNRMreturn: begin
                    if Up then AlertMessage := 'Upstream' else AlertMessage := 'Downstream';
                    AlertMessage := AlertMessage + ' SNRM rose again to ' + FloatToStrF(Value, ffFixed, 0, 2) + ' dB';
                    SNRMReturnDone := True;
                    TriggerSNRM := GetTriggerFP(atSNRM, SNRMTrigger.Text);  //Reset the SNRM trigger
                    if SNRMemail.Checked then DoEMail := True;
                  end;
    atCRC:        begin
                    if Up then AlertMessage := 'Upstream' else AlertMessage := 'Downstream';
                    if CRCps.Checked then AlertMessage := AlertMessage + ' CRC/sample'
                    else AlertMessage := AlertMessage + ' CRC/min';
                    AlertMessage := AlertMessage + ' rose to ' + FloatToStrF(Value, ffFixed, 0, 2);
                    if not KeepLevelsBox.Checked then TriggerCRC := Value;         //New higher trigger
                    if CRCemail.Checked then DoEMail := True;
                  end;
    atFEC:        begin
                    if Up then AlertMessage := 'Upstream' else AlertMessage := 'Downstream';
                    if FECps.Checked then AlertMessage := AlertMessage + ' FEC/sample'
                    else AlertMessage := AlertMessage + ' FEC/min';
                    AlertMessage := AlertMessage + ' rose to ' + FloatToStrF(Value, ffFixed, 0, 2);
                    if not KeepLevelsBox.Checked then TriggerFEC := Value;      //New higher trigger
                    if FECemail.Checked then DoEMail := True;
                  end;
    atES:         begin
                    if Up then AlertMessage := 'Upstream' else AlertMessage := 'Downstream';
                    AlertMessage := AlertMessage + ' ES/hour rose to ' + FloatToStrF(Value, ffFixed, 0, 2);
                    if not KeepLevelsBox.Checked then TriggerES := Value;      //New higher trigger
                    if ESemail.Checked then DoEMail := True;
                  end;
    atResync:     begin
                    AlertMessage := 'Modem/router re-sync';
                    if Resyncemail.Checked then
                    begin
                      DoEMail := True;
                      EmailResync := True;
                    end;
                  end;
    atTest:       begin
                    AlertMessage := 'Test alert';
                  end;
    atDS:         begin
                    AlertMessage := MiscString;
                    if DSemail.Checked then DoEMail := True;
                  end;
    atFTPFail:    begin
                    AlertMessage := 'Four ' + MiscString + 'FTP upload failures in succession. ' + MiscString + ' uploading disabled.';
                    if FTPemail.Checked then DoEMail := True;
                  end;
  end;
  if SendEventlogBox.Checked and not (AlertType = atTest) then
  begin
    MiscString := 'Alert: ' + AlertMessage;
    LogEvent(leMisc);
  end;
  if (SendEmailAlerts and DoEmail) or (AlertType = atTest) then
  begin
    Success := False;
    SMTP := TSMTPSend.Create;
    MessageList := TStringList.Create;
    try
      try
        if EMailSubject.Text = '' then EMailSubject.Text := 'DSLstats alert';
        MessageList.Add('From: ' + EMailFrom.Text);
        MessageList.Add('To: ' + EMailAddress.Text);
        Offset := GetLocalTimeOffset;
        if Offset = 0 then S := '+0000'
        else begin
          if Offset < 0 then S := '+' else S := '-';  //UTC = Local Time + GetLocalTimeOffset
          Offset := Abs(Offset);
          N := Offset div 60;
          if N < 10 then S := S + '0' + IntToStr(N) else S := S + IntToStr(N);
          N := Offset mod 60;
          if N < 10 then S := S + '0' + IntToStr(N) else S := S + IntToStr(N);
        end;
        MessageList.Add('Date: ' + FormatDateTime('ddd, d mmm yyyy hh:nn:ss', Now) + ' ' + S);
        MessageList.Add('Subject: ' + EMailSubject.Text);
        MessageList.Add('');
        MessageList.Add('');
        MessageList.Add(AlertMessage);
        SMTP.Username := EMailUsername.Text;
        SMTP.Password := EMailPassword.Text;
        SMTP.TargetHost := SMTPServer.Text;
        SMTP.TargetPort := SMTPPort.Text;
        SMTP.AutoTLS := True;
        if StartTLSButton.Checked then SMTP.StartTLS
        else if SSLButton.Checked then SMTP.FullSSL := True;
        if SMTP.Login then
        begin
          Success := SMTP.MailFrom(EMailFrom.Text, Length(EMailFrom.Text));
          if Success then
          begin
            Success := SMTP.MailTo(EMailAddress.Text);
            if Success then
            begin
              Success := SMTP.MailData(MessageList);
              if Success then MiscString := 'Email alert sent: "Alert: ' + AlertMessage + '"'
              else MiscString := 'Email alert failed (unknown reason)';
            end
            else MiscString := 'Email alert failed (invalid "To" address)';
          end
          else MiscString := 'Email alert failed (invalid "From" address)';
          SMTP.Logout;
        end
        else begin
          MiscString := 'Email alert failed (unable to login to SMTP server)';

        end;
        if EmailResync and not Success then
        begin
          Inc(Tries);
          if Tries >= 4 then
          begin
            Tries := 0;
            EmailResync := False;
            MiscString := 'Four attempts to send email alert for a resync failed.';
            LogEvent(leMisc);
          end;
        end
        else begin
          LogEvent(leMisc);
          EmailResync := False;
          Tries := 0;
        end;
      except
        on E: Exception do
        begin
          MiscString := 'Failed to sent email alert - ' + E.Message;
          LogEvent(leMisc);
        end;
      end;
    finally
      SMTP.Free;
      MessageList.Free;
    end;
  end;
end;

// **** Pulse the system tray icon ****

//Start pulsing
procedure Tmainform.StartPulsing;
begin
  TrayIcon1.Icon.LoadFromResourceName(HInstance, IconColour1);
  IconColour := IconColour1;
  Timer9.Enabled := True;
end;

//Stop pulsing
procedure Tmainform.StopPulsing;
begin
  TrayIcon1.Icon.LoadFromResourceName(HInstance, 'BLUE');
  IconColour := 'BLUE';
  Timer9.Enabled := False;
end;

//Switch to other colour
procedure Tmainform.Timer9Timer(Sender: TObject);
begin
  if IconColour = IconColour2 then
  begin
    TrayIcon1.Icon.LoadFromResourceName(HInstance, IconColour1);
    IconColour := IconColour1;
  end
  else begin
    TrayIcon1.Icon.LoadFromResourceName(HInstance, IconColour2);
    IconColour := IconColour2;
  end;
end;

// **** MISCELLANEOUS PROCEDURES ****

//Convert seconds to a string
function Tmainform.Timestring(Secs: Longword): string;
var
  Days, Hours, Minutes, Seconds: Longword;
  S: string;
begin
  Result := '';
  Minutes := Secs div 60;
  Seconds := Secs mod 60;
  Hours := Minutes div 60;
  Minutes := Minutes mod 60;
  if Hours > 23 then
  begin
    Days := Hours div 24;
    Hours := Hours mod 24;
  end
  else Days := 0;
  if Days > 0 then
  begin
    S := IntToStr(Days) + ' day';
    if Days > 1 then S := S + 's';
  end
  else S := '';
  S := S + ' ' + IntToStr(Hours) + ' hour';
  if Hours > 1 then S := S + 's';
  S := S + ' ' + IntToStr(Minutes) + ' min ' + IntToStr(Seconds) + ' sec';
  Result := S;
end;

//Display online help
procedure Tmainform.HelpbuttonClick(Sender: TObject);
begin
  try
    OpenURL('http://dslstats.me.uk');
  except
    on E: Exception do ShowMessage('Unable to contact website. Internet connection down?');
  end;
end;

//Toggle popup hints
procedure Tmainform.ShowHintsBoxChange(Sender: TObject);
begin
  Mainform.ShowHint := ShowHintsBox.Checked;
end;

//Trigger GetIPAddress two seconds after timer was enabled (so it happens during a quiescent period)
procedure Tmainform.Timer6Timer(Sender: TObject);
begin
  GetIPAddress(False);
  Timer6.Enabled := False;
end;

//Get IP address manually
procedure Tmainform.GetIPButtonClick(Sender: TObject);
begin
  GetIPAddress(True);
end;

//Get IP address from web site
procedure Tmainform.GetIPAddress(Manual: Boolean);
var
  IPList: TStringList;
  HTTPClient: TFPHTTPClient;
  N: Integer;
  S: string;
begin
  IPList := TStringList.Create;
  HTTPClient := TFPHTTPClient.Create(nil);
  try
    IPList.Text := HTTPClient.Get('http://dslstats.me.uk/cgi-bin/ipaddress.py');
    if IPList.Count > 1 then
    for N := 0 to IPList.Count - 1 do
    begin
      S := IPList[N] + ' ';
      if S[1] in ['0'..'9'] then
      begin
        if Manual or (S <> IPAddress) then  //Manual request or IP address has changed
        begin
          IPAddress := S;
          LogEvent(leIPAddress);
        end;
        Break;
      end;
    end
    else LogEvent(leNoIPaddress);
  except
    on E: Exception do LogEvent(leNoipaddress);
  end;
  IPList.Free;
  HTTPClient.Free;
end;

//
function TMainform.Encrypt(Value: string): string;
var
  EStream: TBlowFishEncryptStream;
  S1: TStringStream;
begin
  Result := '';
  S1 := TStringStream.Create('');
  try
  EStream := TBlowfishEncryptStream.Create(LC, S1);
  EStream.WriteAnsiString(Value);
  EStream.Free;
  Result := EncodeStringBase64(S1.DataString);
  except
    on E: Exception do
    begin
      Result := '';
      EStream.Free;
    end;
  end;
  S1.Free;
end;

//
function TMainform.Decrypt(Value: string): string;
var
  S2: TStringStream;
  DStream: TBlowFishDecryptStream;
begin
  Result := '';
  try
    Value := DecodeStringBase64(Value);
  except
    on E: Exception do
    begin
      Result := '';
      Exit;
    end;
  end;
  try
    S2 := TStringStream.Create(Value);
    DStream := TBlowfishDecryptStream.Create(LC, S2);
    Result := DStream.ReadAnsiString;
  except
    on E: Exception do
      Result := '';
  end;
  S2.Free;
  DStream.Free;
end;

//Add an entry to the event log
procedure Tmainform.LogEvent(LE: TLogEvent);
var
  Eventtext: string;
begin
  case LE of
    leAdvancedTweaking: EventText := 'Advanced tweak command sent';
    leAutosnapshot: Eventtext := 'Auto snapshots taken';
    leBadpassword: Eventtext := 'Incorrect login name or password';
    leBusy: Eventtext := 'Previous sample still being processed';
    leCLIfail: Eventtext := 'CLI command failed to execute';
    leDayreset: EventText := 'Reset the current day''s traffic';
    leDeletefailed: EventText := 'Failed to delete snapshots in folder ' + DeletingDir;
    leDownloadFailed: EventText := 'File download failed';
    leDeletesucceeded: EventText := 'Deleted snapshots in folder ' + DeletingDir;
    leDownloadSucceeded: EventText := 'Updated version downloaded';
    leErrorsReset: EventText := 'Average error data reset';
    leErrorsnotsaved: EventText := 'Unable to save average errors text';
    leGraphRestore: EventText := 'Restored XML file ' + XMLFileEdit.Text;
    leIPAddress: EventText := 'IP address is now ' + IPAddress;
    leIsrunningfailed: EventText := 'Unable to determine if HG612-Modem-Stats programs are running';
    leLoginException: Eventtext := 'Exception during telnet login';
    leMisc: EventText := MiscString;
    leMonthreset: EventText := 'Reset the current month''s traffic';
    leNoadvancedtweak: EventText := 'Attempt to apply advanced tweaks failed';
    leNobits: Eventtext := 'No bitloading data received';
    leNoCLI: EventText := 'Unable to login to CLI';
    leNodata: Eventtext := 'No data received';
    leNographrestore: EventText := 'Failed to restore XML file ' + XMLFileEdit.Text;
    leNohlog: Eventtext := 'No HLog data received';
    leNoIPAddress: EventText := 'Unable to determine IP address';
    leNologin: Eventtext := 'Unable to login to modem/router';
    leNopbparams: Eventtext := 'No pbParams data received';
    leNoqln: Eventtext := 'No QLN data received';
    leNoresponse: Eventtext := 'Modem/router failed to respond to telnet request';
    leNoreturntonormal: EventText := 'Failed to return to normal operation';
    leReturntonormal: EventText := 'Returned to normal operation';
    leNorouterresponse: Eventtext := 'Modem/router failed to respond';
    leNospeciallogin: Eventtext := 'Special login failed';
    leNostats: Eventtext := 'No stats received';
    leNosecondstage: Eventtext := 'Second stage login failed';
    leNoshowtime: Eventtext := 'No DSL connection';
    leNosnr: Eventtext := 'No SNR per tone data received';
    leNouptime: Eventtext := 'No uptime data received';
    leNoTraffic: EventText := 'No traffic data received';
    leNovendor: Eventtext := 'No vendor data received';
    lePause: Eventtext := 'Recording paused';
    leReadonlydir: Eventtext := 'Snapshot directory is read-only';
    leResume: Eventtext := 'Recording resumed';
    leResync: Eventtext := 'Resync command issued';
    leSamplemissed: EventText := IsRunning;
    leSnapshotFailed: EventText := GraphType + ' snapshot failed';
    leStart: Eventtext := 'Recording started';
    leStop: Eventtext := 'Recording stopped';
    leSynchronise: EventText := 'Option "HG612-Modem-Stats co-operation" was set';
    leXMLoadfailed: EventText := 'Unable to restore XML file';
    leXMLsavefailed: EventText := 'Unable to save XML file';
    leTimeout: EventText := 'Telnet timeout';
    leUnknown: EventText := 'Unknown telnet error';
    leUnsync: EventText := 'Option "HG612-Modem-Stats co-operation" was unset';
    else Eventtext := 'Unknown Event';
  end;
  Eventlist.Lines.Add(FormatDateTime('dd mmm yyyy hh:mm:ss', Now) + chr(9) + Eventtext);
  if SaveAfterBox.Checked then MSnapButtonClick(SaveLogButton);
  if Eventlist.Lines.Count > 500 then
  begin
    ManualSaveTextPages(EventList);
    EventList.Clear;
    MiscString := 'Event Log saved to Snapshots folder, then cleared';
    LogEvent(leMisc);
  end;
end;

//Add an entry to the Line Attenuation log
procedure TMainform.LogLAttenuation(Up, Down: string);
begin
  LAttenuationLog.Lines.Add(FormatDateTime('dd mmm yyyy hh:mm:ss', Now) + chr(9) + Down + Chr(9) + Up);
end;

//Add an entry to the Signal Attenuation log
procedure TMainform.LogSAttenuation(Up, Down: string);
begin
  SAttenuationLog.Lines.Add(FormatDateTime('dd mmm yyyy hh:mm:ss', Now) + chr(9) + Down + Chr(9) + Up);
end;

//Clear the Attenuation logs
procedure Tmainform.ClearLogButtonClick(Sender: TObject);
var
  N: Integer;
  S: string;
begin
  if VDSL2 then
  begin
    LAttenuationLog.Clear;
    S := 'Line attenuation (dB)    ';
    for N := 0 to 4 do
      if BandsReported[0, N] then S := S + Chr(9) + 'U' + IntToStr(N);
    for N := 0 to 4 do
      if BandsReported[1, N] then S := S + Chr(9) + 'D' + IntToStr(N);
    LAttenuationLog.Lines.Add(S);
    SAttenuationLog.Clear;
    S := 'Signal attenuation (dB) ';
    for N := 0 to 4 do
      if BandsReported[0, N] then S := S + Chr(9) + 'U' + IntToStr(N);
    for N := 0 to 4 do
      if BandsReported[1, N] then S := S + Chr(9) + 'D' + IntToStr(N);
    SAttenuationLog.Lines.Add(S);
  end
  else begin
    with LAttenuationLog do
    if Lines.Count > 2 then
      for N := Lines.Count - 1 downto 2 do Lines.Delete(N);
    StatsArray[1, 7] := '';   //So next sample will log the present value
    StatsArray[2, 7] := '';
    with SAttenuationLog do
    if Lines.Count > 2 then
      for N := Lines.Count - 1 downto 2 do Lines.Delete(N);
    StatsArray[1, 8] := '';   //So next sample will log the present value
    StatsArray[2, 8] := '';
  end;
  PreviousLAttLine := '';
  PreviousSAttLine := '';
end;

// **** WEBSERVER ****

{ THTTPServerThread }

constructor THTTPServerThread.Create(APort: Word; const OnRequest: THTTPServerRequestHandler);
begin
  FServer := TFPHTTPServer.Create(nil);
  FServer.Threaded := True;
  FServer.Port := APort;
  FServer.OnRequest := OnRequest;
  inherited Create(False);              //Was True
end;

procedure THTTPServerThread.Execute;
begin
  try
    FServer.Active := True;
  finally
    FreeandNil(FServer);
  end;
end;

procedure THTTPServerThread.DoTerminate;
begin
  inherited DoTerminate;
  FServer.Active := False;
end;

{ Mainform webserver procedures }

//Handle request to webserver
procedure TMainform.DoHandleRequest(Sender: TObject; var ARequest: TFPHTTPConnectionRequest; var AResponse: TFPHTTPConnectionResponse);
begin
  FHandler.HandleRequest(ARequest, AResponse);
end;

//Restrict entry to numerals
procedure Tmainform.WebPortEditKeyPress(Sender: TObject; var Key: char);
begin
  if not (Key in ['0'..'9', Chr(8), Chr(127)]) then Key := Chr(0);
end;

//Set up webserver
procedure TMainform.SetupWebserver;
begin
  WebserverPath := IncludeTrailingPathDelimiter(WebserverDirEdit.Directory);
  MakeWebserverDir;
  RegisterFileLocation('files', WebserverPath);
  MimeTypesFile := DataDir + 'mime.types';
  if FirstSetup then
  begin
    FHandler := TFPCustomFileModule.CreateNew(Self);
    FHandler.BaseURL := 'files/';
    FirstSetup := False;
  end;
  WebserverRunning := False;
end;

//Set up webserver directory
procedure TMainform.MakeWebserverDir;
begin
  if not ForceDirectories(WebserverPath) then
  begin
    MiscString := 'Unable to create folder ' + WebserverPath + '. Webserver disabled.';
    LogEvent(leMisc);
    WebserverBox.Checked := False;
  end
  else begin   //Copy the html files to WebserverPath folder
    {$IFNDEF Windows}
    FpChmod(WebserverPath, &775);
    {$ENDIF}
    try
      Fileutil.CopyFile(DataDir + 'snrm.html', WebserverPath + 'snrm.html');
      Fileutil.CopyFile(DataDir + 'snrmpbup.html', WebserverPath + 'snrmpbup.html');
      Fileutil.CopyFile(DataDir + 'snrmpbdown.html', WebserverPath + 'snrmpbdown.html');
      Fileutil.CopyFile(DataDir + 'connspeed.html', WebserverPath + 'connspeed.html');
      Fileutil.CopyFile(DataDir + 'bitloading.html', WebserverPath + 'bitloading.html');
      Fileutil.CopyFile(DataDir + 'crc.html', WebserverPath + 'crc.html');
      Fileutil.CopyFile(DataDir + 'es.html', WebserverPath + 'es.html');
      Fileutil.CopyFile(DataDir + 'ses.html', WebserverPath + 'ses.html');
      Fileutil.CopyFile(DataDir + 'fec.html', WebserverPath + 'fec.html');
      Fileutil.CopyFile(DataDir + 'statstemplate.html', WebserverPath + 'statstemplate.html');
      Fileutil.CopyFile(DataDir + 'fullstatstemplate.html', WebserverPath + 'fullstatstemplate.html');
    except
      on E: Exception do
      begin
        MiscString := 'Unable to copy webserver templates to ' + WebserverPath + ' . Webserver disabled.';
        LogEvent(leMisc);
        WebserverBox.Checked := False;
      end;
    end;
  end;
end;

//Toggle webserver on/off
procedure Tmainform.WebserverBoxChange(Sender: TObject);
var
  Value: LongWord;
begin
  if Starting then Exit;  //Don't do this until after dslstats.ini is read and the webserver set up
  if WebserverBox.Checked then
  begin
    if WebserverRunning then Exit;
    try
      Value := StrToInt(WebPortEdit.Text);
      if (Value < 49152) or (Value > 65535) then
      begin
        ShowMessage('Webserver port number must be in range 49152 - 65535');
        WebserverBox.Checked := False;
        Exit;
      end;
    except
      ShowMessage('Webserver port number must be in range 49152 - 65535');
      WebserverBox.Checked := False;
      Exit;
    end;
    WebPortEdit.Enabled := False;
    SetupWebserver;
    if FServerThread = nil then FServerThread := THTTPServerThread.Create(Value, @DoHandleRequest);
    FServerThread.Start;
    MiscString := 'Webserver started';
    LogEvent(leMisc);
    WebserverRunning := True;
    WebserverSaveBox.Checked := True;
    WebserverSaveBox.Enabled := False;
  end
  else begin
    WebPortEdit.Enabled := True;
    if WebserverRunning then
    begin
      FServerThread.Terminate;
      if DeleteWebBox.Checked then DeleteWebserverDir(WebserverPath);     //Delete the webserver folder and contents
      MiscString := 'Webserver stopped';
      LogEvent(leMisc);
      WebserverRunning := False;
      WebserverSaveBox.Enabled := True;
      WebserverSaveBox.Checked := False;
    end;
  end;
end;

procedure Tmainform.WebserverSaveBoxChange(Sender: TObject);
begin
  if WebserverSaveBox.Checked then SetupWebserver
  else begin
    if DeleteWebBox.Checked then DeleteWebserverDir(WebserverPath);
  end;
end;

//Process change of snapshot directory
procedure Tmainform.SnapDirEditChange(Sender: TObject);
var
  OldPath: string;
begin
  OldPath := SnapshotsPath;
  SnapshotsPath := IncludeTrailingPathDelimiter(SnapDirEdit.Directory);
  if ForceDirectories(SnapshotsPath) then
  begin
    {$IFNDEF Windows}
    FpChmod(SnapshotsPath, &775);
    {$ENDIF}
    MiscString := 'Snapshots folder is ' + SnapshotsPath;
    LogEvent(leMisc);
  end
  else begin
    MiscString := 'Unable to create new directory "' + SnapshotsPath + '. Previous location retained.';
    LogEvent(leMisc);
    SnapshotsPath := OldPath;
    SnapDirEdit.Directory := DataStorePath;
  end;
end;

//Process change of webserver directory
procedure Tmainform.WebserverDirEditChange(Sender: TObject);
var
  OldDir, NewDir: string;
  Running: Boolean;
begin
  if Starting then Exit;
  OldDir := IncludeTrailingPathDelimiter(WebserverPath);
  NewDir := IncludeTrailingPathDelimiter(WebserverDirEdit.Directory);
  if NewDir = OldDir then Exit;
  Running := WebserverRunning;
  if Running then WebserverBox.Checked := False;  //Stop webserver while making the changeover
  WebserverPath := NewDir;
  if ForceDirectories(WebserverPath) then   //Directory needs to exist before the next line
  begin
    RegisterFileLocation('files', WebserverPath);
    RemoveDir(WebserverPath);
    if OldDir <> IncludeTrailingPathDelimiter('') then DeleteWebserverDir(OldDir); //Don't delete if OldDir doesn't exist
  end
  else begin
    WebserverPath := OldDir;
    ForceDirectories(OldDir);
    {$IFNDEF Windows}
    FpChmod(OldDir, &775);
    {$ENDIF}
    MiscString := 'Could not create new webserver folder. Previous folder retained.';
    LogEvent(leMisc);
  end;
  WebserverBox.Checked := Running;       //Restart webserver if it was running
  MiscString := 'Webserver folder is now ' + IncludeTrailingPathDelimiter(WebserverPath);
  LogEvent(leMisc);
end;

//Confirm change of webpage refresh period
procedure Tmainform.ConfirmRefreshButtonClick(Sender: TObject);
var
  Filename: string;
  HTMLList: TStringList;
  Success: Boolean;

  //Edit one HTML template to change refresh period
  procedure EditFile;
  var
    LineText: string;
    Index, Len: Byte;
    C: Char;
  begin
    HTMLList.LoadFromFile(DataDir + Filename);
    LineText := HTMLList[1];
    Index := Pos('content=', LineText) + 6;
    repeat
      Inc(Index);
      C := LineText[Index];
    until (C in ['0'..'9']);
    Len := 0;
    repeat
      Inc(Len);
      C := LineText[Index + Len];
    until not (C in ['0'..'9']);
    LineText := LineText[1..Index - 1] + IntToStr(RefreshPeriod) + LineText[Index + Len..Length(LineText)];
    HTMLList[1] := LineText;
    HTMLList.SaveToFile(DataDir + FileName);
    if not Starting then
    begin
      if Filename = 'combined.template' then Fileutil.CopyFile(DataDir + Filename, WebserverPath + 'combined.html')
      else Fileutil.CopyFile(DataDir + Filename, WebserverPath + Filename);
    end;
  end;

begin
  if SameButton.Checked then
  begin
    RefreshEdit.Value := SampleRateEdit.Value;
    RefreshEdit.Enabled := False;
  end
  else RefreshEdit.Enabled := True;
  if RefreshPeriod = RefreshEdit.Value then Exit;
  Success := False;
  HTMLList := TStringList.Create;
  RefreshPeriod := RefreshEdit.Value;
  try
    Filename := 'bitloading.html';
    EditFile;
    Filename := 'connspeed.html';
    EditFile;
    Filename := 'crc.html';
    EditFile;
    Filename := 'es.html';
    EditFile;
    Filename := 'ses.html';
    EditFile;
    Filename := 'fec.html';
    EditFile;
    Filename := 'fullstatstemplate.html';
    EditFile;
    Filename := 'snrm.html';
    EditFile;
    Filename := 'snrmpbdown.html';
    EditFile;
    Filename := 'snrmpbup.html';
    EditFile;
    Filename := 'statstemplate.html';
    EditFile;
    Filename := 'combined.template';
    EditFile;
    Success := True;
  except
    on E: Exception do
    begin
      MiscString := 'Unable to read all HTML templates in ' + DataDir + '. Refresh period unchanged';
      LogEvent(leMisc);
    end;
  end;
  HTMLList.Free;
  if Success then
  begin
    MiscString := 'Webpage refresh period changed';
    LogEvent(leMisc);
  end;
end;

//Delete webserver directory and contents
procedure TMainForm.DeleteWebserverDir(Path: string);
var
  Rec: TSearchRec;
begin
  if DirectoryExists(Path) then
  begin
    Path := IncludeTrailingPathDelimiter(Path);
    if Sysutils.FindFirst(Path + '*.*', faAnyFile - faDirectory, Rec) = 0 then
    begin
      if not Sysutils.DeleteFile(Path + Rec.Name) then
      begin
        MiscString := 'Unable to delete file ' + Rec.Name;
        LogEvent(leMisc);
      end;
      while Sysutils.FindNext(Rec) = 0 do
      begin
        if not Sysutils.DeleteFile(Path + Rec.Name) then
        begin
          MiscString := 'Unable to delete file ' + Rec.Name;
          LogEvent(leMisc);
        end;
      end;
    end;
    if RemoveDir(Path) then
    begin
      MiscString := 'Deleted folder ' + Path + ' with its contents';
      LogEvent(leMisc);
    end
    else begin
      MiscString := 'Unable to delete folder ' + Path;
      LogEvent(leMisc);
    end;
  end;
end;

//Save snapshots for webserver
procedure Tmainform.PopulateWebserver;
var
  WebList, CombinedList: TStringList;
  SLineNum, HLineNum, N: Byte;
  CombineOK: Boolean;
begin
  WebserverPath := IncludeTrailingPathDelimiter(WebserverDirEdit.Directory);
  CombineOK := False;
  if not (WebserverBox.Checked or WebserverSaveBox.Checked) then Exit;
  if not ForceDirectories(WebserverPath) then
  begin
    MiscString := 'Unable to create directory "' + WebserverPath + '. Webserver disabled.';
    LogEvent(leMisc);
    WebserverBox.Checked := False;
    WebserverSaveBox.Checked := False;
    Exit;
  end;
  CombinedList := TStringList.Create;
  if CombineWeb.Checked then
  try
    CombinedList.LoadFromFile(DataDir + 'combined.template');
    CombineOK := True;
  except
    on E: Exception do
    begin
      MiscString := 'Unable to load file "combined.template" (file missing?). No combined web page created.';
      LogEvent(leMisc);
    end;
  end;
  if CombineOK then
  begin
    if SNRMweb.Checked then
    begin
      CombinedList.Add('<h4>SNR margin</h4>');
      CombinedList.Add('<img alt="" src="snrm.png" hspace="20"><br><br>');
    end;
    if SNRMpbweb.Checked then
    begin
      CombinedList.Add('<h4>SNRM per band upstream</h4>');
      CombinedList.Add('<img alt="" src="snrmpbup.png" hspace="20"><br><br>');
      CombinedList.Add('<h4>SNRM per band downstream</h4>');
      CombinedList.Add('<img alt="" src="snrmpbdown.png" hspace="20"><br><br>');
    end;
    if Connspeedweb.Checked then
    begin
      CombinedList.Add('<h4>Connection speed</h4>');
      CombinedList.Add('<img alt="" src="connspeed.png" hspace="20"><br><br>');
    end;
    if Bitloadingweb.Checked then
    begin
      CombinedList.Add('<h4>Bitloading</h4>');
      CombinedList.Add('<img alt="" src="bitloading.png" hspace="20"><br><br>');
    end;
    if CRCweb.Checked then
    begin
      CombinedList.Add('<h4>CRC errors</h4>');
      CombinedList.Add('<img alt="" src="crc.png" hspace="20"><br><br>');
    end;
    if ESweb.Checked then
    begin
      CombinedList.Add('<h4>ES</h4>');
      CombinedList.Add('<img alt="" src="es.png" hspace="20"><br><br>');
    end;
    if SESweb.Checked then
    begin
      CombinedList.Add('<h4>SES</h4>');
      CombinedList.Add('<img alt="" src="ses.png" hspace="20"><br><br>');
    end;
    if FECweb.Checked then
    begin
      CombinedList.Add('<h4>FEC errors</h4>');
      CombinedList.Add('<img alt="" src="fec.png" hspace="20"><br><br>');
    end;
    if Statsweb.Checked then
    begin
      CombinedList.Add('<h4>Stats summary</h4>');
      CombinedList.Add('<PRE>');
      for N := 0 to StatsText.Lines.Count - 1 do CombinedList.Add(StatsText.Lines[N]);
      CombinedList.Add('</PRE>');
    end;
    if Fullstatsweb.Checked then
    begin
      CombinedList.Add('<h4>Connection stats</h4>');
      CombinedList.Add('<PRE>');
      for N := 0 to StatsList.Lines.Count - 1 do CombinedList.Add(StatsList.Lines[N]);
      CombinedList.Add('</PRE>');
    end;
    CombinedList.Add('<br></body></html>');
    try
      CombinedList.SaveToFile(WebserverPath + 'combined.html');
    except
      on E: Exception do
      begin
        MiscString := 'Unable to save combined web page';
        LogEvent(leMisc);
      end;
    end;
  end;
  if AllTextWeb.Checked then    //Save text files
  begin
    if StatsList.Lines.Count > 0 then StatsList.Lines.SaveToFile(WebserverPath + 'fullstats.txt');
    if SNRList.Lines.Count > 0 then SNRList.Lines.SaveToFile(WebserverPath + 'snrpertone.txt');
    if QLNList.Lines.Count > 0 then QLNList.Lines.SaveToFile(WebserverPath + 'qln.txt');
    if HLogList.Lines.Count > 0 then HLogList.Lines.SaveToFile(WebserverPath + 'hlog.txt');
    if BitsList.Lines.Count > 0 then BitsList.Lines.SaveToFile(WebserverPath + 'bitloading.txt');
    if ParamsList.Lines.Count > 0 then ParamsList.Lines.SaveToFile(WebserverPath + 'pbparams.txt');
    if TrafficList.Lines.Count > 0 then TrafficList.Lines.SaveToFile(WebserverPath + 'traffic.txt');
    if StatsText.Lines.Count > 0 then StatsText.Lines.SaveToFile(WebserverPath + 'stats.txt');
  end;
  WebList := TStringList.Create;
  if AllActiveWeb.Checked then DoSnapshots(SnapallBox, True)   //Snapshot all active graphs
  else if AllConfiguredWeb.Checked then DoSnapshots(SnapallConfiguredBox, True);   //Snapshot all graphs configured to autosave
  try
    if SNRMweb.Checked and DoMargin.Checked then
    begin
      Marginchart.SaveToPNGFile(WebserverPath + 'snrm.png');
    end;
    if SNRMpbweb.Checked and DoSNRMperband.Checked then
    begin
      SNRMpbChartup.SaveToPNGFile(WebserverPath + 'snrmpbup.png');
      SNRMpbChartdown.SaveToPNGFile(WebserverPath + 'snrmpbdown.png');
    end;
    if Connspeedweb.Checked and DoSpeed.Checked then
    begin
      Speedchart1.SaveToPNGFile(WebserverPath + 'connspeed.png');
    end;
    if Bitloadingweb.Checked and DoBitloading.Checked then
    begin
      Bitchart1.SaveToPNGFile(WebserverPath + 'bitloading.png');
    end;
    if CRCweb.Checked and DoCRC.Checked then
    begin
      CRCchart.SaveToPNGFile(WebserverPath + 'crc.png');
    end;
    if ESweb.Checked and DoES.Checked then
    begin
      ESchart.SaveToPNGFile(WebserverPath + 'es.png');
    end;
    if SESweb.Checked and DoSES.Checked then
    begin
      SESchart.SaveToPNGFile(WebserverPath + 'ses.png');
    end;
    if FECweb.Checked and DoFEC.Checked then
    begin
      FECchart.SaveToPNGFile(WebserverPath + 'fec.png');
    end;
    if StatsWeb.Checked then
    begin
      WebList.LoadFromFile(WebserverPath + 'statstemplate.html');
      N := StatsText.Lines.Count;
      if N > 10 then
      begin
        for HLineNum := 0 to WebList.Count - 1 do
        if Pos('</PRE>', WebList[HLineNum]) > 0 then
        begin
          for SLineNum := StatsText.Lines.Count - 1 downto 0 do
            WebList.Insert(HLineNum, StatsText.Lines[SLineNum]);
          Break;
        end;
      end;
      WebList.SaveToFile(WebserverPath + 'stats.html');
    end;
    if FullstatsWeb.Checked then
    begin
      WebList.LoadFromFile(WebserverPath + 'fullstatstemplate.html');
      N := StatsList.Lines.Count;
      if N > 10 then
      begin
        for HLineNum := 0 to WebList.Count - 1 do
        if Pos('</PRE>', WebList[HLineNum]) > 0 then
        begin
          for SLineNum := StatsList.Lines.Count - 1 downto 0 do
            WebList.Insert(HLineNum, StatsList.Lines[SLineNum]);
          Break;
        end;
      end;
      WebList.SaveToFile(WebserverPath + 'fullstats.html');
    end;
  except
    on E: Exception do
    begin
      MiscString := 'Some webserver snapshot(s) could not be saved';
      LogEvent(leMisc);
    end;
  end;
  WebList.Free;
  CombinedList.Free;
end;

//Enable graphs as necessary
procedure Tmainform.DSstatsChange(Sender: TObject);
begin
  DoMargin.Checked := True;
  if DSbits.Checked then DoBitloading.Checked := True;
  if DSsnr.Checked then DoSNRpertone.Checked := True;
  if DShlog.Checked then DoHLog.Checked := True;
  if DSqln.Checked then DoQLN.Checked := True;
end;

//If webserver upload is enabled, sample time must be set to one minute
procedure Tmainform.DoUploadWSChange(Sender: TObject);
begin
  if DoUploadWS.Checked then SampleRateEdit.Value := 60;
end;

// **** Data Store and Data Upload ****

//Process change of data store location
procedure Tmainform.DataFolderEditChange(Sender: TObject);
var
  OldPath: string;
begin
  OldPath := DataStorePath;
  DataStorePath := IncludeTrailingPathDelimiter(DataFolderEdit.Directory);
  if ForceDirectories(DataStorePath) then
  begin
    {$IFNDEF Windows}
    FpChmod(DataStorePath, &775);
    {$ENDIF}
    MiscString := 'Data store folder is ' + DataStorePath;
    LogEvent(leMisc);
  end
  else begin
    MiscString := 'Unable to create new directory "' + DataStorePath + '. Previous location retained.';
    LogEvent(leMisc);
    DataStorePath := OldPath;
    DataFolderEdit.Directory := DataStorePath;
  end;
end;

//Reveal or obscure data store upload password
procedure Tmainform.DSRevealFTPpasswordChange(Sender: TObject);
begin
  if DSRevealFTPpassword.Checked then DSUploadPassword.EchoMode := emNormal
  else DSUploadPassword.EchoMode := emPassword;
end;

//Reveal or obscure webserver upload password
procedure Tmainform.WSRevealFTPpasswordChange(Sender: TObject);
begin
  if WSRevealFTPPassword.Checked then WSUploadPassword.EchoMode := emNormal
  else WSUploadPassword.EchoMode := emPassword;
end;

//Reveal or obscure snapshots upload password
procedure Tmainform.SSRevealFTPpasswordChange(Sender: TObject);
begin
  if SSRevealFTPpassword.Checked then SSUploadPassword.EchoMode := emNormal
  else SSUploadPassword.EchoMode := emPassword;
end;

//Limit characters in port box to numerals + backspace and delete
procedure Tmainform.WSFTPPortKeyPress(Sender: TObject; var Key: char);
begin
  if not (Key in ['0'..'9', #8, #127]) then Key := #0;
end;

//Make all versions of the curl location equal
procedure Tmainform.DSCurlLocationChange(Sender: TObject);
begin
  if Sender = DSCurlLocation then
  begin
    WSCurlLocation.Filename := DSCurlLocation.Filename;
    SSCurlLocation.Filename := DSCurlLocation.Filename;
  end
  else if Sender = WSCurlLocation then
  begin
    DSCurlLocation.Filename := WSCurlLocation.Filename;
    SSCurlLocation.Filename := WSCurlLocation.Filename;
  end
  else if Sender = SSCurlLocation then
  begin
    DSCurlLocation.Filename := SSCurlLocation.Filename;
    WSCurlLocation.Filename := SSCurlLocation.Filename;
  end;
end;

//If snapshots are uploaded then name order must be selected
procedure Tmainform.DoUploadSSChange(Sender: TObject);
begin
  if DoUploadSS.Checked then NameOrder.Checked := True;
end;

//Upload files by FTP using curl

//Helper function which does most of the work
function TMainForm.RunProcess(Proc: TProcess; out OutputString, StderrString: string; out ExitStatus: Integer): Integer;
var
  NumBytes, BytesRead, Available, OutputLength, StderrLength, StderrNumBytes, StderrBytesRead: Integer;
const
  READ_BYTES = 65536;
begin
  Result := -1;
  try
    try
      Proc.Options := Proc.Options + [poUsePipes, poNoConsole];
      BytesRead := 0;
      OutputLength := 0;
      StderrBytesRead := 0;
      StderrLength := 0;
      Proc.Execute;
      while Proc.Running do
      begin
        Available := Proc.Output.NumBytesAvailable;
        if Available > 0 then
        begin
          if (BytesRead + Available > OutputLength) then
          begin
            OutputLength := BytesRead + READ_BYTES;
            SetLength(OutputString, OutputLength);
          end;
          NumBytes := Proc.Output.Read(OutputString[BytesRead + 1], Available);
          if NumBytes > 0 then Inc(BytesRead, NumBytes);
        end
        else if Assigned(Proc.Stderr) and (Proc.Stderr.NumBytesAvailable > 0) then
        begin
          Available := Proc.Stderr.NumBytesAvailable;
          if (StderrBytesRead + Available > StderrLength) then
          begin
            StderrLength := StderrBytesRead + READ_BYTES;
            SetLength(StderrString, StderrLength);
          end;
          StderrNumBytes := Proc.Stderr.Read(StderrString[StderrBytesRead + 1], Available);
          if StderrNumBytes > 0 then Inc(StderrBytesRead, StderrNumBytes);
        end
        else Sleep(100);
      end;
      Available := Proc.Output.NumBytesAvailable;
      while Available > 0 do
      begin
        if (BytesRead + Available > OutputLength) then
        begin
          OutputLength := BytesRead + READ_BYTES;
          SetLength(OutputString, OutputLength);
        end;
        NumBytes := Proc.Output.Read(OutputString[BytesRead + 1], Available);
        if NumBytes > 0 then Inc(BytesRead, NumBytes);
        Available := Proc.Output.NumBytesAvailable;
      end;
      SetLength(OutputString, BytesRead);
      while Assigned(Proc.Stderr) and (Proc.Stderr.NumBytesAvailable > 0) do
      begin
        Available := Proc.Stderr.NumBytesAvailable;
        if (StderrBytesRead + Available > StderrLength) then
        begin
          StderrLength := StderrBytesRead + READ_BYTES;
          SetLength(StderrString, StderrLength);
        end;
        StderrNumBytes := Proc.Stderr.Read(StderrString[StderrBytesRead + 1], Available);
        if StderrNumBytes > 0 then Inc(StderrBytesRead, StderrNumBytes);
      end;
      SetLength(StderrString, StderrBytesRead);
      ExitStatus := Proc.ExitStatus;
      Result := 0;
    except
      on E:Exception do
      begin
        Result := 1;
        SetLength(OutputString, BytesRead);
      end;
    end;
  finally
    Proc.Free;
  end;
end;

//Set up the uploads
function TMainform.CurlUploadFiles(Source: TUploadSource; LocalDir, RemoteURL, RemoteDir, Username, Password, Port: string;
     FTPFiles: TStringList): Boolean;
var
  N: Word;
  CurlString, DayString, CommandString, OutputString, StderrString: string;
  Params: array[0..7] of string;
  ExitStatus: Integer;

  function TopandTail(InString: string): string;
  begin
    Result := '';
    if InString = '' then Exit;
    if (InString[1] = '/') or (InString[1] = '\') then InString := RightStr(InString, Length(InString) - 1);
    if InString = '' then Exit;
    if (InString[Length(InString)] = '/') or (InString[Length(InString)] = '\') then InString := LeftStr(InString, Length(InString) - 1);
    Result := InString;
  end;

begin
  Result := False;
  N := 0;
  if FTPFiles.Count = 0 then
  begin
    MiscString := 'No files were specified for FTP upload';
    LogEvent(leMisc);
    Exit;
  end;
  LocalDir := IncludeTrailingPathDelimiter(LocalDir);
  {$IFDEF Windows}
  LocalDir := AnsiReplaceStr(LocalDir, '\', '/');    //Change backslash to slash to suit curl
  {$ENDIF}
  RemoteURL := TopandTail(RemoteURL);
  RemoteDir := TopandTail(RemoteDir);
  DayString := FormatDateTime('yyyy"-"mm"-"dd', TNow);
  {$IFDEF Windows}
  CurlString := 'curl';
  {$ELSE}
  CurlString := WSCurlLocation.Filename;
  if not FileExists(WSCurlLocation.Filename) then
  begin
    MiscString := 'Curl executable not found. Is it installed, and is the location correct?';
    LogEvent(leMisc);
    Exit;
  end;
  {$ENDIF}
  Params[0] := '-k';
  Params[1] := '--connect-timeout 5';
  Params[2] := '--max-time 60';
  if (Source = usDataStore) or (Source = usDSTest) then
  begin
    if DSFTPButton.Checked then       //Plain FTP
    begin
      Params[3] := '--silent';
      Params[4] := '"ftp://';
    end
    else if DSTLSButton.Checked then  //FTP with TLS v1.2
    begin
      Params[3] := '--tlsv1.2';
      Params[4] := '"ftp://';
    end
    else if DSFTPSButton.Checked then //FTPS
    begin
      Params[3] := '--ssl';
      Params[4] := '"ftp://';
    end
    else begin                        //SFTP
      Params[3] := '--silent';
      Params[4] := '"sftp://';
    end;
  end
  else if (Source = usSnapshots) or (Source = usSSTest) then
  begin
    if SSFTPButton.Checked then       //Plain FTP
    begin
      Params[3] := '--silent';        //Just a do-nothing parameter
      Params[4] := '"ftp://';
    end
    else if SSTLSButton.Checked then  //FTP with TLS v1.2
    begin
      Params[3] := '--tlsv1.2';
      Params[4] := '"ftp://';
    end
    else if SSFTPSButton.Checked then //FTPS
    begin
      Params[3] := '--ssl';
      Params[4] := '"ftp://';
    end
    else begin                        //SFTP
      Params[3] := '--silent';
      Params[4] := '"sftp://';
    end;
  end
  else if (Source = usWebserver) or (Source = usWSTest) then
  begin
    if WSFTPButton.Checked then       //Plain FTP
    begin
      Params[3] := '--silent';        //Just a do-nothing parameter
      Params[4] := '"ftp://';
    end
    else if WSTLSButton.Checked then  //FTP with TLS v1.2
    begin
      Params[3] := '--tlsv1.2';
      Params[4] := '"ftp://';
    end
    else if WSFTPSButton.Checked then //FTPS
    begin
      Params[3] := '--ssl';
      Params[4] := '"ftp://';
    end
    else begin                        //SFTP
      Params[3] := '--silent';
      Params[4] := '"sftp://';
    end;
  end;
  Params[4] := Params[4] + RemoteURL;
  if Port > '' then Params[4] := Params[4] + ':' + Port;
  Params[4] := Params[4] + '/';
  Params[5] := '--user "' +  Username + ':' + Password + '"';
  Params[6] := '-T "' + LocalDir + '{' + FTPFiles[0];
  for N := 1 to FTPFiles.Count - 1 do
    Params[6] := Params[6] + ',' + FTPFiles[N];
  Params[6] := Params[6] + '}"';
  Params[7] := '--ftp-create-dirs';

  //Test login first
  CommandString := CurlString + ' ' + Params[0] + ' ' + Params[1] + ' ' + Params[3] + ' ' + Params[4] + '" ' + Params[5];
  FTPProcess := TProcess.Create(nil);
  FTPProcess.CommandLine := CommandString;
  RunProcess(FTPProcess, OutputString, StderrString, ExitStatus);
  if ExitStatus <> 0 then
  begin
    MiscString := 'Unable to login to FTP server. Upload aborted.';
    LogEvent(leMisc);
    Exit;
  end;

  //Now upload the files
  if RemoteDir > '' then Params[4] := Params[4] + RemoteDir + '/'
  else Params[4] := Params[4] + '/';
  if (Source = usDataStore){ or (Source = usSnapshots)} then Params[4] := Params[4] + DayString + '/';
  Params[4] := Params[4] + '"';
  CommandString := CurlString + ' ' + Params[0] + ' ' + Params[1] + ' ' + Params[2] + ' ' + Params[3] + ' ' + Params[4]
                  + ' ' + Params[5] + ' ' + Params[6] + ' ' + Params[7];
  FTPProcess := TProcess.Create(nil);
  FTPProcess.CommandLine := CommandString;
  RunProcess(FTPProcess, OutputString, StderrString, ExitStatus);
  if ExitStatus <> 0 then
  begin
    MiscString := 'FTP upload failed';
    LogEvent(leMisc);
    Exit;
  end;
  Result := True;
end;

//If data upload is enabled then data save and single stats file must also be enabled
procedure Tmainform.DoUploadDataChange(Sender: TObject);
begin
  if DoUploadData.Checked then
  begin
    DoSaveData.Checked := True;
    SingleButton.Checked := True;
  end;
end;

//If "Include in upload" is enabled then the item must also be saved
{procedure Tmainform.DSbitsUploadChange(Sender: TObject);
begin
  if (Sender = DSbitsUpload) and DSbitsUpload.Checked then DSbits.Checked := True;
  if (Sender = DSsnrUpload) and DSsnrUpload.Checked then DSsnr.Checked := True;
  if (Sender = DSqlnUpload) and DSqlnUpload.Checked then DSqln.Checked := True;
  if (Sender = DShlogUpload) and DShlogUpload.Checked then DShlog.Checked := True;
end;  }

//Disallow switching to multiple stats files if DoUploadData is checked;
procedure Tmainform.MultipleButtonChange(Sender: TObject);
begin
  if MultipleButton.Checked and DoUploadData.Checked then SingleButton.Checked := True;
end;

//Disallow "Include in upload" if item is not being saved
{procedure Tmainform.DSbitsChange(Sender: TObject);
begin
  if (Sender = DSbits) and not DSbits.Checked then DSbitsUpload.Checked := False;
  if (Sender = DSsnr) and not DSsnr.Checked then DSsnrUpload.Checked := False;
  if (Sender = DSqln) and not DSqln.Checked then DSqlnUpload.Checked := False;
  if (Sender = DShlog) and not DShlog.Checked then DShlogUpload.Checked := False;
end; }

//Upload two small files to test the data store upload configuration
procedure Tmainform.TestUploadButtonClick(Sender: TObject);
var
  TestList: TStringList;
begin
  TestList := TStringList.Create;
  TestList.Add('FTPtest1');
  TestList.Add('FTPtest2');
  try
    if CurlUploadFiles(usDSTest, DataDir, DSUploadURL.Text, DSUploadDir.Text, DSUploadUsername.Text, DSUploadPassword.Text, DSFTPPort.Text, TestList) then
    begin
      FTPResult.Caption := 'Test succeeded';
      FTPResult.Font.Color := clGreen;
    end
    else begin
      FTPResult.Caption := 'Test upload failed - see event log';
      FTPResult.Font.Color := clRed;
    end;
  finally
    TestList.Free;
  end;
end;

//Upload two small files to test the webserver upload configuration
procedure Tmainform.WSTestUploadButtonClick(Sender: TObject);
var
  TestList: TStringList;
begin
  TestList := TStringList.Create;
  TestList.Add('FTPtest1');
  TestList.Add('FTPtest2');
  try
    if CurlUploadFiles(usWSTest, DataDir, WSUploadURL.Text, WSUploadDir.Text, WSUploadUsername.Text, WSUploadPassword.Text, WSFTPPort.Text, TestList) then
    begin
      WSFTPResult.Caption := 'Test succeeded';
      WSFTPResult.Font.Color := clGreen;
    end
    else begin
      WSFTPResult.Caption := 'Test upload failed - see event log';
      WSFTPResult.Font.Color := clRed;
    end;
  finally
    TestList.Free;
  end;
end;

//Upload two small files to test the snapshots upload configuration
procedure Tmainform.SSTestUploadButtonClick(Sender: TObject);
var
  TestList: TStringList;
begin
  TestList := TStringList.Create;
  TestList.Add('FTPtest1');
  TestList.Add('FTPtest2');
  try
    if CurlUploadFiles(usSSTest, DataDir, SSUploadURL.Text, SSUploadDir.Text, SSUploadUsername.Text, SSUploadPassword.Text, SSFTPPort.Text, TestList) then
    begin
      SSFTPResult.Caption := 'Test succeeded';
      SSFTPResult.Font.Color := clGreen;
    end
    else begin
      SSFTPResult.Caption := 'Test upload failed - see event log';
      SSFTPResult.Font.Color := clRed;
    end;
  finally
    TestList.Free;
  end;
end;

procedure Tmainform.WSUploadDirKeyPress(Sender: TObject; var Key: char);
begin
  if Key = #32 then Key := #0;
end;

//Clear any previous test result when changing the details
procedure Tmainform.DSUploadURLChange(Sender: TObject);
begin
  if (Sender as TControl).Parent = DSUploadTab then FTPResult.Caption := ''
  else if (Sender as TControl).Parent = WSUploadTab then WSFTPResult.Caption := ''
  else if (Sender as TControl).Parent = SSUploadTab then SSFTPResult.Caption := '';
end;

end.


